package ext {

[Uses("ext.SClassManager")]
[Native("Ext.ClassManager", require)]
/**
 * <a href="#!/api/Ext.ClassManager" rel="Ext.ClassManager" class="docClass">Ext.ClassManager</a> manages all classes and handles mapping from string class name to
 * actual class objects throughout the whole framework. It is not generally accessed directly, rather through
 * these convenient shorthands:
 *
 * <ul>
 * <li><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a></li>
 * <li><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a></li>
 * <li><a href="#!/api/Ext-method-widget" rel="Ext-method-widget" class="docClass">Ext.widget</a></li>
 * <li><a href="#!/api/Ext-method-getClass" rel="Ext-method-getClass" class="docClass">Ext.getClass</a></li>
 * <li><a href="#!/api/Ext-method-getClassName" rel="Ext-method-getClassName" class="docClass">Ext.getClassName</a></li>
 * </ul>
 *
 *
 * <h1>Basic syntax:</h1>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>(className, properties);
 * </code></pre>
 *
 * <p>in which <code>properties</code> is an object represent a collection of properties that apply to the class. See
 * <a href="#!/api/Ext.ClassManager-method-create" rel="Ext.ClassManager-method-create" class="docClass">create</a> for more detailed instructions.</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Person', {
 *      name: 'Unknown',
 *
 *      constructor: function(name) {
 *          if (name) {
 *              this.name = name;
 *          }
 *      },
 *
 *      eat: function(foodType) {
 *          alert("I'm eating: " + foodType);
 *
 *          return this;
 *      }
 * });
 *
 * var aaron = new Person("Aaron");
 * aaron.eat("Sandwich"); // alert("I'm eating: Sandwich");
 * </code></pre>
 *
 * <p><a href="#!/api/Ext.Class" rel="Ext.Class" class="docClass">Ext.Class</a> has a powerful set of extensible <a href="#!/api/Ext.Class-static-method-registerPreprocessor" rel="Ext.Class-static-method-registerPreprocessor" class="docClass">pre-processors</a> which takes care of
 * everything related to class creation, including but not limited to inheritance, mixins, configuration, statics, etc.</p>
 *
 * <h1>Inheritance:</h1>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Developer', {
 *      extend: 'Person',
 *
 *      constructor: function(name, isGeek) {
 *          this.isGeek = isGeek;
 *
 *          // Apply a method from the parent class' prototype
 *          this.callParent([name]);
 *      },
 *
 *      code: function(language) {
 *          alert("I'm coding in: " + language);
 *
 *          this.eat("Bugs");
 *
 *          return this;
 *      }
 * });
 *
 * var jacky = new Developer("Jacky", true);
 * jacky.code("JavaScript"); // alert("I'm coding in: JavaScript");
 *                           // alert("I'm eating: Bugs");
 * </code></pre>
 *
 * <p>See <a href="#!/api/Ext.Base-method-callParent" rel="Ext.Base-method-callParent" class="docClass">Ext.Base.callParent</a> for more details on calling superclass' methods</p>
 *
 * <h1>Mixins:</h1>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('CanPlayGuitar', {
 *      playGuitar: function() {
 *         alert("F#...G...D...A");
 *      }
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('CanComposeSongs', {
 *      composeSongs: function() { ... }
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('CanSing', {
 *      sing: function() {
 *          alert("For he's a jolly good fellow...")
 *      }
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Musician', {
 *      extend: 'Person',
 *
 *      mixins: {
 *          canPlayGuitar: 'CanPlayGuitar',
 *          canComposeSongs: 'CanComposeSongs',
 *          canSing: 'CanSing'
 *      }
 * })
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('CoolPerson', {
 *      extend: 'Person',
 *
 *      mixins: {
 *          canPlayGuitar: 'CanPlayGuitar',
 *          canSing: 'CanSing'
 *      },
 *
 *      sing: function() {
 *          alert("Ahem....");
 *
 *          this.mixins.canSing.sing.call(this);
 *
 *          alert("[Playing guitar at the same time...]");
 *
 *          this.playGuitar();
 *      }
 * });
 *
 * var me = new CoolPerson("Jacky");
 *
 * me.sing(); // alert("Ahem...");
 *            // alert("For he's a jolly good fellow...");
 *            // alert("[Playing guitar at the same time...]");
 *            // alert("F#...G...D...A");
 * </code></pre>
 *
 * <h1>Config:</h1>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('SmartPhone', {
 *      config: {
 *          hasTouchScreen: false,
 *          operatingSystem: 'Other',
 *          price: 500
 *      },
 *
 *      isExpensive: false,
 *
 *      constructor: function(config) {
 *          this.initConfig(config);
 *      },
 *
 *      applyPrice: function(price) {
 *          this.isExpensive = (price &gt; 500);
 *
 *          return price;
 *      },
 *
 *      applyOperatingSystem: function(operatingSystem) {
 *          if (!(/^(iOS|Android|BlackBerry)$/i).test(operatingSystem)) {
 *              return 'Other';
 *          }
 *
 *          return operatingSystem;
 *      }
 * });
 *
 * var iPhone = new SmartPhone({
 *      hasTouchScreen: true,
 *      operatingSystem: 'iOS'
 * });
 *
 * iPhone.getPrice(); // 500;
 * iPhone.getOperatingSystem(); // 'iOS'
 * iPhone.getHasTouchScreen(); // true;
 *
 * iPhone.isExpensive; // false;
 * iPhone.setPrice(600);
 * iPhone.getPrice(); // 600
 * iPhone.isExpensive; // true;
 *
 * iPhone.setOperatingSystem('AlienOS');
 * iPhone.getOperatingSystem(); // 'Other'
 * </code></pre>
 *
 * <h1>Statics:</h1>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Computer', {
 *      statics: {
 *          factory: function(brand) {
 *             // 'this' in static methods refer to the class itself
 *              return new this(brand);
 *          }
 *      },
 *
 *      constructor: function() { ... }
 * });
 *
 * var dellComputer = Computer.factory('Dell');
 * </code></pre>
 *
 * <p>Also see <a href="#!/api/Ext.Base-method-statics" rel="Ext.Base-method-statics" class="docClass">Ext.Base.statics</a> and <a href="#!/api/Ext.Base-property-self" rel="Ext.Base-property-self" class="docClass">Ext.Base.self</a> for more details on accessing
 * static properties within class methods</p>
 */
public const ClassManager:ext.SClassManager;
}