package ext {

[Native("Ext.Factory", require)]
/**
 * Manages factories for families of classes (classes with a common <code>alias</code> prefix). The
 * factory for a class family is a function stored as a <code>static</code> on <code><a href="#!/api/Ext.Factory" rel="Ext.Factory" class="docClass">Ext.Factory</a></code>. These
 * are created either by directly calling <code><a href="#!/api/Ext.Factory-static-method-define" rel="Ext.Factory-static-method-define" class="docClass">Ext.Factory.define</a></code> or by using the
 * <code><a href="#!/api/Ext.mixin.Factoryable" rel="Ext.mixin.Factoryable" class="docClass">Ext.mixin.Factoryable</a></code> interface.
 *
 * <p>To illustrate, consider the layout system's use of aliases. The <code>hbox</code> layout maps to
 * the <code>"layout.hbox"</code> alias that one typically provides via the <code>layout</code> config on a
 * Container.</p>
 *
 * <p>Under the covers this maps to a call like this:</p>
 *
 * <pre><code> Ext.Factory.layout('hbox');
 * </code></pre>
 *
 * <p>Or possibly:</p>
 *
 * <pre><code> Ext.Factory.layout({
 *      type: 'hbox'
 *  });
 * </code></pre>
 *
 * <p>The value of the <code>layout</code> config is passed to the <code>Ext.Factory.layout</code> function. The
 * exact signature of a factory method matches <code><a href="#!/api/Ext.Factory-method-create" rel="Ext.Factory-method-create" class="docClass">create</a></code>.</p>
 *
 * <p>To define this factory directly, one could call <code><a href="#!/api/Ext.Factory-static-method-define" rel="Ext.Factory-static-method-define" class="docClass">Ext.Factory.define</a></code> like so:</p>
 *
 * <pre><code> <a href="#!/api/Ext.Factory-static-method-define" rel="Ext.Factory-static-method-define" class="docClass">Ext.Factory.define</a>('layout', 'auto');  // "layout.auto" is the default type
 * </code></pre>
 */
public class Factory {
  /**
   * For example, the layout alias family could be defined like this:
   *
   * <pre><code> <a href="#!/api/Ext.Factory-static-method-define" rel="Ext.Factory-static-method-define" class="docClass">Ext.Factory.define</a>('layout', {
   *      defaultType: 'auto'
   *  });
   * </code></pre>
   *
   * <p>To define multiple families at once:</p>
   *
   * <pre><code> <a href="#!/api/Ext.Factory-static-method-define" rel="Ext.Factory-static-method-define" class="docClass">Ext.Factory.define</a>({
   *      layout: {
   *          defaultType: 'auto'
   *      }
   *  });
   * </code></pre>
   * @since 5.0.0
   * @param type The alias prefix for type (e.g., "layout.").
   * @param config An object specifying the config for the <code><a href="#!/api/Ext.Factory" rel="Ext.Factory" class="docClass">Ext.Factory</a></code>
   * to be created. If a string is passed it is treated as the <code>defaultType</code>.
   */
  public static native function define(type:String, config:* = undefined):Function;

  /**
   * @param config @inheritDoc
   */
  public native function Factory(config:ext.Factory = null);

  [ExtConfig]
  /**
   * The prefix to apply to <code>type</code> values to form a complete alias. This defaults to the
   * proper value in most all cases and should not need to be specified.
   * @since 5.0.0
   */
  public native function get aliasPrefix():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set aliasPrefix(value:String):void;

  [ExtConfig]
  /**
   * The config property to set when the factory is given a config that is a string.
   * @default "type"
   * @since 5.0.0
   */
  public native function get defaultProperty():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultProperty(value:String):void;

  [ExtConfig]
  /**
   * An optional type to use if none is given to the factory at invocation. This is a
   * suffix added to the <code>aliasPrefix</code>. For example, if <code>aliasPrefix="layout."</code> and
   * <code>defaultType="hbox"</code> the default alias is <code>"layout.hbox"</code>. This is an alternative
   * to <code>xclass</code> so only one should be provided.
   * @default null
   * @since 5.0.0
   */
  public native function get defaultType():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultType(value:String):void;

  [ExtConfig]
  /**
   * The property that identifies an object as instance vs a config.
   * @default "isInstance"
   * @since 5.0.0
   */
  public native function get instanceProp():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set instanceProp(value:String):void;

  [ExtConfig]
  /**
   * The full classname of the type of instance to create when none is provided to the
   * factory. This is an alternative to <code>defaultType</code> so only one should be specified.
   * @default null
   * @since 5.0.0
   */
  public native function get xclass():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set xclass(value:String):void;

  /**
   * Creates an instance of this class family given configuration options.
   * @since 5.0.0
   * @param config The configuration or instance (if an Object) or
   * just the type (if a String) describing the instance to create.
   * @param {String} [config.xclass] The full class name of the class to create.
   * @param {String} [config.type] The type string to add to the alias prefix for this
   * factory.
   * @param defaultType The type to create if no type is contained in the
   * <code>config</code>, or an object containing a default set of configs.
   * @return The newly created instance.
   */
  public native function create(config:* = undefined, defaultType:* = undefined):Object;

  /**
   * Returns an instance of an ID generator based on the ID you pass in.
   * @return Ext.data.identifier.* The data identifier
   */
  public native function dataIdentifier(id:String):Object;
}
}