package ext {

[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.Img", require)]
/**
 * Simple helper class for easily creating image components. This renders an image tag to
 * the DOM with the configured src.
 *
 * <p><p><img src="" alt="Ext.Img component" width="" height=""/></p></p>
 *
 * <h2>Example usage:</h2>
 *
 * <pre><code>var changingImage = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.Img" rel="Ext.Img" class="docClass">Ext.Img</a>', {
 *     src: 'http://www.sencha.com/img/20110215-feat-html5.png',
 *     width: 184,
 *     height: 90,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 *
 * // change the src of the image programmatically
 * changingImage.setSrc('http://www.sencha.com/img/20110215-feat-perf.png');
 * </code></pre>
 *
 * <p>By default, only an img element is rendered and that is this component's primary
 * <a href="#!/api/Ext.Component-method-getEl" rel="Ext.Component-method-getEl" class="docClass">element</a>. If the <a href="#!/api/Ext.Component-cfg-autoEl" rel="Ext.Component-cfg-autoEl" class="docClass">Ext.Component.autoEl</a> property
 * is other than 'img' (the default), the a child img element will be added to the primary
 * element. This can be used to create a wrapper element around the img.</p>
 *
 * <h2>Wrapping the img in a div:</h2>
 *
 * <pre><code>var wrappedImage = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.Img" rel="Ext.Img" class="docClass">Ext.Img</a>', {
 *     src: 'http://www.sencha.com/img/20110215-feat-html5.png',
 *     autoEl: 'div', // wrap in a div
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <h2>Using a glyph</h2>
 *
 * <pre><code>var glyphImage = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.Img" rel="Ext.Img" class="docClass">Ext.Img</a>', {
 *     glyph: 'xf015&#64;FontAwesome',     // the "home" icon
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <h2>Image Dimensions</h2>
 *
 * <p>You should include height and width dimensions for any image owned by a parent
 * container.  By omitting dimensions, an owning container will not know how to
 * size and position the image in the initial layout.</p>
 */
public class Img extends ext.Component {
  /**
   * @param config @inheritDoc
   */
  public function Img(config:ext.Img = null) {
    super();
  }

  [ExtConfig]
  /**
   * The descriptive text for non-visual UI description.
   * @default ''
   * @see #setAlt()
   */
  public native function get alt():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set alt(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A numeric unicode character code to serve as the image.  If this option is used
   * The image will be rendered using a div with innerHTML set to the html entity
   * for the given character code.  The default font-family for glyphs can be set
   * globally using <a href="#!/api/Ext-method-setGlyphFontFamily" rel="Ext-method-setGlyphFontFamily" class="docClass">Ext.setGlyphFontFamily()</a>. Alternatively,
   * this config option accepts a string with the charCode and font-family separated by
   * the <code>&#64;</code> symbol. For example '65&#64;My Font Family'.
   * @see #getGlyph()
   * @see #setGlyph()
   */
  public native function get glyph():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set glyph(value:*):void;

  [ExtConfig]
  /**
   * Optional CSS classes to add to the img element.
   * @default ''
   */
  public native function get imgCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set imgCls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The source of this image. See <a href="#!/api/Ext-method-resolveResource" rel="Ext-method-resolveResource" class="docClass">Ext.resolveResource</a> for
   * details on locating application resources.
   * @see #getSrc()
   * @see #setSrc()
   */
  public native function get src():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set src(value:String):void;

  [ExtConfig]
  /**
   * Specifies addtional information about the image.
   * @default ''
   * @see #setTitle()
   */
  public native function get title():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set title(value:String):void;

  /**
   * Returns the value of <a href="#!/api/Ext.Img-cfg-glyph" rel="Ext.Img-cfg-glyph" class="docClass">glyph</a>.
   */
  public native function getGlyph():*;

  /**
   * Returns the size of the image as an object.
   * @since 6.2.0
   * @return The size and aspect ratio of the image.
   * <ul>
   *   <li><code>aspect</code> : Number
   * The aspect ration of the image (<code>width / height</code>).
   *   </li>
   *   <li><code>height</code> : Number
   * The height of the image.
   *   </li>
   *   <li><code>width</code> : Number
   * The width of the image.
   *   </li>
   * </ul>
   */
  public native function getNaturalSize():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.Img-cfg-src" rel="Ext.Img-cfg-src" class="docClass">src</a>.
   */
  public native function getSrc():String;

  /**
   * Updates the <a href="#!/api/Ext.Img-cfg-alt" rel="Ext.Img-cfg-alt" class="docClass">alt</a> of the image.
   */
  public native function setAlt(alt:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Img-cfg-glyph" rel="Ext.Img-cfg-glyph" class="docClass">glyph</a>.
   * @param glyph The new value.
   */
  public native function setGlyph(glyph:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Img-cfg-src" rel="Ext.Img-cfg-src" class="docClass">src</a>.
   * @param src The new value.
   */
  public native function setSrc(src:String):void;

  /**
   * Updates the <a href="#!/api/Ext.Img-cfg-title" rel="Ext.Img-cfg-title" class="docClass">title</a> of the image.
   */
  public native function setTitle(title:String):void;
}
}