package ext {

[Native]
/**
 * <a href="#!/api/Ext.ClassManager" rel="Ext.ClassManager" class="docClass">Ext.ClassManager</a> manages all classes and handles mapping from string class name to
 * actual class objects throughout the whole framework. It is not generally accessed directly, rather through
 * these convenient shorthands:
 *
 * <ul>
 * <li><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a></li>
 * <li><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a></li>
 * <li><a href="#!/api/Ext-method-widget" rel="Ext-method-widget" class="docClass">Ext.widget</a></li>
 * <li><a href="#!/api/Ext-method-getClass" rel="Ext-method-getClass" class="docClass">Ext.getClass</a></li>
 * <li><a href="#!/api/Ext-method-getClassName" rel="Ext-method-getClassName" class="docClass">Ext.getClassName</a></li>
 * </ul>
 *
 *
 * <h1>Basic syntax:</h1>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>(className, properties);
 * </code></pre>
 *
 * <p>in which <code>properties</code> is an object represent a collection of properties that apply to the class. See
 * <a href="#!/api/Ext.ClassManager-method-create" rel="Ext.ClassManager-method-create" class="docClass">create</a> for more detailed instructions.</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Person', {
 *      name: 'Unknown',
 *
 *      constructor: function(name) {
 *          if (name) {
 *              this.name = name;
 *          }
 *      },
 *
 *      eat: function(foodType) {
 *          alert("I'm eating: " + foodType);
 *
 *          return this;
 *      }
 * });
 *
 * var aaron = new Person("Aaron");
 * aaron.eat("Sandwich"); // alert("I'm eating: Sandwich");
 * </code></pre>
 *
 * <p><a href="#!/api/Ext.Class" rel="Ext.Class" class="docClass">Ext.Class</a> has a powerful set of extensible <a href="#!/api/Ext.Class-static-method-registerPreprocessor" rel="Ext.Class-static-method-registerPreprocessor" class="docClass">pre-processors</a> which takes care of
 * everything related to class creation, including but not limited to inheritance, mixins, configuration, statics, etc.</p>
 *
 * <h1>Inheritance:</h1>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Developer', {
 *      extend: 'Person',
 *
 *      constructor: function(name, isGeek) {
 *          this.isGeek = isGeek;
 *
 *          // Apply a method from the parent class' prototype
 *          this.callParent([name]);
 *      },
 *
 *      code: function(language) {
 *          alert("I'm coding in: " + language);
 *
 *          this.eat("Bugs");
 *
 *          return this;
 *      }
 * });
 *
 * var jacky = new Developer("Jacky", true);
 * jacky.code("JavaScript"); // alert("I'm coding in: JavaScript");
 *                           // alert("I'm eating: Bugs");
 * </code></pre>
 *
 * <p>See <a href="#!/api/Ext.Base-method-callParent" rel="Ext.Base-method-callParent" class="docClass">Ext.Base.callParent</a> for more details on calling superclass' methods</p>
 *
 * <h1>Mixins:</h1>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('CanPlayGuitar', {
 *      playGuitar: function() {
 *         alert("F#...G...D...A");
 *      }
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('CanComposeSongs', {
 *      composeSongs: function() { ... }
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('CanSing', {
 *      sing: function() {
 *          alert("For he's a jolly good fellow...")
 *      }
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Musician', {
 *      extend: 'Person',
 *
 *      mixins: {
 *          canPlayGuitar: 'CanPlayGuitar',
 *          canComposeSongs: 'CanComposeSongs',
 *          canSing: 'CanSing'
 *      }
 * })
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('CoolPerson', {
 *      extend: 'Person',
 *
 *      mixins: {
 *          canPlayGuitar: 'CanPlayGuitar',
 *          canSing: 'CanSing'
 *      },
 *
 *      sing: function() {
 *          alert("Ahem....");
 *
 *          this.mixins.canSing.sing.call(this);
 *
 *          alert("[Playing guitar at the same time...]");
 *
 *          this.playGuitar();
 *      }
 * });
 *
 * var me = new CoolPerson("Jacky");
 *
 * me.sing(); // alert("Ahem...");
 *            // alert("For he's a jolly good fellow...");
 *            // alert("[Playing guitar at the same time...]");
 *            // alert("F#...G...D...A");
 * </code></pre>
 *
 * <h1>Config:</h1>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('SmartPhone', {
 *      config: {
 *          hasTouchScreen: false,
 *          operatingSystem: 'Other',
 *          price: 500
 *      },
 *
 *      isExpensive: false,
 *
 *      constructor: function(config) {
 *          this.initConfig(config);
 *      },
 *
 *      applyPrice: function(price) {
 *          this.isExpensive = (price &gt; 500);
 *
 *          return price;
 *      },
 *
 *      applyOperatingSystem: function(operatingSystem) {
 *          if (!(/^(iOS|Android|BlackBerry)$/i).test(operatingSystem)) {
 *              return 'Other';
 *          }
 *
 *          return operatingSystem;
 *      }
 * });
 *
 * var iPhone = new SmartPhone({
 *      hasTouchScreen: true,
 *      operatingSystem: 'iOS'
 * });
 *
 * iPhone.getPrice(); // 500;
 * iPhone.getOperatingSystem(); // 'iOS'
 * iPhone.getHasTouchScreen(); // true;
 *
 * iPhone.isExpensive; // false;
 * iPhone.setPrice(600);
 * iPhone.getPrice(); // 600
 * iPhone.isExpensive; // true;
 *
 * iPhone.setOperatingSystem('AlienOS');
 * iPhone.getOperatingSystem(); // 'Other'
 * </code></pre>
 *
 * <h1>Statics:</h1>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Computer', {
 *      statics: {
 *          factory: function(brand) {
 *             // 'this' in static methods refer to the class itself
 *              return new this(brand);
 *          }
 *      },
 *
 *      constructor: function() { ... }
 * });
 *
 * var dellComputer = Computer.factory('Dell');
 * </code></pre>
 *
 * <p>Also see <a href="#!/api/Ext.Base-method-statics" rel="Ext.Base-method-statics" class="docClass">Ext.Base.statics</a> and <a href="#!/api/Ext.Base-property-self" rel="Ext.Base-property-self" class="docClass">Ext.Base.self</a> for more details on accessing
 * static properties within class methods</p>
 * <p>Type of singleton ClassManager.</p>
 * @see ext.#ClassManager ext.ClassManager
 */
public class SClassManager {
  [ExtConfig]
  /**
   * A collection of diagnostic methods to decorate the real methods of the class. These
   * methods are applied as an <code>override</code> if this class has debug enabled as defined by
   * <code><a href="#!/api/Ext-method-isDebugEnabled" rel="Ext-method-isDebugEnabled" class="docClass">Ext.isDebugEnabled</a></code>.
   *
   * <p>These will be automatically removed by the Sencha Cmd compiler for production builds.</p>
   *
   * <p>Example usage:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.bar.Class', {
   *      foo: function (a, b, c) {
   *          ...
   *      },
   *
   *      bar: function (a, b) {
   *          ...
   *          return 42;
   *      },
   *
   *      debugHooks: {
   *          foo: function (a, b, c) {
   *              // check arguments...
   *              return this.callParent(arguments);
   *          }
   *      }
   *  });
   * </code></pre>
   *
   * <p>If you specify a <code>$enabled</code> property in the <code>debugHooks</code> object that will be used
   * as the default enabled state for the hooks. If the <code><a href="#!/api/Ext-cfg-manifest" rel="Ext-cfg-manifest" class="docClass">Ext.manifest</a></code> contains
   * a <code>debug</code> object of if <code><a href="#!/api/Ext-cfg-debugConfig" rel="Ext-cfg-debugConfig" class="docClass">Ext.debugConfig</a></code> is specified, the <code>$enabled</code> flag
   * will override its "*" value.</p>
   */
  public native function get debugHooks():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set debugHooks(value:Object):void;

  [ExtConfig]
  /**
   * The object given has properties that describe the versions at which the deprecations
   * apply.
   *
   * <p>The purpose of the <code>deprecated</code> declaration is to enable development mode to give
   * suitable error messages when deprecated methods or properties are used. Methods can
   * always be injected to provide this feedback, but properties can only be handled on
   * some browsers (those that support <code>Object.defineProperty</code>).</p>
   *
   * <p>In some cases, deprecated methods can be restored to their previous behavior or
   * added back if they have been removed.</p>
   *
   * <p>The structure of a <code>deprecated</code> declaration is this:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.bar.Class', {
   *      ...
   *
   *      deprecated: {
   *          // Optional package name - default is the framework (ext or touch)
   *          name: 'foobar',
   *
   *          '5.0': {
   *              methods: {
   *                  // Throws: '"removedMethod" is deprecated.'
   *                  removedMethod: null,
   *
   *                  // Throws: '"oldMethod" is deprecated. Please use "newMethod" instead.'
   *                  oldMethod: 'newMethod',
   *
   *                  // When this block is enabled, this method is applied as an
   *                  // override. Otherwise you get same as "removeMethod".
   *                  method: function () {
   *                      // Do what v5 "method" did. If "method" exists in newer
   *                      // versions callParent can call it. If 5.1 has "method"
   *                      // then it would be next in line, otherwise 5.2 and last
   *                      // would be the current class.
   *                  },
   *
   *                  moreHelpful: {
   *                      message: 'Something helpful to do instead.',
   *                      fn: function () {
   *                          // The v5 "moreHelpful" method to use when enabled.
   *                      }
   *                  }
   *              },
   *              properties: {
   *                  // Throws: '"removedProp" is deprecated.'
   *                  removedProp: null,
   *
   *                  // Throws: '"oldProp" is deprecated. Please use "newProp" instead.'
   *                  oldProp: 'newProp',
   *
   *                  helpful: {
   *                      message: 'Something helpful message about what to do.'
   *                  }
   *                  ...
   *              },
   *              statics: {
   *                  methods: {
   *                      ...
   *                  },
   *                  properties: {
   *                      ...
   *                  },
   *              }
   *          },
   *
   *          '5.1': {
   *              ...
   *          },
   *
   *          '5.2': {
   *              ...
   *          }
   *      }
   *  });
   * </code></pre>
   *
   * <p>The primary content of <code>deprecated</code> are the version number keys. These indicate
   * a version number where methods or properties were deprecated. These versions are
   * compared to the version reported by <code><a href="#!/api/Ext-method-getCompatVersion" rel="Ext-method-getCompatVersion" class="docClass">Ext.getCompatVersion</a></code> to determine the action
   * to take for each "block".</p>
   *
   * <p>When the compatibility version is set to a value less than a version number key,
   * that block is said to be "enabled". For example, if a method was deprecated in
   * version 5.0 but the desired compatibility level is 4.2 then the block is used to
   * patch methods and (to some degree) restore pre-5.0 compatibility.</p>
   *
   * <p>When multiple active blocks have the same method name, each method is applied as
   * an override in reverse order of version. In the above example, if a method appears
   * in the "5.0", "5.1" and "5.2" blocks then the "5.2" method is applied as an override
   * first, followed by the "5.1" method and finally the "5.0" method. This means that
   * the <code>callParent</code> from the "5.0" method calls the "5.1" method which calls the
   * "5.2" method which can (if applicable) call the current version.</p>
   */
  public native function get deprecated():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set deprecated(value:Object):void;

  /**
   * Adds a batch of class name to alias mappings.
   * @param aliases The set of mappings of the form.
   * className : [values...]
   */
  public native function addNameAliasMappings(aliases:Object):void;

  /**
   * @param alternates The set of mappings of the form
   * className : [values...]
   */
  public native function addNameAlternateMappings(alternates:Object):void;

  /**
   * Retrieve a class by its name.
   * @return class
   */
  public native function get(name:String):ext.ExtClass;

  /**
   * Get a reference to the class by its alias.
   * @return class
   */
  public native function getByAlias(alias:String):ext.ExtClass;

  /**
   * Get the class of the provided object; returns null if it's not an instance
   * of any class created with <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>. This is usually invoked by the
   * shorthand <a href="#!/api/Ext-method-getClass" rel="Ext-method-getClass" class="docClass">Ext.getClass</a>.
   *
   * <pre><code>var component = new <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>();
   *
   * <a href="#!/api/Ext-method-getClass" rel="Ext-method-getClass" class="docClass">Ext.getClass</a>(component); // returns <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>
   * </code></pre>
   * @return class
   */
  public native function getClass(object:Object):ext.ExtClass;

  /**
   * Returns the displayName property or className or object. When all else fails, returns "Anonymous".
   */
  public native function getDisplayName(object:Object):String;

  /**
   * Get the name of the class by its reference or its instance. This is
   * usually invoked by the shorthand <a href="#!/api/Ext-method-getClassName" rel="Ext-method-getClassName" class="docClass">Ext.getClassName</a>.
   *
   * <pre><code><a href="#!/api/Ext.ClassManager-method-getName" rel="Ext.ClassManager-method-getName" class="docClass">Ext.ClassManager.getName</a>(<a href="#!/api/Ext.Action" rel="Ext.Action" class="docClass">Ext.Action</a>); // returns "<a href="#!/api/Ext.Action" rel="Ext.Action" class="docClass">Ext.Action</a>"
   * </code></pre>
   * @return className
   */
  public native function getName(object:*):String;

  [Deprecated(replacement = "create", since = "5.0")]
  /**
   * Instantiate a class by either full name, alias or alternate name
   * @param args Additional arguments after the name will be passed to the class' constructor.
   * @return instance
   */
  public native function instantiate(name:String, args:*):Object;

  /**
   * Instantiate a class by its alias. This is usually invoked by the
   * shorthand <a href="#!/api/Ext-method-createByAlias" rel="Ext-method-createByAlias" class="docClass">Ext.createByAlias</a>.
   *
   * <p>If <a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a> is <a href="#!/api/Ext.Loader-method-setConfig" rel="Ext.Loader-method-setConfig" class="docClass">enabled</a> and the class
   * has not been defined yet, it will attempt to load the class via synchronous
   * loading.</p>
   *
   * <pre><code>var window = <a href="#!/api/Ext-method-createByAlias" rel="Ext-method-createByAlias" class="docClass">Ext.createByAlias</a>('widget.window', { width: 600, height: 800 });
   * </code></pre>
   * @param args Additional arguments after the alias will be passed to the
   * class constructor.
   * @return instance
   */
  public native function instantiateByAlias(alias:String, ...args):Object;

  /**
   * Checks if a class has already been created.
   * @return exist
   */
  public native function isCreated(className:String):Boolean;

  /**
   * Sets a name reference to a class.
   * @return this
   */
  public native function set(name:String, value:Object):ext.SClassManager;

  /**
   * Creates a namespace and assign the <code>value</code> to the created object.
   *
   * <pre><code><a href="#!/api/Ext.ClassManager-method-setNamespace" rel="Ext.ClassManager-method-setNamespace" class="docClass">Ext.ClassManager.setNamespace</a>('MyCompany.pkg.Example', someObject);
   *
   * alert(MyCompany.pkg.Example === someObject); // alerts true
   * </code></pre>
   */
  public native function setNamespace(name:String, value:Object):void;
}
}