package ext {

[Native]
/**
 * A collection of useful static methods to deal with function callbacks.
 * <p>Type of singleton FunctionUtil.</p>
 * @see ext.#FunctionUtil ext.FunctionUtil
 */
public class SFunctionUtil {
  /**
   * Create an alias to the provided method property with name <code>methodName</code> of <code>object</code>.
   * Note that the execution scope will still be bound to the provided <code>object</code> itself.
   * @return aliasFn
   */
  public native function alias(object:*, methodName:String):Function;

  /**
   * Create a new function from the provided <code>fn</code>, change <code>this</code> to the provided scope,
   * optionally overrides arguments for the call. Defaults to the arguments passed by
   * the caller.
   *
   * <p><a href="#!/api/Ext-method-bind" rel="Ext-method-bind" class="docClass">Ext.bind</a> is alias for <a href="#!/api/Ext.Function-method-bind" rel="Ext.Function-method-bind" class="docClass">Ext.Function.bind</a></p>
   *
   * <p><strong>NOTE:</strong> This method is deprecated. Use the standard <code>bind</code> method of JavaScript
   * <code>Function</code> instead:</p>
   *
   * <pre><code> function foo () {
   *      ...
   *  }
   *
   *  var fn = foo.bind(this);
   * </code></pre>
   *
   * <p>This method is unavailable natively on IE8 and IE/Quirks but Ext JS provides a
   * "polyfill" to emulate the important features of the standard <code>bind</code> method. In
   * particular, the polyfill only provides binding of "this" and optional arguments.</p>
   * @param fn The function to delegate.
   * @param scope The scope (<code>this</code> reference) in which the function is executed.
   * <strong>If omitted, defaults to the default global environment object (usually the browser window).</strong>
   * @param args Overrides arguments for the call. (Defaults to the arguments passed by the caller)
   * @param appendArgs if True args are appended to call args instead of overriding,
   * if a number the args are inserted at the specified position.
   * @return The new function.
   */
  public native function bind(fn:Function, scope:Object = null, args:Array = null, appendArgs:* = undefined):Function;

  /**
   * Captures the given parameters for a later call to <code><a href="#!/api/Ext-method-callback" rel="Ext-method-callback" class="docClass">Ext.callback</a></code>. This binding is
   * most useful for resolving scopes for example to an <code><a href="#!/api/Ext.app.ViewController" rel="Ext.app.ViewController" class="docClass">Ext.app.ViewController</a></code>.
   *
   * <p>The arguments match that of <code><a href="#!/api/Ext-method-callback" rel="Ext-method-callback" class="docClass">Ext.callback</a></code> except for the <code>args</code> which, if provided
   * to this method, are prepended to any arguments supplied by the eventual caller of
   * the returned function.</p>
   * @since 5.0.0
   * @return A function that, when called, uses <code><a href="#!/api/Ext-method-callback" rel="Ext-method-callback" class="docClass">Ext.callback</a></code> to call the
   * captured <code>callback</code>.
   */
  public native function bindCallback(callback:Object, scope:Object, args:Object, delay:Object, caller:Object):Function;

  /**
   * Create a "clone" of the provided method. The returned method will call the given
   * method passing along all arguments and the "this" pointer and return its result.
   * @return cloneFn
   */
  public native function clone(method:Function):Function;

  /**
   * Wraps the passed function in a barrier function which will call the passed function after the passed number of invocations.
   * @param count The number of invocations which will result in the calling of the passed function.
   * @param fn The function to call after the required number of invocations.
   * @param scope The scope (<code>this</code> reference) in which the function will be called.
   */
  public native function createBarrier(count:Number, fn:Function, scope:Object):void;

  /**
   * Creates a delegate function, optionally with a bound scope which, when called, buffers
   * the execution of the passed function for the configured number of milliseconds.
   * If called again within that period, the impending invocation will be canceled, and the
   * timeout period will begin again.
   * @param fn The function to invoke on a buffered timer.
   * @param buffer The number of milliseconds by which to buffer the invocation of the
   * function.
   * @param scope The scope (<code>this</code> reference) in which.
   * the passed function is executed. If omitted, defaults to the scope specified by the caller.
   * @param args Override arguments for the call. Defaults to the arguments
   * passed by the caller.
   * @return A function which invokes the passed function after buffering for the specified time.
   */
  public native function createBuffered(fn:Function, buffer:Number, scope:Object = null, args:Array = null):Function;

  /**
   * Creates a delegate (callback) which, when called, executes after a specific delay.
   * @param fn The function which will be called on a delay when the returned function is called.
   * Optionally, a replacement (or additional) argument list may be specified.
   * @param delay The number of milliseconds to defer execution by whenever called.
   * @param scope The scope (<code>this</code> reference) used by the function at execution time.
   * @param args Override arguments for the call. (Defaults to the arguments passed by the caller)
   * @param appendArgs if True args are appended to call args instead of overriding,
   * if a number the args are inserted at the specified position.
   * @return A function which, when called, executes the original function after the specified delay.
   */
  public native function createDelayed(fn:Function, delay:Number, scope:Object = null, args:Array = null, appendArgs:* = undefined):Function;

  /**
   * Creates an interceptor function. The passed function is called before the original one. If it returns false,
   * the original one is not called. The resulting function returns the results of the original function.
   * The passed function is called with the parameters of the original function. Example usage:
   *
   * <pre><code>var sayHi = function(name){
   *     alert('Hi, ' + name);
   * };
   *
   * sayHi('Fred'); // alerts "Hi, Fred"
   *
   * // create a new function that validates input without
   * // directly modifying the original function:
   * var sayHiToFriend = <a href="#!/api/Ext.Function-method-createInterceptor" rel="Ext.Function-method-createInterceptor" class="docClass">Ext.Function.createInterceptor</a>(sayHi, function(name){
   *     return name === 'Brian';
   * });
   *
   * sayHiToFriend('Fred');  // no alert
   * sayHiToFriend('Brian'); // alerts "Hi, Brian"
   * </code></pre>
   * @param origFn The original function.
   * @param newFn The function to call before the original.
   * @param scope The scope (<code>this</code> reference) in which the passed function is executed.
   * <strong>If omitted, defaults to the scope in which the original function is called or the browser window.</strong>
   * @param returnValue The value to return if the passed function return <code>false</code>.
   * @default null
   * @return The new function.
   */
  public native function createInterceptor(origFn:Function, newFn:Function, scope:Object = null, returnValue:Object = null):Function;

  /**
   * Create a combined function call sequence of the original function + the passed function.
   * The resulting function returns the results of the original function.
   * The passed function is called with the parameters of the original function. Example usage:
   *
   * <pre><code>var sayHi = function(name){
   *     alert('Hi, ' + name);
   * };
   *
   * sayHi('Fred'); // alerts "Hi, Fred"
   *
   * var sayGoodbye = <a href="#!/api/Ext.Function-method-createSequence" rel="Ext.Function-method-createSequence" class="docClass">Ext.Function.createSequence</a>(sayHi, function(name){
   *     alert('Bye, ' + name);
   * });
   *
   * sayGoodbye('Fred'); // both alerts show
   * </code></pre>
   * @param originalFn The original function.
   * @param newFn The function to sequence.
   * @param scope The scope (<code>this</code> reference) in which the passed function is executed.
   * If omitted, defaults to the scope in which the original function is called or the
   * default global environment object (usually the browser window).
   * @return The new function.
   */
  public native function createSequence(originalFn:Function, newFn:Function, scope:Object = null):Function;

  /**
   * Creates a throttled version of the passed function which, when called repeatedly and
   * rapidly, invokes the passed function only after a certain interval has elapsed since the
   * previous invocation.
   *
   * <p>This is useful for wrapping functions which may be called repeatedly, such as
   * a handler of a mouse move event when the processing is expensive.</p>
   * @param fn The function to execute at a regular time interval.
   * @param interval The interval in milliseconds on which the passed function is executed.
   * @param scope The scope (<code>this</code> reference) in which
   * the passed function is executed. If omitted, defaults to the scope specified by the caller.
   * @return A function which invokes the passed function at the specified interval.
   */
  public native function createThrottled(fn:Function, interval:Number, scope:Object = null):Function;

  /**
   * Calls this function after the number of milliseconds specified, optionally in a specific scope. Example usage:
   *
   * <pre><code>var sayHi = function(name){
   *     alert('Hi, ' + name);
   * }
   *
   * // executes immediately:
   * sayHi('Fred');
   *
   * // executes after 2 seconds:
   * <a href="#!/api/Ext.Function-method-defer" rel="Ext.Function-method-defer" class="docClass">Ext.Function.defer</a>(sayHi, 2000, this, ['Fred']);
   *
   * // this syntax is sometimes useful for deferring
   * // execution of an anonymous function:
   * <a href="#!/api/Ext.Function-method-defer" rel="Ext.Function-method-defer" class="docClass">Ext.Function.defer</a>(function(){
   *     alert('Anonymous');
   * }, 100);
   * </code></pre>
   *
   * <p><a href="#!/api/Ext-method-defer" rel="Ext-method-defer" class="docClass">Ext.defer</a> is alias for <a href="#!/api/Ext.Function-method-defer" rel="Ext.Function-method-defer" class="docClass">Ext.Function.defer</a></p>
   * @param fn The function to defer.
   * @param millis The number of milliseconds for the <code>setTimeout</code> call
   * (if less than or equal to 0 the function is executed immediately).
   * @param scope The scope (<code>this</code> reference) in which the function is executed.
   * <strong>If omitted, defaults to the browser window.</strong>
   * @param args Overrides arguments for the call. Defaults to the arguments passed by the caller.
   * @param appendArgs If <code>true</code> args are appended to call args instead of overriding,
   * or, if a number, then the args are inserted at the specified position.
   * @default false
   * @return The timeout id that can be used with <code>clearTimeout</code>.
   */
  public native function defer(fn:Function, millis:Number, scope:Object = null, args:Array = null, appendArgs:* = false):Number;

  /**
   * A very commonly used method throughout the framework. It acts as a wrapper around another method
   * which originally accepts 2 arguments for <code>name</code> and <code>value</code>.
   * The wrapped function then allows "flexible" value setting of either:
   *
   * <ul>
   * <li><code>name</code> and <code>value</code> as 2 arguments</li>
   * <li>one single object argument with multiple key - value pairs</li>
   * </ul>
   *
   *
   * <p>For example:</p>
   *
   * <pre><code>var setValue = <a href="#!/api/Ext.Function-method-flexSetter" rel="Ext.Function-method-flexSetter" class="docClass">Ext.Function.flexSetter</a>(function(name, value) {
   *     this[name] = value;
   * });
   *
   * // Afterwards
   * // Setting a single name - value
   * setValue('name1', 'value1');
   *
   * // Settings multiple name - value pairs
   * setValue({
   *     name1: 'value1',
   *     name2: 'value2',
   *     name3: 'value3'
   * });
   * </code></pre>
   * @param setter The single value setter method.
   * @param {String} setter.name The name of the value being set.
   * @param {Object} setter.value The value being set.
   */
  public native function flexSetter(setter:Function):Function;

  /**
   * Adds behavior to an existing method that is executed after the
   * original behavior of the function.  For example:
   *
   * <pre><code>var soup = {
   *     contents: [],
   *     add: function(ingredient) {
   *         this.contents.push(ingredient);
   *     }
   * };
   * <a href="#!/api/Ext.Function-method-interceptAfter" rel="Ext.Function-method-interceptAfter" class="docClass">Ext.Function.interceptAfter</a>(soup, "add", function(ingredient){
   *     // Always add a bit of extra salt
   *     this.contents.push("salt");
   * });
   * soup.add("water");
   * soup.add("onions");
   * soup.contents; // will contain: water, salt, onions, salt
   * </code></pre>
   * @param object The target object
   * @param methodName Name of the method to override
   * @param fn Function with the new behavior.  It will
   * be called with the same arguments as the original method.  The
   * return value of this function will be the return value of the
   * new method.
   * @param scope The scope to execute the interceptor function. Defaults to the object.
   * @return The new function just created.
   */
  public native function interceptAfter(object:Object, methodName:String, fn:Function, scope:Object = null):Function;

  /**
   * Adds behavior to an existing method that is executed before the
   * original behavior of the function.  For example:
   *
   * <pre><code>var soup = {
   *     contents: [],
   *     add: function(ingredient) {
   *         this.contents.push(ingredient);
   *     }
   * };
   * <a href="#!/api/Ext.Function-method-interceptBefore" rel="Ext.Function-method-interceptBefore" class="docClass">Ext.Function.interceptBefore</a>(soup, "add", function(ingredient){
   *     if (!this.contents.length &amp;&amp; ingredient !== "water") {
   *         // Always add water to start with
   *         this.contents.push("water");
   *     }
   * });
   * soup.add("onions");
   * soup.add("salt");
   * soup.contents; // will contain: water, onions, salt
   * </code></pre>
   * @param object The target object
   * @param methodName Name of the method to override
   * @param fn Function with the new behavior.  It will
   * be called with the same arguments as the original method.  The
   * return value of this function will be the return value of the
   * new method.
   * @param scope The scope to execute the interceptor function. Defaults to the object.
   * @return The new function just created.
   */
  public native function interceptBefore(object:Object, methodName:String, fn:Function, scope:Object = null):Function;

  /**
   * Calls this function repeatedly at a given interval, optionally in a specific scope.
   *
   * <p><a href="#!/api/Ext-method-defer" rel="Ext-method-defer" class="docClass">Ext.defer</a> is alias for <a href="#!/api/Ext.Function-method-defer" rel="Ext.Function-method-defer" class="docClass">Ext.Function.defer</a></p>
   * @param fn The function to defer.
   * @param millis The number of milliseconds for the <code>setInterval</code> call
   * @param scope The scope (<code>this</code> reference) in which the function is executed.
   * <strong>If omitted, defaults to the browser window.</strong>
   * @param args Overrides arguments for the call. Defaults to the arguments passed by the caller.
   * @param appendArgs If <code>true</code> args are appended to call args instead of overriding,
   * or, if a number, then the args are inserted at the specified position.
   * @default false
   * @return The interval id that can be used with <code>clearInterval</code>.
   */
  public native function interval(fn:Function, millis:Number, scope:Object = null, args:Array = null, appendArgs:* = false):Number;

  /**
   * Returns a wrapper function that caches the return value for previously
   * processed function argument(s).
   *
   * <p>For example:</p>
   *
   * <pre><code> function factorial (value) {
   *      var ret = value;
   *
   *      while (--value &gt; 1) {
   *          ret *= value;
   *      }
   *
   *      return ret;
   *  }
   * </code></pre>
   *
   * <p>Each call to <code>factorial</code> will loop and multiply to produce the answer. Using
   * this function we can wrap the above and cache its answers:</p>
   *
   * <pre><code> factorial = <a href="#!/api/Ext.Function-method-memoize" rel="Ext.Function-method-memoize" class="docClass">Ext.Function.memoize</a>(factorial);
   * </code></pre>
   *
   * <p>The returned function operates in the same manner as before, but results are
   * stored in a cache to avoid calling the wrapped function when given the same
   * arguments.</p>
   *
   * <pre><code> var x = factorial(20);  // first time; call real factorial()
   *  var y = factorial(20);  // second time; return value from first call
   * </code></pre>
   *
   * <p>To support multi-argument methods, you will need to provide a <code>hashFn</code>.</p>
   *
   * <pre><code> function permutation (n, k) {
   *      return factorial(n) / factorial(n - k);
   *  }
   *
   *  permutation = <a href="#!/api/Ext.Function-method-memoize" rel="Ext.Function-method-memoize" class="docClass">Ext.Function.memoize</a>(permutation, null, function (n, k) {
   *      n + '-' + k;
   *  });
   * </code></pre>
   *
   * <p>In this case, the <code>memoize</code> of <code>factorial</code> is sufficient optimization, but the
   * example is simply to illustrate how to generate a unique key for an expensive,
   * multi-argument method.</p>
   *
   * <p><strong>IMPORTANT</strong>: This cache is unbounded so be cautious of memory leaks if the
   * <code>memoize</code>d function is kept indefinitely or is given an unbounded set of
   * possible arguments.</p>
   * @since 6.0.0
   * @param fn Function to wrap.
   * @param scope Optional scope in which to execute the wrapped function.
   * @param hashFn Optional function used to compute a hash key for
   * storing the result, based on the arguments to the original function.
   * @return The caching wrapper function.
   */
  public native function memoize(fn:Function, scope:Object, hashFn:Function):Function;

  /**
   * Create a new function from the provided <code>fn</code>, the arguments of which are pre-set to <code>args</code>.
   * New arguments passed to the newly created callback when it's invoked are appended after the pre-set ones.
   * This is especially useful when creating callbacks.
   *
   * <p>For example:</p>
   *
   * <pre><code>var originalFunction = function(){
   *     alert(<a href="#!/api/Ext.Array-method-from" rel="Ext.Array-method-from" class="docClass">Ext.Array.from</a>(arguments).join(' '));
   * };
   *
   * var callback = <a href="#!/api/Ext.Function-method-pass" rel="Ext.Function-method-pass" class="docClass">Ext.Function.pass</a>(originalFunction, ['Hello', 'World']);
   *
   * callback(); // alerts 'Hello World'
   * callback('by Me'); // alerts 'Hello World by Me'
   * </code></pre>
   *
   * <p><a href="#!/api/Ext-method-pass" rel="Ext-method-pass" class="docClass">Ext.pass</a> is alias for <a href="#!/api/Ext.Function-method-pass" rel="Ext.Function-method-pass" class="docClass">Ext.Function.pass</a></p>
   * @param fn The original function.
   * @param args The arguments to pass to new callback.
   * @param scope The scope (<code>this</code> reference) in which the function is executed.
   * @return The new callback function.
   */
  public native function pass(fn:Function, args:Array, scope:Object = null):Function;
}
}