package ext {

[Native]
/**
 * A collection of useful static methods to deal with numbers
 * <p>Type of singleton NumberUtil.</p>
 * @see ext.#NumberUtil ext.NumberUtil
 */
public class SNumberUtil {
  /**
   * Coerces a given index into a valid index given a <code>length</code>.
   *
   * <p>Negative indexes are interpreted starting at the end of the collection. That is,
   * a value of -1 indicates the last item, or equivalent to <code>length - 1</code>.</p>
   *
   * <p>When handling methods that take "begin" and "end" arguments like most array or
   * string methods, this method can be used like so:</p>
   *
   * <pre><code> function foo (array, begin, end) {
   *      var range = <a href="#!/api/Ext.Number-method-clipIndices" rel="Ext.Number-method-clipIndices" class="docClass">Ext.Number.clipIndices</a>(array.length, [begin, end]);
   *
   *      begin = range[0];
   *      end   = range[1];
   *
   *      // 0 &lt;= begin &lt;= end &lt;= array.length
   *
   *      var length = end - begin;
   *  }
   * </code></pre>
   *
   * <p>For example:</p>
   *
   * <pre><code> +---+---+---+---+---+---+---+---+
   *  |   |   |   |   |   |   |   |   |  length = 8
   *  +---+---+---+---+---+---+---+---+
   *    0   1   2   3   4   5   6   7
   *   -8  -7  -6  -5  -4  -3  -2  -1
   *
   *  console.log(<a href="#!/api/Ext.Number-method-clipIndices" rel="Ext.Number-method-clipIndices" class="docClass">Ext.Number.clipIndices</a>(8, [3, 10]); // logs "[3, 8]"
   *  console.log(<a href="#!/api/Ext.Number-method-clipIndices" rel="Ext.Number-method-clipIndices" class="docClass">Ext.Number.clipIndices</a>(8, [-5]);    // logs "[3, 8]"
   *  console.log(<a href="#!/api/Ext.Number-method-clipIndices" rel="Ext.Number-method-clipIndices" class="docClass">Ext.Number.clipIndices</a>(8, []);
   *  console.log(<a href="#!/api/Ext.Number-method-clipIndices" rel="Ext.Number-method-clipIndices" class="docClass">Ext.Number.clipIndices</a>(8, []);
   * </code></pre>
   * @param options An object with different option flags.
   * @param {Boolean} [options.count] The second number in <code>indices</code> is the
   * count not and an index.
   * @default false
   * @param {Boolean} [options.inclusive] The second number in <code>indices</code> is
   * "inclusive" meaning that the item should be considered in the range. Normally,
   * the second number is considered the first item outside the range or as an
   * "exclusive" bound.
   * @default false
   * @param {Boolean} [options.wrap] Wraps negative numbers backwards from the
   * end of the array. Passing <code>false</code> simply clips negative index values at 0.
   * @default true
   * @return The normalized <code>[begin, end]</code> array where <code>end</code> is now
   * exclusive such that <code>length = end - begin</code>. Both values are between 0 and the
   * given <code>length</code> and <code>end</code> will not be less-than <code>begin</code>.
   */
  public native function clipIndices(length:Number, indices:Array, options:Object = null):Array;

  /**
   * Checks whether or not the passed number is within a desired range.  If the number is already within the
   * range it is returned, otherwise the min or max value is returned depending on which side of the range is
   * exceeded. Note that this method returns the constrained value but does not change the current number.
   * @param number The number to check
   * @param min The minimum number in the range
   * @param max The maximum number in the range
   * @return The constrained value if outside the range, otherwise the current value
   */
  public native function constrain(number:Number, min:Number, max:Number):Number;

  /**
   * Corrects floating point numbers that overflow to a non-precise
   * value because of their floating nature, for example <code>0.1 + 0.2</code>
   * @param n The number
   * @return The correctly rounded number
   */
  public native function correctFloat(n:Number):Number;

  /**
   * Validate that a value is numeric and convert it to a number if necessary. Returns the specified default value if
   * it is not.
   *
   * <pre><code><a href="#!/api/Ext.Number-method-from" rel="Ext.Number-method-from" class="docClass">Ext.Number.from</a>('1.23', 1); // returns 1.23
   * <a href="#!/api/Ext.Number-method-from" rel="Ext.Number-method-from" class="docClass">Ext.Number.from</a>('abc', 1); // returns 1
   * </code></pre>
   * @param defaultValue The value to return if the original value is non-numeric
   * @return value, if numeric, defaultValue otherwise
   */
  public native function from(value:Object, defaultValue:Number):Number;

  /**
   * Determines if two numbers <code>n1</code> and <code>n2</code> are equal within a given
   * margin of precision <code>epsilon</code>.
   * @param n1 First number.
   * @param n2 Second number.
   * @param epsilon Margin of precision.
   * @return <code>true</code>, if numbers are equal. <code>false</code> otherwise.
   */
  public native function isEqual(n1:Number, n2:Number, epsilon:Number):Boolean;

  /**
   * Determines if the value passed is a number and also finite.
   * This a Polyfill version of Number.isFinite(),differently than
   * the isFinite() function, this method doesn't convert the parameter to a number.
   * @since 6.2
   * @param value Number to be tested.
   * @return <code>true</code>, if the parameter is a number and finite, <code>false</code> otherwise.
   */
  public native function isFinite(value:Number):Boolean;

  /**
   * Returns the base 10 logarithm of a number.
   * This will use Math.log10, if available (ES6).
   * @param x The number.
   * @return Base 10 logarithm of the number 'x'.
   */
  public native function log10(x:Number):Number;

  /**
   * Returns a random integer between the specified range (inclusive)
   * @param from Lowest value to return.
   * @param to Highest value to return.
   * @return A random integer within the specified range.
   */
  public native function randomInt(from:Number, to:Number):Number;

  /**
   * Round a number to the nearest interval.
   * @since 6.2.0
   * @param value The value to round.
   * @param interval The interval to round to.
   * @return The rounded value.
   */
  public native function roundToNearest(value:Number, interval:Number):Number;

  /**
   * Returns the sign of the given number. See also MDN for Math.sign documentation
   * for the standard method this method emulates.
   * @param x The number.
   * @return The sign of the number <code>x</code>, indicating whether the number is
   * positive (1), negative (-1) or zero (0).
   */
  public native function sign(x:Number):Number;

  /**
   * Snaps the passed number between stopping points based upon a passed increment value.
   *
   * <p>The difference between this and <a href="#!/api/Ext.Number-method-snapInRange" rel="Ext.Number-method-snapInRange" class="docClass">snapInRange</a> is that <a href="#!/api/Ext.Number-method-snapInRange" rel="Ext.Number-method-snapInRange" class="docClass">snapInRange</a> uses the minValue
   * when calculating snap points:</p>
   *
   * <pre><code>r = <a href="#!/api/Ext.Number-method-snap" rel="Ext.Number-method-snap" class="docClass">Ext.Number.snap</a>(56, 2, 55, 65);        // Returns 56 - snap points are zero based
   *
   * r = <a href="#!/api/Ext.Number-method-snapInRange" rel="Ext.Number-method-snapInRange" class="docClass">Ext.Number.snapInRange</a>(56, 2, 55, 65); // Returns 57 - snap points are based from minValue
   * </code></pre>
   * @param value The unsnapped value.
   * @param increment The increment by which the value must move.
   * @param minValue The minimum value to which the returned value must be constrained. Overrides the increment.
   * @param maxValue The maximum value to which the returned value must be constrained. Overrides the increment.
   * @return The value of the nearest snap target.
   */
  public native function snap(value:Number, increment:Number, minValue:Number, maxValue:Number):Number;

  /**
   * Snaps the passed number between stopping points based upon a passed increment value.
   *
   * <p>The difference between this and <a href="#!/api/Ext.Number-method-snap" rel="Ext.Number-method-snap" class="docClass">snap</a> is that <a href="#!/api/Ext.Number-method-snap" rel="Ext.Number-method-snap" class="docClass">snap</a> does not use the minValue
   * when calculating snap points:</p>
   *
   * <pre><code>r = <a href="#!/api/Ext.Number-method-snap" rel="Ext.Number-method-snap" class="docClass">Ext.Number.snap</a>(56, 2, 55, 65);        // Returns 56 - snap points are zero based
   *
   * r = <a href="#!/api/Ext.Number-method-snapInRange" rel="Ext.Number-method-snapInRange" class="docClass">Ext.Number.snapInRange</a>(56, 2, 55, 65); // Returns 57 - snap points are based from minValue
   * </code></pre>
   * @param value The unsnapped value.
   * @param increment The increment by which the value must move.
   * @param minValue The minimum value to which the returned value must be constrained.
   * @default 0
   * @param maxValue The maximum value to which the returned value must be constrained.
   * @default Infinity
   * @return The value of the nearest snap target.
   */
  public native function snapInRange(value:Number, increment:Number, minValue:Number = 0, maxValue:Number = Infinity):Number;

  /**
   * Formats a number using fixed-point notation
   * @param value The number to format
   * @param precision The number of digits to show after the decimal point
   */
  public native function toFixed(value:Number, precision:Number):void;
}
}