package ext {

[Native]
/**
 * A collection of useful static methods to deal with objects.
 * <p>Type of singleton ObjectUtil.</p>
 * @see ext.#ObjectUtil ext.ObjectUtil
 */
public class SObjectUtil {
  /**
   * Returns a new object with the given object as the prototype chain. This method is
   * designed to mimic the ECMA standard <code>Object.create</code> method and is assigned to that
   * function when it is available.
   *
   * <p><strong>NOTE</strong> This method does not support the property definitions capability of the
   * <code>Object.create</code> method. Only the first argument is supported.</p>
   * @param object The prototype chain for the new object.
   */
  public native function chain(object:Object):void;

  /**
   * This method removes all keys from the given object.
   * @param object The object from which to remove all keys.
   * @return The given object.
   */
  public native function clear(object:Object):Object;

  /**
   * Iterates through an object and invokes the given callback function for each iteration.
   * The iteration can be stopped by returning <code>false</code> in the callback function. For example:
   *
   * <pre><code>var person = {
   *     name: 'Jacky'
   *     hairColor: 'black'
   *     loves: ['food', 'sleeping', 'wife']
   * };
   *
   * <a href="#!/api/Ext.Object-method-each" rel="Ext.Object-method-each" class="docClass">Ext.Object.each</a>(person, function(key, value, myself) {
   *     console.log(key + ":" + value);
   *
   *     if (key === 'hairColor') {
   *         return false; // stop the iteration
   *     }
   * });
   * </code></pre>
   * @param object The object to iterate
   * @param fn The callback function.
   * @param {String} fn.key
   * @param {Object} fn.value
   * @param {Object} fn.object The object itself
   * @param scope The execution scope (<code>this</code>) of the callback function
   */
  public native function each(object:Object, fn:Function, scope:Object = null):void;

  /**
   * Iterates through an object and invokes the given callback function for each iteration.
   * The iteration can be stopped by returning <code>false</code> in the callback function. For example:
   *
   * <pre><code>var items = {
   *     1: 'Hello',
   *     2: 'World'
   * };
   *
   * <a href="#!/api/Ext.Object-method-eachValue" rel="Ext.Object-method-eachValue" class="docClass">Ext.Object.eachValue</a>(items, function (value) {
   *     console.log("Value: " + value);
   * });
   * </code></pre>
   *
   * <p>This will log 'Hello' and 'World' in no particular order. This method is useful
   * in cases where the keys are not important to the processing, just the values.</p>
   * @param object The object to iterate
   * @param fn The callback function.
   * @param {Object} fn.value The value of
   * @param scope The execution scope (<code>this</code>) of the callback function
   */
  public native function eachValue(object:Object, fn:Function, scope:Object = null):void;

  /**
   * Shallow compares the contents of 2 objects using strict equality. Objects are
   * considered equal if they both have the same set of properties and the
   * value for those properties equals the other in the corresponding object.
   *
   * <pre><code>// Returns true
   * <a href="#!/api/Ext.Object-method-equals" rel="Ext.Object-method-equals" class="docClass">Ext.Object.equals</a>({
   *     foo: 1,
   *     bar: 2
   * }, {
   *     foo: 1,
   *     bar: 2
   * });
   * </code></pre>
   * @return <code>true</code> if the objects are equal.
   */
  public native function equals(object1:Object, object2:Object):Boolean;

  /**
   * Freezes the given object making it immutable. This operation is by default shallow
   * and does not effect objects referenced by the given object.
   * @param obj The object to freeze.
   * @param deep Pass <code>true</code> to freeze sub-objects recursively.
   * @default false
   * @return The given object <code>obj</code>.
   */
  public native function freeze(obj:Object, deep:Boolean = false):Object;

  /**
   * Converts a query string back into an object.
   *
   * <p>Non-recursive:</p>
   *
   * <pre><code><a href="#!/api/Ext.Object-method-fromQueryString" rel="Ext.Object-method-fromQueryString" class="docClass">Ext.Object.fromQueryString</a>("foo=1&amp;bar=2"); // returns {foo: '1', bar: '2'}
   * <a href="#!/api/Ext.Object-method-fromQueryString" rel="Ext.Object-method-fromQueryString" class="docClass">Ext.Object.fromQueryString</a>("foo=&amp;bar=2"); // returns {foo: '', bar: '2'}
   * <a href="#!/api/Ext.Object-method-fromQueryString" rel="Ext.Object-method-fromQueryString" class="docClass">Ext.Object.fromQueryString</a>("some%20price=%24300"); // returns {'some price': '$300'}
   * <a href="#!/api/Ext.Object-method-fromQueryString" rel="Ext.Object-method-fromQueryString" class="docClass">Ext.Object.fromQueryString</a>("colors=red&amp;colors=green&amp;colors=blue"); // returns {colors: ['red', 'green', 'blue']}
   * </code></pre>
   *
   * <p>Recursive:</p>
   *
   * <pre><code><a href="#!/api/Ext.Object-method-fromQueryString" rel="Ext.Object-method-fromQueryString" class="docClass">Ext.Object.fromQueryString</a>(
   *     "username=Jacky&amp;"+
   *     "dateOfBirth[day]=1&amp;dateOfBirth[month]=2&amp;dateOfBirth[year]=1911&amp;"+
   *     "hobbies[0]=coding&amp;hobbies[1]=eating&amp;hobbies[2]=sleeping&amp;"+
   *     "hobbies[3][0]=nested&amp;hobbies[3][1]=stuff", true);
   *
   * // returns
   * {
   *     username: 'Jacky',
   *     dateOfBirth: {
   *         day: '1',
   *         month: '2',
   *         year: '1911'
   *     },
   *     hobbies: ['coding', 'eating', 'sleeping', ['nested', 'stuff']]
   * }
   * </code></pre>
   * @param queryString The query string to decode
   * @param recursive Whether or not to recursively decode the string. This format is supported by
   * PHP / Ruby on Rails servers and similar.
   * @default false
   */
  public native function fromQueryString(queryString:String, recursive:Boolean = false):Object;

  /**
   * Returns all keys of the given object as an array.
   * @return An array of keys from the object or any of its prototypes.
   */
  public native function getAllKeys(object:Object):Array;

  /**
   * Returns the first matching key corresponding to the given value.
   * If no matching value is found, null is returned.
   *
   * <pre><code>var person = {
   *     name: 'Jacky',
   *     loves: 'food'
   * };
   *
   * alert(<a href="#!/api/Ext.Object-method-getKey" rel="Ext.Object-method-getKey" class="docClass">Ext.Object.getKey</a>(person, 'food')); // alerts 'loves'
   * </code></pre>
   * @param value The value to find
   */
  public native function getKey(object:Object, value:Object):String;

  /**
   * Returns the <code>hasOwnProperty</code> keys of the given object as an array.
   *
   * <pre><code>var values = <a href="#!/api/Ext.Object-method-getKeys" rel="Ext.Object-method-getKeys" class="docClass">Ext.Object.getKeys</a>({
   *     name: 'Jacky',
   *     loves: 'food'
   * }); // ['name', 'loves']
   * </code></pre>
   * @return An array of keys from the object
   */
  public native function getKeys(object:Object):Array;

  /**
   * Gets the total number of this object's own properties
   *
   * <pre><code>var size = <a href="#!/api/Ext.Object-method-getSize" rel="Ext.Object-method-getSize" class="docClass">Ext.Object.getSize</a>({
   *     name: 'Jacky',
   *     loves: 'food'
   * }); // size equals 2
   * </code></pre>
   * @return size
   */
  public native function getSize(object:Object):Number;

  /**
   * Gets all values of the given object as an array.
   *
   * <pre><code>var values = <a href="#!/api/Ext.Object-method-getValues" rel="Ext.Object-method-getValues" class="docClass">Ext.Object.getValues</a>({
   *     name: 'Jacky',
   *     loves: 'food'
   * }); // ['Jacky', 'food']
   * </code></pre>
   * @return An array of values from the object
   */
  public native function getValues(object:Object):Array;

  /**
   * Checks if there are any properties on this object.
   * @return <code>true</code> if there no properties on the object.
   */
  public native function isEmpty(object:Object):Boolean;

  /**
   * Merges any number of objects recursively without referencing them or their children.
   *
   * <pre><code>var extjs = {
   *     companyName: 'Ext JS',
   *     products: ['Ext JS', 'Ext GWT', 'Ext Designer'],
   *     isSuperCool: true,
   *     office: {
   *         size: 2000,
   *         location: 'Palo Alto',
   *         isFun: true
   *     }
   * };
   *
   * var newStuff = {
   *     companyName: 'Sencha Inc.',
   *     products: ['Ext JS', 'Ext GWT', 'Ext Designer', 'Sencha Touch', 'Sencha Animator'],
   *     office: {
   *         size: 40000,
   *         location: 'Redwood City'
   *     }
   * };
   *
   * var sencha = <a href="#!/api/Ext.Object-method-merge" rel="Ext.Object-method-merge" class="docClass">Ext.Object.merge</a>(extjs, newStuff);
   *
   * // extjs and sencha then equals to
   * {
   *     companyName: 'Sencha Inc.',
   *     products: ['Ext JS', 'Ext GWT', 'Ext Designer', 'Sencha Touch', 'Sencha Animator'],
   *     isSuperCool: true,
   *     office: {
   *         size: 40000,
   *         location: 'Redwood City',
   *         isFun: true
   *     }
   * }
   * </code></pre>
   * @param destination The object into which all subsequent objects are merged.
   * @param object Any number of objects to merge into the destination.
   * @return merged The destination object with all passed objects merged in.
   */
  public native function merge(destination:Object, ...object):Object;

  /**
   * Converts a <code>name</code> - <code>value</code> pair to an array of objects with support for nested structures. Useful to construct
   * query strings. For example:
   *
   * <pre><code>var objects = <a href="#!/api/Ext.Object-method-toQueryObjects" rel="Ext.Object-method-toQueryObjects" class="docClass">Ext.Object.toQueryObjects</a>('hobbies', ['reading', 'cooking', 'swimming']);
   *
   * // objects then equals:
   * [
   *     { name: 'hobbies', value: 'reading' },
   *     { name: 'hobbies', value: 'cooking' },
   *     { name: 'hobbies', value: 'swimming' },
   * ];
   *
   * var objects = <a href="#!/api/Ext.Object-method-toQueryObjects" rel="Ext.Object-method-toQueryObjects" class="docClass">Ext.Object.toQueryObjects</a>('dateOfBirth', {
   *     day: 3,
   *     month: 8,
   *     year: 1987,
   *     extra: {
   *         hour: 4
   *         minute: 30
   *     }
   * }, true); // Recursive
   *
   * // objects then equals:
   * [
   *     { name: 'dateOfBirth[day]', value: 3 },
   *     { name: 'dateOfBirth[month]', value: 8 },
   *     { name: 'dateOfBirth[year]', value: 1987 },
   *     { name: 'dateOfBirth[extra][hour]', value: 4 },
   *     { name: 'dateOfBirth[extra][minute]', value: 30 },
   * ];
   * </code></pre>
   * @param recursive True to traverse object recursively
   * @default false
   */
  public native function toQueryObjects(name:String, value:*, recursive:Boolean = false):Array;

  /**
   * Takes an object and converts it to an encoded query string.
   *
   * <p>Non-recursive:</p>
   *
   * <pre><code><a href="#!/api/Ext.Object-method-toQueryString" rel="Ext.Object-method-toQueryString" class="docClass">Ext.Object.toQueryString</a>({foo: 1, bar: 2}); // returns "foo=1&amp;bar=2"
   * <a href="#!/api/Ext.Object-method-toQueryString" rel="Ext.Object-method-toQueryString" class="docClass">Ext.Object.toQueryString</a>({foo: null, bar: 2}); // returns "foo=&amp;bar=2"
   * <a href="#!/api/Ext.Object-method-toQueryString" rel="Ext.Object-method-toQueryString" class="docClass">Ext.Object.toQueryString</a>({'some price': '$300'}); // returns "some%20price=%24300"
   * <a href="#!/api/Ext.Object-method-toQueryString" rel="Ext.Object-method-toQueryString" class="docClass">Ext.Object.toQueryString</a>({date: new Date(2011, 0, 1)}); // returns "date=%222011-01-01T00%3A00%3A00%22"
   * <a href="#!/api/Ext.Object-method-toQueryString" rel="Ext.Object-method-toQueryString" class="docClass">Ext.Object.toQueryString</a>({colors: ['red', 'green', 'blue']}); // returns "colors=red&amp;colors=green&amp;colors=blue"
   * </code></pre>
   *
   * <p>Recursive:</p>
   *
   * <pre><code><a href="#!/api/Ext.Object-method-toQueryString" rel="Ext.Object-method-toQueryString" class="docClass">Ext.Object.toQueryString</a>({
   *     username: 'Jacky',
   *     dateOfBirth: {
   *         day: 1,
   *         month: 2,
   *         year: 1911
   *     },
   *     hobbies: ['coding', 'eating', 'sleeping', ['nested', 'stuff']]
   * }, true); // returns the following string (broken down and url-decoded for ease of reading purpose):
   * // username=Jacky
   * //    &amp;dateOfBirth[day]=1&amp;dateOfBirth[month]=2&amp;dateOfBirth[year]=1911
   * //    &amp;hobbies[0]=coding&amp;hobbies[1]=eating&amp;hobbies[2]=sleeping&amp;hobbies[3][0]=nested&amp;hobbies[3][1]=stuff
   * </code></pre>
   * @param object The object to encode
   * @param recursive Whether or not to interpret the object in recursive format.
   * (PHP / Ruby on Rails servers and similar).
   * @default false
   * @return queryString
   */
  public native function toQueryString(object:Object, recursive:Boolean = false):String;
}
}