package ext {

[Uses("ext.Base")]
[Native("Ext.Template", require)]
/**
 * Represents an HTML fragment template. Templates may be <a href="#!/api/Ext.Template-method-compile" rel="Ext.Template-method-compile" class="docClass">precompiled</a> for greater performance.
 *
 * <p>An instance of this class may be created by passing to the constructor either a single argument, or multiple
 * arguments:</p>
 *
 * <h1>Single argument: String/Array</h1>
 *
 * <p>The single argument may be either a String or an Array:</p>
 *
 * <ul>
 * <li><p>String:</p>
 *
 * <pre><code>var t = new <a href="#!/api/Ext.Template" rel="Ext.Template" class="docClass">Ext.Template</a>("&lt;div&gt;Hello {0}.&lt;/div&gt;");
 * t.append('some-element', ['foo']);
 * </code></pre></li>
 * <li><p>Array:</p>
 *
 * <p>An Array will be combined with <code>join('')</code>.</p>
 *
 * <pre><code>var t = new <a href="#!/api/Ext.Template" rel="Ext.Template" class="docClass">Ext.Template</a>([
 *     '&lt;div name="{id}"&gt;',
 *         '&lt;span class="{cls}"&gt;{name:trim} {value:ellipsis(10)}&lt;/span&gt;',
 *     '&lt;/div&gt;',
 * ]);
 * t.compile();
 * t.append('some-element', {id: 'myid', cls: 'myclass', name: 'foo', value: 'bar'});
 * </code></pre></li>
 * </ul>
 *
 *
 * <h1>Multiple arguments: String, Object, Array, ...</h1>
 *
 * <p>Multiple arguments will be combined with <code>join('')</code>.</p>
 *
 * <pre><code>var t = new <a href="#!/api/Ext.Template" rel="Ext.Template" class="docClass">Ext.Template</a>(
 *     '&lt;div name="{id}"&gt;',
 *         '&lt;span class="{cls}"&gt;{name} {value}&lt;/span&gt;',
 *     '&lt;/div&gt;',
 *     // a configuration object:
 *     {
 *         compiled: true,      // <a href="#!/api/Ext.Template-method-compile" rel="Ext.Template-method-compile" class="docClass">compile</a> immediately
 *     }
 * );
 * </code></pre>
 *
 * <h1>Notes</h1>
 *
 * <ul>
 * <li>For a list of available format functions, see <a href="#!/api/Ext.util.Format" rel="Ext.util.Format" class="docClass">Ext.util.Format</a>.</li>
 * <li><code>disableFormats</code> reduces <code><a href="#!/api/Ext.Template-method-apply" rel="Ext.Template-method-apply" class="docClass">apply</a></code> time when no formatting is required.</li>
 * </ul>
 */
public class Template extends ext.Base {
  /**
   * Creates a template from the passed element's value (<em>display:none</em> textarea, preferred) or innerHTML.
   * @param el A DOM element or its id
   * @param config Config object
   * @return The created template
   */
  public static native function from(el:*, config:Object = null):ext.Template;

  /**
   * Creates new template.
   * @param config List of strings to be concatenated into template or template Configs.
   */
  public function Template(...config) {
    super();
  }

  /**
   * <code>true</code> in this class to identify an object as an instantiated Template, or subclass thereof.
   * @default true
   */
  public native function get isTemplate():Boolean;

  /**
   * @private
   */
  public native function set isTemplate(value:Boolean):void;

  [ExtConfig]
  /**
   * True to immediately compile the template. Defaults to false.
   */
  public native function get compiled():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set compiled(value:Boolean):void;

  [ExtConfig]
  /**
   * True to disable format functions in the template. If the template doesn't contain
   * format functions, setting disableFormats to true will reduce apply time. Defaults to false.
   * @default false
   */
  public native function get disableFormats():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set disableFormats(value:Boolean):void;

  /**
   * Applies the supplied <code>values</code> to the template and appends the new node(s) to the specified <code>el</code>.
   *
   * <p>For example usage see <a href="#!/api/Ext.Template" rel="Ext.Template" class="docClass">Ext.Template class docs</a>.</p>
   * @param el The context element
   * @param values The template values. See <a href="#!/api/Ext.Template-method-applyTemplate" rel="Ext.Template-method-applyTemplate" class="docClass">applyTemplate</a> for details.
   * @param returnElement true to return an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a>.
   * @return The new node or Element
   */
  public native function append(el:*, values:*, returnElement:Boolean = false):*;

  /**
   * Returns an HTML fragment of this template with the specified values applied.
   * @param values The template values. Can be an array if your params are numeric:
   * <pre><code>var tpl = new <a href="#!/api/Ext.Template" rel="Ext.Template" class="docClass">Ext.Template</a>('Name: {0}, Age: {1}');
   * tpl.apply(['John', 25]);
   * </code></pre>
   * <p>or an object:</p>
   * <pre><code>var tpl = new <a href="#!/api/Ext.Template" rel="Ext.Template" class="docClass">Ext.Template</a>('Name: {name}, Age: {age}');
   * tpl.apply({name: 'John', age: 25});
   * </code></pre>
   * @return The HTML fragment
   */
  public native function apply(values:*):String;

  /**
   * Appends the result of this template to the provided output array.
   * @param values The template values. See <a href="#!/api/Ext.Template-method-apply" rel="Ext.Template-method-apply" class="docClass">apply</a>.
   * @param out The array to which output is pushed.
   * @return The given out array.
   */
  public native function applyOut(values:*, out:Array):Array;

  /**
   * Alias for <a href="#!/api/Ext.Template-method-apply" rel="Ext.Template-method-apply" class="docClass">apply</a>.
   * @param values The template values. Can be an array if your params are numeric:
   * <pre><code>var tpl = new <a href="#!/api/Ext.Template" rel="Ext.Template" class="docClass">Ext.Template</a>('Name: {0}, Age: {1}');
   * tpl.apply(['John', 25]);
   * </code></pre>
   * <p>or an object:</p>
   * <pre><code>var tpl = new <a href="#!/api/Ext.Template" rel="Ext.Template" class="docClass">Ext.Template</a>('Name: {name}, Age: {age}');
   * tpl.apply({name: 'John', age: 25});
   * </code></pre>
   * @return The HTML fragment
   */
  public native function applyTemplate(values:*):String;

  /**
   * Compiles the template into an internal function, eliminating the RegEx overhead.
   * @return this
   */
  public native function compile():ext.Template;

  /**
   * Applies the supplied values to the template and inserts the new node(s) after el.
   * @param el The context element
   * @param values The template values. See <a href="#!/api/Ext.Template-method-applyTemplate" rel="Ext.Template-method-applyTemplate" class="docClass">applyTemplate</a> for details.
   * @param returnElement true to return a <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a>.
   * @return The new node or Element
   */
  public native function insertAfter(el:*, values:*, returnElement:Boolean = false):*;

  /**
   * Applies the supplied values to the template and inserts the new node(s) before el.
   * @param el The context element
   * @param values The template values. See <a href="#!/api/Ext.Template-method-applyTemplate" rel="Ext.Template-method-applyTemplate" class="docClass">applyTemplate</a> for details.
   * @param returnElement true to return a <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a>.
   * @return The new node or Element
   */
  public native function insertBefore(el:*, values:*, returnElement:Boolean = false):*;

  /**
   * Applies the supplied values to the template and inserts the new node(s) as the first child of el.
   * @param el The context element
   * @param values The template values. See <a href="#!/api/Ext.Template-method-applyTemplate" rel="Ext.Template-method-applyTemplate" class="docClass">applyTemplate</a> for details.
   * @param returnElement true to return a <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a>.
   * @return The new node or Element
   */
  public native function insertFirst(el:*, values:*, returnElement:Boolean = false):*;

  /**
   * Applies the supplied values to the template and overwrites the content of el with the new node(s).
   * @param el The context element
   * @param values The template values. See <a href="#!/api/Ext.Template-method-applyTemplate" rel="Ext.Template-method-applyTemplate" class="docClass">applyTemplate</a> for details.
   * @param returnElement true to return a <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a>.
   * @return The new node or Element
   */
  public native function overwrite(el:*, values:*, returnElement:Boolean = false):*;

  /**
   * Sets the HTML used as the template and optionally compiles it.
   * @param compile True to compile the template.
   * @return this
   */
  public native function set(html:String, compile:Boolean = false):ext.Template;
}
}