package ext {

[Uses("ext.Base")]
[Native("Ext.ZIndexManager", require)]
/**
 * A class that manages a group of <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">Ext.Component.floating</a> Components and
 * provides z-order management, and Component activation behavior, including masking
 * below the active (topmost) Component.
 *
 * <p><a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">Floating</a> Components which are rendered directly
 * into the document (such as <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Window</a>s) which are
 * <a href="#!/api/Ext.Component-method-show" rel="Ext.Component-method-show" class="docClass">show</a>n are managed by a
 * <a href="#!/api/Ext.WindowManager" rel="Ext.WindowManager" class="docClass">global instance</a>.</p>
 *
 * <p><a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">Floating</a> Components which are descendants of
 * <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> <em>Containers</em> (for example a
 * <a href="#!/api/Ext.view.BoundList" rel="Ext.view.BoundList" class="docClass">BoundList</a> within an <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Window</a>,
 * or a <a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">Menu</a>), are managed by a ZIndexManager owned by that floating Container. Therefore
 * ComboBox dropdowns within Windows will have managed z-indices guaranteed to be correct, relative to the Window.</p>
 */
public class ZIndexManager extends ext.Base {
  /**
   * Brings the specified Component to the front of any other active Components in this ZIndexManager.
   * @param comp The id of the Component or a <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> instance.
   * @param preventFocus Pass <code>true</code> to prevent the component being focused when moved to front.
   * @return True if the component was brought to the front, else false
   * if it was already in front, or another component remains at the front due to configuration (eg
   * <a href="#!/api/Ext.util.Floating-cfg-alwaysOnTop" rel="Ext.util.Floating-cfg-alwaysOnTop" class="docClass">Ext.util.Floating.alwaysOnTop</a>, or if the component was not found.
   */
  public native function bringToFront(comp:*, preventFocus:Boolean):Boolean;

  /**
   * Executes the specified function once for every Component in this ZIndexManager, passing each
   * Component as the only parameter. Returning false from the function will stop the iteration.
   * @param fn The function to execute for each item
   * @param scope The scope (this reference) in which the function
   * is executed. Defaults to the current Component in the iteration.
   */
  public native function each(fn:Function, scope:Object = null):void;

  /**
   * Executes the specified function once for every Component in this ZIndexManager, passing each
   * Component as the only parameter. Returning false from the function will stop the iteration.
   * The components are passed to the function starting at the bottom and proceeding to the top.
   * @param fn The function to execute for each item
   * @param scope The scope (this reference) in which the function
   * is executed. Defaults to the current Component in the iteration.
   */
  public native function eachBottomUp(fn:Function, scope:Object = null):void;

  /**
   * Executes the specified function once for every Component in this ZIndexManager, passing each
   * Component as the only parameter. Returning false from the function will stop the iteration.
   * The components are passed to the function starting at the top and proceeding to the bottom.
   * @param fn The function to execute for each item
   * @param scope The scope (this reference) in which the function
   * is executed. Defaults to the current Component in the iteration.
   */
  public native function eachTopDown(fn:Function, scope:Object = null):void;

  /**
   * Gets a registered Component by id.
   * @param id The id of the Component or a <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> instance
   */
  public native function get(id:*):ext.Component;

  /**
   * Gets the currently-active Component in this ZIndexManager.
   * @return The active Component
   */
  public native function getActive():ext.Component;

  /**
   * Returns zero or more Components in this ZIndexManager using the custom search function passed to this method.
   * The function should accept a single <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> reference as its only argument and should
   * return true if the Component matches the search criteria, otherwise it should return false.
   * @param fn The search function
   * @param scope The scope (<code>this</code> reference) in which the function is executed.
   * Defaults to the Component being tested. That gets passed to the function if not specified.
   * @return An array of zero or more matching floating components.
   */
  public native function getBy(fn:Function, scope:Object = null):Array;

  /**
   * Hides all Components managed by this ZIndexManager.
   */
  public native function hideAll():void;

  /**
   * Registers a floating <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> with this ZIndexManager. This should not
   * need to be called under normal circumstances. Floating Components (such as Windows,
   * BoundLists and Menus) are automatically registered with a
   * <a href="#!/api/Ext.Component-property-zIndexManager" rel="Ext.Component-property-zIndexManager" class="docClass">zIndexManager</a> at render time.
   *
   * <p>Where this may be useful is moving Windows between two ZIndexManagers. For example,
   * to bring the <a href="#!/api/Ext.MessageBox" rel="Ext.MessageBox" class="docClass">Ext.MessageBox</a> dialog under the same manager as the Desktop's
   * ZIndexManager in the desktop sample app:</p>
   *
   * <pre><code>MyDesktop.getDesktop().getManager().register(<a href="#!/api/Ext.MessageBox" rel="Ext.MessageBox" class="docClass">Ext.MessageBox</a>);
   * </code></pre>
   * @param comp The Component to register.
   */
  public native function register(comp:ext.Component):void;

  /**
   * Sends the specified Component to the back of other active Components in this ZIndexManager.
   * @param comp The id of the Component or a <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> instance
   * @return The Component
   */
  public native function sendToBack(comp:*):ext.Component;

  /**
   * Unregisters a <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> from this ZIndexManager. This should not
   * need to be called. Components are automatically unregistered upon destruction.
   * See <a href="#!/api/Ext.ZIndexManager-method-register" rel="Ext.ZIndexManager-method-register" class="docClass">register</a>.
   * @param comp The Component to unregister.
   */
  public native function unregister(comp:ext.Component):void;
}
}