package ext.app {
import ext.Base;
import ext.ExtClass;
import ext.data.Store;

[Uses("ext.app.BaseController")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.app.Controller", require)]
/**
 * Controllers are the glue that binds an application together. That said, their main
 * purpose is to listen for events (usually from views) and take some action. Here's how
 * we might create a Controller to manage Users:
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.controller.Users', {
 *      extend: '<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>',
 *
 *      init: function() {
 *          console.log('Initialized Users! This happens before ' +
 *                      'the Application launch() function is called');
 *      }
 *  });
 * </code></pre>
 *
 * <p>The init function is a special method that is called when your application boots. It is
 * called before the <a href="#!/api/Ext.app.Application" rel="Ext.app.Application" class="docClass">Application</a>'s launch function is executed.
 * This creates an area you can run code prior to Viewport creation.</p>
 *
 * <p>The controller's <a href="#!/api/Ext.app.Controller-method-control" rel="Ext.app.Controller-method-control" class="docClass">control</a> function
 * makes it easy to listen to events on your view classes and take some action with a
 * handler function. Let's update our Users controller to tell us when the panel is
 * rendered:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.controller.Users', {
 *      extend: '<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>',
 *
 *      control: {
 *          'viewport &gt; panel': {
 *              render: 'onPanelRendered'
 *          }
 *      }
 *
 *      onPanelRendered: function() {
 *          console.log('The panel was rendered');
 *      }
 *  });
 * </code></pre>
 *
 * <p>The <a href="#!/api/Ext.app.BaseController-method-control" rel="Ext.app.BaseController-method-control" class="docClass">control method</a> has now set up
 * listeners on views in our application. The control method uses the ComponentQuery
 * engine to quickly and easily get references to components on the page. If you are not
 * familiar with ComponentQuery yet, be sure to check out the
 * <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">documentation</a>. In brief, it allows us to pass a
 * CSS-like selector that will find every matching component on the page.</p>
 *
 * <p>In our init function above, we supplied 'viewport > panel', which translates to "find me
 * every Panel that is a direct child of a Viewport". We then supplied an object that maps
 * event names (just 'render' in this case) to handler functions. In short, whenever
 * a component that matches our selector fires a 'render' event, our
 * onPanelRendered function is called.</p>
 *
 * <h2>Event domains</h2>
 *
 * <p>In Ext JS 4.2, we introduced the concept of event domains. In terms of MVC, an event
 * domain is one or more base classes that fire events to which a Controller wants to
 * listen. Besides Component event domain that encompass <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>-descended
 * Views, Controllers now can listen to events from data Stores, Ext Direct Providers,
 * other Controllers, and <a href="#!/api/Ext.GlobalEvents" rel="Ext.GlobalEvents" class="docClass">Ext.GlobalEvents</a>. This feature provides a way to communicate
 * between parts of the whole application without the need to bind controllers together
 * tightly, and allows to develop and test application parts in isolation.</p>
 *
 * <p>See usage examples in <a href="#!/api/Ext.app.Controller-method-listen" rel="Ext.app.Controller-method-listen" class="docClass">listen</a> method documentation.</p>
 *
 * <h2>Using refs</h2>
 *
 * <p>One of the most useful parts of Controllers is the ref system. These use
 * <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> to make it really easy to get references to Views on your
 * page. Let's look at an example of this now:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.controller.Users', {
 *      extend: '<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>',
 *
 *      refs: [{
 *          ref: 'list',
 *          selector: 'grid'
 *      }],
 *
 *      control: {
 *          'button': {
 *              click: 'refreshGrid'
 *          }
 *      },
 *
 *      refreshGrid: function() {
 *          this.getList().store.load();
 *      }
 *  });
 * </code></pre>
 *
 * <p>This example assumes the existence of a <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Grid</a> on the page, which
 * contains a single button to refresh the Grid when clicked. In our refs array, we set up
 * a reference to the grid. There are two parts to this - the 'selector', which is a
 * <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">ComponentQuery</a> selector which finds any grid on the page and
 * assigns it to the reference 'list'.</p>
 *
 * <p>By giving the reference a name, we get a number of things for free. The first is the
 * getList function that we use in the refreshGrid method above. This is generated
 * automatically by the Controller based on the name of our ref, which was capitalized and
 * prepended with get to go from 'list' to 'getList'.</p>
 *
 * <p>The way this works is that the first time getList is called by your code, the
 * ComponentQuery selector is run and the first component that matches the selector
 * ('grid' in this case) will be returned. All future calls to getList will use a cached
 * reference to that grid. Usually it is advised to use a specific ComponentQuery selector
 * that will only match a single View in your application (in the case above our selector
 * will match any grid on the page).</p>
 *
 * <p>Bringing it all together, we configure control
 * to listen to any click on a <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">button</a> and call our refreshGrid
 * function (again, this will match any button on the page so we advise a more specific
 * selector than just 'button', but have left it this way for simplicity). When the button
 * is clicked we use out getList function to refresh the grid.</p>
 *
 * <p>You can create any number of refs and control any number of components this way, simply
 * adding more functions to your Controller as you go. For an example of real-world usage
 * of Controllers see the Feed Viewer example in the examples/app/feed-viewer folder in
 * the SDK download.</p>
 *
 * <h2>Generated getter methods</h2>
 *
 * <p>Refs aren't the only thing that generate convenient getter methods. Controllers often
 * have to deal with Models and Stores so the framework offers a couple of easy ways to
 * get access to those too. Let's look at another example:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.controller.Users', {
 *      extend: '<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>',
 *
 *      models: ['User'],
 *      stores: ['AllUsers', 'AdminUsers'],
 *
 *      init: function() {
 *          var User, allUsers, ed;
 *
 *          User = this.getUserModel();
 *          allUsers = this.getAllUsersStore();
 *
 *          ed = new User({ name: 'Ed' });
 *          allUsers.add(ed);
 *      }
 *  });
 * </code></pre>
 *
 * <p>By specifying Models and Stores that the Controller cares about, it again dynamically
 * loads them from the appropriate locations (app/model/User.js, app/store/AllUsers.js and
 * app/store/AdminUsers.js in this case) and creates getter functions for them all. The
 * example above will create a new User model instance and add it to the AllUsers Store.
 * Of course, you could do anything in this function but in this case we just did
 * something simple to demonstrate the functionality.</p>
 *
 * <h2>Further Reading</h2>
 *
 * <p>For more information about writing Ext JS 5 applications, please see the
 * <a href="../../../application_architecture/application_architecture.html">Application Architecture</a>.
 * Also see the <a href="#!/api/Ext.app.Application" rel="Ext.app.Application" class="docClass">Ext.app.Application</a> documentation.</p>
 */
public class Controller extends ext.app.BaseController {
  /**
   * Creates new Controller.
   * @param config Configuration object.
   */
  public function Controller(config:ext.app.Controller = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The <a href="#!/api/Ext.app.Application" rel="Ext.app.Application" class="docClass">Ext.app.Application</a> for this controller accessible via the getApplication method.
   * @see #getApplication()
   */
  public native function get application():ext.app.Application;

  [ExtConfig]
  /**
   * Array of models to require from AppName.model namespace. For example:
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>("MyApp.controller.Foo", {
   *      extend: "<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>",
   *      models: ['User', 'Vehicle']
   *  });
   * </code></pre>
   *
   * <p>This is equivalent to:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>("MyApp.controller.Foo", {
   *      extend: "<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>",
   *      requires: ['MyApp.model.User', 'MyApp.model.Vehicle'],
   *
   *      getUserModel: function() {
   *          return this.getModel("User");
   *      },
   *
   *      getVehicleModel: function() {
   *          return this.getModel("Vehicle");
   *      }
   *  });
   * </code></pre>
   *
   * <p><strong>Note:</strong> If the model has a different namespace than that of the
   * application you will need to specify the full class name as well as define a path
   * in the <a href="#!/api/Ext.Loader-cfg-paths" rel="Ext.Loader-cfg-paths" class="docClass">Loader's paths</a> config or
   * <a href="#!/api/Ext.Loader-method-setPath" rel="Ext.Loader-method-setPath" class="docClass">setPath</a> method.</p>
   */
  public native function get models():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set models(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The refs config creates a getter method on the controller that internally
   * uses <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> to fetch the component instance using the configured
   * selector.  The following example will add the <code>getList</code> method to
   * the controller and will return the first component in the application
   * hierarchy with an xtype of "grid".  By default, <em>undefined</em> will be returned
   * when the query does not locate the target component.
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.controller.Foo', {
   *     extend: '<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>',
   *
   *     refs: [{
   *         ref: 'list',
   *         selector: 'grid'
   *     }]
   * });
   * </code></pre>
   *
   * <p>The following fields may be used in the ref definition:</p>
   *
   * <ul>
   * <li><code>ref</code> - name of the reference.</li>
   * <li><code>selector</code> - <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector to access the component.</li>
   * <li><code>autoCreate</code> - True to create the component automatically if not found on
   * page.</li>
   * <li><code>forceCreate</code> - True to force the creation of the component every time
   * reference is accessed (when <code>get&lt;REFNAME&gt;</code> is called).</li>
   * <li><code>xtype</code> - Used to create the component by its xtype with <code>autoCreate</code> or
   * <code>forceCreate</code>. If you don't provide <code>xtype</code>, an <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> instance will
   * be created.</li>
   * </ul>
   *
   *
   * <p>The following example will create a <code>getList</code> and <code>getUser</code> method on the
   * controller.</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.controller.Foo', {
   *     extend: '<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>',
   *
   *     refs: [{
   *         list: 'grid',
   *         user: {
   *             autoCreate: true,
   *             selector: 'form',
   *             xtype: 'form'
   *         }
   *     }]
   * });
   * </code></pre>
   * @see #getRefs()
   * @see #setRefs()
   */
  public native function get refs():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set refs(value:*):void;

  [ExtConfig]
  /**
   * Array of stores to require from AppName.store namespace and to generate getter methods for.
   * For example:
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>("MyApp.controller.Foo", {
   *      extend: "<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>",
   *      stores: ['Users', 'Vehicles']
   *  });
   * </code></pre>
   *
   * <p>This is equivalent to:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>("MyApp.controller.Foo", {
   *      extend: "<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>",
   *
   *      requires: [
   *          'MyApp.store.Users',
   *          'MyApp.store.Vehicles'
   *      ]
   *
   *      getUsersStore: function() {
   *          return this.getStore("Users");
   *      },
   *
   *      getVehiclesStore: function() {
   *          return this.getStore("Vehicles");
   *      }
   *  });
   * </code></pre>
   *
   * <p><strong>Note:</strong> If the store has a different namespace than that of the
   * application you will need to specify the full class name as well as define a path
   * in the <a href="#!/api/Ext.Loader-cfg-paths" rel="Ext.Loader-cfg-paths" class="docClass">Loader's paths</a> config or
   * <a href="#!/api/Ext.Loader-method-setPath" rel="Ext.Loader-method-setPath" class="docClass">setPath</a> method.</p>
   */
  public native function get stores():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set stores(value:*):void;

  [ExtConfig]
  /**
   * Array of views to require from AppName.view namespace and to generate getter methods for.
   * For example:
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>("MyApp.controller.Foo", {
   *      extend: "<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>",
   *      views: ['List', 'Detail']
   *  });
   * </code></pre>
   *
   * <p>This is equivalent to:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>("MyApp.controller.Foo", {
   *      extend: "<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>",
   *      requires: ['MyApp.view.List', 'MyApp.view.Detail'],
   *
   *      getListView: function() {
   *          return this.getView("List");
   *      },
   *
   *      getDetailView: function() {
   *          return this.getView("Detail");
   *      }
   *  });
   * </code></pre>
   *
   * <p><strong>Note:</strong> If the view has a different namespace than that of the
   * application you will need to specify the full class name as well as define a path
   * in the <a href="#!/api/Ext.Loader-cfg-paths" rel="Ext.Loader-cfg-paths" class="docClass">Loader's paths</a> config or
   * <a href="#!/api/Ext.Loader-method-setPath" rel="Ext.Loader-method-setPath" class="docClass">setPath</a> method.</p>
   */
  public native function get views():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set views(value:*):void;

  /**
   * Allow the controller to resume receiving events from the event bus.
   * Routes will also be able to begin firing on this controller.
   * Also see <a href="#!/api/Ext.app.Controller-method-deactivate" rel="Ext.app.Controller-method-deactivate" class="docClass">deactivate</a>.
   */
  public native function activate():void;

  /**
   * Registers one or more <a href="#!/api/Ext.app.Controller-cfg-refs" rel="Ext.app.Controller-cfg-refs" class="docClass">references</a>.
   */
  public native function addRef(refs:*):void;

  /**
   * Prevent this controller from receiving events from the event bus.
   * Routes will also not be triggered on inactive controllers unless
   * the <a href="#!/api/Ext.app.route.Route-cfg-allowInactive" rel="Ext.app.route.Route-cfg-allowInactive" class="docClass">Ext.app.route.Route.allowInactive</a> flag is set.
   * Also see <a href="#!/api/Ext.app.Controller-method-activate" rel="Ext.app.Controller-method-activate" class="docClass">activate</a>.
   */
  public native function deactivate():void;

  /**
   * Returns the value of <a href="#!/api/Ext.app.Controller-cfg-application" rel="Ext.app.Controller-cfg-application" class="docClass">application</a>.
   */
  public native function getApplication():ext.app.Application;

  /**
   * Returns instance of a <a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Controller</a> with the given id.
   * When controller doesn't exist yet, it's created. Note that this method depends
   * on Application instance and will return undefined when Application is not
   * accessible. The only exception is when this Controller instance's id is requested;
   * in that case we always return the instance even if Application is no available.
   * @return controller instance or undefined.
   */
  public native function getController(id:String):ext.app.Controller;

  /**
   * Returns a <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> class with the given name.
   * @return A class ultimately derived from <code><a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a></code>.
   */
  public native function getModel(name:String):ext.ExtClass;

  /**
   * Returns instance of a <a href="#!/api/Ext.app.Profile" rel="Ext.app.Profile" class="docClass">Profile</a> with the given name.
   * @return a profile instance.
   */
  public native function getProfile(name:String):String;

  /**
   * Returns the value of <a href="#!/api/Ext.app.Controller-cfg-refs" rel="Ext.app.Controller-cfg-refs" class="docClass">refs</a>.
   */
  public native function getRefs():*;

  /**
   * Returns instance of a <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a> with the given name.
   * When store doesn't exist yet, it's created.
   * @return a store instance.
   */
  public native function getStore(name:String):ext.data.Store;

  /**
   * Returns a View class with the given name.  To create an instance of the view,
   * you can use it like it's used by Application to create the Viewport:
   *
   * <pre><code>this.getView('Viewport').create();
   * </code></pre>
   * @return a view class.
   */
  public native function getView(view:String):ext.Base;

  /**
   * Returns <code>true</code> if a <a href="#!/api/Ext.app.Controller-cfg-refs" rel="Ext.app.Controller-cfg-refs" class="docClass">reference</a> is registered.
   * @param ref The name of the ref to check for.
   */
  public native function hasRef(ref:String):Boolean;

  /**
   * A template method that is called when your application boots. It is called before the
   * <a href="#!/api/Ext.app.Application" rel="Ext.app.Application" class="docClass">Application</a>'s launch function is executed so gives a hook point
   * to run any code before your Viewport is created.
   */
  public native function init(application:ext.app.Application):void;

  /**
   * Checks if this controller is active. See <a href="#!/api/Ext.app.Controller-method-activate" rel="Ext.app.Controller-method-activate" class="docClass">activate</a> &amp;
   * <a href="#!/api/Ext.app.Controller-method-deactivate" rel="Ext.app.Controller-method-deactivate" class="docClass">deactivate</a>.
   * @return <code>true</code> if this controller is active.
   */
  public native function isActive():Boolean;

  /**
   * A template method like <a href="#!/api/Ext.app.Controller-method-init" rel="Ext.app.Controller-method-init" class="docClass">init</a>, but called after the viewport is created.
   * This is called after the <a href="#!/api/Ext.app.Application-method-launch" rel="Ext.app.Application-method-launch" class="docClass">launch</a> method of Application
   * is executed.
   */
  public native function onLaunch(application:ext.app.Application):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Controller-cfg-application" rel="Ext.app.Controller-cfg-application" class="docClass">application</a>.
   * @param application The new value.
   */
  public native function setApplication(application:ext.app.Application):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Controller-cfg-refs" rel="Ext.app.Controller-cfg-refs" class="docClass">refs</a>.
   * @param refs The new value.
   */
  public native function setRefs(refs:*):void;
}
}