package ext.app {
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.app.Profile", require)]
/**
 * A Profile represents a range of devices that fall under a common category. For the vast majority of apps that use
 * device profiles, the app defines a Phone profile and a Tablet profile. Doing this enables you to easily customize
 * the experience for the different sized screens offered by those device types.
 *
 * <p>Only one Profile can be active at a time, and each Profile defines a simple <a href="#!/api/Ext.app.Profile-method-isActive" rel="Ext.app.Profile-method-isActive" class="docClass">isActive</a> function that should
 * return either true or false. The first Profile to return true from its isActive function is set as your Application's
 * <a href="#!/api/Ext.app.Application-cfg-currentProfile" rel="Ext.app.Application-cfg-currentProfile" class="docClass">current profile</a>.</p>
 *
 * <p>A Profile can define any number of <a href="#!/api/Ext.app.Profile-cfg-models" rel="Ext.app.Profile-cfg-models" class="docClass">models</a>, <a href="#!/api/Ext.app.Profile-cfg-views" rel="Ext.app.Profile-cfg-views" class="docClass">views</a>, <a href="#!/api/Ext.app.Profile-cfg-controllers" rel="Ext.app.Profile-cfg-controllers" class="docClass">controllers</a> and <a href="#!/api/Ext.app.Profile-cfg-stores" rel="Ext.app.Profile-cfg-stores" class="docClass">stores</a> which
 * will be loaded if the Profile is activated. It can also define a <a href="#!/api/Ext.app.Profile-method-launch" rel="Ext.app.Profile-method-launch" class="docClass">launch</a> function that will be called after
 * all of its dependencies have been loaded, just before the <a href="#!/api/Ext.app.Application-method-launch" rel="Ext.app.Application-method-launch" class="docClass">application launch</a>
 * function is called.</p>
 *
 * <h2>Sample Usage</h2>
 *
 * <p>First you need to tell your Application about your Profile(s):</p>
 *
 * <pre><code>Ext.application({
 *     name: 'MyApp',
 *     profiles: ['Phone', 'Tablet']
 * });
 * </code></pre>
 *
 * <p>This will load app/profile/Phone.js and app/profile/Tablet.js. Here's how we might define the Phone profile:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.profile.Phone', {
 *     extend: '<a href="#!/api/Ext.app.Profile" rel="Ext.app.Profile" class="docClass">Ext.app.Profile</a>',
 *
 *     views: ['Main'],
 *
 *     isActive: function() {
 *         return <a href="#!/api/Ext.os-method-is" rel="Ext.os-method-is" class="docClass">Ext.os.is</a>('Phone');
 *     }
 * });
 * </code></pre>
 *
 * <p>The isActive function returns true if we detect that we are running on a phone device. If that is the case the
 * Application will set this Profile active and load the 'Main' view specified in the Profile's <a href="#!/api/Ext.app.Profile-cfg-views" rel="Ext.app.Profile-cfg-views" class="docClass">views</a> config.</p>
 *
 * <h2>Class Specializations</h2>
 *
 * <p>Because Profiles are specializations of an application, all of the models, views, controllers and stores defined
 * in a Profile are expected to be namespaced under the name of the Profile. Here's an expanded form of the example
 * above:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.profile.Phone', {
 *     extend: '<a href="#!/api/Ext.app.Profile" rel="Ext.app.Profile" class="docClass">Ext.app.Profile</a>',
 *
 *     views: ['Main'],
 *     controllers: ['Signup'],
 *     models: ['MyApp.model.Group'],
 *
 *     isActive: function() {
 *         return <a href="#!/api/Ext.os-method-is" rel="Ext.os-method-is" class="docClass">Ext.os.is</a>('Phone');
 *     }
 * });
 * </code></pre>
 *
 * <p>In this case, the Profile is going to load <em>app/view/phone/Main.js</em>, <em>app/controller/phone/Signup.js</em> and
 * <em>app/model/Group.js</em>. Notice that in each of the first two cases the name of the profile ('phone' in this case) was
 * injected into the class names. In the third case we specified the full Model name (for Group) so the Profile name
 * was not injected.</p>
 *
 * <p>For a fuller understanding of the ideas behind Profiles and how best to use them in your app, we suggest you read
 * the <a href="/touch/2.4/core_concepts/device_profiles.html">device profiles guide</a>.</p>
 */
public class Profile extends ext.mixin.Observable {
  /**
   * Creates a new Profile instance
   */
  public function Profile(config:ext.app.Profile = null) {
    super();
  }

  /**
   * <code>true</code> to identify an object as an instance of <code><a href="#!/api/Ext.app.Profile" rel="Ext.app.Profile" class="docClass">Ext.app.Profile</a></code>
   * @default true
   */
  public native function get isProfile():Boolean;

  /**
   * @private
   */
  public native function set isProfile(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The <a href="#!/api/Ext.app.Application" rel="Ext.app.Application" class="docClass">Application</a> instance to which this Profile is
   * bound. This is set automatically.
   * @see #getApplication()
   */
  public native function get application():ext.app.Application;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Any additional <a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Controllers</a> to load for this profile.
   * Note that each item here will be prepended with the Profile namespace when loaded.
   *
   * <p>Example usage:</p>
   *
   * <pre><code>controllers: [
   *     'Users',
   *     'MyApp.controller.Products'
   * ]
   * </code></pre>
   *
   * <p>This will load <em>MyApp.controller.tablet.Users</em> and <em>MyApp.controller.Products</em>.</p>
   * @default []
   * @see #getControllers()
   * @see #setControllers()
   */
  public native function get controllers():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set controllers(value:Array):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @default {$value: null, lazy: true}
   * @see #getMainView()
   * @see #setMainView()
   */
  public native function get mainView():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set mainView(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Any additional <a href="#!/api/Ext.app.Application-cfg-models" rel="Ext.app.Application-cfg-models" class="docClass">Models</a> to load for this profile.
   * Note that each item here will be prepended with the Profile namespace when loaded.
   *
   * <p>Example usage:</p>
   *
   * <pre><code>models: [
   *     'Group',
   *     'MyApp.model.User'
   * ]
   * </code></pre>
   *
   * <p>This will load <em>MyApp.model.tablet.Group</em> and <em>MyApp.model.User</em>.</p>
   * @default []
   * @see #getModels()
   * @see #setModels()
   */
  public native function get models():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set models(value:Array):void;

  [ExtConfig]
  /**
   * The name of this Profile. Defaults to the last section of the class name (e.g. a profile
   * called MyApp.profile.Phone will default the name to 'Phone').
   * @see #setName()
   */
  public native function get name():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set name(value:String):void;

  [ExtConfig]
  /**
   * The namespace that this Profile's classes can be found in. Defaults to the lowercase
   * Profile <a href="#!/api/Ext.app.Profile-cfg-name" rel="Ext.app.Profile-cfg-name" class="docClass">name</a>, for example a Profile called MyApp.profile.Phone will by default have a 'phone'
   * namespace, which means that this Profile's additional models, stores, views and controllers will be loaded
   * from the MyApp.model.phone.<em>, MyApp.store.phone.</em>, MyApp.view.phone.<em> and MyApp.controller.phone.</em> namespaces
   * respectively.
   * @see #setNamespace()
   */
  public native function get namespace():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set namespace(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Any additional <a href="#!/api/Ext.app.Application-cfg-stores" rel="Ext.app.Application-cfg-stores" class="docClass">Stores</a> to load for this profile.
   * Note that each item here will be prepended with the Profile namespace when loaded.
   *
   * <p>Example usage:</p>
   *
   * <pre><code>stores: [
   *     'Users',
   *     'MyApp.store.Products'
   * ]
   * </code></pre>
   *
   * <p>This will load <em>MyApp.store.tablet.Users</em> and <em>MyApp.store.Products</em>.</p>
   * @default []
   * @see #getStores()
   * @see #setStores()
   */
  public native function get stores():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set stores(value:Array):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This config allows the active profile to define a set of <code>xtypes</code> and map them
   * to desired classes and default configurations. Normally an <code>xtype</code> is statically
   * declared by a <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">component</a> in its class definition. This
   * mechanism allows the active profile to control a set of these types.
   *
   * <p>Example:</p>
   *
   * <pre><code> views: {
   *      // The "main" xtype maps to MyApp.view.tablet.Main
   *      //
   *      main: 'MyApp.view.tablet.Main',
   *
   *      // The "inbox" xtype maps to a subclass of MyApp.view.Inbox (created
   *      // by this mechanism) that sets the "mode" config to "compact".
   *      //
   *      inbox: {
   *          xclass: 'MyApp.view.Inbox',
   *          mode: 'compact'
   *      }
   *  }
   * </code></pre>
   *
   * <p>Note that class names used in this form must be full class names, unlike the
   * historical usage of <code>views</code>. Further, these views cannot be accessed using the
   * <code>getView</code> method but rather via their assigned <code>xtype</code>.</p>
   *
   * <p>The historical usage of this config is enabled when an array is passed. In this
   * case, these are simply additional <a href="#!/api/Ext.app.Application-cfg-views" rel="Ext.app.Application-cfg-views" class="docClass">views</a> to
   * load for this profile. Note that each item here will be prepended with the
   * Profile namespace when loaded.</p>
   *
   * <p>Example usage:</p>
   *
   * <pre><code>views: [
   *     'Main',
   *     'MyApp.view.Login'
   * ]
   * </code></pre>
   *
   * <p>This will load <em>MyApp.view.tablet.Main</em> and <em>MyApp.view.Login</em>. While supported,
   * this usage is discouraged in favor of <code>xtype</code> mapping.</p>
   * @default []
   * @see #getViews()
   * @see #setViews()
   */
  public native function get views():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set views(value:*):void;

  /**
   * Returns the value of <a href="#!/api/Ext.app.Profile-cfg-application" rel="Ext.app.Profile-cfg-application" class="docClass">application</a>.
   */
  public native function getApplication():ext.app.Application;

  /**
   * Returns the value of <a href="#!/api/Ext.app.Profile-cfg-controllers" rel="Ext.app.Profile-cfg-controllers" class="docClass">controllers</a>.
   */
  public native function getControllers():Array;

  /**
   * Returns the value of <a href="#!/api/Ext.app.Profile-cfg-mainView" rel="Ext.app.Profile-cfg-mainView" class="docClass">mainView</a>.
   */
  public native function getMainView():String;

  /**
   * Returns the value of <a href="#!/api/Ext.app.Profile-cfg-models" rel="Ext.app.Profile-cfg-models" class="docClass">models</a>.
   */
  public native function getModels():Array;

  /**
   * Returns the value of <a href="#!/api/Ext.app.Profile-cfg-stores" rel="Ext.app.Profile-cfg-stores" class="docClass">stores</a>.
   */
  public native function getStores():Array;

  /**
   * Returns the value of <a href="#!/api/Ext.app.Profile-cfg-views" rel="Ext.app.Profile-cfg-views" class="docClass">views</a>.
   */
  public native function getViews():*;

  /**
   * Determines whether or not this Profile is active on the device isActive is executed on. Should return true if
   * this profile is meant to be active on this device, false otherwise. Each Profile should implement this function
   * (the default implementation just returns false).
   * @return True if this Profile should be activated on the device it is running on, false otherwise
   */
  public native function isActive():Boolean;

  /**
   * The launch function is called by the <a href="#!/api/Ext.app.Application" rel="Ext.app.Application" class="docClass">Application</a> if this Profile's <a href="#!/api/Ext.app.Profile-method-isActive" rel="Ext.app.Profile-method-isActive" class="docClass">isActive</a>
   * function returned true. This is typically the best place to run any profile-specific app launch code. Example
   * usage:
   *
   * <pre><code>launch: function() {
   *     <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('MyApp.view.tablet.Main');
   * }
   * </code></pre>
   */
  public native function launch():void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Profile-cfg-application" rel="Ext.app.Profile-cfg-application" class="docClass">application</a>.
   * @param application The new value.
   */
  public native function setApplication(application:ext.app.Application):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Profile-cfg-controllers" rel="Ext.app.Profile-cfg-controllers" class="docClass">controllers</a>.
   * @param controllers The new value.
   */
  public native function setControllers(controllers:Array):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Profile-cfg-mainView" rel="Ext.app.Profile-cfg-mainView" class="docClass">mainView</a>.
   * @param mainView The new value.
   */
  public native function setMainView(mainView:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Profile-cfg-models" rel="Ext.app.Profile-cfg-models" class="docClass">models</a>.
   * @param models The new value.
   */
  public native function setModels(models:Array):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Profile-cfg-name" rel="Ext.app.Profile-cfg-name" class="docClass">name</a>.
   * @param name The new value.
   */
  public native function setName(name:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Profile-cfg-namespace" rel="Ext.app.Profile-cfg-namespace" class="docClass">namespace</a>.
   * @param namespace The new value.
   */
  public native function setNamespace(namespace:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Profile-cfg-stores" rel="Ext.app.Profile-cfg-stores" class="docClass">stores</a>.
   * @param stores The new value.
   */
  public native function setStores(stores:Array):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Profile-cfg-views" rel="Ext.app.Profile-cfg-views" class="docClass">views</a>.
   * @param views The new value.
   */
  public native function setViews(views:*):void;

  /**
   * This method is called once the profile is determined to be the active profile. This
   * initialization is performed before controllers are initialized and therefore also
   * before launch.
   * @since 6.0.1
   */
  protected native function init():void;
}
}