package ext.app {
import ext.Base;
import ext.container.Container;
import ext.data.Session;
import ext.data.Store;
import ext.mixin.IFactoryable;
import ext.mixin.IIdentifiable;

[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.mixin.Identifiable")]
[Native("Ext.app.ViewModel", require)]
[ExtConfig]
/**
 * This class manages arbitrary data and its relationship to data models. Instances of
 * <code>ViewModel</code> are associated with some <code>Component</code> and then used by their child items
 * for the purposes of Data Binding.
 *
 * <h1>Binding</h1>
 *
 * <p>The most commonly used aspect of a <code>ViewModel</code> is the <code>bind</code> method. This method takes
 * a "bind descriptor" (see below) and a callback to call when the data indicated by the
 * bind descriptor either becomes available or changes.</p>
 *
 * <p>The <code>bind</code> method, based on the bind descriptor given, will return different types of
 * "binding" objects. These objects maintain the connection between the requested data and
 * the callback. Bindings ultimately derive from <code><a href="#!/api/Ext.app.bind.BaseBinding" rel="Ext.app.bind.BaseBinding" class="docClass">Ext.app.bind.BaseBinding</a></code>
 * which provides several methods to help manage the binding.</p>
 *
 * <p>Perhaps the most important method is <code>destroy</code>. When the binding is no longer needed
 * it is important to remember to <code>destroy</code> it. Leaking bindings can cause performance
 * problems or worse when callbacks are called at unexpected times.</p>
 *
 * <p>The types of bindings produced by <code>bind</code> are:</p>
 *
 * <ul>
 * <li><code><a href="#!/api/Ext.app.bind.Binding" rel="Ext.app.bind.Binding" class="docClass">Ext.app.bind.Binding</a></code></li>
 * <li><code><a href="#!/api/Ext.app.bind.Multi" rel="Ext.app.bind.Multi" class="docClass">Ext.app.bind.Multi</a></code></li>
 * <li><code><a href="#!/api/Ext.app.bind.TemplateBinding" rel="Ext.app.bind.TemplateBinding" class="docClass">Ext.app.bind.TemplateBinding</a></code></li>
 * </ul>
 *
 *
 * <h2>Bind Descriptors</h2>
 *
 * <p>A "bind descriptor" is a value (a String, an Object or an array of these) that describe
 * the desired data. Any piece of data in the <code>ViewModel</code> can be described by a bind
 * descriptor.</p>
 *
 * <h3>Textual Bind Descriptors</h3>
 *
 * <p>The simplest and most common form of bind descriptors are strings that look like an
 * <code><a href="#!/api/Ext.Template" rel="Ext.Template" class="docClass">Ext.Template</a></code> containing text and tokens surrounded by "{}" with dot notation inside
 * to traverse objects and their properties.</p>
 *
 * <p>For example:</p>
 *
 * <ul>
 * <li><code>'Hello {user.name}!'</code></li>
 * <li><code>'You have selected "{selectedItem.text}".'</code></li>
 * <li><code>'{!isDisabled}'</code></li>
 * <li><code>'{a &gt; b ? "Bigger" : "Smaller"}'</code></li>
 * <li><code>'{user.groups}'</code></li>
 * </ul>
 *
 *
 * <p>All except the last are <code><a href="#!/api/Ext.app.bind.TemplateBinding" rel="Ext.app.bind.TemplateBinding" class="docClass">template bindings</a></code>
 * which use the familiar <code><a href="#!/api/Ext.Template" rel="Ext.Template" class="docClass">Ext.Template</a></code> syntax with some slight differences. For more on
 * templates see <code><a href="#!/api/Ext.app.bind.Template" rel="Ext.app.bind.Template" class="docClass">Ext.app.bind.Template</a></code>.</p>
 *
 * <p>The last descriptor is called a "direct bind descriptor". This special form of
 * bind maps one-to-one to some piece of data in the <code>ViewModel</code> and is managed by the
 * <code><a href="#!/api/Ext.app.bind.Binding" rel="Ext.app.bind.Binding" class="docClass">Ext.app.bind.Binding</a></code> class.</p>
 *
 * <h4>Two-Way Descriptors</h4>
 *
 * <p>A direct bind descriptor may be able to write back a value to the <code>ViewModel</code> as well
 * as retrieve one. When this is the case, they are said to be "two-way". For example:</p>
 *
 * <pre><code> var binding = viewModel.bind('{s}', function(s) { console.log('s=' + s); });
 *
 *  binding.setValue('abc');
 * </code></pre>
 *
 * <p>Direct use of <code>ViewModel</code> in this way is not commonly needed because <code><a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a></code>
 * automates this process. For example, a <code>textfield</code> component understands when it is
 * given a "two-way" binding and automatically synchronizes its value bidirectionally using
 * the above technique. For example:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-widget" rel="Ext-method-widget" class="docClass">Ext.widget</a>({
 *      items: [{
 *          xtype: 'textfield',
 *          bind: '{s}'  // a two-way / direct bind descriptor
 *      }]
 *  });
 * </code></pre>
 *
 * <h3>Object and Array Descriptors / Multi-Bind</h3>
 *
 * <p>With two exceptions (see below) an Object is interpreted as a "shape" to produce by
 * treating each of its properties as individual bind descriptors. An object of the same
 * shape is passed as the value of the bind except that each property is populated with
 * the appropriate value. Of course, this definition is recursive, so these properties
 * may also be objects.</p>
 *
 * <p>For example:</p>
 *
 * <pre><code> viewModel.bind({
 *          x: '{x}',
 *          foo: {
 *              bar: 'Hello {foo.bar}'
 *          }
 *      },
 *      function (obj) {
 *          //  obj = {
 *          //      x: 42,
 *          //      foo: {
 *          //          bar: 'Hello foobar'
 *          //      }
 *          //  }
 *      });
 * </code></pre>
 *
 * <p>Arrays are handled in the same way. Each element of the array is considered a bind
 * descriptor (recursively) and the value produced for the binding is an array with each
 * element set to the bound property.</p>
 *
 * <h3>Bind Options</h3>
 *
 * <p>One exception to the "object is a multi-bind" rule is when that object contains a
 * <code>bindTo</code> property. When an object contains a <code>bindTo</code> property the object is understood
 * to contain bind options and the value of <code>bindTo</code> is considered the actual bind
 * descriptor.</p>
 *
 * <p>For example:</p>
 *
 * <pre><code> viewModel.bind({
 *          bindTo: '{x}',
 *          single: true
 *      },
 *      function (x) {
 *          console.log('x: ' + x); // only called once
 *      });
 * </code></pre>
 *
 * <p>The available bind options depend on the type of binding, but since all bindings
 * derive from <code><a href="#!/api/Ext.app.bind.BaseBinding" rel="Ext.app.bind.BaseBinding" class="docClass">Ext.app.bind.BaseBinding</a></code> its options are always applicable.
 * For a list of the other types of bindings, see above.</p>
 *
 * <h4>Deep Binding</h4>
 *
 * <p>When a direct bind is made and the bound property is an object, by default the binding
 * callback is only called when that reference changes. This is the most efficient way to
 * understand a bind of this type, but sometimes you may need to be notified if any of the
 * properties of that object change.</p>
 *
 * <p>To do this, we create a "deep bind":</p>
 *
 * <pre><code> viewModel.bind({
 *          bindTo: '{someObject}',
 *          deep: true
 *      },
 *      function (someObject) {
 *          // called when reference changes or *any* property changes
 *      });
 * </code></pre>
 *
 * <h4>Binding Timings</h4>
 *
 * <p>The <code>ViewModel</code> has a <a href="#!/api/Ext.app.ViewModel-cfg-scheduler" rel="Ext.app.ViewModel-cfg-scheduler" class="docClass">scheduler</a> attached that is used to coordinate the firing of bindings.
 * It serves 2 main purposes:
 * - To coordinate dependencies between bindings. This means bindings will be fired in an order such that
 * the any dependencies for a binding are fired before the binding itself.
 * - To batch binding firings. The scheduler runs on a short timer, so the following code will only trigger
 * a single binding (the last), the changes in between will never be triggered.</p>
 *
 * <pre><code>viewModel.bind('{val}', function(v) {
 *     console.log(v);
 * });
 * viewModel.set('val', 1);
 * viewModel.set('val', 2);
 * viewModel.set('val', 3);
 * viewModel.set('val', 4);
 * </code></pre>
 *
 * <p>The <code>ViewModel</code> can be forced to process by calling <code><a href="#!/api/Ext.app.ViewModel-method-notify" rel="Ext.app.ViewModel-method-notify" class="docClass">notify</a></code>, which will force the
 * scheduler to run immediately in the current state.</p>
 *
 * <pre><code>viewModel.bind('{val}', function(v) {
 *     console.log(v);
 * });
 * viewModel.set('val', 1);
 * viewModel.notify();
 * viewModel.set('val', 2);
 * viewModel.notify();
 * viewModel.set('val', 3);
 * viewModel.notify();
 * viewModel.set('val', 4);
 * viewModel.notify();
 * </code></pre>
 *
 * <h4>Models, Stores and Associations</h4>
 *
 * <p>A <a href="#!/api/Ext.data.Session" rel="Ext.data.Session" class="docClass">Session</a> manages model instances and their associations.
 * The <code>ViewModel</code> may be used with or without a <code>Session</code>. When a <code>Session</code> is attached, the
 * <code>ViewModel</code> will always consult the <code>Session</code> to ask about records and stores. The <code>Session</code>
 * ensures that only a single instance of each model Type/Id combination is created. This is
 * important when tracking changes in models so that we always have the same reference.</p>
 *
 * <p>A <code>ViewModel</code> provides functionality to easily consume the built in data package types
 * <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> and <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>, as well as their associations.</p>
 *
 * <h3>Model Links</h3>
 *
 * <p>A model can be described declaratively using a <a href="#!/api/Ext.app.ViewModel-cfg-links" rel="Ext.app.ViewModel-cfg-links" class="docClass">&lt;code&gt;link&lt;/code&gt;</a>. In the example code below,
 * We ask the <code>ViewModel</code> to construct a record of type <code>User</code> with <code>id: 17</code>. The model will be loaded
 * from the server and the bindings will trigger once the load has completed. Similarly, we could also
 * attach a model instance to the <code>ViewModel</code> data directly.</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.model.User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['name']
 * });
 *
 * var rec = new MyApp.model.User({
 *     id: 12,
 *     name: 'Foo'
 * });
 *
 * var viewModel = new <a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">Ext.app.ViewModel</a>({
 *     links: {
 *         theUser: {
 *             type: 'User',
 *             id: 17
 *         }
 *     },
 *     data: {
 *         otherUser: rec
 *     }
 * });
 * viewModel.bind('{theUser.name}', function(v) {
 *     console.log(v);
 * });
 * viewModel.bind('{otherUser.name}', function(v) {
 *     console.log(v);
 * });
 * </code></pre>
 *
 * <h3>Model Fields</h3>
 *
 * <p>Bindings have the functionality to inspect the parent values and resolve the underlying
 * value dynamically. This behavior allows model fields to be interrogated as part of a binding.</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.model.User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['name', 'age']
 * });
 *
 * var viewModel = new <a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">Ext.app.ViewModel</a>({
 *     links: {
 *         theUser: {
 *             type: 'User',
 *             id: 22
 *         }
 *     }
 * });
 *
 * // Server responds with:
 * {
 *     "id": 22,
 *     "name": "Foo",
 *     "age": 100
 * }
 *
 * viewModel.bind('Hello {name}, you are {age} years old', function(v) {
 *     console.log(v);
 * });
 * </code></pre>
 *
 * <h3>Associations</h3>
 *
 * <p>In the same way as fields, the bindings can also traverse associations in a bind statement.
 * The <code>ViewModel</code> will handle the asynchronous loading of data and only present the value once
 * the full path has been loaded. For more information on associations see <a href="#!/api/Ext.data.schema.OneToOne" rel="Ext.data.schema.OneToOne" class="docClass">OneToOne</a> and
 * <a href="#!/api/Ext.data.schema.ManyToOne" rel="Ext.data.schema.ManyToOne" class="docClass">ManyToOne</a> associations.</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['name']
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Order', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['date', {
 *         name: 'userId',
 *         reference: 'User'
 *     }]
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('OrderItem', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['price', 'qty', {
 *         name: 'orderId',
 *         reference: 'Order'
 *     }]
 * });
 *
 * var viewModel = new <a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">Ext.app.ViewModel</a>({
 *     links: {
 *         orderItem: {
 *             type: 'OrderItem',
 *             id: 13
 *         }
 *     }
 * });
 * // The viewmodel will handle both ways of loading the data:
 * // a) If the data is loaded inline in a nested fashion it will
 * //    not make requests for extra data
 * // b) Only loading a single model at a time. So the Order will be loaded once
 * //    the OrderItem returns. The User will be loaded once the Order loads.
 * viewModel.bind('{orderItem.order.user.name}', function(name) {
 *     console.log(name);
 * });
 * </code></pre>
 *
 * <h3>Stores</h3>
 *
 * <p>Stores can be created as part of the <code>ViewModel</code> definition. The definitions are processed
 * like bindings which allows for very powerful dynamic functionality.</p>
 *
 * <p>It is important to ensure that you name viewModel's data keys uniquely. If data is not named<br/>
 * uniquely, binds and formulas may receive information from an unintended data source.<br/>
 * This applies to keys in the viewModel's data block, stores, and links configs.</p>
 *
 * <pre><code>var viewModel = new <a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">Ext.app.ViewModel</a>({
 *     stores: {
 *         users: {
 *             model: 'User',
 *             autoLoad: true,
 *             filters: [{
 *                 property: 'createdDate',
 *                 value: '{createdFilter}',
 *                 operator: '&gt;'
 *             }]
 *         }
 *     }
 * });
 * // Later on in our code, we set the date so that the store is created.
 * viewModel.set('createdFilter', <a href="#!/api/Ext.Date-method-subtract" rel="Ext.Date-method-subtract" class="docClass">Ext.Date.subtract</a>(new Date(), <a href="#!/api/Ext.Date-property-DAY" rel="Ext.Date-property-DAY" class="docClass">Ext.Date.DAY</a>, 7));
 * </code></pre>
 *
 * <p>See <a href="#!/api/Ext.app.ViewModel-cfg-stores" rel="Ext.app.ViewModel-cfg-stores" class="docClass">stores</a> for more detail.</p>
 *
 * <h4>Formulas</h4>
 *
 * <p>Formulas allow for calculated <code>ViewModel</code> data values. The dependencies for these formulas
 * are automatically determined so that the formula will not be processed until the required
 * data is present.</p>
 *
 * <pre><code>var viewModel = new <a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">Ext.app.ViewModel</a>({
 *     formulas: {
 *         fullName: function(get) {
 *             return get('firstName') + ' ' + get('lastName');
 *         }
 *     },
 *     data: {firstName: 'John', lastName: 'Smith'}
 * });
 *
 * viewModel.bind('{fullName}', function(v) {
 *     console.log(v);
 * });
 * </code></pre>
 *
 * <p>See <a href="#!/api/Ext.app.ViewModel-cfg-formulas" rel="Ext.app.ViewModel-cfg-formulas" class="docClass">formulas</a> for more detail.</p>
 */
public class ViewModel extends ext.Base implements ext.mixin.IFactoryable, ext.mixin.IIdentifiable {
  /**
   * @param config @inheritDoc
   */
  public function ViewModel(config:ext.app.ViewModel = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This object holds the arbitrary data that populates the <code>ViewModel</code> and is
   * then available for binding.
   * @default true
   * @since 5.0.0
   * @see #getData()
   * @see #setData()
   */
  public native function get data():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set data(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An object that defines named values whose value is managed by function calls.
   * The names of the properties of this object are assigned as values in the
   * ViewModel.
   *
   * <p>For example:</p>
   *
   * <pre><code> formulas: {
   *      xy: function (get) { return get('x') * get('y'); }
   *  }
   * </code></pre>
   *
   * <p>For more details about defining a formula, see <code><a href="#!/api/Ext.app.bind.Formula" rel="Ext.app.bind.Formula" class="docClass">Ext.app.bind.Formula</a></code>.</p>
   * @since 5.0.0
   * @see #getFormulas()
   * @see #setFormulas()
   */
  public native function get formulas():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set formulas(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Links provide a way to assign a simple name to a more complex bind. The primary
   * use for this is to assign names to records in the data model.
   *
   * <pre><code> links: {
   *      theUser: {
   *          type: 'User',
   *          id: 12
   *      }
   *  }
   * </code></pre>
   *
   * <p>It is also possible to force a new phantom record to be created by not specifying an
   * id but passing <code>create: true</code> as part of the descriptor. This is often useful when
   * creating a new record for a child session.</p>
   *
   * <pre><code>links: {
   *     newUser: {
   *         type: 'User',
   *         create: true
   *     }
   * }
   * </code></pre>
   *
   * <p><code>create</code> can also be an object containing initial data for the record.</p>
   *
   * <pre><code>links: {
   *     newUser: {
   *         type: 'User',
   *         create: {
   *             firstName: 'John',
   *             lastName: 'Smith'
   *         }
   *     }
   * }
   * </code></pre>
   *
   * <p>While that is the typical use, the value of each property in <code>links</code> may also be
   * a bind descriptor (see <code><a href="#!/api/Ext.app.ViewModel-method-bind" rel="Ext.app.ViewModel-method-bind" class="docClass">bind</a></code> for the various forms of bind
   * descriptors).</p>
   * @since 5.0.0
   * @see #getLinks()
   * @see #setLinks()
   */
  public native function get links():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set links(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The parent <code>ViewModel</code> of this <code>ViewModel</code>. Once set, this cannot be changed.
   * @since 5.0.0
   * @see #getParent()
   */
  public native function get parent():ext.app.ViewModel;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The schema to use for getting information about entities.
   * @default 'default'
   * @see #getSchema()
   * @see #setSchema()
   */
  public native function get schema():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set schema(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The session used to manage the data model (records and stores).
   * @since 5.0.0
   * @see #getSession()
   * @see #setSession()
   */
  public native function get session():ext.data.Session;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set session(value:ext.data.Session):void;

  [ExtConfig]
  /**
   * A declaration of <code><a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a></code> configurations that are first processed as
   * binds to produce an effective store configuration.
   *
   * <p>A simple store definition. We can reference this in our bind statements using the
   * <code>{users}</code> as we would with other data values.</p>
   *
   * <pre><code>new <a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">Ext.app.ViewModel</a>({
   *     stores: {
   *         users: {
   *             model: 'User',
   *             autoLoad: true
   *         }
   *     }
   * });
   * </code></pre>
   *
   * <p>This store definition contains a dynamic binding. The store will not be created until
   * the initial value for groupId is set. Once that occurs, the store is created with the appropriate
   * filter configuration. Subsequently, once we change the group value, the old filter will be
   * overwritten with the new value.</p>
   *
   * <pre><code>var viewModel = new <a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">Ext.app.ViewModel</a>({
   *     stores: {
   *         users: {
   *             model: 'User',
   *             filters: [{
   *                 property: 'groupId',
   *                 value: '{groupId}'
   *             }]
   *         }
   *     }
   * });
   * viewModel.set('groupId', 1); // This will trigger the store creation with the filter.
   * viewModel.set('groupId', 2); // The filter value will be changed.
   * </code></pre>
   *
   * <p>This store uses <a href="#!/api/Ext.data.ChainedStore" rel="Ext.data.ChainedStore" class="docClass">store chaining</a> to create a store backed by the
   * data in another store. By specifying a string as the store, it will bind our creation and backing
   * to the other store. This functionality is especially useful when wanting to display a different "view"
   * of a store, for example a different sort order or different filters.</p>
   *
   * <pre><code>var viewModel = new <a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">Ext.app.ViewModel</a>({
   *     stores: {
   *         allUsers: {
   *             model: 'User',
   *             autoLoad: true
   *         },
   *         children: {
   *             source: '{allUsers}',
   *             filters: [{
   *                 property: 'age',
   *                 value: 18,
   *                 operator: '&lt;'
   *             }]
   *         }
   *     }
   * });
   * </code></pre>
   * @since 5.0.0
   * @see #setStores()
   */
  public native function get stores():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set stores(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The Container that owns this <code>ViewModel</code> instance.
   * @since 5.0.0
   * @see #getView()
   * @see #setView()
   */
  public native function get view():ext.container.Container;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set view(value:ext.container.Container):void;

  /**
   * @inheritDoc
   */
  public native function get factoryConfig():Object;

  /**
   * @inheritDoc
   */
  public native function set factoryConfig(value:Object):void;

  /**
   * This method requests that data in this <code>ViewModel</code> be delivered to the specified
   * <code>callback</code>. The data desired is given in a "bind descriptor" which is the first
   * argument.
   *
   * <p>A simple call might look like this:</p>
   *
   * <pre><code>var binding = vm.bind('{foo}', this.onFoo, this);
   *
   * binding.destroy();  // when done with the binding
   * </code></pre>
   *
   * <p>Options for the binding can be provided in the last argument:</p>
   *
   * <pre><code>var binding = vm.bind('{foo}', this.onFoo, this, {
   *     deep: true
   * });
   * </code></pre>
   *
   * <p>Alternatively, bind options can be combined with the bind descriptor using only
   * the first argument:</p>
   *
   * <pre><code>var binding = vm.bind({
   *     bindTo: '{foo}',  // the presence of bindTo identifies this form
   *     deep: true
   * }, this.onFoo, this);
   * </code></pre>
   *
   * <p>See the class documentation for more details on Bind Descriptors and options.</p>
   * @param descriptor The bind descriptor. See class description
   * for details.
   * @param callback The function to call with the value of the bound property.
   * @param scope The scope (<code>this</code> pointer) for the <code>callback</code>.
   * @param options Additional options to configure the <a href="#!/api/Ext.app.bind.Binding" rel="Ext.app.bind.Binding" class="docClass">binding</a>.
   * If this parameter is provided, the <code>bindTo</code> form of combining options and bind descriptor is not
   * recognized.
   * @return The binding.
   */
  public native function bind(descriptor:*, callback:Function, scope:Object = null, options:Object = null):*;

  /**
   * Get a value from the data for this viewmodel.
   * @param path The path of the data to retrieve.
   * <p>   var value = vm.get('theUser.address.city');</p>
   * @return The data stored at the passed path.
   */
  public native function get(path:String):*;

  /**
   * Returns the value of <a href="#!/api/Ext.app.ViewModel-cfg-data" rel="Ext.app.ViewModel-cfg-data" class="docClass">data</a>.
   * @since 5.0.0
   */
  public native function getData():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.app.ViewModel-cfg-formulas" rel="Ext.app.ViewModel-cfg-formulas" class="docClass">formulas</a>.
   * @since 5.0.0
   */
  public native function getFormulas():Object;

  /**
   * Retrieves the id of this component. Will autogenerate an id if one has not already been set.
   * @return id
   */
  public native function getId():String;

  /**
   * Returns the value of <a href="#!/api/Ext.app.ViewModel-cfg-links" rel="Ext.app.ViewModel-cfg-links" class="docClass">links</a>.
   * @since 5.0.0
   */
  public native function getLinks():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.app.ViewModel-cfg-parent" rel="Ext.app.ViewModel-cfg-parent" class="docClass">parent</a>.
   * @since 5.0.0
   */
  public native function getParent():ext.app.ViewModel;

  /**
   * Returns the value of <a href="#!/api/Ext.app.ViewModel-cfg-schema" rel="Ext.app.ViewModel-cfg-schema" class="docClass">schema</a>.
   */
  public native function getSchema():*;

  /**
   * Gets the session attached to this (or a parent) ViewModel. See the <a href="#!/api/Ext.app.ViewModel-cfg-session" rel="Ext.app.ViewModel-cfg-session" class="docClass">session</a> configuration.
   * @return The session. <code>null</code> if no session exists.
   */
  public native function getSession():ext.data.Session;

  /**
   * Gets a store configured via the <a href="#!/api/Ext.app.ViewModel-cfg-stores" rel="Ext.app.ViewModel-cfg-stores" class="docClass">stores</a> configuration.
   * @param key The name of the store.
   * @return The store. <code>null</code> if no store exists.
   */
  public native function getStore(key:String):ext.data.Store;

  /**
   * Returns the value of <a href="#!/api/Ext.app.ViewModel-cfg-view" rel="Ext.app.ViewModel-cfg-view" class="docClass">view</a>.
   * @since 5.0.0
   */
  public native function getView():ext.container.Container;

  /**
   * Create a link to a reference. See the <a href="#!/api/Ext.app.ViewModel-cfg-links" rel="Ext.app.ViewModel-cfg-links" class="docClass">links</a> configuration.
   * @param key The name for the link.
   * @param reference The reference descriptor.
   */
  public native function linkTo(key:String, reference:Object):void;

  /**
   * Forces all bindings in this ViewModel hierarchy to evaluate immediately. Use this to do a synchronous flush
   * of all bindings.
   */
  public native function notify():void;

  /**
   * Set  a value in the data for this viewmodel.
   * @param path The path of the value to set, or an object literal to set
   * at the root of the viewmodel.
   * @param value The data to set at the value. If the value is an object literal,
   * any required paths will be created.
   * <pre><code>// Set a single property at the root level
   * viewModel.set('expiry', <a href="#!/api/Ext.Date-method-add" rel="Ext.Date-method-add" class="docClass">Ext.Date.add</a>(new Date(), <a href="#!/api/Ext.Date-property-DAY" rel="Ext.Date-property-DAY" class="docClass">Ext.Date.DAY</a>, 7));
   * console.log(viewModel.get('expiry'));
   * // Sets a single property in user.address, does not overwrite any hierarchy.
   * viewModel.set('user.address.city', 'London');
   * console.log(viewModel.get('user.address.city'));
   * // Sets 2 properties of "user". Overwrites any existing hierarchy.
   * viewModel.set('user', {firstName: 'Foo', lastName: 'Bar'});
   * console.log(viewModel.get('user.firstName'));
   * // Sets a single property at the root level. Overwrites any existing hierarchy.
   * viewModel.set({rootKey: 1});
   * console.log(viewModel.get('rootKey'));
   * </code></pre>
   */
  public native function set(path:*, value:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.ViewModel-cfg-data" rel="Ext.app.ViewModel-cfg-data" class="docClass">data</a>.
   * @since 5.0.0
   * @param data The new value.
   */
  public native function setData(data:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.ViewModel-cfg-formulas" rel="Ext.app.ViewModel-cfg-formulas" class="docClass">formulas</a>.
   * @since 5.0.0
   * @param formulas The new value.
   */
  public native function setFormulas(formulas:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.ViewModel-cfg-links" rel="Ext.app.ViewModel-cfg-links" class="docClass">links</a>.
   * @since 5.0.0
   * @param links The new value.
   */
  public native function setLinks(links:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.ViewModel-cfg-parent" rel="Ext.app.ViewModel-cfg-parent" class="docClass">parent</a>.
   * @since 5.0.0
   * @param parent The new value.
   */
  public native function setParent(parent:ext.app.ViewModel):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.ViewModel-cfg-schema" rel="Ext.app.ViewModel-cfg-schema" class="docClass">schema</a>.
   * @param schema The new value.
   */
  public native function setSchema(schema:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.ViewModel-cfg-session" rel="Ext.app.ViewModel-cfg-session" class="docClass">session</a>.
   * @since 5.0.0
   * @param session The new value.
   */
  public native function setSession(session:ext.data.Session):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.ViewModel-cfg-stores" rel="Ext.app.ViewModel-cfg-stores" class="docClass">stores</a>.
   * @since 5.0.0
   * @param stores The new value.
   */
  public native function setStores(stores:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.ViewModel-cfg-view" rel="Ext.app.ViewModel-cfg-view" class="docClass">view</a>.
   * @since 5.0.0
   * @param view The new value.
   */
  public native function setView(view:ext.container.Container):void;
}
}