package ext.container {
import ext.mixin.IResponsive;

[Uses("ext.container.Container")]
[Uses("ext.mixin.Responsive")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.container.Viewport", require)]
/**
 * A specialized container representing the viewable application area (the browser viewport).
 *
 * <p>The Viewport renders itself to the document body, and automatically sizes itself to the size of
 * the browser viewport and manages window resizing. There may only be one Viewport created
 * in a page.</p>
 *
 * <p>Like any <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Container</a>, a Viewport will only perform sizing and positioning
 * on its child Components if you configure it with a <a href="#!/api/Ext.container.Viewport-cfg-layout" rel="Ext.container.Viewport-cfg-layout" class="docClass">layout</a>.</p>
 *
 * <p>A Common layout used with Viewports is <a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">border layout</a>, but if the
 * required layout is simpler, a different layout should be chosen.</p>
 *
 * <p>For example, to simply make a single child item occupy all available space, use
 * <a href="#!/api/Ext.layout.container.Fit" rel="Ext.layout.container.Fit" class="docClass">fit layout</a>.</p>
 *
 * <p>To display one "active" item at full size from a choice of several child items, use
 * <a href="#!/api/Ext.layout.container.Card" rel="Ext.layout.container.Card" class="docClass">card layout</a>.</p>
 *
 * <p>Inner layouts are available because all <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Panel</a>s
 * added to the Viewport, either through its <a href="#!/api/Ext.container.Viewport-cfg-items" rel="Ext.container.Viewport-cfg-items" class="docClass">items</a>, or the <a href="#!/api/Ext.container.Viewport-method-add" rel="Ext.container.Viewport-method-add" class="docClass">add</a>
 * method of any of its child Panels may themselves have a layout.</p>
 *
 * <p>The Viewport does not provide scrolling, so child Panels within the Viewport should provide
 * for scrolling if needed using the <a href="#!/api/Ext.container.Viewport-cfg-scrollable" rel="Ext.container.Viewport-cfg-scrollable" class="docClass">scrollable</a> config.</p>
 *
 * <p>An example showing a classic application border layout:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.container.Viewport" rel="Ext.container.Viewport" class="docClass">Ext.container.Viewport</a>', {
 *     layout: 'border',
 *     items: [{
 *         region: 'north',
 *         html: '&lt;h1 class="x-panel-header"&gt;Page Title&lt;/h1&gt;',
 *         border: false,
 *         margin: '0 0 5 0'
 *     }, {
 *         region: 'west',
 *         collapsible: true,
 *         title: 'Navigation',
 *         width: 150
 *         // could use a TreePanel or AccordionLayout for navigational items
 *     }, {
 *         region: 'south',
 *         title: 'South Panel',
 *         collapsible: true,
 *         html: 'Information goes here',
 *         split: true,
 *         height: 100,
 *         minHeight: 100
 *     }, {
 *         region: 'east',
 *         title: 'East Panel',
 *         collapsible: true,
 *         split: true,
 *         width: 150
 *     }, {
 *         region: 'center',
 *         xtype: 'tabpanel', // TabPanel itself has no title
 *         activeTab: 0,      // First tab active by default
 *         items: {
 *             title: 'Default Tab',
 *             html: 'The first tab\'s content. Others may be added dynamically'
 *         }
 *     }]
 * });
 * </code></pre>
 *
 * <p>Alternatively you can turn any normal Container (or Component) into a Viewport using
 * the <code><a href="#!/api/Ext.plugin.Viewport" rel="Ext.plugin.Viewport" class="docClass">viewport plugin</a></code>.</p>
 */
public class Viewport extends ext.container.Container implements ext.mixin.IResponsive {
  /**
   * @param config @inheritDoc
   */
  public function Viewport(config:ext.container.Viewport = null) {
    super();
  }

  /**
   * <code>true</code> in this class to identify an object as an instantiated Viewport, or subclass thereof.
   */
  public native function get isViewport():Boolean;

  [ExtConfig]
  /**
   * The maximum zoom scale. Only applicable for touch devices. Set this to 1 to
   * disable zooming.
   * @default 10
   */
  public native function get maxUserScale():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maxUserScale(value:Number):void;

  [ExtConfig]
  /**
   * This object consists of keys that represent the conditions on which configs
   * will be applied. For example:
   *
   * <pre><code> responsiveConfig: {
   *      landscape: {
   *          region: 'west'
   *      },
   *      portrait: {
   *          region: 'north'
   *      }
   *  }
   * </code></pre>
   *
   * <p>In this case the keys ("landscape" and "portrait") are the criteria (or "rules")
   * and the object to their right contains the configs that will apply when that
   * rule is true.</p>
   *
   * <p>These rules can be any valid JavaScript expression but the following values
   * are considered in scope:</p>
   *
   * <ul>
   * <li><code>landscape</code> - True if the device orientation is landscape (always <code>true</code> on
   * desktop devices).</li>
   * <li><code>portrait</code> - True if the device orientation is portrait (always <code>false</code> on
   * desktop devices).</li>
   * <li><code>tall</code> - True if <code>width</code> &lt; <code>height</code> regardless of device type.</li>
   * <li><code>wide</code> - True if <code>width</code> > <code>height</code> regardless of device type.</li>
   * <li><code>width</code> - The width of the viewport in pixels.</li>
   * <li><code>height</code> - The height of the viewport in pixels.</li>
   * <li><code>platform</code> - An object containing various booleans describing the platform
   * (see <code><a href="#!/api/Ext-property-platformTags" rel="Ext-property-platformTags" class="docClass">Ext.platformTags</a></code>). The properties of this
   * object are also available implicitly (without "platform." prefix) but this
   * sub-object may be useful to resolve ambiguity (for example, if one of the
   * <code><a href="#!/api/Ext.mixin.Responsive-cfg-responsiveFormulas" rel="Ext.mixin.Responsive-cfg-responsiveFormulas" class="docClass">responsiveFormulas</a></code> overlaps and hides any of these properties).
   * Previous to Ext JS 5.1, the <code>platformTags</code> were only available using this
   * prefix.</li>
   * </ul>
   *
   *
   * <p>A more complex example:</p>
   *
   * <pre><code> responsiveConfig: {
   *      'desktop || width &gt; 800': {
   *          region: 'west'
   *      },
   *
   *      '!(desktop || width &gt; 800)': {
   *          region: 'north'
   *      }
   *  }
   * </code></pre>
   *
   * <p><strong>NOTE</strong>: If multiple rules set a single config (like above), it is important
   * that the rules be mutually exclusive. That is, only one rule should set each
   * config. If multiple rules are actively setting a single config, the order of
   * these (and therefore the config's value) is unspecified.</p>
   *
   * <p>For a config to participate as a <code>responsiveConfig</code> it must have a "setter"
   * method. In the above example, a "setRegion" method must exist.</p>
   * @since 5.0.0
   */
  public native function get responsiveConfig():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set responsiveConfig(value:Object):void;

  [ExtConfig]
  /**
   * It is common when using <code>responsiveConfig</code> to have recurring expressions that
   * make for complex configurations. Using <code>responsiveFormulas</code> allows you to cut
   * down on this repetition by adding new properties to the "scope" for the rules
   * in a <code>responsiveConfig</code>.
   *
   * <p>For example:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.view.main.Main', {
   *      extend: '<a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a>',
   *
   *      mixins: [
   *          '<a href="#!/api/Ext.mixin.Responsive" rel="Ext.mixin.Responsive" class="docClass">Ext.mixin.Responsive</a>'
   *      ],
   *
   *      responsiveFormulas: {
   *          small: 'width &lt; 600',
   *
   *          medium: 'width &gt;= 600 &amp;&amp; width &lt; 800',
   *
   *          large: 'width &gt;= 800',
   *
   *          tuesday: function (context) {
   *              return (new Date()).getDay() === 2;
   *          }
   *      }
   *  });
   * </code></pre>
   *
   * <p>With the above declaration, any <code>responsiveConfig</code> can now use these values
   * like so:</p>
   *
   * <pre><code> responsiveConfig: {
   *      small: {
   *          hidden: true
   *      },
   *      'medium &amp;&amp; !tuesday': {
   *          hidden: false,
   *          region: 'north'
   *      },
   *      large: {
   *          hidden: false,
   *          region: 'west'
   *      }
   *  }
   * </code></pre>
   * @since 5.0.1
   */
  public native function get responsiveFormulas():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set responsiveFormulas(value:Object):void;

  /**
   * This method removes this instance from the Responsive collection.
   */
  override public native function destroy(...params):void;
}
}