package ext.data {

[Uses("ext.data.ProxyStore")]
[Uses("ext.data.AbstractStore")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.BufferedStore", require)]
/**
 * A BufferedStore maintains a sparsely populated map of pages corresponding to an extremely large server-side dataset.
 *
 * <p>Use a BufferedStore when the dataset size is so large that the database and network latency, and client memory requirements
 * preclude caching the entire dataset in a regular <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a>.</p>
 *
 * <p>When using a BufferedStore <em>not all of the dataset is present in the client</em>. Only pages which have been
 * requested by the UI (usually a <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">GridPanel</a>) and surrounding pages will be present. Retention
 * of viewed pages in the BufferedStore after they have been scrolled out of view is configurable. See <a href="#!/api/Ext.data.BufferedStore-cfg-leadingBufferZone" rel="Ext.data.BufferedStore-cfg-leadingBufferZone" class="docClass">leadingBufferZone</a>,
 * <a href="#!/api/Ext.data.BufferedStore-cfg-trailingBufferZone" rel="Ext.data.BufferedStore-cfg-trailingBufferZone" class="docClass">trailingBufferZone</a> and <a href="#!/api/Ext.data.BufferedStore-cfg-purgePageCount" rel="Ext.data.BufferedStore-cfg-purgePageCount" class="docClass">purgePageCount</a>.</p>
 *
 * <p>To use a BufferedStore, initiate the loading process by loading the first page. The number of rows rendered are
 * determined automatically, and the range of pages needed to keep the cache primed for scrolling is
 * requested and cached.
 * Example:</p>
 *
 * <pre><code>myBufferedStore.loadPage(1); // Load page 1
 * </code></pre>
 *
 * <p>A <a href="#!/api/Ext.grid.plugin.BufferedRenderer" rel="Ext.grid.plugin.BufferedRenderer" class="docClass">BufferedRenderer</a> is instantiated which will monitor the scrolling in the grid, and
 * refresh the view's rows from the page cache as needed. It will also pull new data into the page
 * cache when scrolling of the view draws upon data near either end of the prefetched data.</p>
 *
 * <p>The margins which trigger view refreshing from the prefetched data are <a href="#!/api/Ext.grid.plugin.BufferedRenderer-cfg-numFromEdge" rel="Ext.grid.plugin.BufferedRenderer-cfg-numFromEdge" class="docClass">Ext.grid.plugin.BufferedRenderer.numFromEdge</a>,
 * <a href="#!/api/Ext.grid.plugin.BufferedRenderer-cfg-leadingBufferZone" rel="Ext.grid.plugin.BufferedRenderer-cfg-leadingBufferZone" class="docClass">Ext.grid.plugin.BufferedRenderer.leadingBufferZone</a> and <a href="#!/api/Ext.grid.plugin.BufferedRenderer-cfg-trailingBufferZone" rel="Ext.grid.plugin.BufferedRenderer-cfg-trailingBufferZone" class="docClass">Ext.grid.plugin.BufferedRenderer.trailingBufferZone</a>.</p>
 *
 * <p>The margins which trigger loading more data into the page cache are, <a href="#!/api/Ext.data.BufferedStore-cfg-leadingBufferZone" rel="Ext.data.BufferedStore-cfg-leadingBufferZone" class="docClass">leadingBufferZone</a> and
 * <a href="#!/api/Ext.data.BufferedStore-cfg-trailingBufferZone" rel="Ext.data.BufferedStore-cfg-trailingBufferZone" class="docClass">trailingBufferZone</a>.</p>
 *
 * <p>By default, only 5 pages of data (in addition to the pages which over the visible region) are cached in the page cache,
 * with old pages being evicted from the cache as the view moves down through the dataset. This is controlled by the
 * <a href="#!/api/Ext.data.BufferedStore-cfg-purgePageCount" rel="Ext.data.BufferedStore-cfg-purgePageCount" class="docClass">purgePageCount</a> setting.</p>
 *
 * <p>Setting this value to zero means that no pages are <em>ever</em> scrolled out of the page cache, and
 * that eventually the whole dataset may become present in the page cache. This is sometimes desirable
 * as long as datasets do not reach astronomical proportions.</p>
 *
 * <p>Selection state may be maintained across page boundaries by configuring the SelectionModel not to discard
 * records from its collection when those Records cycle out of the Store's primary collection. This is done
 * by configuring the SelectionModel like this:</p>
 *
 * <pre><code>selModel: {
 *     pruneRemoved: false
 * }
 * </code></pre>
 */
public class BufferedStore extends ext.data.ProxyStore {
  /**
   * @param config @inheritDoc
   */
  public function BufferedStore(config:ext.data.BufferedStore = null) {
    super();
  }

  /**
   * <code>true</code> in this class to identify an object as an instantiated BufferedStore, or subclass thereof.
   * @default true
   */
  public native function get isBufferedStore():Boolean;

  /**
   * @private
   */
  public native function set isBufferedStore(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The number of extra rows to keep cached on the leading side of scrolling buffer
   * as scrolling proceeds. A larger number means fewer replenishments from the server.
   * @default 200
   * @see #getLeadingBufferZone()
   * @see #setLeadingBufferZone()
   */
  public native function get leadingBufferZone():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set leadingBufferZone(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The number of pages <em>in addition to twice the required buffered range</em> to keep in the prefetch cache before purging least recently used records.
   *
   * <p>For example, if the height of the view area and the configured <a href="#!/api/Ext.data.BufferedStore-cfg-trailingBufferZone" rel="Ext.data.BufferedStore-cfg-trailingBufferZone" class="docClass">trailingBufferZone</a> and <a href="#!/api/Ext.data.BufferedStore-cfg-leadingBufferZone" rel="Ext.data.BufferedStore-cfg-leadingBufferZone" class="docClass">leadingBufferZone</a> require that there
   * are three pages in the cache, then a <code>purgePageCount</code> of 5 ensures that up to 11 pages can be in the page cache any any one time. This is enough
   * to allow the user to scroll rapidly between different areas of the dataset without evicting pages which are still needed.</p>
   *
   * <p>A value of 0 indicates to never purge the prefetched data.</p>
   * @default 5
   * @see #getPurgePageCount()
   * @see #setPurgePageCount()
   */
  public native function get purgePageCount():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set purgePageCount(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The number of extra records to keep cached on the trailing side of scrolling buffer
   * as scrolling proceeds. A larger number means fewer replenishments from the server.
   * @default 25
   * @see #getTrailingBufferZone()
   * @see #setTrailingBufferZone()
   */
  public native function get trailingBufferZone():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set trailingBufferZone(value:Number):void;

  /**
   * We are using applyData so that we can return nothing and prevent the <code>this.data</code>
   * property to be overridden.
   */
  public native function applyData(data:*):void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.BufferedStore-cfg-leadingBufferZone" rel="Ext.data.BufferedStore-cfg-leadingBufferZone" class="docClass">leadingBufferZone</a>.
   */
  public native function getLeadingBufferZone():Number;

  /**
   * Determines the page from a record index
   * @param index The record index
   * @return The page the record belongs to
   */
  public native function getPageFromRecordIndex(index:Number):Number;

  /**
   * Returns the value of <a href="#!/api/Ext.data.BufferedStore-cfg-purgePageCount" rel="Ext.data.BufferedStore-cfg-purgePageCount" class="docClass">purgePageCount</a>.
   */
  public native function getPurgePageCount():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.data.BufferedStore-cfg-trailingBufferZone" rel="Ext.data.BufferedStore-cfg-trailingBufferZone" class="docClass">trailingBufferZone</a>.
   */
  public native function getTrailingBufferZone():Number;

  [Deprecated(replacement = "getRange")]
  /**
   * Guarantee a specific range, this will load the store with a range (that
   * must be the <code>pageSize</code> or smaller) and take care of any loading that may
   * be necessary.
   */
  public native function guaranteeRange(start:Object, end:Object, callback:Object, scope:Object, options:Object):void;

  /**
   * Get the index of the record within the store.
   *
   * <p>When store is filtered, records outside of filter will not be found.</p>
   * @param record The <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> object to find.
   * @return The index of the passed Record. Returns -1 if not found.
   */
  public native function indexOf(record:ext.data.Model):Number;

  /**
   * Get the index within the store of the Record with the passed id.
   *
   * <p>Like <a href="#!/api/Ext.data.BufferedStore-method-indexOf" rel="Ext.data.BufferedStore-method-indexOf" class="docClass">indexOf</a>, this method is effected by filtering.</p>
   * @param id The id of the Record to find.
   * @return The index of the Record. Returns -1 if not found.
   */
  public native function indexOfId(id:String):Number;

  /**
   * Prefetches data into the store using its configured <a href="#!/api/Ext.data.BufferedStore-cfg-proxy" rel="Ext.data.BufferedStore-cfg-proxy" class="docClass">proxy</a>.
   * @param options config object, passed into the <a href="#!/api/Ext.data.operation.Operation" rel="Ext.data.operation.Operation" class="docClass">Ext.data.operation.Operation</a> object before loading.
   * See <a href="#!/api/Ext.data.BufferedStore-method-load" rel="Ext.data.BufferedStore-method-load" class="docClass">load</a>
   */
  public native function prefetch(options:Object = null):void;

  /**
   * Prefetches a page of data.
   * @param page The page to prefetch
   * @param options config object, passed into the <a href="#!/api/Ext.data.operation.Operation" rel="Ext.data.operation.Operation" class="docClass">Ext.data.operation.Operation</a> object before loading.
   * See <a href="#!/api/Ext.data.BufferedStore-method-load" rel="Ext.data.BufferedStore-method-load" class="docClass">load</a>
   */
  public native function prefetchPage(page:Number, options:Object = null):void;

  /**
   * Ensures that the specified range of rows is present in the cache.
   *
   * <p>Converts the row range to a page range and then only load pages which are not already
   * present in the page cache.</p>
   */
  public native function prefetchRange(start:Object, end:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.BufferedStore-cfg-leadingBufferZone" rel="Ext.data.BufferedStore-cfg-leadingBufferZone" class="docClass">leadingBufferZone</a>.
   * @param leadingBufferZone The new value.
   */
  public native function setLeadingBufferZone(leadingBufferZone:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.BufferedStore-cfg-purgePageCount" rel="Ext.data.BufferedStore-cfg-purgePageCount" class="docClass">purgePageCount</a>.
   * @param purgePageCount The new value.
   */
  public native function setPurgePageCount(purgePageCount:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.BufferedStore-cfg-trailingBufferZone" rel="Ext.data.BufferedStore-cfg-trailingBufferZone" class="docClass">trailingBufferZone</a>.
   * @param trailingBufferZone The new value.
   */
  public native function setTrailingBufferZone(trailingBufferZone:Number):void;
}
}