package ext.data {
import ext.IMixin;
import ext.util.Collection;

[Uses("ext.Mixin")]
[Uses("ext.data.LocalStore")]
[Uses("ext.Base")]
[Mixin("ext.data.LocalStore")]
/**
 * A mixin that provides common store methods for <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> &amp; <a href="#!/api/Ext.data.ChainedStore" rel="Ext.data.ChainedStore" class="docClass">Ext.data.ChainedStore</a>.
 * @see ext.data.LocalStore
 */
public interface ILocalStore extends ext.IMixin {
  /**
   * Adds Model instance to the Store. This method accepts either:
   *
   * <ul>
   * <li>An array of Model instances or Model configuration objects.</li>
   * <li>Any number of Model instance or Model configuration object arguments.</li>
   * </ul>
   *
   *
   * <p>The new Model instances will be added at the end of the existing collection.</p>
   *
   * <p>Sample usage:</p>
   *
   * <pre><code>myStore.add({some: 'data'}, {some: 'other data'});
   * </code></pre>
   *
   * <p>Note that if this Store is sorted, the new Model instances will be inserted
   * at the correct point in the Store to maintain the sort order.</p>
   * @param model An array of Model instances
   * or Model configuration objects, or variable number of Model instance or config arguments.
   * @return The model instances that were added
   */
  function add(...model):Array;

  /**
   * Runs the aggregate function for all the records in the store.
   *
   * <p>When store is filtered, only items within the filter are aggregated.</p>
   * @param fn The function to execute. The function is called with a single parameter,
   * an array of records for that group.
   * @param scope The scope to execute the function in. Defaults to the store.
   * @param grouped True to perform the operation for each group
   * in the store. The value returned will be an object literal with the key being the group
   * name and the group average being the value. The grouped parameter is only honored if
   * the store has a groupField.
   * @param field The field to get the value from
   * @return An object literal with the group names and their appropriate values.
   */
  function aggregate(fn:Function, scope:Object = null, grouped:Boolean = false, field:String = null):Object;

  /**
   * Gets the average value in the store.
   *
   * <p>When store is filtered, only items within the filter are aggregated.</p>
   * @param field The field in each record
   * @param grouped True to perform the operation for each group
   * in the store. The value returned will be an object literal with the key being the group
   * name and the group average being the value. The grouped parameter is only honored if
   * the store has a groupField.
   * @return The average value, if no items exist, 0.
   */
  function average(field:String, grouped:Boolean = false):Object;

  /**
   * Collects unique values for a particular dataIndex from this store.
   *
   * <p>Note that the <code>filtered</code> option can also be passed as a separate parameter for
   * compatibility with previous versions.</p>
   *
   * <pre><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
   *     fields: ['name'],
   *     data: [{
   *         name: 'Larry'
   *     }, {
   *         name: 'Darryl'
   *     }, {
   *         name: 'Darryl'
   *     }]
   * });
   *
   * store.collect('name');
   * // returns ["Larry", "Darryl"]
   * </code></pre>
   * @param property The property to collect
   * @param includeOptions An object which contains options which modify how the store is traversed.
   * @param {Boolean} [includeOptions.allowNull] Pass true to allow null, undefined or empty string values.
   * @param {Boolean} [includeOptions.filtered] Pass <code>true</code> to collect from all records, even ones which are filtered.
   * @return An array of the unique values
   */
  function collect(property:String, includeOptions:Object = null):Array;

  /**
   * Gets the count of items in the store.
   *
   * <p>When store is filtered, only items within the filter are counted.</p>
   * @param grouped True to perform the operation for each group
   * in the store. The value returned will be an object literal with the key being the group
   * name and the count for each group being the value. The grouped parameter is only honored if
   * the store has a groupField.
   * @return the count
   */
  function count(grouped:Boolean = false):Number;

  /**
   * Calls the specified function for each <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">record</a> in the store.
   *
   * <p>When store is filtered, only loops over the filtered records.</p>
   * @param fn The function to call. The <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Record</a> is passed as the first parameter.
   * Returning <code>false</code> aborts and exits the iteration.
   * @param scope The scope (<code>this</code> reference) in which the function is executed.
   * Defaults to the current <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">record</a> in the iteration.
   * @param includeOptions An object which contains options which modify how the store is traversed.
   * @param {Boolean} [includeOptions.filtered] Pass <code>true</code> to include filtered out nodes in the iteration.
   * <p>Note that the <code>filtered</code> option can also be passed as a separate parameter for
   * compatibility with previous versions.</p>
   */
  function each(fn:Function, scope:Object = null, includeOptions:Object = null):void;

  /**
   * Convenience function for getting the first model instance in the store.
   *
   * <p>When store is filtered, will return first item within the filter.</p>
   * @param grouped True to perform the operation for each group
   * in the store. The value returned will be an object literal with the key being the group
   * name and the first record being the value. The grouped parameter is only honored if
   * the store has a groupField.
   * @return The first model instance in the store, or undefined
   */
  function first(grouped:Boolean = false):*;

  /**
   * Get the index of the record within the store.
   *
   * <p>When store is filtered, records outside of filter will not be found.</p>
   * @param record The <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> object to find.
   * @return The index of the passed Record. Returns -1 if not found.
   */
  function indexOf(record:ext.data.Model):Number;

  /**
   * Get the index within the store of the Record with the passed id.
   *
   * <p>Like <a href="#!/api/Ext.data.LocalStore-method-indexOf" rel="Ext.data.LocalStore-method-indexOf" class="docClass">indexOf</a>, this method is affected by filtering.</p>
   * @param id The id of the Record to find.
   * @return The index of the Record. Returns -1 if not found.
   */
  function indexOfId(id:String):Number;

  /**
   * Inserts Model instances into the Store at the given index and fires the add event.
   * See also <a href="#!/api/Ext.data.LocalStore-method-add" rel="Ext.data.LocalStore-method-add" class="docClass">add</a>.
   * @param index The start index at which to insert the passed Records.
   * @param records An <code><a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a></code> instance, the
   * data needed to populate an instance or an array of either of these.
   * @return records The added records
   */
  function insert(index:Number, records:*):Array;

  /**
   * Convenience function for getting the last model instance in the store.
   *
   * <p>When store is filtered, will return last item within the filter.</p>
   * @param grouped True to perform the operation for each group
   * in the store. The value returned will be an object literal with the key being the group
   * name and the last record being the value. The grouped parameter is only honored if
   * the store has a groupField.
   * @return The last model instance in the store, or undefined
   */
  function last(grouped:Boolean = false):*;

  /**
   * Gets the maximum value in the store.
   *
   * <p>When store is filtered, only items within the filter are aggregated.</p>
   * @param field The field in each record
   * @param grouped True to perform the operation for each group
   * in the store. The value returned will be an object literal with the key being the group
   * name and the maximum in the group being the value. The grouped parameter is only honored if
   * the store has a groupField.
   * @return The maximum value, if no items exist, undefined.
   */
  function max(field:String, grouped:Boolean = false):Object;

  /**
   * Gets the minimum value in the store.
   *
   * <p>When store is filtered, only items within the filter are aggregated.</p>
   * @param field The field in each record
   * @param grouped True to perform the operation for each group
   * in the store. The value returned will be an object literal with the key being the group
   * name and the minimum in the group being the value. The grouped parameter is only honored if
   * the store has a groupField.
   * @return The minimum value, if no items exist, undefined.
   */
  function min(field:String, grouped:Boolean = false):Object;

  /**
   * Query all the cached records in this Store by name/value pair.
   * The parameters will be used to generated a filter function that is given
   * to the queryBy method.
   *
   * <p>This method complements queryBy by generating the query function automatically.</p>
   *
   * <p>This method is not affected by filtering, it will always search <em>all</em> records in the store
   * regardless of filtering.</p>
   * @param property The property to create the filter function for
   * @param value The string/regex to compare the property value to
   * @param anyMatch True to match any part of the string, not just the
   * beginning.
   * @default false
   * @param caseSensitive <code>true</code> to create a case-sensitive regex.
   * @default false
   * @param exactMatch True to force exact match (^ and $ characters
   * added to the regex). Ignored if <code>anyMatch</code> is <code>true</code>.
   * @default false
   * @return The matched records
   */
  function query(property:String, value:*, anyMatch:Boolean = false, caseSensitive:Boolean = false, exactMatch:Boolean = false):ext.util.Collection;

  /**
   * Query all the cached records in this Store using a filtering function. The specified function
   * will be called with each record in this Store. If the function returns <code>true</code> the record is
   * included in the results.
   *
   * <p>This method is not affected by filtering, it will always search <em>all</em> records in the store
   * regardless of filtering.</p>
   * @param fn The function to be called. It will be passed the following parameters:
   * @param {ext.data.Model} fn.record The record to test for filtering. Access field values
   *  using <a href="#!/api/Ext.data.Model-method-get" rel="Ext.data.Model-method-get" class="docClass">Ext.data.Model.get</a>.
   * @param {Object} fn.id The ID of the Record passed.
   * @param scope The scope (this reference) in which the function is executed
   * Defaults to this Store.
   * @return The matched records
   */
  function queryBy(fn:Function, scope:Object = null):ext.util.Collection;

  /**
   * Sums the value of <code>field</code> for each <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">record</a> in store
   * and returns the result.
   *
   * <p>When store is filtered, only sums items within the filter.</p>
   * @param field A field in each record
   * @param grouped True to perform the operation for each group
   * in the store. The value returned will be an object literal with the key being the group
   * name and the sum for that group being the value. The grouped parameter is only honored if
   * the store has a groupField.
   * @return The sum
   */
  function sum(field:String, grouped:Boolean = false):Number;
}
}