package ext.data {
import ext.Base;
import ext.data.operation.DestroyOperation;
import ext.data.operation.ReadOperation;
import ext.data.proxy.DataProxy;
import ext.data.schema.Schema;

[Uses("ext.Base")]
[Native("Ext.data.Model", require)]
/**
 * A Model or Entity represents some object that your application manages. For example, one
 * might define a Model for Users, Products, Cars, or other real-world object that we want
 * to model in the system. Models are used by <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">stores</a>, which are in
 * turn used by many of the data-bound components in Ext.
 *
 * <h1>Fields</h1>
 *
 * <p>Models are defined as a set of fields and any arbitrary methods and properties relevant
 * to the model. For example:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: [
 *         {name: 'name',  type: 'string'},
 *         {name: 'age',   type: 'int', convert: null},
 *         {name: 'phone', type: 'string'},
 *         {name: 'alive', type: 'boolean', defaultValue: true, convert: null}
 *     ],
 *
 *     changeName: function() {
 *         var oldName = this.get('name'),
 *             newName = oldName + " The Barbarian";
 *
 *         this.set('name', newName);
 *     }
 * });
 * </code></pre>
 *
 * <p>Now we can create instances of our User model and call any model logic we defined:</p>
 *
 * <pre><code>var user = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('User', {
 *     id   : 'ABCD12345',
 *     name : 'Conan',
 *     age  : 24,
 *     phone: '555-555-5555'
 * });
 *
 * user.changeName();
 * user.get('name'); //returns "Conan The Barbarian"
 * </code></pre>
 *
 * <p>By default, the built in field types such as number and boolean coerce string values
 * in the raw data by virtue of their <a href="#!/api/Ext.data.field.Field-method-convert" rel="Ext.data.field.Field-method-convert" class="docClass">Ext.data.field.Field.convert</a> method.
 * When the server can be relied upon to send data in a format that does not need to be
 * converted, disabling this can improve performance. The <a href="#!/api/Ext.data.reader.Json" rel="Ext.data.reader.Json" class="docClass">Json</a>
 * and <a href="#!/api/Ext.data.reader.Array" rel="Ext.data.reader.Array" class="docClass">Array</a> readers are likely candidates for this
 * optimization. To disable field conversions you simply specify <code>null</code> for the field's
 * <a href="#!/api/Ext.data.field.Field-cfg-convert" rel="Ext.data.field.Field-cfg-convert" class="docClass">convert config</a>.</p>
 *
 * <h2>The "id" Field and <code>idProperty</code></h2>
 *
 * <p>A Model definition always has an <em>identifying field</em> which should yield a unique key
 * for each instance. By default, a field named "id" will be created with a
 * <a href="#!/api/Ext.data.field.Field-cfg-mapping" rel="Ext.data.field.Field-cfg-mapping" class="docClass">mapping</a> of "id". This happens because of the default
 * <a href="#!/api/Ext.data.Model-cfg-idProperty" rel="Ext.data.Model-cfg-idProperty" class="docClass">idProperty</a> provided in Model definitions.</p>
 *
 * <p>To alter which field is the identifying field, use the <a href="#!/api/Ext.data.Model-cfg-idProperty" rel="Ext.data.Model-cfg-idProperty" class="docClass">idProperty</a> config.</p>
 *
 * <h1>Validators</h1>
 *
 * <p>Models have built-in support for field validators. Validators are added to models as in
 * the follow example:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: [
 *         { name: 'name',     type: 'string' },
 *         { name: 'age',      type: 'int' },
 *         { name: 'phone',    type: 'string' },
 *         { name: 'gender',   type: 'string' },
 *         { name: 'username', type: 'string' },
 *         { name: 'alive',    type: 'boolean', defaultValue: true }
 *     ],
 *
 *     validators: {
 *         age: 'presence',
 *         name: { type: 'length', min: 2 },
 *         gender: { type: 'inclusion', list: ['Male', 'Female'] },
 *         username: [
 *             { type: 'exclusion', list: ['Admin', 'Operator'] },
 *             { type: 'format', matcher: /([a-z]+)[0-9]{2,3}/i }
 *         ]
 *     }
 * });
 * </code></pre>
 *
 * <p>The derived type of <code><a href="#!/api/Ext.data.field.Field" rel="Ext.data.field.Field" class="docClass">Ext.data.field.Field</a></code> can also provide validation. If <code>validators</code>
 * need to be duplicated on multiple fields, instead consider creating a custom field type.</p>
 *
 * <h2>Validation</h2>
 *
 * <p>The results of the validators can be retrieved via the "associated" validation record:</p>
 *
 * <pre><code>var instance = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('User', {
 *     name: 'Ed',
 *     gender: 'Male',
 *     username: 'edspencer'
 * });
 *
 * var validation = instance.getValidation();
 * </code></pre>
 *
 * <p>The returned object is an instance of <code><a href="#!/api/Ext.data.Validation" rel="Ext.data.Validation" class="docClass">Ext.data.Validation</a></code> and has as its fields the
 * result of the field <code>validators</code>. The validation object is "dirty" if there are one or
 * more validation errors present.</p>
 *
 * <p>This record is also available when using data binding as a "pseudo-association" called
 * "validation". This pseudo-association can be hidden by an explicitly declared
 * association by the same name (for compatibility reasons), but doing so is not
 * recommended.</p>
 *
 * <p>The <code><a href="#!/api/Ext.Component-cfg-modelValidation" rel="Ext.Component-cfg-modelValidation" class="docClass">Ext.Component.modelValidation</a></code> config can be used to enable automatic
 * binding from the "validation" of a record to the form fields that may be bound to its
 * values.</p>
 *
 * <h1>Associations</h1>
 *
 * <p>Models often have associations with other Models. These associations can be defined by
 * fields (often called "foreign keys") or by other data such as a many-to-many (or "matrix").
 * See <a href="#!/api/Ext.data.schema.Association" rel="Ext.data.schema.Association" class="docClass">Ext.data.schema.Association</a> for information about configuring and using associations.</p>
 *
 * <h1>Using a Proxy</h1>
 *
 * <p>Models are great for representing types of data and relationships, but sooner or later we're going to want to load or
 * save that data somewhere. All loading and saving of data is handled via a <a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">Proxy</a>, which
 * can be set directly on the Model:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'name', 'email'],
 *
 *     proxy: {
 *         type: 'rest',
 *         url : '/users'
 *     }
 * });
 * </code></pre>
 *
 * <p>Here we've set up a <a href="#!/api/Ext.data.proxy.Rest" rel="Ext.data.proxy.Rest" class="docClass">Rest Proxy</a>, which knows how to load and save data to and from a
 * RESTful backend. Let's see how this works:</p>
 *
 * <pre><code>var user = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('User', {name: 'Ed Spencer', email: 'ed&#64;sencha.com'});
 *
 * user.save(); //POST /users
 * </code></pre>
 *
 * <p>Calling <a href="#!/api/Ext.data.Model-method-save" rel="Ext.data.Model-method-save" class="docClass">save</a> on the new Model instance tells the configured RestProxy that we wish to persist this Model's
 * data onto our server. RestProxy figures out that this Model hasn't been saved before because it doesn't have an id,
 * and performs the appropriate action - in this case issuing a POST request to the url we configured (/users). We
 * configure any Proxy on any Model and always follow this API - see <a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">Ext.data.proxy.Proxy</a> for a full list.</p>
 *
 * <p>Loading data via the Proxy is accomplished with the static <code>load</code> method:</p>
 *
 * <pre><code>//Uses the configured RestProxy to make a GET request to /users/123
 * User.load(123, {
 *     success: function(user) {
 *         console.log(user.getId()); //logs 123
 *     }
 * });
 * </code></pre>
 *
 * <p>Models can also be updated and destroyed easily:</p>
 *
 * <pre><code>//the user Model we loaded in the last snippet:
 * user.set('name', 'Edward Spencer');
 *
 * //tells the Proxy to save the Model. In this case it will perform a PUT request to /users/123 as this Model already has an id
 * user.save({
 *     success: function() {
 *         console.log('The User was updated');
 *     }
 * });
 *
 * //tells the Proxy to destroy the Model. Performs a DELETE request to /users/123
 * user.erase({
 *     success: function() {
 *         console.log('The User was destroyed!');
 *     }
 * });
 * </code></pre>
 *
 * <h1>HTTP Parameter names when using a <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">Ajax proxy</a></h1>
 *
 * <p>By default, the model ID is specified in an HTTP parameter named <code>id</code>. To change the
 * name of this parameter use the Proxy's <a href="#!/api/Ext.data.proxy.Ajax-cfg-idParam" rel="Ext.data.proxy.Ajax-cfg-idParam" class="docClass">idParam</a>
 * configuration.</p>
 *
 * <p>Parameters for other commonly passed values such as
 * <a href="#!/api/Ext.data.proxy.Ajax-cfg-pageParam" rel="Ext.data.proxy.Ajax-cfg-pageParam" class="docClass">page number</a> or
 * <a href="#!/api/Ext.data.proxy.Ajax-cfg-startParam" rel="Ext.data.proxy.Ajax-cfg-startParam" class="docClass">start row</a> may also be configured.</p>
 *
 * <h1>Usage in Stores</h1>
 *
 * <p>It is very common to want to load a set of Model instances to be displayed and manipulated in the UI. We do this by
 * creating a <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a>:</p>
 *
 * <pre><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     model: 'User'
 * });
 *
 * //uses the Proxy we set up on Model to load the Store data
 * store.load();
 * </code></pre>
 *
 * <p>A Store is just a collection of Model instances - usually loaded from a server somewhere. Store can also maintain a
 * set of added, updated and removed Model instances to be synchronized with the server via the Proxy. See the <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store docs</a> for more information on Stores.</p>
 */
public class Model extends ext.Base {
  /**
   * Record event type 'commit'.
   * @see http://docs.sencha.com/extjs/6.0/6.0.1-classic/#!/api/Ext.data.Model-static-property-COMMIT Sencha Docs Ext JS 6.0
   */
  public static const COMMIT:String = "commit";

  /**
   * The default proxy to use for instances of this Model when no proxy is configured
   * on the instance.  When specified, the model will use this proxy instead of
   * requesting one from the <a href="#!/api/Ext.data.Session" rel="Ext.data.Session" class="docClass">Session</a>.
   *
   * <p>Can be a string "type", or a <a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">Proxy</a> config object.</p>
   *
   * <p>This proxy is not inherited by subclasses.</p>
   * @default 'memory'
   */
  public static native function get defaultProxy():*;

  /**
   * @private
   */
  public static native function set defaultProxy(value:*):void;

  /**
   * Returns the configured Proxy for this Model.
   * @return The proxy
   */
  public static native function getProxy():ext.data.proxy.DataProxy;

  /**
   * Asynchronously loads a model instance by id. Any processing of the loaded
   * record should be done in a callback.
   *
   * <p>Sample usage:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.User', {
   *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
   *     fields: [
   *         {name: 'id', type: 'int'},
   *         {name: 'name', type: 'string'}
   *     ]
   * });
   *
   * MyApp.User.load(10, {
   *     scope: this,
   *     failure: function(record, operation) {
   *         //do something if the load failed
   *     },
   *     success: function(record, operation) {
   *         //do something if the load succeeded
   *     },
   *     callback: function(record, operation, success) {
   *         //do something whether the load succeeded or failed
   *     }
   * });
   * </code></pre>
   * @param id The ID of the model to load.
   * <strong>NOTE:</strong> The model returned must have an ID matching the param in the load
   * request.
   * @param options The options param is an
   * <a href="#!/api/Ext.data.operation.Read" rel="Ext.data.operation.Read" class="docClass">Ext.data.operation.Read</a> config object containing success, failure and
   * callback functions, plus optional scope.
   * @param {Function} options.success A function to be called when the
   * model is processed by the proxy successfully.
   * The callback is passed the following parameters:
   * @param {ext.data.Model} options.success.record The record.
   * @param {ext.data.operation.Operation} options.success.operation The operation.
   * @param {Function} options.failure A function to be called when the
   * model is unable to be processed by the server.
   * The callback is passed the following parameters:
   * @param {ext.data.Model} options.failure.record The record.
   * @param {ext.data.operation.Operation} options.failure.operation The operation.
   * @param {Function} options.callback A function to be called whether the proxy
   * transaction was successful or not.
   * The callback is passed the following parameters:
   * @param {ext.data.Model} options.callback.record The record.
   * @param {ext.data.operation.Operation} options.callback.operation The
   * operation.
   * @param {Boolean} options.callback.success <code>true</code> if the operation was
   * successful.
   * @param {Object} options.scope The scope in which to execute the callback
   * functions.  Defaults to the model instance.
   * @param session The session for this record.
   * @return The newly created model. Note that the model will
   * (probably) still be loading once it is returned from this method. To do any
   * post-processing on the data, the appropriate place to do see is in the
   * callback.
   */
  public static native function load(id:*, options:Object = null, session:ext.data.Session = null):ext.data.Model;

  /**
   * Sets the Proxy to use for this model. Accepts any options that can be accepted by
   * <a href="#!/api/Ext-method-createByAlias" rel="Ext-method-createByAlias" class="docClass">Ext.createByAlias</a>.
   * @param proxy The proxy
   */
  public static native function setProxy(proxy:*):ext.data.proxy.DataProxy;

  /**
   * This method adds the given set of fields to this model class.
   * @since 5.0.0
   * @param newFields The new fields to add. Based on the <code>name</code>
   * of a field this may replace a previous field definition.
   */
  protected static native function addFields(newFields:*):void;

  /**
   * Removes the given set of fields from this model.
   * @since 5.0.0
   * @param removeFields The names of fields to remove or <code>true</code>
   * to remove all existing fields. Removes are processed first followed by adds so
   * if a field name appears in <code>newFields</code> as well that field will effectively be
   * added (however, in that case there is no need to include the field in this
   * array).
   */
  protected static native function removeFields(removeFields:*):void;

  /**
   * This method replaces the specified set of fields with a given set of new fields.
   * Fields should normally be considered immutable, but if the timing is right (that
   * is, before derived classes are declared), it is permissible to change the fields
   * collection.
   * @since 5.0.0
   * @param newFields The new fields to add. Based on the <code>name</code>
   * of a field this may replace a previous field definition.
   * @param removeFields The names of fields to remove or <code>true</code>
   * to remove all existing fields. Removes are processed first followed by adds so
   * if a field name appears in <code>newFields</code> as well that field will effectively be
   * added (however, in that case there is no need to include the field in this
   * array).
   */
  protected static native function replaceFields(newFields:*, removeFields:*):void;

  /**
   * @param data
   * @param session
   */
  public function Model(data:Object, session:ext.data.Session = null) {
    super();
  }

  /**
   * This value is initially "R" or "C" indicating the initial CRUD state. As the
   * record changes and the various joined parties (stores, sessions, etc.) are notified
   * this value is updated prior to these calls. In other words, the joined parties
   * are notified after the <code>crudState</code> is updated. This means that the <code>crudState</code>
   * property may be briefly out of sync with underlying changes if this state is used
   * outside of these notifications.
   *
   * <p>The possible states have these meanings:</p>
   *
   * <ul>
   * <li>"R" - The record is in a cleanly retrieved (unmodified) state.</li>
   * <li>"C" - The record is in a newly created (<code>phantom</code>) state.</li>
   * <li>"U" - The record is in an updated, <code>modified</code> (<code>dirty</code>) state.</li>
   * <li>"D" - The record is in a <code>dropped</code> state.</li>
   * </ul>
   * @default 'R'
   * @since 6.2.0
   */
  public native function get crudState():String;

  /**
   * This value is initially <code>null</code> indicating there is no previous CRUD state. As the
   * record changes and the various joined parties (stores, sessions, etc.) are notified
   * this value is updated for the <em>subsequent</em> calls. In other words, the joined parties
   * are notified and then <code>crudStateWas</code> is modified for the next update.
   *
   * <p>The value of this property has the same meaning as <code>crudState</code>.</p>
   * @since 6.2.0
   */
  public native function get crudStateWas():String;

  /**
   * An object hash representing the data for this Record. Every field name in the Record definition is represented by a
   * property of that name in this object. Note that unless you specified a field with
   * name "id" in the Record definition, this will <b>not</b> contain an <tt>id</tt> property.
   */
  public native function get data():Object;

  /**
   * True if this record has been modified.
   * @default false
   */
  public native function get dirty():Boolean;

  /**
   * True if this record is pending delete on the server. This is set by the <code>drop</code>
   * method and transmitted to the server by the <code>save</code> method.
   * @default false
   */
  public native function get dropped():Boolean;

  /**
   * Internal flag used to track whether or not the model instance is currently being edited.
   * @default false
   */
  public native function get editing():Boolean;

  /**
   * The short name of this entity class. This name is derived from the <code>namespace</code> of
   * the associated <code>schema</code> and this class name. By default, a class is not given a
   * shortened name.
   *
   * <p>All entities in a given <code>schema</code> must have a unique <code>entityName</code>.</p>
   *
   * <p>For more details see "Relative Naming" in <a href="#!/api/Ext.data.schema.Schema" rel="Ext.data.schema.Schema" class="docClass">Ext.data.schema.Schema</a>.</p>
   */
  public native function get entityName():String;

  /**
   * @private
   */
  public native function set entityName(value:String):void;

  /**
   * True if this record has been erased on the server. This flag is set of the <code>erase</code>
   * method.
   * @default false
   */
  public native function get erased():Boolean;

  /**
   * This property is incremented on each modification of a record.
   * @default 1
   * @since 5.0.0
   */
  public native function get generation():Number;

  /**
   * The value <code>true</code> to identify this class and its subclasses.
   * @default true
   */
  public native function get isEntity():Boolean;

  /**
   * The value <code>true</code> to identify this class and its subclasses.
   * @default true
   */
  public native function get isModel():Boolean;

  /**
   * A hash of field values which holds the initial values of fields before a set of edits
   * are <a href="#!/api/Ext.data.Model-method-commit" rel="Ext.data.Model-method-commit" class="docClass">committed</a>.
   */
  public native function get modified():Object;

  /**
   * @private
   */
  public native function set modified(value:Object):void;

  /**
   * True when the record does not yet exist in a server-side database. Any record which
   * has a real database identity set as its <code>idProperty</code> is NOT a phantom -- it's real.
   * @default false
   */
  public native function get phantom():Boolean;

  /**
   * @private
   */
  public native function set phantom(value:Boolean):void;

  /**
   * The <a href="#!/api/Ext.data.Session" rel="Ext.data.Session" class="docClass">Ext.data.Session</a> for this record.
   */
  public native function get session():ext.data.Session;

  /**
   * The <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a> to which this instance belongs.
   *
   * <p><strong>Note:</strong> If this instance is bound to multiple stores, this property
   * will reference only the first.</p>
   */
  public native function get store():ext.data.Store;

  /**
   * @private
   */
  public native function set store(value:ext.data.Store):void;

  [Deprecated(message = "Use &lt;code>hasMany/hasOne/belongsTo&lt;/code>.\n", since = "6.2.0")]
  [ExtConfig]
  /**
   * An array of <a href="#!/api/Ext.data.schema.Association" rel="Ext.data.schema.Association" class="docClass">associations</a> for this model.
   *
   * <p>For further documentation, see <a href="#!/api/Ext.data.schema.Association" rel="Ext.data.schema.Association" class="docClass">Ext.data.schema.Association</a>.</p>
   */
  public native function get associations():Array;

  [ExtConfig]
  [ArrayElementType("ext.data.schema.Association")]
  /**
   * @private
   */
  public native function set associations(value:Array):void;

  [ExtConfig]
  /**
   * One or more <code>Ext.data.schema.BelongsTo</code> associations for this model.
   */
  public native function get belongsTo():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set belongsTo(value:*):void;

  [ExtConfig]
  /**
   * The name of the property a server will use to send back a client-generated id in a
   * <code>create</code> or <code>update</code> <code><a href="#!/api/Ext.data.operation.Operation" rel="Ext.data.operation.Operation" class="docClass">operation</a></code>.
   *
   * <p>If specified, this property cannot have the same name as any other field.</p>
   *
   * <p>For example:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Person', {
   *      idProperty: 'id',  // this is the default value (for clarity)
   *
   *      clientIdProperty: 'clientId',
   *
   *      identifier: 'negative', // to generate -1, -2 etc on the client
   *
   *      fields: [ 'name' ]
   *  });
   *
   *  var person = new Person({
   *      // no id provided, so -1 is generated
   *      name: 'Clark Kent'
   *  });
   * </code></pre>
   *
   * <p>The server is given this data during the <code>create</code>:</p>
   *
   * <pre><code> {
   *      id: -1,
   *      name: 'Clark Kent'
   *  }
   * </code></pre>
   *
   * <p>The server allocates a real id and responds like so:</p>
   *
   * <pre><code> {
   *      id: 427,
   *      clientId: -1
   *  }
   * </code></pre>
   *
   * <p>This property is most useful when creating multiple entities in a single call to
   * the server in a <code><a href="#!/api/Ext.data.operation.Create" rel="Ext.data.operation.Create" class="docClass">create operation</a></code>. Alternatively,
   * the server could respond with records that correspond one-to-one to those sent in
   * the <code>operation</code>.</p>
   *
   * <p>For example the client could send a <code>create</code> with this data:</p>
   *
   * <pre><code> [ { id: -1, name: 'Clark Kent' },
   *    { id: -2, name: 'Peter Parker' },
   *    { id: -3, name: 'Bruce Banner' } ]
   * </code></pre>
   *
   * <p>And the server could respond in the same order:</p>
   *
   * <pre><code> [ { id: 427 },      // updates id = -1
   *    { id: 428 },      // updates id = -2
   *    { id: 429 } ]     // updates id = -3
   * </code></pre>
   *
   * <p>Or using <code>clientIdProperty</code> the server could respond in arbitrary order:</p>
   *
   * <pre><code> [ { id: 427, clientId: -3 },
   *    { id: 428, clientId: -1 },
   *    { id: 429, clientId: -2 } ]
   * </code></pre>
   *
   * <p><strong>IMPORTANT:</strong> When upgrading from previous versions be aware that this property
   * used to perform the role of <code><a href="#!/api/Ext.data.writer.Writer-cfg-clientIdProperty" rel="Ext.data.writer.Writer-cfg-clientIdProperty" class="docClass">Ext.data.writer.Writer.clientIdProperty</a></code> as
   * well as that described above. To continue send a client-generated id as other than
   * the <code>idProperty</code>, set <code>clientIdProperty</code> on the <code>writer</code>. A better solution, however,
   * is most likely a properly configured <code>identifier</code> as that would work better with
   * associations.</p>
   */
  public native function get clientIdProperty():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set clientIdProperty(value:String):void;

  [ExtConfig]
  /**
   * Set to <code>false</code> to prevent any converters from being called on fields specified in
   * a <a href="#!/api/Ext.data.Model-method-set" rel="Ext.data.Model-method-set" class="docClass">set</a> operation.
   *
   * <p><strong>Note:</strong> Setting the config to <code>false</code> will only prevent the convert / calculate
   * call when the set <code>fieldName</code> param matches the field's <code>name</code>.  In the
   * following example the calls to set <code>salary</code> will not execute the convert method
   * on <code>set</code> while the calls to set <code>vested</code> will execute the convert method on the
   * initial read as well as on <code>set</code>.</p>
   *
   * <p>Example model definition:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.model.Employee', {
   *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
   *     fields: ['yearsOfService', {
   *         name: 'salary',
   *         convert: function (val) {
   *             var startingBonus = val * .1;
   *             return val + startingBonus;
   *         }
   *     }, {
   *         name: 'vested',
   *         convert: function (val, record) {
   *             return record.get('yearsOfService') &gt;= 4;
   *         },
   *         depends: 'yearsOfService'
   *     }],
   *     convertOnSet: false
   * });
   *
   * var tina = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('MyApp.model.Employee', {
   *     salary: 50000,
   *     yearsOfService: 3
   * });
   *
   * console.log(tina.get('salary')); // logs 55000
   * console.log(tina.get('vested')); // logs false
   *
   * tina.set({
   *     salary: 60000,
   *     yearsOfService: 4
   * });
   * console.log(tina.get('salary')); // logs 60000
   * console.log(tina.get('vested')); // logs true
   * </code></pre>
   * @default true
   */
  public native function get convertOnSet():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set convertOnSet(value:Boolean):void;

  [ExtConfig]
  /**
   * An Array of <code><a href="#!/api/Ext.data.field.Field" rel="Ext.data.field.Field" class="docClass">Ext.data.field.Field</a></code> config objects, simply the field
   * <a href="#!/api/Ext.data.field.Field-cfg-name" rel="Ext.data.field.Field-cfg-name" class="docClass">name</a>, or a mix of config objects and strings.
   * If just a name is given, the field type defaults to <code>auto</code>.
   *
   * <p>In a <a href="#!/api/Ext.data.field.Field" rel="Ext.data.field.Field" class="docClass">Field</a> config object you may pass the alias of
   * the <code>Ext.data.field.&#42;</code> type using the <code>type</code> config option.</p>
   *
   * <pre><code>// two fields are set:
   * // - an 'auto' field with a name of 'firstName'
   * // - and an <a href="#!/api/Ext.data.field.Integer" rel="Ext.data.field.Integer" class="docClass">Ext.data.field.Integer</a> field with a name of 'age'
   * fields: ['firstName', {
   *     type: 'int',
   *     name: 'age'
   * }]
   * </code></pre>
   *
   * <p>Fields will automatically be created at read time for any for any keys in the
   * data passed to the Model's <a href="#!/api/Ext.data.Model-cfg-proxy" rel="Ext.data.Model-cfg-proxy" class="docClass">proxy's</a>
   * <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">reader</a> whose name is not explicitly configured in
   * the <code>fields</code> config.</p>
   *
   * <p>Extending a Model class will inherit all the <code>fields</code> from the superclass /
   * ancestor classes.</p>
   */
  public native function get fields():Array;

  [ExtConfig]
  [ArrayElementType("ext.data.field.DataField")]
  /**
   * @private
   */
  public native function set fields(value:Array):void;

  [ExtConfig]
  /**
   * One or more <code><a href="#!/api/Ext.data.schema.HasMany" rel="Ext.data.schema.HasMany" class="docClass">Ext.data.schema.HasMany</a></code> associations for this model.
   */
  public native function get hasMany():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hasMany(value:*):void;

  [ExtConfig]
  /**
   * One or more <code><a href="#!/api/Ext.data.schema.HasOne" rel="Ext.data.schema.HasOne" class="docClass">Ext.data.schema.HasOne</a></code> associations for this model.
   */
  public native function get hasOne():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hasOne(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the field treated as this Model's unique id.
   *
   * <p>If changing the idProperty in a subclass, the generated id field will replace the one
   * generated by the superclass, for example;</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Super', {
   *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
   *      fields: ['name']
   *  });
   *
   *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Sub', {
   *      extend: 'Super',
   *      idProperty: 'customId'
   *  });
   *
   *  var fields = Super.getFields();
   *  // Has 2 fields, "name" &amp; "id"
   *  console.log(fields[0].name, fields[1].name, fields.length);
   *
   *  fields = Sub.getFields();
   *  // Has 2 fields, "name" &amp; "customId", "id" is replaced
   *  console.log(fields[0].name, fields[1].name, fields.length);
   * </code></pre>
   *
   * <p>The data values for this field must be unique or there will be id value collisions
   * in the <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a>.</p>
   * @default 'id'
   * @see #getIdProperty()
   */
  public native function get idProperty():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set idProperty(value:String):void;

  [ExtConfig]
  /**
   * The id generator to use for this model. The <code>identifier</code> generates values for the
   * <a href="#!/api/Ext.data.Model-cfg-idProperty" rel="Ext.data.Model-cfg-idProperty" class="docClass">idProperty</a> when no value is given. Records with client-side generated
   * values for <a href="#!/api/Ext.data.Model-cfg-idProperty" rel="Ext.data.Model-cfg-idProperty" class="docClass">idProperty</a> are called <a href="#!/api/Ext.data.Model-property-phantom" rel="Ext.data.Model-property-phantom" class="docClass">phantom</a> records since they are
   * not yet known to the server.
   *
   * <p>This can be overridden at the model level to provide a custom generator for a model.
   * The simplest form of this would be:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.MyModel', {
   *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
   *      requires: ['<a href="#!/api/Ext.data.identifier.Sequential" rel="Ext.data.identifier.Sequential" class="docClass">Ext.data.identifier.Sequential</a>'],
   *      identifier: 'sequential',
   *      ...
   *  });
   * </code></pre>
   *
   * <p>The above would generate <a href="#!/api/Ext.data.identifier.Sequential" rel="Ext.data.identifier.Sequential" class="docClass">sequential</a> id's such
   * as 1, 2, 3 etc..</p>
   *
   * <p>Another useful id generator is <a href="#!/api/Ext.data.identifier.Uuid" rel="Ext.data.identifier.Uuid" class="docClass">Ext.data.identifier.Uuid</a>:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.MyModel', {
   *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
   *      requires: ['<a href="#!/api/Ext.data.identifier.Uuid" rel="Ext.data.identifier.Uuid" class="docClass">Ext.data.identifier.Uuid</a>'],
   *      identifier: 'uuid',
   *      ...
   *  });
   * </code></pre>
   *
   * <p>An id generator can also be further configured:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.MyModel', {
   *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
   *      identifier: {
   *          type: 'sequential',
   *          seed: 1000,
   *          prefix: 'ID_'
   *      }
   *  });
   * </code></pre>
   *
   * <p>The above would generate id's such as ID_1000, ID_1001, ID_1002 etc..</p>
   *
   * <p>If multiple models share an id space, a single generator can be shared:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.MyModelX', {
   *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
   *      identifier: {
   *          type: 'sequential',
   *          id: 'xy'
   *      }
   *  });
   *
   *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.MyModelY', {
   *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
   *      identifier: {
   *          type: 'sequential',
   *          id: 'xy'
   *      }
   *  });
   * </code></pre>
   *
   * <p>For more complex, shared id generators, a custom generator is the best approach.
   * See <a href="#!/api/Ext.data.identifier.Generator" rel="Ext.data.identifier.Generator" class="docClass">Ext.data.identifier.Generator</a> for details on creating custom id generators.</p>
   */
  public native function get identifier():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set identifier(value:*):void;

  [ExtConfig]
  /**
   * A config object for a <a href="#!/api/Ext.data.schema.ManyToMany" rel="Ext.data.schema.ManyToMany" class="docClass">ManyToMany</a> association.
   * See the class description for <a href="#!/api/Ext.data.schema.ManyToMany" rel="Ext.data.schema.ManyToMany" class="docClass">ManyToMany</a> for
   * configuration examples.
   */
  public native function get manyToMany():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set manyToMany(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The <a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">proxy</a> to use for this class.
   * @see #getProxy()
   */
  public native function get proxy():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set proxy(value:*):void;

  [ExtConfig]
  /**
   * The <code><a href="#!/api/Ext.data.schema.Schema" rel="Ext.data.schema.Schema" class="docClass">Ext.data.schema.Schema</a></code> to which this entity and its associations belong.
   * @default 'default'
   */
  public native function get schema():ext.data.schema.Schema;

  [ExtConfig]
  /**
   * @private
   */
  public native function set schema(value:ext.data.schema.Schema):void;

  [ExtConfig]
  /**
   * If specified this property is used to concatenate multiple errors for each field
   * as reported by the <code>validators</code>.
   * @default null
   */
  public native function get validationSeparator():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set validationSeparator(value:String):void;

  [ExtConfig]
  /**
   * An array of <a href="#!/api/Ext.data.validator.Validator" rel="Ext.data.validator.Validator" class="docClass">validators</a> for this model.
   */
  public native function get validators():Array;

  [ExtConfig]
  [ArrayElementType("ext.data.validator.Validator")]
  /**
   * @private
   */
  public native function set validators(value:Array):void;

  [ExtConfig]
  /**
   * If specified, this is the name of the property that contains the entity "version".
   * The version property is used to manage a long-running transaction and allows the
   * detection of simultaneous modification.
   *
   * <p>The way a version property is used is that the client receives the version as it
   * would any other entity property. When saving an entity, this property is always
   * included in the request and the server uses the value in a "conditional update".
   * If the current version of the entity on the server matches the version property
   * sent by the client, the update is allowed. Otherwise, the update fails.</p>
   *
   * <p>On successful update, both the client and server increment the version. This is
   * done on the server in the conditional update and on the client when it receives a
   * success on its update request.</p>
   */
  public native function get versionProperty():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set versionProperty(value:String):void;

  /**
   * Aborts a pending <a href="#!/api/Ext.data.Model-method-load" rel="Ext.data.Model-method-load" class="docClass">load</a> operation. If the record is not loading, this does nothing.
   */
  public native function abort():void;

  /**
   * Begins an edit. While in edit mode, no events (e.g.. the <code>update</code> event) are
   * relayed to the containing store. When an edit has begun, it must be followed by
   * either <code>endEdit</code> or <code>cancelEdit</code>.
   */
  public native function beginEdit():void;

  /**
   * Cancels all changes made in the current edit operation.
   */
  public native function cancelEdit():void;

  /**
   * Creates a clone of this record. States like <code>dropped</code>, <code>phantom</code> and <code>dirty</code> are
   * all preserved in the cloned record.
   * @param session The session to which the new record
   * belongs.
   * @return The cloned record.
   */
  public native function clone(session:ext.data.Session = null):ext.data.Model;

  /**
   * Usually called by the <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> which owns the model instance. Commits all changes made to the
   * instance since either creation or the last commit operation.
   *
   * <p>Developers should subscribe to the <a href="#!/api/Ext.data.Store-event-update" rel="Ext.data.Store-event-update" class="docClass">Ext.data.Store.update</a> event to have their code notified of commit
   * operations.</p>
   * @param silent Pass <code>true</code> to skip notification of the owning store of the change.
   * @default false
   * @param modifiedFieldNames Array of field names changed during sync with server if known.
   * Omit or pass <code>null</code> if unknown. An empty array means that it is known that no fields were modified
   * by the server's response.
   * Defaults to false.
   */
  public native function commit(silent:Boolean = false, modifiedFieldNames:Array = null):void;

  /**
   * Creates a clean copy of this record. The returned record will not consider any its
   * fields as modified.
   *
   * <p>To generate a phantom instance with a new id pass <code>null</code>:</p>
   *
   * <pre><code>var rec = record.copy(null); // clone the record but no id (one is generated)
   * </code></pre>
   * @param newId A new id, defaults to the id of the instance being copied.
   * See <code><a href="#!/api/Ext.data.Model-cfg-idProperty" rel="Ext.data.Model-cfg-idProperty" class="docClass">idProperty</a></code>.
   * @param session The session to which the new record
   * belongs.
   */
  public native function copy(newId:String = null, session:ext.data.Session = null):ext.data.Model;

  /**
   * Marks this record as <code>dropped</code> and waiting to be deleted on the server. When a
   * record is dropped, it is automatically removed from all association stores and
   * any child records associated to this record are also dropped (a "cascade delete")
   * depending on the <code>cascade</code> parameter.
   * @since 5.0.0
   * @param cascade Pass <code>false</code> to disable the cascade to drop child
   * records.
   * @default true
   */
  public native function drop(cascade:Boolean = true):void;

  /**
   * Ends an edit. If any data was modified, the containing store is notified
   * (ie, the store's <code>update</code> event will fire).
   * @param silent True to not notify any stores of the change.
   * @param modifiedFieldNames Array of field names changed during edit.
   */
  public native function endEdit(silent:Boolean = false, modifiedFieldNames:Array = null):void;

  /**
   * @param options Options to pass to the proxy.
   * @param {Function} options.success A function to be called when the
   * model is processed by the proxy successfully.
   * The callback is passed the following parameters:
   * @param {ext.data.Model} options.success.record The record.
   * @param {ext.data.operation.Operation} options.success.operation The operation.
   * @param {Function} options.failure A function to be called when the
   * model is unable to be processed by the server.
   * The callback is passed the following parameters:
   * @param {ext.data.Model} options.failure.record The record.
   * @param {ext.data.operation.Operation} options.failure.operation The operation.
   * @param {Function} options.callback A function to be called whether the proxy
   * transaction was successful or not.
   * The callback is passed the following parameters:
   * @param {ext.data.Model} options.callback.record The record.
   * @param {ext.data.operation.Operation} options.callback.operation The operation.
   * @param {Boolean} options.callback.success <code>true</code> if the operation was successful.
   * @param {Object} options.scope The scope in which to execute the callback
   * functions.  Defaults to the model instance.
   * @return The destroy operation
   */
  public native function erase(options:Object = null):ext.data.operation.DestroyOperation;

  /**
   * Returns the value of the given field.
   * @param fieldName The name of the field.
   * @return The value of the specified field.
   */
  public native function get(fieldName:String):Object;

  /**
   * Gets all of the data from this Models <em>loaded</em> associations. It does this
   * recursively. For example if we have a User which hasMany Orders, and each Order
   * hasMany OrderItems, it will return an object like this:
   *
   * <pre><code>{
   *     orders: [
   *         {
   *             id: 123,
   *             status: 'shipped',
   *             orderItems: [
   *                 ...
   *             ]
   *         }
   *     ]
   * }
   * </code></pre>
   * @param result The object on to which the associations will be added. If
   * no object is passed one is created. This object is then returned.
   * @param options An object containing options describing the data
   * desired.
   * @param {Boolean} [options.associated] Pass <code>true</code> to include associated data from
   * other associated records.
   * @default true
   * @param {Boolean} [options.changes] Pass <code>true</code> to only include fields that
   * have been modified. Note that field modifications are only tracked for fields that
   * are not declared with <code>persist</code> set to <code>false</code>. In other words, only persistent
   * fields have changes tracked so passing <code>true</code> for this means <code>options.persist</code> is
   * redundant.
   * @default false
   * @param {Boolean} [options.critical] Pass <code>true</code> to include fields set as <code>critical</code>.
   * This is only meaningful when <code>options.changes</code> is <code>true</code> since critical fields may
   * not have been modified.
   * @param {Boolean} [options.persist] Pass <code>true</code> to only return persistent fields.
   * This is implied when <code>options.changes</code> is set to <code>true</code>.
   * @param {Boolean} [options.serialize] Pass <code>true</code> to invoke the <code>serialize</code>
   * method on the returned fields.
   * @default false
   * @return The nested data set for the Model's loaded associations.
   */
  public native function getAssociatedData(result:Object = null, options:* = undefined):Object;

  /**
   * Gets an object of only the fields that have been modified since this record was
   * created or committed. Only persistent fields are tracked in the <code>modified</code> set so
   * this method will only return changes to persistent fields.
   *
   * <p>For more control over the returned data, see <code><a href="#!/api/Ext.data.Model-method-getData" rel="Ext.data.Model-method-getData" class="docClass">getData</a></code>.</p>
   */
  public native function getChanges():Object;

  [ArrayElementType("ext.data.field.DataField")]
  /**
   * Returns the array of fields that are declared as critical (must always send).
   */
  public native function getCriticalFields():Array;

  /**
   * Gets all values for each field in this model and returns an object containing the
   * current data. This can be tuned by passing an <code>options</code> object with various
   * properties describing the desired result. Passing <code>true</code> simply returns all fields
   * <em>and</em> all associated record data.
   * @param options An object containing options describing the data
   * desired. If <code>true</code> is passed it is treated as an object with <code>associated</code> set to
   * <code>true</code>.
   * @param {Boolean} [options.associated] Pass <code>true</code> to include associated data.
   * This is equivalent to pass <code>true</code> as the only argument. See <code>getAssociatedData</code>.
   * @default false
   * @param {Boolean} [options.changes] Pass <code>true</code> to only include fields that
   * have been modified. Note that field modifications are only tracked for fields that
   * are not declared with <code>persist</code> set to <code>false</code>. In other words, only persistent
   * fields have changes tracked so passing <code>true</code> for this means <code>options.persist</code> is
   * redundant.
   * @default false
   * @param {Boolean} [options.critical] Pass <code>true</code> to include fields set as <code>critical</code>.
   * This is only meaningful when <code>options.changes</code> is <code>true</code> since critical fields may
   * not have been modified.
   * @param {Boolean} [options.persist] Pass <code>true</code> to only return persistent fields.
   * This is implied when <code>options.changes</code> is set to <code>true</code>.
   * @param {Boolean} [options.serialize] Pass <code>true</code> to invoke the <code>serialize</code>
   * method on the returned fields.
   * @default false
   * @return An object containing all the values in this model.
   */
  public native function getData(options:* = undefined):Object;

  [ArrayElementType("ext.data.field.DataField")]
  /**
   * Get the fields array for this model.
   * @return The fields array
   */
  public native function getFields():Array;

  /**
   * @private
   */
  public native function getFieldsMap():Object;

  /**
   * Returns the unique ID allocated to this model instance as defined by <code>idProperty</code>.
   * @return The id
   */
  public native function getId():*;

  /**
   * Get the idProperty for this model.
   * @return The idProperty
   */
  public native function getIdProperty():String;

  /**
   * Returns the original value of a modified field. If there is no modified value,
   * <code>undefined</code> will be return. Also see <a href="#!/api/Ext.data.Model-method-isModified" rel="Ext.data.Model-method-isModified" class="docClass">isModified</a>.
   * @param fieldName The name of the field for which to return the original value.
   * @return modified
   */
  public native function getModified(fieldName:String):Object;

  /**
   * This method returns the value of a field given its name prior to its most recent
   * change.
   * @param fieldName The field's <a href="#!/api/Ext.data.field.Field-cfg-name" rel="Ext.data.field.Field-cfg-name" class="docClass">name</a>.
   * @return The value of the given field prior to its current value. <code>undefined</code>
   * if there is no previous value;
   */
  public native function getPrevious(fieldName:String):Object;

  /**
   * Returns the configured Proxy for this Model.
   * @return The proxy
   */
  public native function getProxy():ext.data.proxy.DataProxy;

  [ArrayElementType("ext.data.field.DataField")]
  /**
   * Returns the array of fields that are declared as non-persist or "transient".
   * @since 5.0.0
   */
  public native function getTransientFields():Array;

  /**
   * Returns the <code><a href="#!/api/Ext.data.Validation" rel="Ext.data.Validation" class="docClass">Ext.data.Validation</a></code> record holding the results of this record's
   * <code>validators</code>. This record is lazily created on first request and is then kept on
   * this record to be updated later.
   *
   * <p>See the class description for more about <code>validators</code>.</p>
   * @since 5.0.0
   * @param refresh Pass <code>false</code> to not call the <code>refresh</code> method on the
   * validation instance prior to returning it. Pass <code>true</code> to force a <code>refresh</code> of the
   * validation instance. By default the returned record is only refreshed if changes
   * have been made to this record.
   * @return The <code>Validation</code> record for this record.
   */
  public native function getValidation(refresh:Boolean = false):ext.data.Model;

  /**
   * Checks whether this model is loading data from the <a href="#!/api/Ext.data.Model-cfg-proxy" rel="Ext.data.Model-cfg-proxy" class="docClass">proxy</a>.
   * @return <code>true</code> if in a loading state.
   */
  public native function isLoading():Boolean;

  /**
   * Returns true if the passed field name has been <code><a href="#!/api/Ext.data.Model-property-modified" rel="Ext.data.Model-property-modified" class="docClass">modified</a></code> since the load or last commit.
   * @param fieldName The field's <a href="#!/api/Ext.data.field.Field-cfg-name" rel="Ext.data.field.Field-cfg-name" class="docClass">name</a>.
   */
  public native function isModified(fieldName:String):Boolean;

  /**
   * Checks if the model is valid. See <a href="#!/api/Ext.data.Model-method-getValidation" rel="Ext.data.Model-method-getValidation" class="docClass">getValidation</a>.
   * @return True if the model is valid.
   */
  public native function isValid():Boolean;

  /**
   * @param options Options to pass to the proxy.
   * @param {Function} options.success A function to be called when the
   * model is processed by the proxy successfully.
   * The callback is passed the following parameters:
   * @param {ext.data.Model} options.success.record The record.
   * @param {ext.data.operation.Operation} options.success.operation The operation.
   * @param {Function} options.failure A function to be called when the
   * model is unable to be processed by the server.
   * The callback is passed the following parameters:
   * @param {ext.data.Model} options.failure.record The record.
   * @param {ext.data.operation.Operation} options.failure.operation The operation.
   * @param {Function} options.callback A function to be called whether the proxy
   * transaction was successful or not.
   * The callback is passed the following parameters:
   * @param {ext.data.Model} options.callback.record The record.
   * @param {ext.data.operation.Operation} options.callback.operation The operation.
   * @param {Boolean} options.callback.success <code>true</code> if the operation was successful.
   * @param {Object} options.scope The scope in which to execute the callback
   * functions.  Defaults to the model instance.
   * @return The read operation.
   */
  public native function load(options:Object = null):ext.data.operation.ReadOperation;

  /**
   * Called when the model id is changed.
   * @param id The new id.
   * @param oldId The old id.
   */
  public native function onIdChanged(id:Object, oldId:Object):void;

  /**
   * Usually called by the <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> to which this model instance has been <a href="#!/api/Ext.data.Model-method-join" rel="Ext.data.Model-method-join" class="docClass">joined</a>. Rejects
   * all changes made to the model instance since either creation, or the last commit operation. Modified fields are
   * reverted to their original values.
   *
   * <p>Developers should subscribe to the <a href="#!/api/Ext.data.Store-event-update" rel="Ext.data.Store-event-update" class="docClass">Ext.data.Store.update</a> event to have their code notified of reject
   * operations.</p>
   * @param silent <code>true</code> to skip notification of the owning store of the change.
   * @default false
   */
  public native function reject(silent:Boolean = false):void;

  /**
   * @param options Options to pass to the proxy.
   * @param {Function} options.success A function to be called when the
   * model is processed by the proxy successfully.
   * The callback is passed the following parameters:
   * @param {ext.data.Model} options.success.record The record.
   * @param {ext.data.operation.Operation} options.success.operation The operation.
   * @param {Function} options.failure A function to be called when the
   * model is unable to be processed by the server.
   * The callback is passed the following parameters:
   * @param {ext.data.Model} options.failure.record The record.
   * @param {ext.data.operation.Operation} options.failure.operation The operation.
   * @param {Function} options.callback A function to be called whether the proxy
   * transaction was successful or not.
   * The callback is passed the following parameters:
   * @param {ext.data.Model} options.callback.record The record.
   * @param {ext.data.operation.Operation} options.callback.operation The operation.
   * @param {Boolean} options.callback.success <code>true</code> if the operation was successful.
   * @param {Object} options.scope The scope in which to execute the callback
   * functions.  Defaults to the model instance.
   * @return The operation instance for saving this model.  The type of operation returned
   * depends on the model state at the time of the action.
   * <ul>
   * <li><a href="#!/api/Ext.data.Model-property-phantom" rel="Ext.data.Model-property-phantom" class="docClass">phantom</a> model - <a href="#!/api/Ext.data.operation.Create" rel="Ext.data.operation.Create" class="docClass">Ext.data.operation.Create</a></li>
   * <li><a href="#!/api/Ext.data.Model-method-isModified" rel="Ext.data.Model-method-isModified" class="docClass">modified</a> model - <a href="#!/api/Ext.data.operation.Update" rel="Ext.data.operation.Update" class="docClass">Ext.data.operation.Update</a></li>
   * <li><a href="#!/api/Ext.data.Model-property-dropped" rel="Ext.data.Model-property-dropped" class="docClass">dropped</a> model - <a href="#!/api/Ext.data.operation.Destroy" rel="Ext.data.operation.Destroy" class="docClass">Ext.data.operation.Destroy</a></li>
   * </ul>
   */
  public native function save(options:Object = null):*;

  /**
   * Sets the given field to the given value. For example:
   *
   * <pre><code> record.set('name', 'value');
   * </code></pre>
   *
   * <p>This method can also be passed an object containing multiple values to set at once.
   * For example:</p>
   *
   * <pre><code> record.set({
   *      name: 'value',
   *      age: 42
   *  });
   * </code></pre>
   *
   * <p>The following store events are fired when the modified record belongs to a store:</p>
   *
   * <ul>
   * <li><a href="#!/api/Ext.data.Store-event-beginupdate" rel="Ext.data.Store-event-beginupdate" class="docClass">beginupdate</a></li>
   * <li><a href="#!/api/Ext.data.Store-event-update" rel="Ext.data.Store-event-update" class="docClass">update</a></li>
   * <li><a href="#!/api/Ext.data.Store-event-endupdate" rel="Ext.data.Store-event-endupdate" class="docClass">endupdate</a></li>
   * </ul>
   * @param fieldName The field to set, or an object containing key/value
   * pairs.
   * @param newValue The value for the field (if <code>fieldName</code> is a string).
   * @param options Options for governing this update.
   * @param {Boolean} [options.convert] Set to <code>false</code> to  prevent any converters from
   * being called during the set operation. This may be useful when setting a large bunch of
   * raw values.
   * @default true
   * @param {Boolean} [options.dirty] Pass <code>false</code> if the field values are to be
   * understood as non-dirty (fresh from the server). When <code>true</code>, this change will be
   * reflected in the <code>modified</code> collection.
   * @default true
   * @param {Boolean} [options.commit] Pass <code>true</code> to call the <a href="#!/api/Ext.data.Model-method-commit" rel="Ext.data.Model-method-commit" class="docClass">commit</a> method
   * after setting fields. If this option is passed, the usual after change processing will
   * be bypassed. <a href="#!/api/Ext.data.Model-method-commit" rel="Ext.data.Model-method-commit" class="docClass">Commit</a> will be called even if there are no field changes.
   * @default false
   * @param {Boolean} [options.silent] Pass <code>true</code> to suppress notification of any
   * changes made by this call. Use with caution.
   * @default false
   * @return The array of modified field names or null if nothing was modified.
   */
  public native function set(fieldName:String, newValue:*, options:Object = null):void;

  /**
   * Sets the model instance's id field to the given id.
   * @param id The new id.
   * @param options See <a href="#!/api/Ext.data.Model-method-set" rel="Ext.data.Model-method-set" class="docClass">set</a>.
   */
  public native function setId(id:*, options:Object = null):void;

  /**
   * Set the session for this record.
   * @param session The session
   */
  public native function setSession(session:ext.data.Session):void;

  /**
   * Returns a url-suitable string for this model instance. By default this just returns the name of the Model class
   * followed by the instance ID - for example an instance of MyApp.model.User with ID 123 will return 'user/123'.
   * @return The url string for this model instance.
   */
  public native function toUrl():String;

  /**
   * Tells this model instance that it has been removed from the store.
   * @param store The store from which this model has been removed.
   */
  public native function unjoin(store:ext.data.Store):void;

  [Deprecated(message = "Use &lt;code>getValidation&lt;/code> instead.\n", since = "5.0")]
  /**
   * Validates the current data against all of its configured <a href="#!/api/Ext.data.Model-cfg-validators" rel="Ext.data.Model-cfg-validators" class="docClass">validators</a>. The
   * returned collection holds an object for each reported problem from a <code>validator</code>.
   * @return The errors collection.
   */
  public native function validate():ext.data.DataErrorCollection;

  /**
   * Return a unique observable ID. Model is not observable but tree nodes (<code><a href="#!/api/Ext.data.NodeInterface" rel="Ext.data.NodeInterface" class="docClass">Ext.data.NodeInterface</a></code>) are, so
   * they must be globally unique within the <a href="#!/api/Ext.data.Model-property-observableType" rel="Ext.data.Model-property-observableType" class="docClass">observableType</a>.
   */
  protected native function getObservableId():String;

  /**
   * Tells this model instance that an observer is looking at it.
   * @param item The store to which this model has been added.
   */
  protected native function join(item:ext.data.Store):void;

  /**
   * This method is called by the <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Ext.data.reader.Reader</a> after loading a model from
   * the server. This is after processing any inline associations that are available.
   */
  protected native function onLoad():void;
}
}