package ext.data {
import ext.Base;

[Uses("ext.Base")]
[Native]
/**
 * This class is used to create JSONP requests. JSONP is a mechanism that allows for making
 * requests for data cross domain. JSONP is basically a <code>&lt;script&gt;</code> node with the source of the url executing
 * a function that was created by <a href="#!/api/Ext.data.JsonP" rel="Ext.data.JsonP" class="docClass">Ext.data.JsonP</a>. Once the resource has loaded, the <code>&lt;script&gt;</code> node will be destroyed.
 *
 * <p>If you have a request such as:</p>
 *
 * <pre><code><a href="#!/api/Ext.data.JsonP-method-request" rel="Ext.data.JsonP-method-request" class="docClass">Ext.data.JsonP.request</a>({
 *     url : 'foo.php'
 * });
 * </code></pre>
 *
 * <p><a href="#!/api/Ext.data.JsonP" rel="Ext.data.JsonP" class="docClass">Ext.data.JsonP</a> will create a <code>&lt;script&gt;</code> node in the <code>&lt;head&gt;</code> with the <code>src</code> attribute pointing to
 * <code>foo.php?callback=Ext.data.JsonP.callback1</code>. The <code>foo.php</code> script will have to detect the <code>callback</code> URL parameter
 * and return valid JavaScript:</p>
 *
 * <pre><code>Ext.data.JsonP.callback1({"foo":"bar"});
 * </code></pre>
 *
 * <p>A simple PHP example would look like:</p>
 *
 * <pre><code>&lt;?php
 *
 * $data = array('foo' =&gt; 'bar');
 *
 * if (!empty($_REQUEST['callback'])) {
 *     header('Content-Type: application/javascript');
 *     echo $_REQUEST['callback'] . '(';
 * }
 *
 * echo json_encode($data);
 *
 * if (!empty($_REQUEST['callback']) {
 *     echo ');';
 * }
 *
 * ?&gt;
 * </code></pre>
 *
 * <p>More information is available <a href="http://en.wikipedia.org/wiki/JSONP">here</a>. You can also use <a href="http://www.jsonplint.com">JSONPLint</a> to test your JSONP.</p>
 * <p>Type of singleton JsonP.</p>
 * @see ext.data.#JsonP ext.data.JsonP
 */
public class SJsonP extends ext.Base {
  /**
   * Specifies the GET parameter that will be sent to the server containing the function name to be executed when
   * the request completes. Defaults to <tt>callback</tt>. Thus, a common request will be in the form of
   * url?callback=Ext.data.JsonP.callback1
   * @default 'callback'
   */
  public native function get callbackKey():String;

  /**
   * @private
   */
  public native function set callbackKey(value:String):void;

  /**
   * True to add a unique cache-buster param to requests. Defaults to <tt>true</tt>.
   * @default true
   */
  public native function get disableCaching():Boolean;

  /**
   * @private
   */
  public native function set disableCaching(value:Boolean):void;

  /**
   * Change the parameter which is sent went disabling caching through a cache buster. Defaults to <tt>'_dc'</tt>.
   * @default '_dc'
   */
  public native function get disableCachingParam():String;

  /**
   * @private
   */
  public native function set disableCachingParam(value:String):void;

  /**
   * A default timeout for any JsonP requests. If the request has not completed in this time the
   * failure callback will be fired. The timeout is in ms. Defaults to <tt>30000</tt>.
   * @default 30000
   */
  public native function get timeout():Number;

  /**
   * @private
   */
  public native function set timeout(value:Number):void;

  /**
   * Abort a request. If the request parameter is not specified all open requests will
   * be aborted.
   * @param request The request to abort
   */
  public native function abort(request:* = undefined):void;

  /**
   * Makes a JSONP request.
   * @param options An object which may contain the following properties. Note that options will
   * take priority over any defaults that are specified in the class.
   * <ul>
   * <li><b>url</b> : String <div class="sub-desc">The URL to request.</div></li>
   * <li><b>params</b> : Object (Optional)<div class="sub-desc">An object containing a series of
   * key value pairs that will be sent along with the request.</div></li>
   * <li><b>timeout</b> : Number (Optional) <div class="sub-desc">See <a href="#!/api/Ext.data.JsonP-property-timeout" rel="Ext.data.JsonP-property-timeout" class="docClass">timeout</a></div></li>
   * <li><b>callbackKey</b> : String (Optional) <div class="sub-desc">See <a href="#!/api/Ext.data.JsonP-property-callbackKey" rel="Ext.data.JsonP-property-callbackKey" class="docClass">callbackKey</a></div></li>
   * <li><b>callbackName</b> : String (Optional) <div class="sub-desc">The function name to use for this request.
   * By default this name will be auto-generated: Ext.data.JsonP.callback1, Ext.data.JsonP.callback2, etc.
   * Setting this option to "my_name" will force the function name to be Ext.data.JsonP.my_name.
   * Use this if you want deterministic behavior, but be careful - the callbackName should be different
   * in each JsonP request that you make.</div></li>
   * <li><b>disableCaching</b> : Boolean (Optional) <div class="sub-desc">See <a href="#!/api/Ext.data.JsonP-property-disableCaching" rel="Ext.data.JsonP-property-disableCaching" class="docClass">disableCaching</a></div></li>
   * <li><b>disableCachingParam</b> : String (Optional) <div class="sub-desc">See <a href="#!/api/Ext.data.JsonP-property-disableCachingParam" rel="Ext.data.JsonP-property-disableCachingParam" class="docClass">disableCachingParam</a></div></li>
   * <li><b>success</b> : Function (Optional) <div class="sub-desc">A function to execute if the request succeeds.</div></li>
   * <li><b>failure</b> : Function (Optional) <div class="sub-desc">A function to execute if the request fails.</div></li>
   * <li><b>callback</b> : Function (Optional) <div class="sub-desc">A function to execute when the request
   * completes, whether it is a success or failure.</div></li>
   * <li><b>scope</b> : Object (Optional)<div class="sub-desc">The scope in
   * which to execute the callbacks: The "this" object for the callback function. Defaults to the browser window.</div></li>
   * </ul>
   * @return request An object containing the request details.
   */
  public native function request(options:Object):Object;
}
}