package ext.data {
import ext.util.Collection;

[Uses("ext.data.ProxyStore")]
[Uses("ext.data.LocalStore")]
[Uses("ext.data.AbstractStore")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Mixin")]
[Native("Ext.data.Store", require)]
/**
 * Fires before a prefetch occurs. Return <code>false</code> to cancel.
 * @eventType ext.data.events.Store_operationEvent.BEFORE_PREFETCH
 */
[Event(name = 'onBeforePrefetch', type = 'ext.data.events.Store_operationEvent')]
/**
 * Fired whenever the filter set changes.
 * @eventType ext.data.events.Store_filtersEvent.FILTER_CHANGE
 */
[Event(name = 'onFilterChange', type = 'ext.data.events.Store_filtersEvent')]
/**
 * Fired whenever the grouping in the grid changes.
 * @eventType ext.data.events.Store_grouperEvent.GROUP_CHANGE
 */
[Event(name = 'onGroupChange', type = 'ext.data.events.Store_grouperEvent')]
/**
 * Fires whenever records have been prefetched.
 * @eventType ext.data.events.Store_records_successful_operationEvent.PREFETCH
 */
[Event(name = 'onPrefetch', type = 'ext.data.events.Store_records_successful_operationEvent')]
/**
 * The Store class encapsulates a client side cache of <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> objects. Stores load data via a
 * <a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">Proxy</a>, and also provide functions for <a href="#!/api/Ext.data.Store-method-sort" rel="Ext.data.Store-method-sort" class="docClass">sorting</a>, <a href="#!/api/Ext.data.Store-method-filter" rel="Ext.data.Store-method-filter" class="docClass">filtering</a>
 * and querying the <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">model</a> instances contained within it.
 *
 * <p>Creating a Store is easy - we just tell it the Model and the Proxy to use for loading and saving its data:</p>
 *
 * <pre><code> // Set up a model to use in our Store
 *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *      fields: [
 *          {name: 'firstName', type: 'string'},
 *          {name: 'lastName',  type: 'string'},
 *          {name: 'age',       type: 'int'},
 *          {name: 'eyeColor',  type: 'string'}
 *      ]
 *  });
 *
 *  var myStore = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *      model: 'User',
 *      proxy: {
 *          type: 'ajax',
 *          url: '/users.json',
 *          reader: {
 *              type: 'json',
 *              rootProperty: 'users'
 *          }
 *      },
 *      autoLoad: true
 *  });
 * </code></pre>
 *
 * <p>In the example above we configured an AJAX proxy to load data from the url '/users.json'. We told our Proxy to use a
 * <a href="#!/api/Ext.data.reader.Json" rel="Ext.data.reader.Json" class="docClass">JsonReader</a> to parse the response from the server into Model object - <a href="#!/api/Ext.data.reader.Json" rel="Ext.data.reader.Json" class="docClass">see the docs on JsonReader</a> for details.</p>
 *
 * <h2>Inline data</h2>
 *
 * <p>Stores can also load data inline. Internally, Store converts each of the objects we pass in as <a href="#!/api/Ext.data.Store-cfg-data" rel="Ext.data.Store-cfg-data" class="docClass">data</a> into
 * Model instances:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *      model: 'User',
 *      data : [
 *          {firstName: 'Peter',   lastName: 'Venkman'},
 *          {firstName: 'Egon',    lastName: 'Spengler'},
 *          {firstName: 'Ray',     lastName: 'Stantz'},
 *          {firstName: 'Winston', lastName: 'Zeddemore'}
 *      ]
 *  });
 * </code></pre>
 *
 * <p>Loading inline data using the method above is great if the data is in the correct format already (e.g. it doesn't
 * need to be processed by a <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">reader</a>). If your inline data requires processing to decode
 * the data structure, use a <a href="#!/api/Ext.data.proxy.Memory" rel="Ext.data.proxy.Memory" class="docClass">MemoryProxy</a> instead (see the <a href="#!/api/Ext.data.proxy.Memory" rel="Ext.data.proxy.Memory" class="docClass">MemoryProxy</a> docs for an example).</p>
 *
 * <p>Additional data can also be loaded locally using <a href="#!/api/Ext.data.Store-method-add" rel="Ext.data.Store-method-add" class="docClass">add</a>.</p>
 *
 * <h2>Dynamic Loading</h2>
 *
 * <p>Stores can be dynamically updated by calling the <a href="#!/api/Ext.data.Store-method-load" rel="Ext.data.Store-method-load" class="docClass">load</a> method:</p>
 *
 * <pre><code>store.load({
 *     params: {
 *         group: 3,
 *         type: 'user'
 *     },
 *     callback: function(records, operation, success) {
 *         // do something after the load finishes
 *     },
 *     scope: this
 * });
 * </code></pre>
 *
 * <p>Here a bunch of arbitrary parameters is passed along with the load request and a callback function is set
 * up to do something after the loading is over.</p>
 *
 * <h2>Loading Nested Data</h2>
 *
 * <p>Applications often need to load sets of associated data - for example a CRM system might load a User and her Orders.
 * Instead of issuing an AJAX request for the User and a series of additional AJAX requests for each Order, we can load
 * a nested dataset and allow the Reader to automatically populate the associated models. Below is a brief example, see
 * the <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Ext.data.reader.Reader</a> intro docs for a full explanation:</p>
 *
 * <pre><code> var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *      autoLoad: true,
 *      model: "User",
 *      proxy: {
 *          type: 'ajax',
 *          url: 'users.json',
 *          reader: {
 *              type: 'json',
 *              rootProperty: 'users'
 *          }
 *      }
 *  });
 * </code></pre>
 *
 * <p>Which would consume a response like this:</p>
 *
 * <pre><code> {
 *      "users": [{
 *          "id": 1,
 *          "name": "Peter",
 *          "orders": [{
 *              "id": 10,
 *              "total": 10.76,
 *              "status": "invoiced"
 *         },{
 *              "id": 11,
 *              "total": 13.45,
 *              "status": "shipped"
 *         }]
 *      }]
 *  }
 * </code></pre>
 *
 * <p>See the <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Ext.data.reader.Reader</a> intro docs for a full explanation.</p>
 *
 * <h2>Filtering and Sorting</h2>
 *
 * <p>Stores can be sorted and filtered - in both cases either remotely or locally. The <a href="#!/api/Ext.data.Store-cfg-sorters" rel="Ext.data.Store-cfg-sorters" class="docClass">sorters</a> and
 * <a href="#!/api/Ext.data.Store-cfg-filters" rel="Ext.data.Store-cfg-filters" class="docClass">filters</a> are held inside <a href="#!/api/Ext.util.Collection" rel="Ext.util.Collection" class="docClass">Collection</a> instances to make them easy to manage.
 * Usually it is sufficient to either just specify sorters and filters in the Store configuration or call <a href="#!/api/Ext.data.Store-method-sort" rel="Ext.data.Store-method-sort" class="docClass">sort</a>
 * or <a href="#!/api/Ext.data.Store-method-filter" rel="Ext.data.Store-method-filter" class="docClass">filter</a>:</p>
 *
 * <pre><code> var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *      model: 'User',
 *      sorters: [{
 *          property: 'age',
 *          direction: 'DESC'
 *      }, {
 *          property: 'firstName',
 *          direction: 'ASC'
 *      }],
 *
 *      filters: [{
 *          property: 'firstName',
 *          value: /Peter/
 *      }]
 *  });
 * </code></pre>
 *
 * <p>The new Store will keep the configured sorters and filters in the Collection instances mentioned above. By
 * default, sorting and filtering are both performed locally by the Store - see <a href="#!/api/Ext.data.Store-cfg-remoteSort" rel="Ext.data.Store-cfg-remoteSort" class="docClass">remoteSort</a> and
 * <a href="#!/api/Ext.data.Store-cfg-remoteFilter" rel="Ext.data.Store-cfg-remoteFilter" class="docClass">remoteFilter</a> to allow the server to perform these operations instead.</p>
 *
 * <p>Filtering and sorting after the Store has been instantiated is also easy. Calling <a href="#!/api/Ext.data.Store-method-filter" rel="Ext.data.Store-method-filter" class="docClass">filter</a> adds another filter
 * to the Store and automatically filters the dataset (calling <a href="#!/api/Ext.data.Store-method-filter" rel="Ext.data.Store-method-filter" class="docClass">filter</a> with no arguments simply re-applies all
 * existing filters).</p>
 *
 * <pre><code>store.filter('eyeColor', 'Brown');
 * </code></pre>
 *
 * <p>Change the sorting at any time by calling <a href="#!/api/Ext.data.Store-method-sort" rel="Ext.data.Store-method-sort" class="docClass">sort</a>:</p>
 *
 * <pre><code>store.sort('height', 'ASC');
 * </code></pre>
 *
 * <p>Note that all existing sorters will be removed in favor of the new sorter data (if <a href="#!/api/Ext.data.Store-method-sort" rel="Ext.data.Store-method-sort" class="docClass">sort</a> is called with no
 * arguments, the existing sorters are just reapplied instead of being removed). To keep existing sorters and add new
 * ones, just add them to the Collection:</p>
 *
 * <pre><code>store.sorters.add(new <a href="#!/api/Ext.util.Sorter" rel="Ext.util.Sorter" class="docClass">Ext.util.Sorter</a>({
 *     property : 'shoeSize',
 *     direction: 'ASC'
 * }));
 *
 * store.sort();
 * </code></pre>
 *
 * <h2>Registering with StoreManager</h2>
 *
 * <p>Any Store that is instantiated with a <a href="#!/api/Ext.data.Store-cfg-storeId" rel="Ext.data.Store-cfg-storeId" class="docClass">storeId</a> will automatically be registered with the <a href="#!/api/Ext.data.StoreManager" rel="Ext.data.StoreManager" class="docClass">StoreManager</a>. This makes it easy to reuse the same store in multiple views:</p>
 *
 * <pre><code>//this store can be used several times
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     model: 'User',
 *     storeId: 'usersStore'
 * });
 *
 * new Ext.List({
 *     store: 'usersStore',
 *     //other config goes here
 * });
 *
 * new <a href="#!/api/Ext.view.View" rel="Ext.view.View" class="docClass">Ext.view.View</a>({
 *     store: 'usersStore',
 *     //other config goes here
 * });
 * </code></pre>
 *
 * <h2>Further Reading</h2>
 *
 * <p>Stores are backed up by an ecosystem of classes that enables their operation. To gain a full understanding of these
 * pieces and how they fit together, see:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">Proxy</a> - overview of what Proxies are and how they are used</li>
 * <li><a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> - the core class in the data package</li>
 * <li><a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Reader</a> - used by any subclass of <a href="#!/api/Ext.data.proxy.Server" rel="Ext.data.proxy.Server" class="docClass">ServerProxy</a> to read a response</li>
 * </ul>
 */
public class Store extends ext.data.ProxyStore implements ext.data.ILocalStore {
  /**
   * Creates the store.
   * @param config Config object.
   */
  public function Store(config:ext.data.Store = null) {
    super();
  }

  /**
   * The number of times records have been loaded into the store. This includes loads via
   * <a href="#!/api/Ext.data.Store-method-loadData" rel="Ext.data.Store-method-loadData" class="docClass">loadData</a> &amp; <a href="#!/api/Ext.data.Store-method-loadRecords" rel="Ext.data.Store-method-loadRecords" class="docClass">loadRecords</a>.
   * @default 0
   */
  public native function get loadCount():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to empty the store when loading another page via <a href="#!/api/Ext.data.Store-method-loadPage" rel="Ext.data.Store-method-loadPage" class="docClass">loadPage</a>,
   * <a href="#!/api/Ext.data.Store-method-nextPage" rel="Ext.data.Store-method-nextPage" class="docClass">nextPage</a> or <a href="#!/api/Ext.data.Store-method-previousPage" rel="Ext.data.Store-method-previousPage" class="docClass">previousPage</a>. Setting to false keeps existing records, allowing
   * large data sets to be loaded one page at a time but rendered all together.
   * @default true
   * @see #getClearOnPageLoad()
   * @see #setClearOnPageLoad()
   */
  public native function get clearOnPageLoad():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set clearOnPageLoad(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to clear anything in the <a href="#!/api/Ext.data.Store-property-removed" rel="Ext.data.Store-property-removed" class="docClass">removed</a> record collection when the store loads.
   * @default true
   * @see #getClearRemovedOnLoad()
   * @see #setClearRemovedOnLoad()
   */
  public native function get clearRemovedOnLoad():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set clearRemovedOnLoad(value:Boolean):void;

  [ExtConfig]
  /**
   * Array of Model instances or data objects to load locally. See "Inline data"
   * above for details.
   * @default 0
   * @see #setData()
   */
  public native function get data():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set data(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The session for this store. By specifying a session, it ensures any records that are
   * added to this store are also included in the session. This store does not become a member
   * of the session itself.
   * @since 5.0.0
   * @see #getSession()
   * @see #setSession()
   */
  public native function get session():ext.data.Session;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set session(value:ext.data.Session):void;

  /**
   * Removes all items from the store.
   *
   * <p>Individual record <code><a href="#!/api/Ext.data.Store-event-remove" rel="Ext.data.Store-event-remove" class="docClass">remove</a></code> events are not fired by this method.</p>
   * @return The removed records.
   */
  override public native function removeAll():void;

  /**
   * (Local sort only) Inserts the passed Record into the Store at the index where it
   * should go based on the current sort information.
   */
  public native function addSorted(record:ext.data.Model):void;

  /**
   * Collects unique values for a particular dataIndex from this store.
   *
   * <p>Note that the <code>filtered</code> option can also be passed as a separate parameter for
   * compatibility with previous versions.</p>
   *
   * <pre><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
   *     fields: ['name'],
   *     data: [{
   *         name: 'Larry'
   *     }, {
   *         name: 'Darryl'
   *     }, {
   *         name: 'Darryl'
   *     }]
   * });
   *
   * store.collect('name');
   * // returns ["Larry", "Darryl"]
   * </code></pre>
   * @param property The property to collect
   * @param includeOptions An object which contains options which modify how the store is traversed.
   * @param {Boolean} [includeOptions.allowNull] Pass true to allow null, undefined or empty string values.
   * @param {Boolean} [includeOptions.filtered] Pass <code>true</code> to collect from all records, even ones which are filtered.
   * @return An array of the unique values
   */
  public native function collect(property:String, includeOptions:Object = null):Array;

  /**
   * Commits all Records with <a href="#!/api/Ext.data.Store-method-getModifiedRecords" rel="Ext.data.Store-method-getModifiedRecords" class="docClass">outstanding changes</a>. To handle updates for changes,
   * subscribe to the Store's <a href="#!/api/Ext.data.Store-event-update" rel="Ext.data.Store-event-update" class="docClass">update event</a>, and perform updating when the third parameter is
   * <a href="#!/api/Ext.data.Model-static-property-COMMIT" rel="Ext.data.Model-static-property-COMMIT" class="docClass">Ext.data.Record.COMMIT</a>.
   */
  public native function commitChanges():void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Store-cfg-clearOnPageLoad" rel="Ext.data.Store-cfg-clearOnPageLoad" class="docClass">clearOnPageLoad</a>.
   */
  public native function getClearOnPageLoad():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Store-cfg-clearRemovedOnLoad" rel="Ext.data.Store-cfg-clearRemovedOnLoad" class="docClass">clearRemovedOnLoad</a>.
   */
  public native function getClearRemovedOnLoad():Boolean;

  /**
   * Returns the store's records.
   *
   * <p><strong>Note:</strong> If your store has been filtered, getData() will return a filtered
   * collection.  Use <code>getData().<a href="#!/api/Ext.util.Collection-method-getSource" rel="Ext.util.Collection-method-getSource" class="docClass">getSource()</a></code> to
   * fetch all unfiltered records.</p>
   * @return An <a href="#!/api/Ext.util.Collection" rel="Ext.util.Collection" class="docClass">Ext.util.Collection</a> of records
   * (an empty Collection if no records are held by the store).
   */
  public native function getData():ext.util.Collection;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Store-cfg-session" rel="Ext.data.Store-cfg-session" class="docClass">session</a>.
   * @since 5.0.0
   */
  public native function getSession():ext.data.Session;

  /**
   * Loads an array of data straight into the Store.
   *
   * <p>Using this method is great if the data is in the correct format already (e.g. it doesn't need to be
   * processed by a reader). If your data requires processing to decode the data structure, use a
   * <a href="#!/api/Ext.data.proxy.Memory" rel="Ext.data.proxy.Memory" class="docClass">MemoryProxy</a> or <a href="#!/api/Ext.data.Store-method-loadRawData" rel="Ext.data.Store-method-loadRawData" class="docClass">loadRawData</a>.</p>
   * @param data Array of data to load. Any non-model instances will be cast
   * into model instances first.
   * @param append <code>true</code> to add the records to the existing records in the store, <code>false</code>
   * to remove the old ones first.
   * @default false
   */
  public native function loadData(data:Object, append:Boolean = false):void;

  /**
   * Loads a given 'page' of data by setting the start and limit values appropriately. Internally this just causes a normal
   * load operation, passing in calculated 'start' and 'limit' params.
   * @param page The number of the page to load.
   * @param options See options for <a href="#!/api/Ext.data.Store-method-load" rel="Ext.data.Store-method-load" class="docClass">load</a>.
   */
  public native function loadPage(page:Number, options:Object = null):void;

  /**
   * Loads data via the bound Proxy's reader
   *
   * <p>Use this method if you are attempting to load data and want to utilize the configured data reader.</p>
   *
   * <p>As of 4.2, this method will no longer fire the <a href="#!/api/Ext.data.Store-event-load" rel="Ext.data.Store-event-load" class="docClass">load</a> event.</p>
   * @param data The full JSON object you'd like to load into the Data store.
   * @param append <code>true</code> to add the records to the existing records in the store, <code>false</code>
   * to remove the old ones first.
   * @default false
   * @return <code>true</code> if the reader processed the records correctly. See <a href="#!/api/Ext.data.reader.Reader-cfg-successProperty" rel="Ext.data.reader.Reader-cfg-successProperty" class="docClass">Ext.data.reader.Reader.successProperty</a>.
   * If the reader did not process the records, nothing will be added.
   */
  public native function loadRawData(data:Array, append:Boolean = false):Boolean;

  /**
   * Loads an array of <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">model</a> instances into the store, fires the datachanged event. This should only usually
   * be called internally when loading from the <a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">Proxy</a>, when adding records manually use <a href="#!/api/Ext.data.Store-method-add" rel="Ext.data.Store-method-add" class="docClass">add</a> instead
   * @param records The array of records to load
   * @param options
   * @param {Boolean} [options.addRecords] Pass <code>true</code> to add these records to the existing records, <code>false</code> to remove the Store's existing records first.
   * @default false
   */
  public native function loadRecords(records:Array, options:Object):void;

  /**
   * Loads the next 'page' in the current data set
   * @param options See options for <a href="#!/api/Ext.data.Store-method-load" rel="Ext.data.Store-method-load" class="docClass">load</a>
   */
  public native function nextPage(options:Object):void;

  /**
   * Loads the previous 'page' in the current data set
   * @param options See options for <a href="#!/api/Ext.data.Store-method-load" rel="Ext.data.Store-method-load" class="docClass">load</a>
   */
  public native function previousPage(options:Object):void;

  /**
   * <a href="#!/api/Ext.data.Model-method-reject" rel="Ext.data.Model-method-reject" class="docClass">Rejects</a> outstanding changes on all <a href="#!/api/Ext.data.Store-method-getModifiedRecords" rel="Ext.data.Store-method-getModifiedRecords" class="docClass">modified records</a>
   * and re-insert any records that were removed locally. Any phantom records will be removed.
   */
  public native function rejectChanges():void;

  /**
   * Removes the specified record(s) from the Store, firing the <a href="#!/api/Ext.data.Store-event-remove" rel="Ext.data.Store-event-remove" class="docClass">remove</a>
   * event for the removed records.
   *
   * <p>After all records have been removed a single <code>datachanged</code> is fired.</p>
   * @param records Model instance or
   * array of instances to remove or an array of indices from which to remove records.
   * @param isMove (private)
   * @param silent (private)
   */
  public native function remove(records:*, isMove:Boolean = false, silent:Boolean = false):void;

  /**
   * Removes the model instance(s) at the given index
   * @param index The record index
   * @param count The number of records to delete
   * @default 1
   */
  public native function removeAt(index:Number, count:Number = 1):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Store-cfg-clearOnPageLoad" rel="Ext.data.Store-cfg-clearOnPageLoad" class="docClass">clearOnPageLoad</a>.
   * @param clearOnPageLoad The new value.
   */
  public native function setClearOnPageLoad(clearOnPageLoad:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Store-cfg-clearRemovedOnLoad" rel="Ext.data.Store-cfg-clearRemovedOnLoad" class="docClass">clearRemovedOnLoad</a>.
   * @param clearRemovedOnLoad The new value.
   */
  public native function setClearRemovedOnLoad(clearRemovedOnLoad:Boolean):void;

  /**
   * Loads an array of data directly into the Store.
   *
   * <p>setData() is ideal if your data's format is already in its appropriate format (e.g. it doesn't need to be
   * processed by a reader). If your data's structure requires processing, use a
   * <a href="#!/api/Ext.data.proxy.Memory" rel="Ext.data.proxy.Memory" class="docClass">MemoryProxy</a> or <a href="#!/api/Ext.data.Store-method-loadRawData" rel="Ext.data.Store-method-loadRawData" class="docClass">loadRawData</a>.</p>
   *
   * <p>Use <a href="#!/api/Ext.data.Store-method-loadData" rel="Ext.data.Store-method-loadData" class="docClass">loadData</a>, <a href="#!/api/Ext.data.Store-method-add" rel="Ext.data.Store-method-add" class="docClass">add</a>, or <a href="#!/api/Ext.data.Store-method-insert" rel="Ext.data.Store-method-insert" class="docClass">insert</a> if records need to be
   * appended to the current recordset.</p>
   * @param data Array of data to load. Any non-model instances will be cast
   * into model instances first.
   */
  public native function setData(data:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Store-cfg-session" rel="Ext.data.Store-cfg-session" class="docClass">session</a>.
   * @since 5.0.0
   * @param session The new value.
   */
  public native function setSession(session:ext.data.Session):void;

  /**
   * @inheritDoc
   */
  public native function add(...model):Array;

  /**
   * @inheritDoc
   */
  public native function aggregate(fn:Function, scope:Object = null, grouped:Boolean = false, field:String = null):Object;

  /**
   * @inheritDoc
   */
  public native function average(field:String, grouped:Boolean = false):Object;

  /**
   * @inheritDoc
   */
  public native function count(grouped:Boolean = false):Number;

  /**
   * @inheritDoc
   */
  public native function each(fn:Function, scope:Object = null, includeOptions:Object = null):void;

  /**
   * @inheritDoc
   */
  public native function first(grouped:Boolean = false):*;

  /**
   * @inheritDoc
   */
  public native function indexOf(record:ext.data.Model):Number;

  /**
   * @inheritDoc
   */
  public native function indexOfId(id:String):Number;

  /**
   * @inheritDoc
   */
  public native function insert(index:Number, records:*):Array;

  /**
   * @inheritDoc
   */
  public native function last(grouped:Boolean = false):*;

  /**
   * @inheritDoc
   */
  public native function max(field:String, grouped:Boolean = false):Object;

  /**
   * @inheritDoc
   */
  public native function min(field:String, grouped:Boolean = false):Object;

  /**
   * @inheritDoc
   */
  public native function query(property:String, value:*, anyMatch:Boolean = false, caseSensitive:Boolean = false, exactMatch:Boolean = false):ext.util.Collection;

  /**
   * @inheritDoc
   */
  public native function queryBy(fn:Function, scope:Object = null):ext.util.Collection;

  /**
   * @inheritDoc
   */
  public native function sum(field:String, grouped:Boolean = false):Number;

  /**
   * Called internally when a Proxy has completed a load request
   */
  protected native function onProxyLoad(operation:Object):void;
}
}