package ext.data.field {
import ext.Base;
import ext.ExtClass;
import ext.data.DataErrorCollection;
import ext.data.Model;
import ext.mixin.IFactoryable;

[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.field.Field", require)]
[ExtConfig]
/**
 * Fields are used to define the members of a Model. They aren't instantiated directly;
 * instead, when we create a class that extends <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>, it automatically
 * creates Field instances for each field configured in a <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a>.
 * For example, we might set up a model like this:
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: [
 *         'name', 'email',
 *         { name: 'age', type: 'int' },
 *         { name: 'gender', type: 'string', defaultValue: 'Unknown' }
 *     ]
 * });
 * </code></pre>
 *
 * <p>Four fields will have been created for the User Model - name, email, age and gender.
 * Note that we specified a couple of different formats here; if we only pass in the string
 * name of the field (as with name and email), the field is set up with the 'auto' type.
 * It's as if we'd done this instead:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: [
 *         { name: 'name', type: 'auto' },
 *         { name: 'email', type: 'auto' },
 *         { name: 'age', type: 'int' },
 *         { name: 'gender', type: 'string', defaultValue: 'Unknown' }
 *     ]
 * });
 * </code></pre>
 *
 * <h1>Field Types</h1>
 *
 * <p>Fields come in various types. When declaring a field, the <code>type</code> property is used to
 * specify the type of <code>Field</code> derived class used to manage values.</p>
 *
 * <p>The predefined set of types are:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.data.field.Field" rel="Ext.data.field.Field" class="docClass">auto</a> (Default, implies no conversion)</li>
 * <li><a href="#!/api/Ext.data.field.String" rel="Ext.data.field.String" class="docClass">string</a></li>
 * <li><a href="#!/api/Ext.data.field.Integer" rel="Ext.data.field.Integer" class="docClass">int</a></li>
 * <li><a href="#!/api/Ext.data.field.Number" rel="Ext.data.field.Number" class="docClass">number</a></li>
 * <li><a href="#!/api/Ext.data.field.Boolean" rel="Ext.data.field.Boolean" class="docClass">boolean</a></li>
 * <li><a href="#!/api/Ext.data.field.Date" rel="Ext.data.field.Date" class="docClass">date</a></li>
 * </ul>
 *
 *
 * <h1>Conversion</h1>
 *
 * <p>When reading fields it is often necessary to convert the values received before using
 * them or storing them in records. To handle these cases there is the
 * <code><a href="#!/api/Ext.data.field.Field-method-convert" rel="Ext.data.field.Field-method-convert" class="docClass">convert</a></code> method. This method is passed the received value (as
 * well as the current record instance, but see below) and it returns the value to carry
 * forward.</p>
 *
 * <p>For <code>auto</code> fields there is no <code><a href="#!/api/Ext.data.field.Field-method-convert" rel="Ext.data.field.Field-method-convert" class="docClass">convert</a></code> method. This is for
 * efficiency. For other field types, there are often <code>convert</code> methods. You can provide
 * a <code><a href="#!/api/Ext.data.field.Field-cfg-convert" rel="Ext.data.field.Field-cfg-convert" class="docClass">convert</a></code> config when the field is defined like this:</p>
 *
 * <pre><code> {
 *      name: 'timestamp',
 *
 *      convert: function (value) {
 *          return new Date(value);
 *      }
 *  }
 * </code></pre>
 *
 * <p>While this can be convenient, see below for details on defining Custom Types as that is
 * often a better practice and avoids repeating these functions.</p>
 *
 * <p>Note that when a <code>defaultValue</code> is specified, it will also be passed through to
 * <code>convert</code> (either to the <code><a href="#!/api/Ext.data.field.Field-method-convert" rel="Ext.data.field.Field-method-convert" class="docClass">convert</a></code> method or to the
 * <code><a href="#!/api/Ext.data.field.Field-cfg-convert" rel="Ext.data.field.Field-cfg-convert" class="docClass">convert</a> config)</code>.</p>
 *
 * <h2>Calculated Values</h2>
 *
 * <p>In some cases fields are the result of a calculation from other fields. Historically
 * this was a second role for <code><a href="#!/api/Ext.data.field.Field-method-convert" rel="Ext.data.field.Field-method-convert" class="docClass">convert</a></code> but that has some short
 * comings. The simpler solution is the <code><a href="#!/api/Ext.data.field.Field-cfg-calculate" rel="Ext.data.field.Field-cfg-calculate" class="docClass">calculate</a></code> config.</p>
 *
 * <p>Values produced by <code><a href="#!/api/Ext.data.field.Field-cfg-calculate" rel="Ext.data.field.Field-cfg-calculate" class="docClass">calculate</a></code> and <code><a href="#!/api/Ext.data.field.Field-method-convert" rel="Ext.data.field.Field-method-convert" class="docClass">convert</a></code>
 * are stored in the record as with any other field. In fact, if we define a calculated
 * "firstName" field and log out all of the data, we'll see this:</p>
 *
 * <pre><code>var ed = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('User', { name: 'Ed Spencer' });
 *
 * console.log(ed.data);
 *
 * //outputs this:
 * {
 *     age: 0,
 *     email: "",
 *     firstName: "Ed",  // calculated field
 *     gender: "Unknown",
 *     name: "Ed Spencer"
 * }
 * </code></pre>
 *
 * <h3>Using <code>calculate</code></h3>
 *
 * <pre><code> {
 *      name: 'firstName',
 *
 *      calculate: function (data) {
 *          return data.name.split(' ')[0];
 *      }
 *  }
 * </code></pre>
 *
 * <p>Using <code><a href="#!/api/Ext.data.field.Field-cfg-calculate" rel="Ext.data.field.Field-cfg-calculate" class="docClass">calculate</a></code> is the simplest and safest way to define a
 * calculated field. The most important part of this is that, internally, the code of the
 * supplied function is parsed to extract its dependencies. In this case, the "name" field
 * is the only dependency. This means that "firstName" will only need to be recalculated
 * when "name" is modified.</p>
 *
 * <p><strong>Note:</strong> Fields used by the calculate method must be explicitly defined in the
 * <a href="#!/api/Ext.data.Model-cfg-fields" rel="Ext.data.Model-cfg-fields" class="docClass">#fields</a> of the model.</p>
 *
 * <h3>Using <code>convert</code></h3>
 *
 * <p>Following is the equivalent technique using <code><a href="#!/api/Ext.data.field.Field-cfg-convert" rel="Ext.data.field.Field-cfg-convert" class="docClass">convert</a></code></p>
 *
 * <pre><code> {
 *      name: 'firstName',
 *
 *      convert: function (value, record) {
 *          return record.get('name').split(' ')[0];
 *      },
 *
 *      depends: [ 'name' ]
 *  }
 * </code></pre>
 *
 * <p>When a <code><a href="#!/api/Ext.data.field.Field-method-convert" rel="Ext.data.field.Field-method-convert" class="docClass">convert</a></code> function accepts a 2nd argument (a reference to
 * the record), it is considered a calculated field. If a <code><a href="#!/api/Ext.data.field.Field-cfg-depends" rel="Ext.data.field.Field-cfg-depends" class="docClass">depends</a></code>
 * config is not provided then this field's dependencies are unknown. In this case, the
 * <code><a href="#!/api/Ext.data.field.Field-cfg-depends" rel="Ext.data.field.Field-cfg-depends" class="docClass">depends</a></code> are provided as would be automatically determined with
 * the <code><a href="#!/api/Ext.data.field.Field-cfg-calculate" rel="Ext.data.field.Field-cfg-calculate" class="docClass">calculate</a></code> config.</p>
 *
 * <h3>Updating</h3>
 *
 * <p>Fields modified with the <a href="#!/api/Ext.data.Model-method-set" rel="Ext.data.Model-method-set" class="docClass">set</a> method will have their stored
 * value set using the convert / calculate method when present.</p>
 *
 * <p>For example:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.model.Employee', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: [{
 *         name: 'salary',
 *         convert: function (val) {
 *             var startingBonus = val * .1;
 *             return val + startingBonus;
 *         }
 *     }],
 *     convertOnSet: false
 * });
 *
 * var tina = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('MyApp.model.Employee', {
 *     salary: 50000
 * });
 *
 * console.log(tina.get('salary')); // logs 55000
 *
 * tina.set('salary', 60000);
 * console.log(tina.get('salary')); // logs 60000
 * </code></pre>
 *
 * <p>This default behavior can be disabled by setting the Model's
 * <code><a href="#!/api/Ext.data.Model-cfg-convertOnSet" rel="Ext.data.Model-cfg-convertOnSet" class="docClass">Ext.data.Model.convertOnSet</a></code> config to <code>false</code>.</p>
 *
 * <p><strong>Note:</strong> convertOnSet <code>false</code> only prevents the convert / calculate call when the
 * set <code>fieldName</code> param matches the field's <code><a href="#!/api/Ext.data.field.Field-cfg-name" rel="Ext.data.field.Field-cfg-name" class="docClass">name</a></code>.  See
 * <a href="#!/api/Ext.data.Model-cfg-convertOnSet" rel="Ext.data.Model-cfg-convertOnSet" class="docClass">convertOnSet</a> for additional details.</p>
 *
 * <h3>Dependencies</h3>
 *
 * <p>When a field's <code><a href="#!/api/Ext.data.field.Field-method-convert" rel="Ext.data.field.Field-method-convert" class="docClass">convert</a></code> method processes values from the record
 * (vs. just the field's value), it is best to also provide a <code>depends</code> config as shown
 * above. Fields that provide a <code><a href="#!/api/Ext.data.field.Field-cfg-calculate" rel="Ext.data.field.Field-cfg-calculate" class="docClass">calculate</a></code> method must follow the
 * proper form for using fields so that dependencies can be extracted.</p>
 *
 * <p>Calculated fields are processed after other fields based on their dependencies. Fields
 * with <code><a href="#!/api/Ext.data.field.Field-method-convert" rel="Ext.data.field.Field-method-convert" class="docClass">convert</a></code> methods that use the provided record that do <em>not</em>
 * specify a <code><a href="#!/api/Ext.data.field.Field-cfg-depends" rel="Ext.data.field.Field-cfg-depends" class="docClass">depends</a></code> config are processed as a group after all other
 * fields since such converters can rely on anything in the record. The order of processing
 * these fields with respect to each other is unspecified and should not be relied upon.</p>
 *
 * <h1>Serialization</h1>
 *
 * <p>To handle the inverse scenario of <code>convert</code> there is the <code>serialize</code> method. This
 * method is called to produce the value to send to a server based on the internal value
 * as would be returned from <code>convert</code>. In most cases, these methods should "round trip"
 * a value:</p>
 *
 * <pre><code> assertEqual(value, field.serialize(field.convert(value)));
 * </code></pre>
 *
 * <p>By default, only <code><a href="#!/api/Ext.data.field.Date" rel="Ext.data.field.Date" class="docClass">date</a></code> fields have a <code>serialize</code> method.
 * Other types simply send their value unmodified.</p>
 *
 * <h1>Custom Types</h1>
 *
 * <p>Developers may create their own application-specific data types by deriving from this
 * class. This is typically much better than applying multiple configuration values on
 * field instances as these often become repetitive.</p>
 *
 * <p>To illustrate, we define a "time" field type that stores a time-of-day represented as a
 * number of minutes since Midnight.</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.field.Time', {
 *      extend: '<a href="#!/api/Ext.data.field.Field" rel="Ext.data.field.Field" class="docClass">Ext.data.field.Field</a>',
 *
 *      alias: 'data.field.time',
 *
 *      timeFormat: 'g:i',
 *
 *      convert: function (value) {
 *          if (value &amp;&amp; <a href="#!/api/Ext-method-isString" rel="Ext-method-isString" class="docClass">Ext.isString</a>(value)) {
 *              var date = <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">Ext.Date.parse</a>(value, this.timeFormat);
 *              if (!date) {
 *                  return null;
 *              }
 *              return (date.getHours() - 1) * 60 + date.getMinutes();
 *          }
 *          return value;
 *      }
 *  });
 * </code></pre>
 *
 * <h2>Validation</h2>
 *
 * <p>Custom field types can override the <code><a href="#!/api/Ext.data.field.Field-method-validate" rel="Ext.data.field.Field-method-validate" class="docClass">validate</a></code> method or
 * provide a set of <code><a href="#!/api/Ext.data.field.Field-cfg-validators" rel="Ext.data.field.Field-cfg-validators" class="docClass">validators</a></code>.</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.field.PhoneNumber', {
 *      extend: '<a href="#!/api/Ext.data.field.Field" rel="Ext.data.field.Field" class="docClass">Ext.data.field.Field</a>',
 *
 *      alias: 'data.field.phonenumber',
 *
 *      // Match U.S. phone numbers for example purposes
 *      validators: {
 *          type: 'format',
 *          matcher: /\d{3}\-\d{3}\-\d{4}/
 *      }
 *  });
 * </code></pre>
 *
 * <p>Once the class is defined, fields can be declared using the new type (based on its
 * <code>alias</code>) like so:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.model.PhoneCall', {
 *      fields: [
 *          { name: 'startTime', type: 'time' },
 *          { name: 'phoneNumber', type: 'phonenumber' }
 *      ]
 *  });
 * </code></pre>
 */
public class DataField extends ext.Base implements ext.mixin.IFactoryable {
  /**
   * @param config @inheritDoc
   */
  public function DataField(config:ext.data.field.DataField = null) {
    super();
  }

  /**
   * The type of field.
   */
  public native function get type():String;

  /**
   * @private
   */
  public native function set type(value:String):void;

  /**
   * This property is <code>true</code> if this field has a <code><a href="#!/api/Ext.data.field.Field-cfg-calculate" rel="Ext.data.field.Field-cfg-calculate" class="docClass">calculate</a></code>
   * method or a <code><a href="#!/api/Ext.data.field.Field-method-convert" rel="Ext.data.field.Field-method-convert" class="docClass">convert</a></code> method that operates on the entire
   * record as opposed to just the data value. This property is determined from the
   * <code>length</code> of the <code><a href="#!/api/Ext.data.field.Field-method-convert" rel="Ext.data.field.Field-method-convert" class="docClass">convert</a></code> function which means this is
   * <em>not</em> calculated:
   *
   * <pre><code> convert: function (value) {
   *      return ...
   *  }
   * </code></pre>
   *
   * <p>While this <em>is</em> calculated:</p>
   *
   * <pre><code> convert: function (value, record) {
   *      return ...
   *  }
   * </code></pre>
   *
   * <p><strong>NOTE:</strong> It is recommended for such fields to use <code><a href="#!/api/Ext.data.field.Field-cfg-calculate" rel="Ext.data.field.Field-cfg-calculate" class="docClass">calculate</a></code>
   * or explicitly specify the fields used by <code><a href="#!/api/Ext.data.field.Field-method-convert" rel="Ext.data.field.Field-method-convert" class="docClass">convert</a></code> using
   * <code><a href="#!/api/Ext.data.field.Field-cfg-depends" rel="Ext.data.field.Field-cfg-depends" class="docClass">depends</a></code>.</p>
   * @default false
   */
  public native function get calculated():Boolean;

  /**
   * The default message to present for an invalid field.
   * @default 'This field is invalid'
   * @since 5.0.0
   */
  public native function get defaultInvalidMessage():String;

  /**
   * @private
   */
  public native function set defaultInvalidMessage(value:String):void;

  /**
   * The class (derived from <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>) that defined this field.
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.models.Foo', {
   *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
   *
   *      fields: [
   *          { name: 'bar' }
   *      ],
   *      ...
   *  });
   *
   *  var barField = MyApp.models.Foo.getField('bar');
   *
   *  alert(barField.definedBy === MyApp.models.Foo); // alerts 'true'
   * </code></pre>
   *
   * <p>When a field is inherited, this value will reference the class that originally
   * defined the field.</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.models.Base', {
   *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
   *
   *      fields: [
   *          { name: 'foo' }
   *      ],
   *      ...
   *  });
   *
   *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.models.Derived', {
   *      extend: 'MyApp.models.Base',
   *
   *      fields: [
   *          { name: 'bar' }
   *      ],
   *      ...
   *  });
   *
   *  var fooField = MyApp.models.Derived.getField('foo');
   *
   *  alert(fooField.definedBy === MyApp.models.Base); // alerts 'true'
   * </code></pre>
   */
  public native function get definedBy():ext.ExtClass;

  /**
   * @private
   */
  public native function set definedBy(value:ext.ExtClass):void;

  /**
   * This property is set to <code>true</code> if this is an <a href="#!/api/Ext.data.Model-cfg-idProperty" rel="Ext.data.Model-cfg-idProperty" class="docClass">id</a>
   * field.
   * @default false
   */
  public native function get identifier():Boolean;

  /**
   * The position of this field in the <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> in which it was defined.
   */
  public native function get ordinal():Number;

  /**
   * @private
   */
  public native function set ordinal(value:Number):void;

  /**
   * A regular expression for stripping non-numeric characters from a numeric value.
   * This should be overridden for localization.
   * @default /[\$,%]/g
   */
  public native function get stripRe():RegExp;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Use when converting received data into a <a href="#!/api/Ext.data.field.Integer" rel="Ext.data.field.Integer" class="docClass">&lt;code&gt;int&lt;/code&gt;</a>,
   * <a href="#!/api/Ext.data.field.Number" rel="Ext.data.field.Number" class="docClass">&lt;code&gt;float&lt;/code&gt;</a>, <a href="#!/api/Ext.data.field.Boolean" rel="Ext.data.field.Boolean" class="docClass">&lt;code&gt;bool&lt;/code&gt;</a>
   * or <a href="#!/api/Ext.data.field.String" rel="Ext.data.field.String" class="docClass">&lt;code&gt;string&lt;/code&gt;</a> type. If the value cannot be
   * parsed, <code>null</code> will be used if allowNull is true, otherwise a default value for that type will be used:
   *
   * <ul>
   * <li>for <code>int</code> and <code>float</code> - <code>0</code>.</li>
   * <li>for <code>string</code> - <code>""</code>.</li>
   * <li>for <code>bool</code> - <code>false</code>.</li>
   * </ul>
   *
   *
   * <p>Note that when parsing of <a href="#!/api/Ext.data.field.Date" rel="Ext.data.field.Date" class="docClass">&lt;code&gt;date&lt;/code&gt;</a> type fails, the value will
   * be <code>null</code> regardless of this setting.</p>
   * @default false
   * @see #getAllowNull()
   */
  public native function get allowNull():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowNull(value:Boolean):void;

  [ExtConfig]
  /**
   * This config defines a simple field calculation function. A calculate method only
   * has access to the record data and should return the value of the calculated field.
   * When provided in this way, the <code>depends</code> config is automatically determined by
   * parsing the <code>calculate</code> function. For example:
   *
   * <pre><code> fields: [{
   *      name: 'firstName',
   *      type: 'string'
   *  },{
   *      name: 'lastName',
   *      type: 'string'
   *  },{
   *      name: 'fullName',
   *      calculate: function (data) {
   *          return data.firstName + ' ' + data.lastName;
   *      }
   *  }]
   * </code></pre>
   *
   * <p>The above 'fullName' field is equivalent to:</p>
   *
   * <pre><code> {
   *      name: 'fullName',
   *      convert: function (v, rec) {
   *          return rec.get('firstName') + ' ' + rec.get('lastName');
   *      },
   *      depends: ['firstName', 'lastName']
   *  }
   * </code></pre>
   *
   * <p>The restrictions on form for a <code>calculate</code> method are that the accesses to field
   * values must match the following regular expression (case insensitive):</p>
   *
   * <pre><code> data.([a-z_][a-z0-9_]*)
   *  // where 'data' is the param passed to the calculate method
   * </code></pre>
   *
   * <p>The only advantage of a <code>calculate</code> method over a <code>convert</code> method is automatic
   * determination of <code>depends</code>.</p>
   *
   * <p><strong>Note:</strong> The use of calculate and <a href="#!/api/Ext.data.field.Field-method-convert" rel="Ext.data.field.Field-method-convert" class="docClass">convert</a> are exclusive.  The
   * calculate method will override the convert method if both are configured.</p>
   *
   * <p><strong>Note:</strong> Fields used by the calculate method must be explicitly defined in the
   * <a href="#!/api/Ext.data.Model-cfg-fields" rel="Ext.data.Model-cfg-fields" class="docClass">#fields</a> of the model.</p>
   */
  public native function get calculate():Function;

  [ExtConfig]
  /**
   * @private
   */
  public native function set calculate(value:Function):void;

  [ExtConfig("convert")]
  /**
   * If specified this config overrides the <code><a href="#!/api/Ext.data.field.Field-method-convert" rel="Ext.data.field.Field-method-convert" class="docClass">convert</a></code> method. See
   * also <code><a href="#!/api/Ext.data.field.Field-cfg-calculate" rel="Ext.data.field.Field-cfg-calculate" class="docClass">calculate</a></code> for simple field calculations.
   *
   * <p><strong>Note:</strong> The use of <a href="#!/api/Ext.data.field.Field-cfg-calculate" rel="Ext.data.field.Field-cfg-calculate" class="docClass">calculate</a> and convert are exclusive.  The calculate
   * method will override the convert method if both are configured.</p>
   */
  public native function get convert_():Function;

  [ExtConfig("convert")]
  /**
   * @private
   */
  public native function set convert_(value:Function):void;

  [ExtConfig]
  /**
   * A critical field is a field that must always be sent to the server even if it has
   * not changed. The most common example of such a field is the "id" of a record (see
   * <code><a href="#!/api/Ext.data.Model-cfg-idProperty" rel="Ext.data.Model-cfg-idProperty" class="docClass">Ext.data.Model.idProperty</a></code> but the <code><a href="#!/api/Ext.data.Model-cfg-versionProperty" rel="Ext.data.Model-cfg-versionProperty" class="docClass">Ext.data.Model.versionProperty</a></code>
   * is similarly a <code>critical</code> field.
   * @default false
   */
  public native function get critical():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set critical(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The default value used when the creating an instance from a raw data object,
   * and the property referenced by the <code><a href="#!/api/Ext.data.field.Field-cfg-mapping" rel="Ext.data.field.Field-cfg-mapping" class="docClass">mapping</a></code>
   * does not exist in that data object.
   *
   * <p>The value <code>undefined</code> prevents defaulting in a value.</p>
   * @default undefined
   * @see #getDefaultValue()
   */
  public native function get defaultValue():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultValue(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The field name or names within the <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> on which the value
   * of this field depends, and from which a new value may be calculated. These values
   * are the values used by the <code>convert</code> method. If you do not have a <code>convert</code> method
   * then this config should not be specified.
   *
   * <p>Before using this config you should consider if using a <code>calculate</code> method instead
   * of a <code>convert</code> method would be simpler.</p>
   *
   * <p>Whenever any of the named fields are set using the <a href="#!/api/Ext.data.Model-method-set" rel="Ext.data.Model-method-set" class="docClass">set</a>
   * method, this fields will have its <code>convert</code> method called passing the
   * <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">record</a> so that the dependent value can be calculated from
   * all fields which it needs.</p>
   *
   * <p>For example, to display a person's full name, using two separate <code>firstName</code> and
   * <code>lastName</code> fields, configure the name field like this:</p>
   *
   * <pre><code>{
   *     name: 'name',
   *
   *     // Will be called whenever forename or surname fields are set
   *     convert: function (v, rec) {
   *         return rec.get('firstName') + ' ' + rec.get('lastName');
   *     },
   *
   *     depends: [ 'firstName', 'lastName' ],
   *
   *     // It should not be returned to the server - it's not a database field
   *     persist: false
   * }
   * </code></pre>
   *
   * <p>Note that if you do not want the calculated field to be part of the field set sent
   * back to the server when the store is synchronized, you should configure the field
   * with <code>persist</code> set to <code>false</code>.</p>
   * @see #getDepends()
   */
  public native function get depends():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set depends(value:*):void;

  [ExtConfig]
  /**
   * (Optional) A path expression for use by the <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Ext.data.reader.Reader</a> implementation that is creating the
   * <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> to extract the Field value from the data object. If the path expression is the same
   * as the field name, the mapping may be omitted. A function may be passed to do complex data extraction. The examples
   * below are simple just to demonstrate the capability, typically, a function would not be used to extract such
   * simple data.
   *
   * <p>The form of the mapping expression depends on the Reader being used.</p>
   *
   * <ul>
   * <li><p><a href="#!/api/Ext.data.reader.Json" rel="Ext.data.reader.Json" class="docClass">Ext.data.reader.Json</a></p>
   *
   * <p>The mapping is a string containing the javascript expression to reference the data from an element of the data
   * item's <a href="#!/api/Ext.data.reader.Json-cfg-rootProperty" rel="Ext.data.reader.Json-cfg-rootProperty" class="docClass">rootProperty</a> Array. Defaults to the field name. If a function is passed,
   * a single argument is received which contains the raw json object:</p>
   *
   * <pre><code>// Server returns [{"name": "Foo", "age": 1}, {"name": "Bar", "age": 2}]
   * mapping: function(data) {
   *     return data.name;
   * }
   * </code></pre></li>
   * <li><p><a href="#!/api/Ext.data.reader.Xml" rel="Ext.data.reader.Xml" class="docClass">Ext.data.reader.Xml</a></p>
   *
   * <p>The mapping is an <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">Ext.DomQuery</a> path to the data item relative to the DOM element that represents the
   * <a href="#!/api/Ext.data.reader.Xml-cfg-record" rel="Ext.data.reader.Xml-cfg-record" class="docClass">record</a>. Defaults to the field name. If a function is passed, a single argument
   * is received which contains the record node:</p>
   *
   * <pre><code>// Server returns &lt;Root&gt;&lt;Person&gt;&lt;Name&gt;Foo&lt;/Name&gt;&lt;Age&gt;1&lt;/Age&gt;&lt;/Person&gt;&lt;Person&gt;&lt;Name&gt;Bar&lt;/Name&gt;&lt;Age&gt;2&lt;/Age&gt;&lt;/Person&gt;&lt;/Root&gt;
   * mapping: function(data) {
   *     return data.firstChild.textContent;
   * }
   * </code></pre></li>
   * <li><p><a href="#!/api/Ext.data.reader.Array" rel="Ext.data.reader.Array" class="docClass">Ext.data.reader.Array</a></p>
   *
   * <p>The mapping is a number indicating the Array index of the field's value. Defaults to the field specification's
   * Array position. If a function is passed, a single argument is received which contains the child array.</p>
   *
   * <pre><code>// Server returns [["Foo", 1], ["Bar", 2]]
   * mapping: function(data) {
   *     return data[0];
   * }
   * </code></pre></li>
   * </ul>
   *
   *
   * <p>If a more complex value extraction strategy is required, then configure the Field with a <a href="#!/api/Ext.data.field.Field-cfg-convert" rel="Ext.data.field.Field-cfg-convert" class="docClass">convert</a>
   * function. This is passed the whole row object, and may interrogate it in whatever way is necessary in order to
   * return the desired data.</p>
   * @see #getMapping()
   */
  public native function get mapping():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set mapping(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name by which the field is referenced within the Model. This is referenced by,
   * for example, the <code>dataIndex</code> property in column definition objects passed to
   * <a href="#!/api/Ext.grid.property.HeaderContainer" rel="Ext.grid.property.HeaderContainer" class="docClass">Ext.grid.property.HeaderContainer</a>.
   *
   * <p>Note: In the simplest case, if no properties other than <code>name</code> are required, a
   * field definition may consist of just a String for the field name.</p>
   * @see #getName()
   */
  public native function get name():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set name(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * False to exclude this field from the <a href="#!/api/Ext.data.Model-property-modified" rel="Ext.data.Model-property-modified" class="docClass">Ext.data.Model.modified</a> fields in a
   * record. This will also exclude the field from being written using a
   * <a href="#!/api/Ext.data.writer.Writer" rel="Ext.data.writer.Writer" class="docClass">Ext.data.writer.Writer</a>. This option is useful when fields are used to keep
   * state on the client but do not need to be persisted to the server.
   *
   * <p>Defaults to <code>false</code> for <code>calculated</code> fields and <code>true</code> otherwise.</p>
   * @see #getPersist()
   */
  public native function get persist():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set persist(value:Boolean):void;

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.data.Model-property-entityName" rel="Ext.data.Model-property-entityName" class="docClass">name</a> of the entity referenced by this field.
   * In most databases, this relationship is represented by a "foreign key". That is, a
   * value for such a field matches the value of the <a href="#!/api/Ext.data.Model-cfg-idProperty" rel="Ext.data.Model-cfg-idProperty" class="docClass">id</a>
   * for an entity of this type.
   *
   * <p>For further documentation, see <a href="#!/api/Ext.data.schema.Reference" rel="Ext.data.schema.Reference" class="docClass">Ext.data.schema.Reference</a>.</p>
   */
  public native function get reference():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set reference(value:*):void;

  [ExtConfig("serialize")]
  /**
   * A function which converts the Model's value for this Field into a form which can be used by whatever <a href="#!/api/Ext.data.writer.Writer" rel="Ext.data.writer.Writer" class="docClass">Writer</a>
   * is being used to sync data with the server.
   */
  public native function get serialize_():Function;

  [ExtConfig("serialize")]
  /**
   * @private
   */
  public native function set serialize_(value:Function):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A function which converts a Field's value to a comparable value in order to ensure
   * correct sort ordering.
   *
   * <p>Predefined functions are provided in <a href="#!/api/Ext.data.SortTypes" rel="Ext.data.SortTypes" class="docClass">Ext.data.SortTypes</a>. A custom sort example:</p>
   *
   * <pre><code>// current sort     after sort we want
   * // +-+------+          +-+------+
   * // |1|First |          |1|First |
   * // |2|Last  |          |3|Second|
   * // |3|Second|          |2|Last  |
   * // +-+------+          +-+------+
   *
   * sortType: function(value) {
   *    switch (value.toLowerCase()) // native toLowerCase():
   *    {
   *       case 'first': return 1;
   *       case 'second': return 2;
   *       default: return 3;
   *    }
   * }
   * </code></pre>
   *
   * <p>May also be set to a String value, corresponding to one of the named sort types in
   * <a href="#!/api/Ext.data.SortTypes" rel="Ext.data.SortTypes" class="docClass">Ext.data.SortTypes</a>.</p>
   * @see #getSortType()
   */
  public native function get sortType():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set sortType(value:*):void;

  [ExtConfig]
  /**
   * <code>true</code> if the value of this field is unique amongst all instances. When used with a
   * <code>reference</code> this describes a "one-to-one" relationship. It is almost always the case
   * that a <code>unique</code> field cannot also be <a href="#!/api/Ext.data.field.Field-cfg-allowBlank" rel="Ext.data.field.Field-cfg-allowBlank" class="docClass">nullable</a>.
   * @default false
   */
  public native function get unique():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set unique(value:Boolean):void;

  [ExtConfig]
  /**
   * An array of <a href="#!/api/Ext.data.validator.Validator" rel="Ext.data.validator.Validator" class="docClass">validators</a> for this field. These
   * <code>validators</code> will only be passed a field value to validate.
   */
  public native function get validators():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set validators(value:Array):void;

  /**
   * @inheritDoc
   */
  public native function get factoryConfig():Object;

  /**
   * @inheritDoc
   */
  public native function set factoryConfig(value:Object):void;

  /**
   * A function which converts the value provided by the Reader into the value that will
   * be stored in the record. This method can be overridden by a derived class or set as
   * a <code><a href="#!/api/Ext.data.field.Field-cfg-convert" rel="Ext.data.field.Field-cfg-convert" class="docClass">convert</a></code> config.
   *
   * <p>If configured as <code>null</code>, then no conversion will be applied to the raw data property
   * when this Field is read. This will increase performance. but you must ensure that
   * the data is of the correct type and does not <em>need</em> converting.</p>
   *
   * <p>Example of convert functions:</p>
   *
   * <pre><code>function fullName(v, record){
   *     return record.data.last + ', ' + record.data.first;
   * }
   *
   * function location(v, record){
   *     return !record.data.city ? '' : (record.data.city + ', ' + record.data.state);
   * }
   *
   * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Dude', {
   *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
   *     fields: [
   *         {name: 'fullname',  convert: fullName},
   *         {name: 'firstname', mapping: 'name.first'},
   *         {name: 'lastname',  mapping: 'name.last'},
   *         {name: 'city', defaultValue: 'unknown'},
   *         'state',
   *         {name: 'location',  convert: location}
   *     ]
   * });
   *
   * // create the data store
   * var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
   *     model: 'Dude',
   *     proxy: {
   *         type: 'memory',
   *         reader: {
   *             type: 'json',
   *             rootProperty: 'daRoot',
   *             totalProperty: 'total'
   *         }
   *     }
   * });
   *
   * var myData = [
   *     { key: 1,
   *       name: { first: 'Fat',    last:  'Albert' }
   *       // notice no city, state provided in data object
   *     },
   *     { key: 2,
   *       name: { first: 'Barney', last:  'Rubble' },
   *       city: 'Bedrock', state: 'Stoneridge'
   *     },
   *     { key: 3,
   *       name: { first: 'Cliff',  last:  'Claven' },
   *       city: 'Boston',  state: 'MA'
   *     }
   * ];
   * </code></pre>
   * @param value The data value as read by the Reader, if undefined will use
   * the configured <code>defaultValue</code>.
   * @param record The data object containing the Model as read so far
   * by the Reader. Note that the Model may not be fully populated at this point as the
   * fields are read in the order that they are defined.
   * <a href="#!/api/Ext.data.Model-cfg-fields" rel="Ext.data.Model-cfg-fields" class="docClass">fields</a> array.
   * @return The converted value for storage in the record.
   */
  public native function get convert():Function;

  [ExtConfig]
  /**
   * @private
   */
  public native function set convert(value:Function):void;

  /**
   * Compares two values to retrieve their relative position in sort order, taking into account
   * any <a href="#!/api/Ext.data.field.Field-cfg-sortType" rel="Ext.data.field.Field-cfg-sortType" class="docClass">sortType</a>. Also see <a href="#!/api/Ext.data.field.Field-method-compare" rel="Ext.data.field.Field-method-compare" class="docClass">compare</a>.
   * @param value1 The first value.
   * @param value2 The second value.
   * @return <code>-1</code> if <code>value1</code> is less than <code>value2</code>. <code>1</code> if <code>value1</code> is greater than <code>value2</code>.
   * <code>0</code> otherwise.
   */
  public native function collate(value1:Object, value2:Object):Number;

  /**
   * Compares two values to retrieve their relative position in sort order. Also see
   * <a href="#!/api/Ext.data.field.Field-method-collate" rel="Ext.data.field.Field-method-collate" class="docClass">collate</a>.
   * @param value1 The first value.
   * @param value2 The second value.
   * @return <code>-1</code> if <code>value1</code> is less than <code>value2</code>. <code>1</code> if <code>value1</code> is greater than <code>value2</code>.
   * <code>0</code> otherwise.
   */
  public native function compare(value1:Object, value2:Object):Number;

  /**
   * Gets allowBlank for this field. See <a href="#!/api/Ext.data.field.Field-cfg-allowBlank" rel="Ext.data.field.Field-cfg-allowBlank" class="docClass">allowBlank</a>.
   * @return allowBlank
   */
  public native function getAllowBlank():Boolean;

  /**
   * Gets allowNull for this field. See <a href="#!/api/Ext.data.field.Field-cfg-allowNull" rel="Ext.data.field.Field-cfg-allowNull" class="docClass">allowNull</a>.
   * @return allowNull
   */
  public native function getAllowNull():Boolean;

  /**
   * Gets converter for this field. See <a href="#!/api/Ext.data.field.Field-method-convert" rel="Ext.data.field.Field-method-convert" class="docClass">convert</a>.
   * @return convert
   */
  public native function getConvert():Function;

  /**
   * Gets the defaultValue for this field. See <a href="#!/api/Ext.data.field.Field-cfg-defaultValue" rel="Ext.data.field.Field-cfg-defaultValue" class="docClass">defaultValue</a>.
   * @return defaultValue
   */
  public native function getDefaultValue():Object;

  /**
   * Gets the depends for this field. See <a href="#!/api/Ext.data.field.Field-cfg-depends" rel="Ext.data.field.Field-cfg-depends" class="docClass">depends</a>.
   * @return depends
   */
  public native function getDepends():Array;

  /**
   * Get the mapping for this field. See <a href="#!/api/Ext.data.field.Field-cfg-mapping" rel="Ext.data.field.Field-cfg-mapping" class="docClass">mapping</a>.
   * @return mapping
   */
  public native function getMapping():Object;

  /**
   * Gets the name for this field. See <a href="#!/api/Ext.data.field.Field-cfg-name" rel="Ext.data.field.Field-cfg-name" class="docClass">name</a>.
   * @return name
   */
  public native function getName():String;

  /**
   * Gets the persist for this field. See <a href="#!/api/Ext.data.field.Field-cfg-persist" rel="Ext.data.field.Field-cfg-persist" class="docClass">persist</a>.
   * @return persist
   */
  public native function getPersist():Boolean;

  [Deprecated(message = "Setting sortDir and calling getSortDir were never applied by the\nthe Sorter.  This functionality does not natively exist on field instances.\n", since = "5.1")]
  /**
   * Gets the sortDir for this field.
   * @return sortDir
   */
  public native function getSortDir():String;

  /**
   * Gets the sortType for this field. See <a href="#!/api/Ext.data.field.Field-cfg-sortType" rel="Ext.data.field.Field-cfg-sortType" class="docClass">sortType</a>.
   * @return sortType
   */
  public native function getSortType():Function;

  /**
   * Gets a string representation of the type of this field.
   * @return type
   */
  public native function getType():String;

  /**
   * Checks if this field has a mapping applied.
   * @return <code>true</code> if this field has a mapping.
   */
  public native function hasMapping():Boolean;

  /**
   * Tests whether two values are equal based on this field type.
   * This uses the <a href="#!/api/Ext.data.field.Field-method-compare" rel="Ext.data.field.Field-method-compare" class="docClass">compare</a> method to determine equality, so
   * this method should generally not be overridden.
   * @param value1 The first value.
   * @param value2 The second value.
   * @return <code>true</code> if the values are equal.
   */
  public native function isEqual(value1:Object, value2:Object):Boolean;

  /**
   * A function which converts the Model's value for this Field into a form which can be used by whatever <a href="#!/api/Ext.data.writer.Writer" rel="Ext.data.writer.Writer" class="docClass">Writer</a>
   * is being used to sync data with the server.
   * @param value The Field's value - the value to be serialized.
   * @param record The record being serialized.
   * @return The string that represents the Field's value.
   */
  public native function serialize(value:*, record:ext.data.Model):String;

  /**
   * Validates the passed value for this field.
   * @since 5.0.0
   * @param value The value to validate.
   * @param separator This string is passed if the caller wants all validation
   * messages concatenated with this string between each. This can be handled as a
   * "falsy" value because concatenating with no separator is seldom desirable.
   * @param errors This parameter is passed if the caller
   * wants all validation results individually added to the collection.
   * @return <code>true</code> if the value is valid. A string may be returned if
   * the value is not valid, to indicate an error message. Any other non <code>true</code> value
   * indicates the value is not valid. This method is not implemented by default,
   * subclasses may override it to provide an implementation.
   */
  protected native function validate(value:Object, separator:String = null, errors:ext.data.DataErrorCollection = null):*;
}
}