package ext.data.proxy {
import ext.data.Batch;
import ext.data.operation.Operation;
import ext.mixin.IFactoryable;
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Native("Ext.data.proxy.Proxy", require)]
[ExtConfig]
/**
 * Fires when this proxy's reader provides new metadata. Metadata usually consists
 * of new field definitions, but can include any configuration data required by an
 * application, and can be processed as needed in the event handler.
 * This event is currently only fired for JsonReaders. Note that this event is also
 * propagated by <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>, which is typically where it would be handled.
 * @eventType ext.data.proxy.events.DataProxy_metaEvent.META_CHANGE
 */
[Event(name = 'onMetaChange', type = 'ext.data.proxy.events.DataProxy_metaEvent')]
/**
 * Proxies are used by <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Stores</a> to handle the loading and saving of <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a>
 * data. Usually developers will not need to create or interact with proxies directly.
 *
 * <h1>Types of Proxy</h1>
 *
 * <p>There are two main types of Proxy - <a href="#!/api/Ext.data.proxy.Client" rel="Ext.data.proxy.Client" class="docClass">Client</a> and <a href="#!/api/Ext.data.proxy.Server" rel="Ext.data.proxy.Server" class="docClass">Server</a>.
 * The Client proxies save their data locally and include the following subclasses:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.data.proxy.LocalStorage" rel="Ext.data.proxy.LocalStorage" class="docClass">LocalStorageProxy</a> - saves its data to localStorage if the browser supports it</li>
 * <li><a href="#!/api/Ext.data.proxy.SessionStorage" rel="Ext.data.proxy.SessionStorage" class="docClass">SessionStorageProxy</a> - saves its data to sessionStorage if the browsers supports it</li>
 * <li><a href="#!/api/Ext.data.proxy.Memory" rel="Ext.data.proxy.Memory" class="docClass">MemoryProxy</a> - holds data in memory only, any data is lost when the page is refreshed</li>
 * </ul>
 *
 *
 * <p>The Server proxies save their data by sending requests to some remote server. These proxies include:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">Ajax</a> - sends requests to a server on the same domain</li>
 * <li><a href="#!/api/Ext.data.proxy.JsonP" rel="Ext.data.proxy.JsonP" class="docClass">JsonP</a> - uses JSON-P to send requests to a server on a different domain</li>
 * <li><a href="#!/api/Ext.data.proxy.Rest" rel="Ext.data.proxy.Rest" class="docClass">Rest</a> - uses RESTful HTTP methods (GET/PUT/POST/DELETE) to communicate with server</li>
 * <li><a href="#!/api/Ext.data.proxy.Direct" rel="Ext.data.proxy.Direct" class="docClass">Direct</a> - uses <a href="#!/api/Ext.direct.Manager" rel="Ext.direct.Manager" class="docClass">Ext.direct.Manager</a> to send requests</li>
 * </ul>
 *
 *
 * <p>Proxies operate on the principle that all operations performed are either Create, Read, Update or Delete. These four
 * operations are mapped to the methods <a href="#!/api/Ext.data.proxy.Proxy-method-create" rel="Ext.data.proxy.Proxy-method-create" class="docClass">create</a>, <a href="#!/api/Ext.data.proxy.Proxy-method-read" rel="Ext.data.proxy.Proxy-method-read" class="docClass">read</a>, <a href="#!/api/Ext.data.proxy.Proxy-method-update" rel="Ext.data.proxy.Proxy-method-update" class="docClass">update</a> and <a href="#!/api/Ext.data.proxy.Proxy-method-erase" rel="Ext.data.proxy.Proxy-method-erase" class="docClass">erase</a>
 * respectively. Each Proxy subclass implements these functions.</p>
 *
 * <p>The CRUD methods each expect an <a href="#!/api/Ext.data.operation.Operation" rel="Ext.data.operation.Operation" class="docClass">Operation</a> object as the sole argument. The Operation
 * encapsulates information about the action the Store wishes to perform, the <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">model</a> instances
 * that are to be modified, etc. See the <a href="#!/api/Ext.data.operation.Operation" rel="Ext.data.operation.Operation" class="docClass">Operation</a> documentation for more details. Each CRUD
 * method also accepts a callback function to be called asynchronously on completion.</p>
 *
 * <p>Proxies also support batching of Operations via a <a href="#!/api/Ext.data.Batch" rel="Ext.data.Batch" class="docClass">batch</a> object, invoked by the <a href="#!/api/Ext.data.proxy.Proxy-method-batch" rel="Ext.data.proxy.Proxy-method-batch" class="docClass">batch</a>
 * method.</p>
 */
public class DataProxy extends ext.mixin.Observable implements ext.mixin.IFactoryable {
  /**
   * Creates the Proxy
   * @param config Config object.
   */
  public function DataProxy(config:ext.data.proxy.DataProxy = null) {
    super();
  }

  /**
   * If this property is specified by the target class of this mixin its properties are
   * used to configure the created <code><a href="#!/api/Ext.Factory" rel="Ext.Factory" class="docClass">Ext.Factory</a></code>.
   */
  public native function get factoryConfig():Object;

  /**
   * @private
   */
  public native function set factoryConfig(value:Object):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Proxy, or subclass thereof.
   * @default true
   */
  public native function get isProxy():Boolean;

  /**
   * @private
   */
  public native function set isProxy(value:Boolean):void;

  /**
   * Identifies the proxy as (a)synchronous.
   * @default false
   */
  public native function get isSynchronous():Boolean;

  /**
   * @private
   */
  public native function set isSynchronous(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to batch actions of a particular type when synchronizing the store. Defaults to true.
   * @default true
   * @see #getBatchActions()
   * @see #setBatchActions()
   */
  public native function get batchActions():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set batchActions(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Comma-separated ordering 'create', 'update' and 'destroy' actions when batching. Override this to set a different
   * order for the batched CRUD actions to be executed in. Defaults to 'create,update,destroy'.
   * @default 'create,update,destroy'
   * @see #getBatchOrder()
   * @see #setBatchOrder()
   */
  public native function get batchOrder():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set batchOrder(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the Model to tie to this Proxy. Can be either the string name of the Model, or a reference to the
   * Model constructor. Required.
   * @see #getModel()
   * @see #setModel()
   */
  public native function get model():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set model(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Ext.data.reader.Reader</a> to use to decode the server's response or data read
   * from client. This can either be a Reader instance, a config object or just a
   * valid Reader type name (e.g. 'json', 'xml').
   * @default {type: 'json'}
   * @see #getReader()
   * @see #setReader()
   */
  public native function get reader():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set reader(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The <a href="#!/api/Ext.data.writer.Writer" rel="Ext.data.writer.Writer" class="docClass">Ext.data.writer.Writer</a> to use to encode any request sent to the server or
   * saved to client. This can either be a Writer instance, a config object or just
   * a valid Writer type name (e.g. 'json', 'xml').
   * @default {type: 'json'}
   * @see #getWriter()
   * @see #setWriter()
   */
  public native function get writer():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set writer(value:*):void;

  /**
   * Performs a batch of <a href="#!/api/Ext.data.operation.Operation" rel="Ext.data.operation.Operation" class="docClass">Operations</a>, in the order specified by <a href="#!/api/Ext.data.proxy.Proxy-cfg-batchOrder" rel="Ext.data.proxy.Proxy-cfg-batchOrder" class="docClass">batchOrder</a>. Used
   * internally by <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>'s <a href="#!/api/Ext.data.Store-method-sync" rel="Ext.data.Store-method-sync" class="docClass">sync</a> method. Example usage:
   *
   * <pre><code>myProxy.batch({
   *     create : [myModel1, myModel2],
   *     update : [myModel3],
   *     destroy: [myModel4, myModel5]
   * });
   * </code></pre>
   *
   * <p>Where the myModel* above are <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> instances - in this case 1 and 2 are new instances and
   * have not been saved before, 3 has been saved previously but needs to be updated, and 4 and 5 have already been
   * saved but should now be destroyed.</p>
   *
   * <p>Note that the previous version of this method took 2 arguments (operations and listeners). While this is still
   * supported for now, the current signature is now a single <code>options</code> argument that can contain both operations and
   * listeners, in addition to other options. The multi-argument signature will likely be deprecated in a future release.</p>
   * @param options Object containing one or more properties supported by the batch method:
   * @param {Object} options.operations Object containing the Model instances to act upon, keyed by action name
   * @param {Object} [options.listeners] Event listeners object passed straight through to the Batch -
   * see <a href="#!/api/Ext.data.Batch" rel="Ext.data.Batch" class="docClass">Ext.data.Batch</a> for details
   * @param [options.batch] A <a href="#!/api/Ext.data.Batch" rel="Ext.data.Batch" class="docClass">Ext.data.Batch</a> object (or batch config to apply
   * to the created batch). If unspecified a default batch will be auto-created.
   * @param {Function} [options.callback] The function to be called upon completion of processing the batch.
   * The callback is called regardless of success or failure and is passed the following parameters:
   * @param {ext.data.Batch} options.callback.batch The <a href="#!/api/Ext.data.Batch" rel="Ext.data.Batch" class="docClass">batch</a> that was processed,
   * containing all operations in their current state after processing
   * @param {Object} options.callback.options The options argument that was originally passed into batch
   * @param {Function} [options.success] The function to be called upon successful completion of the batch. The
   * success function is called only if no exceptions were reported in any operations. If one or more exceptions
   * occurred then the <code>failure</code> function will be called instead. The success function is called
   * with the following parameters:
   * @param {ext.data.Batch} options.success.batch The <a href="#!/api/Ext.data.Batch" rel="Ext.data.Batch" class="docClass">batch</a> that was processed,
   * containing all operations in their current state after processing
   * @param {Object} options.success.options The options argument that was originally passed into batch
   * @param {Function} [options.failure] The function to be called upon unsuccessful completion of the batch. The
   * failure function is called when one or more operations returns an exception during processing (even if some
   * operations were also successful). In this case you can check the batch's <a href="#!/api/Ext.data.Batch-property-exceptions" rel="Ext.data.Batch-property-exceptions" class="docClass">exceptions</a> array to see exactly which operations had exceptions. The failure function is called with the
   * following parameters:
   * @param {ext.data.Batch} options.failure.batch The <a href="#!/api/Ext.data.Batch" rel="Ext.data.Batch" class="docClass">batch</a> that was processed,
   * containing all operations in their current state after processing
   * @param {Object} options.failure.options The options argument that was originally passed into batch
   * @param {Object} [options.scope] The scope in which to execute any callbacks (i.e. the <code>this</code> object inside
   * the callback, success and/or failure functions). Defaults to the proxy.
   * @return The newly created Batch
   */
  public native function batch(options:Object):ext.data.Batch;

  /**
   * Performs the given create operation.
   * @param operation The Operation to perform
   */
  public native function create(operation:ext.data.operation.Operation):void;

  /**
   * Performs the given destroy operation.
   * @param operation The Operation to perform
   */
  public native function erase(operation:ext.data.operation.Operation):void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Proxy-cfg-batchActions" rel="Ext.data.proxy.Proxy-cfg-batchActions" class="docClass">batchActions</a>.
   */
  public native function getBatchActions():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Proxy-cfg-batchOrder" rel="Ext.data.proxy.Proxy-cfg-batchOrder" class="docClass">batchOrder</a>.
   */
  public native function getBatchOrder():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Proxy-cfg-model" rel="Ext.data.proxy.Proxy-cfg-model" class="docClass">model</a>.
   */
  public native function getModel():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Proxy-cfg-reader" rel="Ext.data.proxy.Proxy-cfg-reader" class="docClass">reader</a>.
   */
  public native function getReader():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Proxy-cfg-writer" rel="Ext.data.proxy.Proxy-cfg-writer" class="docClass">writer</a>.
   */
  public native function getWriter():*;

  /**
   * Performs the given read operation.
   * @param operation The Operation to perform
   */
  public native function read(operation:ext.data.operation.Operation):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Proxy-cfg-batchActions" rel="Ext.data.proxy.Proxy-cfg-batchActions" class="docClass">batchActions</a>.
   * @param batchActions The new value.
   */
  public native function setBatchActions(batchActions:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Proxy-cfg-batchOrder" rel="Ext.data.proxy.Proxy-cfg-batchOrder" class="docClass">batchOrder</a>.
   * @param batchOrder The new value.
   */
  public native function setBatchOrder(batchOrder:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Proxy-cfg-model" rel="Ext.data.proxy.Proxy-cfg-model" class="docClass">model</a>.
   * @param model The new value.
   */
  public native function setModel(model:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Proxy-cfg-reader" rel="Ext.data.proxy.Proxy-cfg-reader" class="docClass">reader</a>.
   * @param reader The new value.
   */
  public native function setReader(reader:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Proxy-cfg-writer" rel="Ext.data.proxy.Proxy-cfg-writer" class="docClass">writer</a>.
   * @param writer The new value.
   */
  public native function setWriter(writer:*):void;

  /**
   * Performs the given update operation.
   * @param operation The Operation to perform
   */
  public native function update(operation:ext.data.operation.Operation):void;
}
}