package ext.data.proxy {
import ext.data.operation.Operation;

[Uses("ext.data.proxy.ServerProxy")]
[Uses("ext.data.proxy.DataProxy")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.proxy.Direct", require)]
/**
 * This class is used to send requests to the server using <a href="#!/api/Ext.direct.Manager" rel="Ext.direct.Manager" class="docClass">Ext Direct</a>.
 * When a request is made, the transport mechanism is handed off to the appropriate
 * <a href="#!/api/Ext.direct.RemotingProvider" rel="Ext.direct.RemotingProvider" class="docClass">Provider</a> to complete the call.
 *
 * <h1>Specifying the functions</h1>
 *
 * <p>This proxy expects Direct remoting method to be passed in order to be able to complete requests,
 * one Direct function per CRUD method. This is done via <a href="#!/api/Ext.data.proxy.Direct-cfg-api" rel="Ext.data.proxy.Direct-cfg-api" class="docClass">api</a> configuration:</p>
 *
 * <pre><code> api: {
 *      read: 'MyApp.readRecords',
 *      create: 'MyApp.createRecords',
 *      update: 'MyApp.updateRecords',
 *      destroy: 'MyApp.destroyRecords'
 *  }
 * </code></pre>
 *
 * <p>You can also use a <code>prefix</code> config to avoid duplicating full namespaces for Direct functions:</p>
 *
 * <pre><code> api: {
 *      prefix: 'MyApp',
 *      read: 'readRecords',
 *      create: 'createRecords',
 *      update: 'updateRecords',
 *      destroy: 'destroyRecords'
 *  }
 * </code></pre>
 *
 * <p>The preferred way is to specify function names to allow late resolution, however you can
 * pass function references instead if desired:</p>
 *
 * <pre><code> api: {
 *      read: MyApp.readRecords,
 *      create: MyApp.createRecords,
 *      update: MyApp.updateRecords,
 *      destroy: MyApp.destroyRecords
 *  }
 * </code></pre>
 *
 * <p>This method of configuring API is not recommended because this way the Direct functions
 * need to be created very early in the application lifecycle, long before <a href="#!/api/Ext.app.Application" rel="Ext.app.Application" class="docClass">Ext.app.Application</a>
 * instance is initialized.</p>
 *
 * <p>You can also use the <a href="#!/api/Ext.data.proxy.Direct-cfg-directFn" rel="Ext.data.proxy.Direct-cfg-directFn" class="docClass">directFn</a> configuration instead of <a href="#!/api/Ext.data.proxy.Direct-cfg-api" rel="Ext.data.proxy.Direct-cfg-api" class="docClass">api</a>. This will use
 * the same Direct function for all types of requests.</p>
 *
 * <h1>Server API</h1>
 *
 * <p>The server side methods are expected to conform to the following calling conventions:</p>
 *
 * <h2><code>read</code></h2>
 *
 * <p>Accept one argument which is either named arguments in an object (default), or an array
 * of values depending on the <a href="#!/api/Ext.data.proxy.Direct-cfg-paramsAsHash" rel="Ext.data.proxy.Direct-cfg-paramsAsHash" class="docClass">paramsAsHash</a> configuration. Return an array of records
 * or an object with format recognizable by the configured <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Ext.data.reader.Reader</a>
 * instance.</p>
 *
 * <p>Example Direct API declaration:</p>
 *
 * <pre><code> actions: {
 *      MyApp: [{
 *          name: 'readRecords',
 *          params: [],
 *          strict: false
 *      }]
 *  }
 * </code></pre>
 *
 * <p>Example function invocation:</p>
 *
 * <pre><code> MyApp.readRecords(
 *      {
 *          start: 0,
 *          limit: 10
 *      },
 *      // Results are passed to the callback function
 *      function(records) {
 *          console.log(records);
 *          // Logs:  [{ id: 'r0', text: 'foo' }, { id: 'r1', text: 'bar' }]
 *      }
 *  );
 * </code></pre>
 *
 * <h2><code>create</code></h2>
 *
 * <p>Accept one ordered argument which is either an object with data for the new record,
 * or an array of objects for multiple records. Return an array of identifiers for actually
 * created records. See <a href="#!/api/Ext.data.Model-cfg-clientIdProperty" rel="Ext.data.Model-cfg-clientIdProperty" class="docClass">Ext.data.Model.clientIdProperty</a> for more information.</p>
 *
 * <p>Example Direct API declaration:</p>
 *
 * <pre><code> actions: [
 *      MyApp: [{
 *          name: 'createRecords',
 *          len: 1
 *      }]
 *  }
 * </code></pre>
 *
 * <p>Example function invocation:</p>
 *
 * <pre><code> MyApp.createRecords(
 *      [
 *          { id: 0, text: 'foo' },
 *          { id: 1, text: 'bar' }
 *      ],
 *      // Results are passed to the callback function
 *      function(records) {
 *          console.log(records);
 *          // Logs: [{ clientId: 0, id: 'r0' }, { clientId: 1, id: 'r1' }]
 *      }
 *  );
 * </code></pre>
 *
 * <h2><code>update</code></h2>
 *
 * <p>Accept one ordered argument which is either an object with updated data and valid
 * record identifier, or an array of objects for multiple records. Return an array of
 * objects with updated record data.</p>
 *
 * <p>Example Direct API declaration:</p>
 *
 * <pre><code> actions: [
 *      MyApp: [{
 *          name: 'updateRecords',
 *          len: 1
 *      }]
 *  }
 * </code></pre>
 *
 * <p>Example function invocation:</p>
 *
 * <pre><code> MyApp.updateRecords(
 *      [
 *          { id: 'r0', text: 'blerg' },
 *          { id: 'r1', text: 'throbbe' }
 *      ],
 *      // Results are passed to the callback function
 *      function(records) {
 *          console.log(records);
 *          // Logs: [{ id: 'r0', text: 'blerg' }, { id: 'r1', text: 'throbbe }]
 *      }
 *  );
 * </code></pre>
 *
 * <h2><code>destroy</code></h2>
 *
 * <p>Accept one ordered argument which is an array of record identifiers to be deleted.
 * Return an object with at least one <a href="#!/api/Ext.data.reader.Json-cfg-successProperty" rel="Ext.data.reader.Json-cfg-successProperty" class="docClass">Ext.data.reader.Json.successProperty</a>
 * property set to <code>true</code> or <code>false</code>, with more optional properties recognizable by configured
 * <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Ext.data.reader.Reader</a> instance.</p>
 *
 * <p>Example Direct API declaration:</p>
 *
 * <pre><code> actions: [
 *      MyApp: [{
 *          name: 'destroyRecords',
 *          len: 1
 *      }]
 *  }
 * </code></pre>
 *
 * <p>Example function invocation:</p>
 *
 * <pre><code> MyApp.destroyRecords(
 *      [
 *          { id: 'r0' },
 *          { id: 'r1' }
 *      ],
 *      // Results are passed to the callback function
 *      function(result) {
 *          // Default successProperty is `success`
 *          if (!result.success) {
 *              // Handle the error
 *          }
 *      }
 *  );
 * </code></pre>
 *
 * <h2>Read method parameters</h2>
 *
 * <p>Direct proxy provides options to help configure which parameters will be sent to the server
 * for Read operations. By setting the <a href="#!/api/Ext.data.proxy.Direct-cfg-paramsAsHash" rel="Ext.data.proxy.Direct-cfg-paramsAsHash" class="docClass">paramsAsHash</a> option to <code>true</code>, the proxy will
 * send an object literal containing each of the passed parameters. This is the default. When
 * <a href="#!/api/Ext.data.proxy.Direct-cfg-paramsAsHash" rel="Ext.data.proxy.Direct-cfg-paramsAsHash" class="docClass">paramsAsHash</a> is set to <code>false</code>, Proxy will pass the Read function an array of values
 * instead of an object, with the order determined by <a href="#!/api/Ext.data.proxy.Direct-cfg-paramOrder" rel="Ext.data.proxy.Direct-cfg-paramOrder" class="docClass">paramOrder</a> value.</p>
 *
 * <p>Setting <a href="#!/api/Ext.data.proxy.Direct-cfg-paramOrder" rel="Ext.data.proxy.Direct-cfg-paramOrder" class="docClass">paramOrder</a> to any value other than <code>undefined</code> will automatically reset
 * <a href="#!/api/Ext.data.proxy.Direct-cfg-paramsAsHash" rel="Ext.data.proxy.Direct-cfg-paramsAsHash" class="docClass">paramsAsHash</a> to <code>false</code>.</p>
 *
 * <h1>Example Usage</h1>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *      fields: ['firstName', 'lastName']
 *  });
 *
 *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Users', {
 *      extend: '<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>',
 *      model: 'User',
 *      proxy: {
 *          type: 'direct',
 *          directFn: 'MyApp.getUsers',
 *          // Tells the proxy to pass `start` and `limit` as two by-position arguments:
 *          paramOrder: 'start,limit'
 *      }
 *  });
 *
 *  var store = new Users();
 *  store.load();
 * </code></pre>
 */
public class DirectProxy extends ext.data.proxy.ServerProxy {
  /**
   * @param config @inheritDoc
   */
  public function DirectProxy(config:ext.data.proxy.DirectProxy = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The same as <a href="#!/api/Ext.data.proxy.Server-cfg-api" rel="Ext.data.proxy.Server-cfg-api" class="docClass">Ext.data.proxy.Server.api</a>, however instead of providing urls
   * you should provide a Direct function name for each CRUD method.
   *
   * <p>Instead of providing fully qualified names for each function, you can use <code>prefix</code>
   * property to provide a common prefix for all functions:</p>
   *
   * <p>  api: {
   *       prefix: 'MyApp',
   *       read: 'readRecords',
   *       create: 'createRecords',
   *       update: 'updateRecords',
   *       destroy: 'destroyRecords'
   *   }</p>
   *
   * <p>This way function names will be resolved to <code>'MyApp.readRecords'</code>, <code>'MyApp.createRecords'</code>,
   * etc. Note that using <code>prefix</code> and fully qualified function names is <strong>not</strong> supported,
   * and prefix will be used for every function name when configured.</p>
   *
   * <p>See also <a href="#!/api/Ext.data.proxy.Direct-cfg-directFn" rel="Ext.data.proxy.Direct-cfg-directFn" class="docClass">directFn</a>.</p>
   * @see #getApi()
   * @see #setApi()
   */
  override public native function get api():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set api(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Function to call when executing a request. <code>directFn</code> is a simple alternative to defining
   * the api configuration parameter for Stores which will not implement a full CRUD api.
   * The <code>directFn</code> may also be a string reference to the fully qualified name of the function,
   * for example: <code>'MyApp.company.GetProfile'</code>. This can be useful when using dynamic loading.
   * The string will be resolved before calling the function for the first time.
   * @see #getDirectFn()
   * @see #setDirectFn()
   */
  public native function get directFn():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set directFn(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Optional set of fixed parameters to send with every Proxy request, similar to
   * <a href="#!/api/Ext.data.proxy.Direct-cfg-extraParams" rel="Ext.data.proxy.Direct-cfg-extraParams" class="docClass">extraParams</a> but available with all CRUD requests. Also unlike
   * <a href="#!/api/Ext.data.proxy.Direct-cfg-extraParams" rel="Ext.data.proxy.Direct-cfg-extraParams" class="docClass">extraParams</a>, metadata is not mixed with the ordinary data but sent
   * separately in the data packet.
   * You may need to update your server side Ext Direct stack to use this feature.
   * @see #getMetadata()
   * @see #setMetadata()
   */
  public native function get metadata():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set metadata(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A list of params to be passed to server side Read function. Specify the params
   * in the order in which they must be executed on the server-side as either (1) an Array
   * of String values, or (2) a String of params delimited by either whitespace, comma,
   * or pipe. For example, any of the following would be acceptable:
   *
   * <pre><code>paramOrder: ['param1','param2','param3']
   * paramOrder: 'param1 param2 param3'
   * paramOrder: 'param1,param2,param3'
   * paramOrder: 'param1|param2|param'
   * </code></pre>
   * @see #getParamOrder()
   * @see #setParamOrder()
   */
  public native function get paramOrder():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set paramOrder(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Send Read function parameters as a collection of named arguments. Providing a
   * <a href="#!/api/Ext.data.proxy.Direct-cfg-paramOrder" rel="Ext.data.proxy.Direct-cfg-paramOrder" class="docClass">paramOrder</a> nullifies this configuration.
   * @default true
   * @see #getParamsAsHash()
   * @see #setParamsAsHash()
   */
  public native function get paramsAsHash():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set paramsAsHash(value:Boolean):void;

  /**
   * Aborts a running request by operation.
   * @param operation The operation to abort. This parameter
   * is mandatory.
   */
  public native function abort(operation:ext.data.operation.Operation):void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Direct-cfg-directFn" rel="Ext.data.proxy.Direct-cfg-directFn" class="docClass">directFn</a>.
   */
  public native function getDirectFn():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Direct-cfg-metadata" rel="Ext.data.proxy.Direct-cfg-metadata" class="docClass">metadata</a>.
   */
  public native function getMetadata():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Direct-cfg-paramOrder" rel="Ext.data.proxy.Direct-cfg-paramOrder" class="docClass">paramOrder</a>.
   */
  public native function getParamOrder():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Direct-cfg-paramsAsHash" rel="Ext.data.proxy.Direct-cfg-paramsAsHash" class="docClass">paramsAsHash</a>.
   */
  public native function getParamsAsHash():Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Direct-cfg-directFn" rel="Ext.data.proxy.Direct-cfg-directFn" class="docClass">directFn</a>.
   * @param directFn The new value.
   */
  public native function setDirectFn(directFn:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Direct-cfg-metadata" rel="Ext.data.proxy.Direct-cfg-metadata" class="docClass">metadata</a>.
   * @param metadata The new value.
   */
  public native function setMetadata(metadata:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Direct-cfg-paramOrder" rel="Ext.data.proxy.Direct-cfg-paramOrder" class="docClass">paramOrder</a>.
   * @param paramOrder The new value.
   */
  public native function setParamOrder(paramOrder:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Direct-cfg-paramsAsHash" rel="Ext.data.proxy.Direct-cfg-paramsAsHash" class="docClass">paramsAsHash</a>.
   * @param paramsAsHash The new value.
   */
  public native function setParamsAsHash(paramsAsHash:Boolean):void;
}
}