package ext.data.reader {
import ext.data.ResultSet;
import ext.mixin.IFactoryable;
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Native("Ext.data.reader.Reader", require)]
/**
 * Fires when the reader receives improperly encoded data from the server
 * @eventType ext.data.reader.events.DataReader_response_errorEvent.EXCEPTION
 */
[Event(name = 'onException', type = 'ext.data.reader.events.DataReader_response_errorEvent')]
/**
 * Readers are used to interpret data to be loaded into a <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> instance or a <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a> - often in response to an AJAX request. In general there is usually no need to create
 * a Reader instance directly, since a Reader is almost always used together with a <a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">Proxy</a>,
 * and is configured using the Proxy's <a href="#!/api/Ext.data.proxy.Proxy-cfg-reader" rel="Ext.data.proxy.Proxy-cfg-reader" class="docClass">reader</a> configuration property:
 *
 * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     model: 'User',
 *     proxy: {
 *         type: 'ajax',
 *         url : 'users.json',
 *         reader: {
 *             type: 'json',
 *             rootProperty: 'users'
 *         }
 *     },
 * });
 * </code></pre>
 *
 * <p>The above reader is configured to consume a JSON string that looks something like this:</p>
 *
 * <pre><code>{
 *     "success": true,
 *     "users": [
 *         { "name": "User 1" },
 *         { "name": "User 2" }
 *     ]
 * }
 * </code></pre>
 *
 * <h1>Loading Nested Data</h1>
 *
 * <p>Readers have the ability to automatically load deeply-nested data objects based on the <a href="#!/api/Ext.data.schema.Association" rel="Ext.data.schema.Association" class="docClass">associations</a>
 * configured on each Model. Below is an example demonstrating the flexibility of these associations in a
 * fictional CRM system which manages a User, their Orders, OrderItems and Products. First we'll define the models:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>("User", {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: [
 *         'id', 'name'
 *     ],
 *
 *     hasMany: {model: 'Order', name: 'orders'},
 *
 *     proxy: {
 *         type: 'rest',
 *         url : 'users.json',
 *         reader: {
 *             type: 'json',
 *             rootProperty: 'users'
 *         }
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>("Order", {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: [
 *         'id', 'total'
 *     ],
 *
 *     hasMany  : {model: 'OrderItem', name: 'orderItems', associationKey: 'order_items'},
 *     belongsTo: 'User'
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>("OrderItem", {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: [
 *         'id', 'price', 'quantity', 'order_id', 'product_id'
 *     ],
 *
 *     belongsTo: ['Order', {model: 'Product', associationKey: 'product'}]
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>("Product", {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: [
 *         'id', 'name'
 *     ],
 *
 *     hasMany: 'OrderItem'
 * });
 * </code></pre>
 *
 * <p>This may be a lot to take in - basically a User has many Orders, each of which is composed of several OrderItems.
 * Finally, each OrderItem has a single Product. This allows us to consume data like this:</p>
 *
 * <pre><code>{
 *     "users": [
 *         {
 *             "id": 123,
 *             "name": "Ed",
 *             "orders": [
 *                 {
 *                     "id": 50,
 *                     "total": 100,
 *                     "order_items": [
 *                         {
 *                             "id"      : 20,
 *                             "price"   : 40,
 *                             "quantity": 2,
 *                             "product" : {
 *                                 "id": 1000,
 *                                 "name": "MacBook Pro"
 *                             }
 *                         },
 *                         {
 *                             "id"      : 21,
 *                             "price"   : 20,
 *                             "quantity": 3,
 *                             "product" : {
 *                                 "id": 1001,
 *                                 "name": "iPhone"
 *                             }
 *                         }
 *                     ]
 *                 }
 *             ]
 *         }
 *     ]
 * }
 * </code></pre>
 *
 * <p>The JSON response is deeply nested - it returns all Users (in this case just 1 for simplicity's sake), all of the
 * Orders for each User (again just 1 in this case), all of the OrderItems for each Order (2 order items in this case),
 * and finally the Product associated with each OrderItem. Now we can read the data and use it as follows:</p>
 *
 * <pre><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     model: "User"
 * });
 *
 * store.load({
 *     callback: function() {
 *         //the user that was loaded
 *         var user = store.first();
 *
 *         console.log("Orders for " + user.get('name') + ":")
 *
 *         //iterate over the Orders for each User
 *         user.orders().each(function(order) {
 *             console.log("Order ID: " + order.getId() + ", which contains items:");
 *
 *             //iterate over the OrderItems for each Order
 *             order.orderItems().each(function(orderItem) {
 *                 //we know that the Product data is already loaded, so we can use the synchronous getProduct
 *                 //usually, we would use the asynchronous version (see #belongsTo)
 *                 var product = orderItem.getProduct();
 *
 *                 console.log(orderItem.get('quantity') + ' orders of ' + product.get('name'));
 *             });
 *         });
 *     }
 * });
 * </code></pre>
 *
 * <p>Running the code above results in the following:</p>
 *
 * <pre><code>Orders for Ed:
 * Order ID: 50, which contains items:
 * 2 orders of MacBook Pro
 * 3 orders of iPhone
 * </code></pre>
 */
public class DataReader extends ext.mixin.Observable implements ext.mixin.IFactoryable {
  /**
   * Creates new Reader.
   * @param config Config object.
   */
  public function DataReader(config:ext.data.reader.DataReader = null) {
    super();
  }

  /**
   * <code>true</code> in this class to identify an object as an instantiated Reader, or subclass thereof.
   * @default true
   */
  public native function get isReader():Boolean;

  /**
   * @private
   */
  public native function set isReader(value:Boolean):void;

  /**
   * The raw meta data that was most recently read, if any. Meta data can include existing
   * Reader config options like <a href="#!/api/Ext.data.reader.Reader-cfg-totalProperty" rel="Ext.data.reader.Reader-cfg-totalProperty" class="docClass">totalProperty</a>, etc. that get
   * automatically applied to the Reader, and those can still be accessed directly from the Reader
   * if needed. However, meta data is also often used to pass other custom data to be processed
   * by application code. For example, it is common when reconfiguring the data model of a grid to
   * also pass a corresponding column model config to be applied to the grid. Any such data will
   * not get applied to the Reader directly (it just gets passed through and is ignored by Ext).
   * This metaData property gives you access to all meta data that was passed, including any such
   * custom data ignored by the reader.
   *
   * <p>This is a read-only property, and it will get replaced each time a new meta data object is
   * passed to the reader. Note that typically you would handle proxy's
   * <a href="#!/api/Ext.data.proxy.Proxy-event-metachange" rel="Ext.data.proxy.Proxy-event-metachange" class="docClass">metachange</a> event which passes this exact same meta
   * object to listeners. However this property is available if it's more convenient to access it
   * via the reader directly in certain cases.</p>
   */
  public native function get metaData():Object;

  /**
   * The raw data object that was last passed to <a href="#!/api/Ext.data.reader.Reader-method-readRecords" rel="Ext.data.reader.Reader-method-readRecords" class="docClass">readRecords</a>. rawData is populated
   * based on the results of <a href="#!/api/Ext.data.proxy.Server-method-processResponse" rel="Ext.data.proxy.Server-method-processResponse" class="docClass">Ext.data.proxy.Server.processResponse</a>. rawData will
   * maintain a cached copy of the last successfully returned records. In other words,
   * if processResponse is unsuccessful, the records from the last successful response
   * will remain cached in rawData.
   *
   * <p>Since Ext JS 5.1.1 you can use the <a href="#!/api/Ext.data.reader.Reader-cfg-keepRawData" rel="Ext.data.reader.Reader-cfg-keepRawData" class="docClass">keepRawData</a> config option to
   * control this behavior.</p>
   */
  public native function get rawData():Object;

  /**
   * @private
   */
  public native function set rawData(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to automatically parse models nested within other models in a response object. See the
   * <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Ext.data.reader.Reader</a> intro docs for full explanation.
   * @default true
   * @see #getImplicitIncludes()
   * @see #setImplicitIncludes()
   */
  public native function get implicitIncludes():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set implicitIncludes(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Determines if the Reader will keep raw data
   * received from the server in the <a href="#!/api/Ext.data.reader.Reader-property-rawData" rel="Ext.data.reader.Reader-property-rawData" class="docClass">rawData</a> property.
   *
   * <p>While this might seem useful to do additional data processing, keeping raw data
   * might cause adverse effects such as memory leaks. It is recommended to set
   * <code>keepRawData</code> to <code>false</code> if you do not need the raw data.</p>
   *
   * <p>If you need to process data packet to extract additional data such as row summaries,
   * it is recommended to use <a href="#!/api/Ext.data.reader.Reader-cfg-transform" rel="Ext.data.reader.Reader-cfg-transform" class="docClass">transform</a> function for that purpose.</p>
   *
   * <p>Note that starting with Ext JS 6.0 the default behavior has been changed to
   * <strong>not</strong> keep the raw data because of the high potential for memory leaks.</p>
   * @since 5.1.1
   * @see #getKeepRawData()
   * @see #setKeepRawData()
   */
  public native function get keepRawData():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set keepRawData(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the property which contains a response message for exception handling. If you want to return a false success
   * response from the server, maybe due to some server-side validation, the messageProperty can hold the error message. For
   * example:
   *
   * <pre><code>{
   *     "success": false,
   *     "error": "There was an error with your request"
   * }
   * </code></pre>
   *
   * <p>You can retrieve this error message in a callback when loading a <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a> or <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> like:</p>
   *
   * <pre><code>var store = new <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>({
   *     fields : ['foo'],
   *     proxy  : {
   *         type   : 'ajax',
   *         url    : 'data.json',
   *         reader : {
   *             type            : 'json',
   *             rootProperty    : 'data',
   *             messageProperty : 'error'
   *         }
   *     }
   * });
   *
   * store.load({
   *     callback: function(records, operation, success) {
   *         if (success) {
   *             // ...
   *         } else {
   *             var error = operation.getError();
   *
   *             <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Error', error);
   *         }
   *     }
   * });
   * </code></pre>
   *
   * <p>In this example, the callback will execute with <code>success</code> being <code>false</code> and will therefore show the <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a> with
   * the error string returned in the response.</p>
   * @default ''
   * @see #getMessageProperty()
   * @see #setMessageProperty()
   */
  public native function get messageProperty():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set messageProperty(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The model to use for this reader. This config is only required if the reader is being used
   * without a proxy, otherwise the proxy will automatically set the model.
   * @see #getModel()
   * @see #setModel()
   */
  public native function get model():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set model(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to extract the records from a data packet even if the <a href="#!/api/Ext.data.reader.Reader-cfg-successProperty" rel="Ext.data.reader.Reader-cfg-successProperty" class="docClass">successProperty</a> returns false.
   * @default true
   * @see #getReadRecordsOnFailure()
   * @see #setReadRecordsOnFailure()
   */
  public native function get readRecordsOnFailure():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set readRecordsOnFailure(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The property that contains data items corresponding to the
   * Model(s) of the configured Reader. <code>rootProperty</code> varies by Reader type.
   *
   * <h2>JSON Reader</h2>
   *
   * <p><code>rootProperty</code> is a property name. It may also be a dot-separated
   * list of property names if the root is nested. The root JSON array will be
   * used by default.</p>
   *
   * <pre><code>// rootPropety config
   * rootProperty: 'embedded.myresults'
   *
   * // server response
   * {
   *     embedded: {
   *         myresults: [{
   *             name: 'Scott',
   *             age: 22
   *         }, {
   *             name: 'Ramona',
   *             age: 24
   *         }]
   *     },
   *     success: true
   * }
   * </code></pre>
   *
   * <h2>XML Reader</h2>
   *
   * <p><code>rootProperty</code> is a CSS selector. The root XML element will be used
   * by default.</p>
   *
   * <pre><code>// rootProperty config (plus record config)
   * rootProperty: 'myresults',
   * record: 'user'
   *
   * // server response
   * &lt;?xml version="1.0" encoding="UTF-8"?&gt;
   * &lt;embedded&gt;
   *     &lt;myresults&gt;
   *         &lt;user&gt;
   *             &lt;name&gt;Scott&lt;/name&gt;
   *             &lt;age&gt;22&lt;/age&gt;
   *         &lt;/user&gt;
   *         &lt;user&gt;
   *             &lt;name&gt;Ramona&lt;/name&gt;
   *             &lt;age&gt;24&lt;/age&gt;
   *         &lt;/user&gt;
   *     &lt;/myresults&gt;
   * &lt;/embedded&gt;
   * </code></pre>
   *
   * <h2>Array Reader</h2>
   *
   * <p><code>rootProperty</code> is not typically applicable since the data is assumed to be a
   * single-level array of arrays.  However, if the array of records is returned
   * within a JSON response a <code>rootProperty</code> config may be used:</p>
   *
   * <pre><code>// rootProperty config
   * rootProperty: 'embedded.myresults'
   *
   * // server response
   * {
   *     embedded: {
   *         myresults: [['Scott', 22], ['Ramona', 24]]
   *     },
   *     success: true
   * }
   * </code></pre>
   *
   * <h2>rootProperty as a function</h2>
   *
   * <p>The <code>rootProperty</code> may also be a function that returns the root node from
   * the dataset. For example:</p>
   *
   * <pre><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">Ext.data.TreeStore</a>', {
   *     proxy: {
   *         type: 'memory',
   *         reader: {
   *             type: 'json',
   *             rootProperty: function(data){
   *                 // Extract child nodes from the items or children property in the dataset
   *                 return data.items || data.children;
   *             }
   *         }
   *     },
   *     data: {
   *         items: [{
   *             text: 'item 1',
   *             children: [{
   *                 text: 'child A',
   *                 leaf: true
   *             }]
   *         }]
   *     }
   * });
   *
   * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tree.Panel" rel="Ext.tree.Panel" class="docClass">Ext.tree.Panel</a>', {
   *     title: 'rootProperty as a function',
   *     width: 200,
   *     height:150,
   *     store: store,
   *     rootVisible: false,
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
   * });
   * </code></pre>
   * @default ''
   * @see #getRootProperty()
   * @see #setRootProperty()
   */
  public native function get rootProperty():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set rootProperty(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Name of the property from which to retrieve the <code>success</code> attribute, the value of which indicates
   * whether a given request succeeded or failed (typically a boolean or 'true'|'false'). See
   * <a href="#!/api/Ext.data.proxy.Server" rel="Ext.data.proxy.Server" class="docClass">Ext.data.proxy.Server</a>.<a href="#!/api/Ext.data.proxy.Server-event-exception" rel="Ext.data.proxy.Server-event-exception" class="docClass">exception</a> for additional information.
   * @default 'success'
   * @see #getSuccessProperty()
   * @see #setSuccessProperty()
   */
  public native function get successProperty():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set successProperty(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Name of the property from which to retrieve the total number of records in the dataset. This is only needed if
   * the whole dataset is not passed in one go, but is being paged from the remote server.
   * @default 'total'
   * @see #getTotalProperty()
   * @see #setTotalProperty()
   */
  public native function get totalProperty():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set totalProperty(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If a transform function is set, it will be invoked just before <a href="#!/api/Ext.data.reader.Reader-method-readRecords" rel="Ext.data.reader.Reader-method-readRecords" class="docClass">readRecords</a> executes.
   * It is passed the raw (deserialized) data object. The transform function returns a data object, which can be
   * a modified version of the original data object, or a completely new data object. The transform can
   * be a function, or a method name on the Reader instance, or an object with a 'fn' key
   * and an optional 'scope' key.
   *
   * <p>Example usage:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
   *     model: 'User',
   *     proxy: {
   *         type: 'ajax',
   *         url : 'users.json',
   *         reader: {
   *             type: 'json',
   *             transform: {
   *                 fn: function(data) {
   *                     // do some manipulation of the raw data object
   *                     return data;
   *                 },
   *                 scope: this
   *             }
   *         }
   *     },
   * });
   * </code></pre>
   * @see #getTransform()
   * @see #setTransform()
   */
  public native function get transform():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set transform(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the property in a node raw data block which indicates the type of the model to be created from that raw data. Useful for heterogeneous trees.
   *
   * <p>For example, hierarchical geographical data may look like this:</p>
   *
   * <pre><code>{
   *     nodeType: 'Territory',
   *     name: 'EMEA',
   *     children: [{
   *         nodeType: 'Country',
   *         name: 'United Kingdon',
   *         children: [{
   *             nodeType: 'City',
   *             name: 'London'
   *         }]
   *     }]
   * }
   * </code></pre>
   *
   * <p>You would configure the typeProperty in this case to be <code>"nodeType"</code> which would cause the models named "Territory", "Country" and "City" to
   * be used.</p>
   * @default ''
   * @see #getTypeProperty()
   * @see #setTypeProperty()
   */
  public native function get typeProperty():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set typeProperty(value:String):void;

  /**
   * @inheritDoc
   */
  public native function get factoryConfig():Object;

  /**
   * @inheritDoc
   */
  public native function set factoryConfig(value:Object):void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.reader.Reader-cfg-implicitIncludes" rel="Ext.data.reader.Reader-cfg-implicitIncludes" class="docClass">implicitIncludes</a>.
   */
  public native function getImplicitIncludes():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.reader.Reader-cfg-keepRawData" rel="Ext.data.reader.Reader-cfg-keepRawData" class="docClass">keepRawData</a>.
   * @since 5.1.1
   */
  public native function getKeepRawData():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.reader.Reader-cfg-messageProperty" rel="Ext.data.reader.Reader-cfg-messageProperty" class="docClass">messageProperty</a>.
   */
  public native function getMessageProperty():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.reader.Reader-cfg-model" rel="Ext.data.reader.Reader-cfg-model" class="docClass">model</a>.
   */
  public native function getModel():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.reader.Reader-cfg-readRecordsOnFailure" rel="Ext.data.reader.Reader-cfg-readRecordsOnFailure" class="docClass">readRecordsOnFailure</a>.
   */
  public native function getReadRecordsOnFailure():Boolean;

  /**
   * Takes a raw response object (as passed to the <a href="#!/api/Ext.data.reader.Reader-method-read" rel="Ext.data.reader.Reader-method-read" class="docClass">read</a> method) and returns the useful data
   * segment from it. This must be implemented by each subclass.
   * @param response The response object
   * @return The extracted data from the response. For example, a JSON object or an XML document.
   */
  public native function getResponseData(response:Object):Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.reader.Reader-cfg-rootProperty" rel="Ext.data.reader.Reader-cfg-rootProperty" class="docClass">rootProperty</a>.
   */
  public native function getRootProperty():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.reader.Reader-cfg-successProperty" rel="Ext.data.reader.Reader-cfg-successProperty" class="docClass">successProperty</a>.
   */
  public native function getSuccessProperty():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.reader.Reader-cfg-totalProperty" rel="Ext.data.reader.Reader-cfg-totalProperty" class="docClass">totalProperty</a>.
   */
  public native function getTotalProperty():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.reader.Reader-cfg-transform" rel="Ext.data.reader.Reader-cfg-transform" class="docClass">transform</a>.
   */
  public native function getTransform():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.reader.Reader-cfg-typeProperty" rel="Ext.data.reader.Reader-cfg-typeProperty" class="docClass">typeProperty</a>.
   */
  public native function getTypeProperty():String;

  /**
   * Reads the given response object. This method normalizes the different types of response object that may be passed to it.
   * If it's an XMLHttpRequest object, hand off to the subclass' <a href="#!/api/Ext.data.reader.Reader-method-getResponseData" rel="Ext.data.reader.Reader-method-getResponseData" class="docClass">getResponseData</a> method.
   * Else, hand off the reading of records to the <a href="#!/api/Ext.data.reader.Reader-method-readRecords" rel="Ext.data.reader.Reader-method-readRecords" class="docClass">readRecords</a> method.
   * @param response The response object. This may be either an XMLHttpRequest object or a plain JS object
   * @param readOptions Various options that instruct the reader on how to read the data
   * @param {Function} [readOptions.recordCreator] A function to construct the model based on the processed data. By default,
   * this just calls the model constructor and passes the raw data.
   * @return The parsed or default ResultSet object
   */
  public native function read(response:Object, readOptions:Object = null):ext.data.ResultSet;

  /**
   * Abstracts common functionality used by all Reader subclasses. Each subclass is expected to call this function
   * before running its own logic and returning the <a href="#!/api/Ext.data.ResultSet" rel="Ext.data.ResultSet" class="docClass">Ext.data.ResultSet</a> instance. For most Readers additional
   * processing should not be needed.
   * @param data The raw data object
   * @param readOptions See <a href="#!/api/Ext.data.reader.Reader-method-read" rel="Ext.data.reader.Reader-method-read" class="docClass">read</a> for details.
   * @return A ResultSet object
   */
  public native function readRecords(data:Object, readOptions:Object = null):ext.data.ResultSet;

  /**
   * Sets the value of <a href="#!/api/Ext.data.reader.Reader-cfg-implicitIncludes" rel="Ext.data.reader.Reader-cfg-implicitIncludes" class="docClass">implicitIncludes</a>.
   * @param implicitIncludes The new value.
   */
  public native function setImplicitIncludes(implicitIncludes:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.reader.Reader-cfg-keepRawData" rel="Ext.data.reader.Reader-cfg-keepRawData" class="docClass">keepRawData</a>.
   * @since 5.1.1
   * @param keepRawData The new value.
   */
  public native function setKeepRawData(keepRawData:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.reader.Reader-cfg-messageProperty" rel="Ext.data.reader.Reader-cfg-messageProperty" class="docClass">messageProperty</a>.
   * @param messageProperty The new value.
   */
  public native function setMessageProperty(messageProperty:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.reader.Reader-cfg-model" rel="Ext.data.reader.Reader-cfg-model" class="docClass">model</a>.
   * @param model The new value.
   */
  public native function setModel(model:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.reader.Reader-cfg-readRecordsOnFailure" rel="Ext.data.reader.Reader-cfg-readRecordsOnFailure" class="docClass">readRecordsOnFailure</a>.
   * @param readRecordsOnFailure The new value.
   */
  public native function setReadRecordsOnFailure(readRecordsOnFailure:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.reader.Reader-cfg-rootProperty" rel="Ext.data.reader.Reader-cfg-rootProperty" class="docClass">rootProperty</a>.
   * @param rootProperty The new value.
   */
  public native function setRootProperty(rootProperty:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.reader.Reader-cfg-successProperty" rel="Ext.data.reader.Reader-cfg-successProperty" class="docClass">successProperty</a>.
   * @param successProperty The new value.
   */
  public native function setSuccessProperty(successProperty:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.reader.Reader-cfg-totalProperty" rel="Ext.data.reader.Reader-cfg-totalProperty" class="docClass">totalProperty</a>.
   * @param totalProperty The new value.
   */
  public native function setTotalProperty(totalProperty:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.reader.Reader-cfg-transform" rel="Ext.data.reader.Reader-cfg-transform" class="docClass">transform</a>.
   * @param transform The new value.
   */
  public native function setTransform(transform:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.reader.Reader-cfg-typeProperty" rel="Ext.data.reader.Reader-cfg-typeProperty" class="docClass">typeProperty</a>.
   * @param typeProperty The new value.
   */
  public native function setTypeProperty(typeProperty:String):void;

  /**
   * This method provides a hook to do any data transformation before the reading process
   * begins. By default this function just returns what is passed to it. It can be
   * overridden in a subclass to return something else.
   * See <a href="#!/api/Ext.data.reader.Xml" rel="Ext.data.reader.Xml" class="docClass">XmlReader</a> for an example.
   * @param data The data object
   * @return The normalized data object
   */
  protected native function getData(data:Object):Object;
}
}