package ext.data.schema {
import ext.Base;
import ext.ExtClass;
import ext.data.field.DataField;

[Uses("ext.Base")]
[Native("Ext.data.schema.Association", require)]
/**
 * <strong>This class is never created directly. It should be constructed through associations in <code><a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a></code>.</strong>
 *
 * <p>Associations enable you to express relationships between different <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Models</a>. Consider
 * an ecommerce system where Users can place Orders - there is a one to many relationship between these Models,
 * one user can have many orders (including 0 orders). Here is what a sample implementation of this association
 * could look like. This example will be referred to in the following sections.</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: [{
 *         name: 'id',
 *         type: 'int'
 *     }, 'name']
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Order', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: [{
 *         name: 'id',
 *         type: 'int'
 *     }, {
 *         name: 'userId',
 *         type: 'int',
 *         reference: 'User'
 *     }]
 * });
 * </code></pre>
 *
 * <h1>Association Types</h1>
 *
 * <p>Assocations can describe relationships in 3 ways:</p>
 *
 * <h2>Many To One</h2>
 *
 * <p>A single entity (<code>A</code>) has a relationship with many (<code>B</code>) entities. An example of this is
 * an ecommerce system <code>User</code> can have many <code>Order</code> entities.</p>
 *
 * <p>This can be defined using <code><a href="#!/api/Ext.data.schema.ManyToOne" rel="Ext.data.schema.ManyToOne" class="docClass">Ext.data.schema.ManyToOne</a></code> for keyed associations, or
 * <code><a href="#!/api/Ext.data.schema.HasMany" rel="Ext.data.schema.HasMany" class="docClass">Ext.data.schema.HasMany</a></code> for keyless associations.</p>
 *
 * <h2>One To One</h2>
 *
 * <p>A less common form of Many To One, a single entity (<code>A</code>) has a relationship with at most 1 entity (<code>B</code>). This is often
 * used when partitioning data. For example a <code>User</code> may have a single <code>UserInfo</code> object that stores extra
 * metadata about the user.</p>
 *
 * <p>This can be defined using <code><a href="#!/api/Ext.data.schema.OneToOne" rel="Ext.data.schema.OneToOne" class="docClass">Ext.data.schema.OneToOne</a></code> for keyed associations, or
 * <code><a href="#!/api/Ext.data.schema.HasOne" rel="Ext.data.schema.HasOne" class="docClass">Ext.data.schema.HasOne</a></code> for keyless associations.</p>
 *
 * <h2>Many To Many</h2>
 *
 * <p>An entity (<code>A</code>) may have a have a relationship with many (<code>B</code>) entities. That (<code>B</code>) entity may also
 * have a relationship with many <code>A</code> entities. For example a single <code>Student</code> can have many <code>Subject</code> entities and
 * a single <code>Subject</code> can have many <code>Student</code> entities.</p>
 *
 * <p>This can be defined using <code><a href="#!/api/Ext.data.schema.ManyToMany" rel="Ext.data.schema.ManyToMany" class="docClass">Ext.data.schema.ManyToMany</a></code>. Many To Many relationships are readonly unless used with
 * a <code><a href="#!/api/Ext.data.Session" rel="Ext.data.Session" class="docClass">Ext.data.Session</a></code>.</p>
 *
 * <h1>Keyed vs Keyless Associations</h1>
 *
 * <p>Associations can be declared in 2 ways, which are outlined below.</p>
 *
 * <h2>Keyed associations</h2>
 *
 * <p>A keyed association relies on a field in the model matching the id of another model. Membership is driven by the key.
 * This is the type of relationship that is typically used in a relational database.
 * This is declared using the ||reference|| configuration on a model field. An example of this can be seen
 * above for <code>User/Order</code>.</p>
 *
 * <h1>Keyless associations</h1>
 *
 * <p>A keyless association relies on data hierarchy to determine membership. Items are members because they are
 * contained by another entity. This type of relationship is common with NoSQL databases.
 * formats. A simple example definition using <code>User/Order</code>:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: [{
 *         name: 'id',
 *         type: 'int'
 *     }, 'name'],
 *     hasMany: 'Order'
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Order', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: [{
 *         name: 'id',
 *         type: 'int'
 *     }]
 * });
 * </code></pre>
 *
 * <h1>Advantages of Associations</h1>
 *
 * <p>Assocations make it easier to work with Models that share a connection. Some of the main functionality includes:</p>
 *
 * <h2>Generated Accessors/Setters</h2>
 *
 * <p>Associated models will automatically generate named methods that allow for accessing the associated data.
 * The names for these are created using a <a href="#!/api/Ext.data.schema.Schema" rel="Ext.data.schema.Schema" class="docClass">Schema</a>, to provide a consistent and
 * predictable naming structure.</p>
 *
 * <p>Using the example code above, there will be 3 generated methods:
 * + <code>User</code> will have an <code>orders()</code> function that returns a <code><a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a></code> of<code>Orders</code>.
 * + <code>Order</code> will have a <code>getUser</code> method which will return a <code>User</code> Model.
 * + <code>Order</code> will have a <code>setUser</code> method that will accept a <code>User</code> model or a key value.</p>
 *
 * <h2>Nested Loading</h2>
 *
 * <p>Nested loading is the ability to load hierarchical associated data from a remote source within a single request.
 * In the following example, each <code>User</code> in the <code>users</code> store has an <code>orders</code> store. Each <code>orders</code> store is populated
 * with <code>Order</code> models read from the request. Each <code>Order</code> model also has a reference back to the appropriate <code>User</code>.</p>
 *
 * <pre><code>// Sample JSON data returned by /Users
 * [{
 *     "id": 1,
 *     "name": "User Foo",
 *     "orders": [{
 *         "id": 101,
 *         "userId": 1
 *     }, {
 *         "id": 102,
 *         "userId": 1
 *     }, {
 *         "id": 103,
 *         "userId": 1
 *     }]
 * }, {
 *     "id": 2,
 *     "name": "User Bar",
 *     "orders": [{
 *         "id": 201,
 *         "userId": 2
 *     }, {
 *         "id": 202,
 *         "userId": 2
 *     }]
 * }]
 *
 * // Application code
 * var users = new <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>({
 *     model: 'User',
 *     proxy: {
 *         type: 'ajax',
 *         url: '/Users'
 *     }
 * });
 * users.load(function() {
 *     var user1 = users.first(),
 *         user2 = users.last(),
 *         orders1 = user1.orders(),
 *         orders2 = user2.orders();
 *
 *     // 3 orders, same reference back to user1
 *     console.log(orders1.getCount(), orders1.first().getUser() === user1);
 *     // 2 orders, same reference back to user2
 *     console.log(orders2.getCount(), orders2.first().getUser() === user2);
 * });
 * </code></pre>
 *
 * <h2>Binding</h2>
 *
 * <p>Data binding using <a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">ViewModels</a> have functionality to be able to recognize
 * associated data as part of a bind statement. For example:
 * + <code>{user.orders}</code> binds to the orders store for a user.
 * + <code>{order.user.name}</code> binds to the name of the user taken from the order.</p>
 *
 * <h1>Association Concepts</h1>
 *
 * <h2>Roles</h2>
 *
 * <p>The role is used to determine generated names for an association. By default, the role is generated from
 * either the field name (in a keyed association) or the model name. This naming follows a pattern defined by
 * the <code><a href="#!/api/Ext.data.schema.Namer" rel="Ext.data.schema.Namer" class="docClass">Ext.data.schema.Namer</a></code>. To change a specific instance, an explicit role can be specified:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Thread', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'title']
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Post', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'content', {
 *         name: 'threadId',
 *         reference: {
 *             type: 'Thread',
 *             role: 'discussion',
 *             inverse: 'comments'
 *
 *         }
 *     }]
 * });
 * </code></pre>
 *
 * <p>In the above example, the <code>Thread</code> will be decorated with a <code>comments</code> method that returns the store.
 * The <code>Post</code> will be decorated with <code>getDiscussion/setDiscussion</code> methods.</p>
 *
 * <h2>Generated Methods</h2>
 *
 * <p>Associations generate methods to allow reading and manipulation on associated data.</p>
 *
 * <p>On records that have a "to many" relationship, a single methods that returns a <code><a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a></code> is created.
 * See <a href="#!/api/Ext.data.schema.Association-method-storeGetter" rel="Ext.data.schema.Association-method-storeGetter" class="docClass">storeGetter</a>. On records that have a "to one" relationship, 2 methods are generated, a
 * <a href="#!/api/Ext.data.schema.Association-method-recordGetter" rel="Ext.data.schema.Association-method-recordGetter" class="docClass">getter</a> and a <a href="#!/api/Ext.data.schema.Association-method-recordSetter" rel="Ext.data.schema.Association-method-recordSetter" class="docClass">setter</a>.</p>
 *
 * <h2>Reflexive</h2>
 *
 * <p>Associations are reflexive. By declaring one "side" of the relationship, the other is automatically setup. In
 * the example below, there is no code in the <code>Thread</code> entity regarding the association, however by virtue of the
 * declaration in post, <code>Thread</code> is decorated with the appropriate infrastructure to participate in the association.</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Thread', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'title']
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Post', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'content', {
 *         name: 'threadId',
 *         reference: 'Thread'
 *     }]
 * });
 * </code></pre>
 *
 * <h2>Naming</h2>
 *
 * <p>Referring to model names in associations depends on their <a href="#!/api/Ext.data.Model-property-entityName" rel="Ext.data.Model-property-entityName" class="docClass">Ext.data.Model.entityName</a>. See
 * the "Relative Naming" section in the <code><a href="#!/api/Ext.data.schema.Schema" rel="Ext.data.schema.Schema" class="docClass">Ext.data.schema.Schema</a></code> documentation.</p>
 */
public class Association extends ext.Base {
  /**
   * @param config @inheritDoc
   */
  public function Association(config:ext.data.schema.Association = null) {
    super();
  }

  public native function get definedBy():ext.ExtClass;

  public native function get field():ext.data.field.DataField;

  public native function get left():ext.Base;

  public native function get nullable():Boolean;

  /**
   * Points at either <code>left</code> or <code>right</code> objects if one is the owning party in this
   * association or is <code>null</code> if there is no owner.
   */
  public native function get owner():Object;

  public native function get right():ext.Base;

  public native function get schema():ext.data.schema.Schema;

  [ExtConfig]
  /**
   * The name of this association.
   */
  public native function get name():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set name(value:String):void;
}
}