package ext.data.schema {
import ext.Base;
import ext.ExtClass;
import ext.data.Model;
import ext.data.field.DataField;
import ext.mixin.IFactoryable;

[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.schema.Schema", require)]
/**
 * A Schema is a collection of related <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">entities</a> and their respective
 * <a href="#!/api/Ext.data.schema.Association" rel="Ext.data.schema.Association" class="docClass">associations</a>.
 *
 * <h1>Schema Instances</h1>
 *
 * <p>By default a single instance of this class is created which serves as the schema for all
 * entities that do not have an explicit <code><a href="#!/api/Ext.data.Model-cfg-schema" rel="Ext.data.Model-cfg-schema" class="docClass">schema</a></code> config
 * either specified or inherited. This is sufficient in most cases.</p>
 *
 * <p>When an entity does specify a <code><a href="#!/api/Ext.data.Model-cfg-schema" rel="Ext.data.Model-cfg-schema" class="docClass">schema</a></code>, however, that
 * looks up (or creates) an instance for that entity class which is then inherited.</p>
 *
 * <p><strong>Important:</strong> All related entities <em>must</em> belong to a single schema instance in order
 * to properly link up their associations.</p>
 *
 * <h2>Configuring Schemas</h2>
 *
 * <p>The best way to control the configuration of your <code>schema</code> is to define a base class for
 * all of your entities and use the <code><a href="#!/api/Ext.data.Model-cfg-schema" rel="Ext.data.Model-cfg-schema" class="docClass">schema</a></code> config like
 * this:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.model.Base', {
 *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *
 *      // This configures the default schema because we don't assign an "id":
 *      schema: {
 *          // configs go here
 *      }
 *  });
 * </code></pre>
 *
 * <p><strong>Note:</strong> Only one explicit configuration can be applied to the default schema. In most
 * applications this will not be an issue.</p>
 *
 * <p>By using a base class for your entities you can ensure that the default schema is fully
 * configured before declaration of your classes proceeds. This is especially helpful if
 * you need to set the <code>namespace</code> for your schema (see below).</p>
 *
 * <h2>Relative Naming</h2>
 *
 * <p>When describing associations between entities, it is desirable to use shorthand names
 * that do not contain the common namespace portion. This is called the <code>entityName</code> as
 * opposed to its class name. By default, the <code>entityName</code> is the full class name. However,
 * if a namespace is used, the common portion can be discarded and we can derive a shorter name.
 * In the following code, <code>"MyApp.model.Foo"</code> has an <code>entityName</code> of <code>"Foo"</code> and the schema has
 * a <code>namespace</code> of "MyApp.model".</p>
 *
 * <p>If you use deeper nesting for entities, you may need to set the <code>namespace</code> config to
 * account for this. For example:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.model.Base', {
 *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *
 *      schema: {
 *          namespace: 'MyApp.model'
 *      }
 *  });
 * </code></pre>
 *
 * <p>Your derived classes now will generate proper default <code>entityName</code> values even if they
 * have further namespaces. For example, "MyApp.model.foo.Thing" will produce "foo.Thing"
 * as the <code>entityName</code> given the above as a base class.</p>
 *
 * <h1>Association Naming</h1>
 *
 * <p>There are various terms involved when describing associations. Perhaps the simplest
 * example that will clarify these terms is that of the common many-to-many association
 * of User and Group.</p>
 *
 * <ul>
 * <li><p><code>entityName</code> - The names "User" and "Group" are the <code>entityName</code> values associated
 * with these two classes. These are derived from their full classnames (perhaps
 * something like "App.model.User" and "App.model.Group").</p></li>
 * <li><p><code>associationName</code> - When talking about associations, especially the many-to-many
 * variety, it is important to give them names. Associations are not owned by either of
 * the entities involved, so this name is similar to an <code>entityName</code>. In the case of
 * "User" and "Group", the default <code>associationName</code> would be "GroupUsers".</p></li>
 * <li><p><code>left</code> and <code>right</code> - Associations describe a relationship between two entities. To
 * talk about specific associations we would use the <code>entityName</code> of the parties (such
 * as "User" or "Group"). When discussing associations in the abstract, however, it is
 * very helpful to be able to talk about the entities in an association in a general way.
 * In the case of the "GroupUsers" association, "User" is said to be the <code>left</code> while
 * "Group" is said to be the <code>right</code>. In a many-to-many association the selection of
 * <code>left</code> and <code>right</code> is arbitrary. When a foreign-key is involved, the <code>left</code> entity
 * is the one containing the foreign-key.</p></li>
 * </ul>
 *
 *
 * <h2>Custom Naming Conventions</h2>
 *
 * <p>One of the jobs the the <code>Schema</code> is to manage name generation (such as <code>entityName</code>).
 * This job is delegated to a class called the <code>namer</code>. If you need to generate names in
 * other ways, you can provide a custom <code>namer</code> for your classes:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.model.Base', {
 *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *
 *      schema: {
 *          namespace: 'MyApp.model',
 *          namer: 'custom'
 *      }
 *  });
 * </code></pre>
 *
 * <p>This will create a class using the alias "namer.custom". For example:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.model.CustomNamer', {
 *      extend: '<a href="#!/api/Ext.data.schema.Namer" rel="Ext.data.schema.Namer" class="docClass">Ext.data.schema.Namer</a>',
 *
 *      alias: 'namer.custom',
 *      ...
 *  });
 * </code></pre>
 *
 * <p>For details see the documentation for <a href="#!/api/Ext.data.schema.Namer" rel="Ext.data.schema.Namer" class="docClass">Namer</a>.</p>
 */
public class Schema extends ext.Base implements ext.mixin.IFactoryable {
  /**
   * Returns the <code>Schema</code> instance given its <code>id</code> or config object. If only the <code>id</code>
   * is specified, that <code>Schema</code> instance is looked up and returned. If there is no
   * instance already created, the <code>id</code> is assumed to be the <code>type</code>. For example:
   *
   * <pre><code> schema: 'foo'
   * </code></pre>
   *
   * <p>Would be created from the alias <code>"schema.foo"</code> and assigned the <code>id</code> of "foo"
   * as well.</p>
   * @param config The id, type or config object of the schema.
   * @param {String} [config.type] The type alias of the schema. A "schema." prefix
   * is added to this string, if provided, to complete the alias. This should match
   * match the "alias" of some class derived from <code><a href="#!/api/Ext.data.schema.Schema" rel="Ext.data.schema.Schema" class="docClass">Ext.data.schema.Schema</a></code>.
   * @return The previously existing or newly created
   * instance.
   */
  public static native function get(config:*):ext.data.schema.Schema;

  /**
   * @param config @inheritDoc
   */
  public function Schema(config:ext.data.schema.Schema = null) {
    super();
  }

  /**
   * The number of <a href="#!/api/Ext.data.schema.Association" rel="Ext.data.schema.Association" class="docClass">associations</a>
   * in this <code>schema</code>.
   * @default 0
   */
  public native function get assocCount():Number;

  /**
   * The number of <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">entities</a> in this
   * <code>schema</code>.
   * @default 0
   */
  public native function get entityCount():Number;

  /**
   * If this property is specified by the target class of this mixin its properties are
   * used to configure the created <code><a href="#!/api/Ext.Factory" rel="Ext.Factory" class="docClass">Ext.Factory</a></code>.
   */
  public native function get factoryConfig():Object;

  /**
   * @private
   */
  public native function set factoryConfig(value:Object):void;

  /**
   * The name of the schema's type. This should be the suffix of the <code>alias</code> for this
   * class following the "schema." prefix. For example, if the <code>alias</code> for a schema is
   * "schema.foo" then <code>type</code> should "foo". If an <code>alias</code> is specified on the derived
   * class, this property is set automatically.
   * @default 'default'
   */
  public native function get type():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This config is used to initialize the <code><a href="#!/api/Ext.data.Model-cfg-identifier" rel="Ext.data.Model-cfg-identifier" class="docClass">Ext.data.Model.identifier</a></code> config
   * for classes that do not define one.
   * @see #getDefaultIdentifier()
   * @see #setDefaultIdentifier()
   */
  public native function get defaultIdentifier():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set defaultIdentifier(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Specifies or configures the name generator for the schema.
   * @default 'default'
   * @see #getNamer()
   * @see #setNamer()
   */
  public native function get namer():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set namer(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The namespace for entity classes in this schema.
   * @see #getNamespace()
   * @see #setNamespace()
   */
  public native function get namespace():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set namespace(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This is a template used to produce <code><a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">Ext.data.proxy.Proxy</a></code> configurations for
   * Models that do not define an explicit <code><a href="#!/api/Ext.data.Model-cfg-proxy" rel="Ext.data.Model-cfg-proxy" class="docClass">proxy</a></code>.
   *
   * <p>This template is processed with the Model class as the data object which means
   * any static properties of the Model are available. The most useful of these are</p>
   *
   * <ul>
   * <li><code>prefix</code> - The <code>urlPrefix</code> property of this instance.</li>
   * <li><code>entityName</code> - The <a href="#!/api/Ext.data.Model-property-entityName" rel="Ext.data.Model-property-entityName" class="docClass">name</a> of the Model
   *   (for example, "User").</li>
   * <li><code>schema</code> - This instance.</li>
   * </ul>
   * @default {type: 'ajax', url: '{prefix}/{entityName}'}
   * @see #getProxy()
   * @see #setProxy()
   */
  public native function get proxy():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set proxy(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This is the URL prefix used for all requests to the server. It could be something
   * like "/~api". This value is included in the <code>proxy</code> template data as "prefix".
   * @default ""
   * @see #getUrlPrefix()
   * @see #setUrlPrefix()
   */
  public native function get urlPrefix():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set urlPrefix(value:String):void;

  /**
   * Returns an <code>Association</code> by name.
   * @param name The name of the association.
   * @return The association instance.
   */
  public native function getAssociation(name:String):ext.data.schema.Association;

  /**
   * Returns the value of <a href="#!/api/Ext.data.schema.Schema-cfg-defaultIdentifier" rel="Ext.data.schema.Schema-cfg-defaultIdentifier" class="docClass">defaultIdentifier</a>.
   */
  public native function getDefaultIdentifier():Object;

  /**
   * Returns an entity by name.
   * @param name The name of the entity
   * @return The entity class.
   */
  public native function getEntity(name:String):ext.data.Model;

  /**
   * Get the entity name taking into account the <a href="#!/api/Ext.data.schema.Schema-cfg-namespace" rel="Ext.data.schema.Schema-cfg-namespace" class="docClass">namespace</a>.
   * @param cls The model class or name of the class.
   * @return The entity name
   */
  public native function getEntityName(cls:*):String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.schema.Schema-cfg-namer" rel="Ext.data.schema.Schema-cfg-namer" class="docClass">namer</a>.
   */
  public native function getNamer():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.schema.Schema-cfg-namespace" rel="Ext.data.schema.Schema-cfg-namespace" class="docClass">namespace</a>.
   */
  public native function getNamespace():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.schema.Schema-cfg-proxy" rel="Ext.data.schema.Schema-cfg-proxy" class="docClass">proxy</a>.
   */
  public native function getProxy():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.schema.Schema-cfg-urlPrefix" rel="Ext.data.schema.Schema-cfg-urlPrefix" class="docClass">urlPrefix</a>.
   */
  public native function getUrlPrefix():String;

  /**
   * Checks if the passed entity has attached associations that need to be read when
   * using nested loading.
   * @param name The name, instance, or Model class.
   * @return <code>true</code> if there are associations attached to the entity.
   */
  public native function hasAssociations(name:*):Boolean;

  /**
   * Checks if an entity is defined
   * @param entity The name or model
   * @return True if this entity is defined
   */
  public native function hasEntity(entity:*):Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.data.schema.Schema-cfg-defaultIdentifier" rel="Ext.data.schema.Schema-cfg-defaultIdentifier" class="docClass">defaultIdentifier</a>.
   * @param defaultIdentifier The new value.
   */
  public native function setDefaultIdentifier(defaultIdentifier:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.schema.Schema-cfg-namer" rel="Ext.data.schema.Schema-cfg-namer" class="docClass">namer</a>.
   * @param namer The new value.
   */
  public native function setNamer(namer:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.schema.Schema-cfg-namespace" rel="Ext.data.schema.Schema-cfg-namespace" class="docClass">namespace</a>.
   * @param namespace The new value.
   */
  public native function setNamespace(namespace:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.schema.Schema-cfg-proxy" rel="Ext.data.schema.Schema-cfg-proxy" class="docClass">proxy</a>.
   * @param proxy The new value.
   */
  public native function setProxy(proxy:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.schema.Schema-cfg-urlPrefix" rel="Ext.data.schema.Schema-cfg-urlPrefix" class="docClass">urlPrefix</a>.
   * @param urlPrefix The new value.
   */
  public native function setUrlPrefix(urlPrefix:String):void;

  /**
   * Adds an entry from a <a href="#!/api/Ext.data.schema.ManyToMany" rel="Ext.data.schema.ManyToMany" class="docClass">matrix config</a> declared by an
   * entity.
   *
   * <p>This is the ideal method to override in a derived class if the standard, default
   * naming conventions need to be adjusted. In the override, apply whatever logic is
   * appropriate to determine the missing values and pass along the proper results to
   * this method in the <code>callParent</code>.</p>
   * @param entityType A class derived from <code><a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a></code>.
   * @param matrixName The name of the matrix association.
   * @param relation A base name for the matrix. For information about the
   * meaning of this see Ext.data.Schema.ManyToMany.
   * @param left The descriptor for the "left" of the matrix.
   * @param {String} left.type The type of the entity on the "left" of the matrix.
   * @param {String} [left.field] The name of the field in the matrix table for the "left"
   * side entity. If not provided, this defaults to the <code>left.type</code> name
   * <a href="#!/api/Ext.util.Inflector-method-singularize" rel="Ext.util.Inflector-method-singularize" class="docClass">singularized</a> and uncapitalized followed by
   * "Id". For example, "userId" for a <code>left.type</code> of "Users".
   * @param {String} [left.role] The name of the relationship from the <code>left.type</code> to the
   * <code>right.type</code>. If not provided, this defaults to the <code>left.type</code> name
   * <a href="#!/api/Ext.util.Inflector-method-pluralize" rel="Ext.util.Inflector-method-pluralize" class="docClass">pluralized</a> and uncapitalized. For example,
   * "users" for a <code>left.type</code> of "User".
   * @param right The descriptor for the "right" of the matrix.
   * @param {String} right.type The type of the entity on the "right" of the matrix.
   * @param {String} [right.field] The name of the field in the matrix table for the
   * "right" side entity. If not provided, this defaults in the same way as <code>left.field</code>
   * except this is based on <code>right.type</code>.
   * @param {String} [right.role] The name of the relationship from the <code>right.type</code> to
   * the <code>left.type</code>. If not provided, this defaults in the same way as <code>left.role</code>
   * except this is based on <code>right.type</code>.
   */
  protected native function addMatrix(entityType:ext.ExtClass, matrixName:String, relation:String = null, left:Object = null, right:Object = null):void;

  /**
   * Adds a <a href="#!/api/Ext.data.field.Field-cfg-reference" rel="Ext.data.field.Field-cfg-reference" class="docClass">reference</a> field association for an entity
   * to this <code>schema</code>.
   *
   * <p>This is the ideal method to override in a derived class if the standard, default
   * naming conventions need to be adjusted. In the override, apply whatever logic is
   * appropriate to determine the missing values and pass along the proper results to
   * this method in the <code>callParent</code>.</p>
   * @param entityType A class derived from <code><a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a></code>.
   * @param referenceField The <code>field</code> with the <code>reference</code> config.
   * @param association The name of the association. If empty or null, this
   * will be derived from <code>entityType</code>, <code>role</code>, <code>inverse</code> and
   * <code>referenceField.unique</code>.
   * @param role The name of the relationship from <code>entityType</code> to the target
   * <code>type</code>. If not specified, the default is the <code>referenceField.name</code> (minus any "Id"
   * suffix if present).
   * @param inverse The name of the relationship from the target <code>type</code>
   * to the <code>entityType</code>. If not specified, this is derived from the
   * <a href="#!/api/Ext.data.Model-property-entityName" rel="Ext.data.Model-property-entityName" class="docClass">entityName</a> of the <code>entityType</code>
   * (<a href="#!/api/Ext.util.Inflector-method-singularize" rel="Ext.util.Inflector-method-singularize" class="docClass">singularized</a> or
   * <a href="#!/api/Ext.util.Inflector-method-pluralize" rel="Ext.util.Inflector-method-pluralize" class="docClass">pluralized</a> based on <code>referenceField.unique</code>).
   * @param type The <a href="#!/api/Ext.data.Model-property-entityName" rel="Ext.data.Model-property-entityName" class="docClass">entityName</a> of the target
   * of the reference.
   * @param descr The <code>reference</code> descriptor from the <code>referenceField</code> if one
   * was given in the field definition.
   * @param unique Indicates if the reference is one-to-one.
   * @default false
   */
  protected native function addReference(entityType:ext.ExtClass, referenceField:ext.data.field.DataField, association:String = null, role:String = null, inverse:String = null, type:String = null, descr:Object = null, unique:Boolean = false):void;
}
}