package ext.data.writer {
import ext.Base;
import ext.data.Model;
import ext.data.Request;
import ext.data.operation.Operation;
import ext.mixin.IFactoryable;

[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.writer.Writer", require)]
/**
 * Base Writer class used by most subclasses of <a href="#!/api/Ext.data.proxy.Server" rel="Ext.data.proxy.Server" class="docClass">Ext.data.proxy.Server</a>. This class
 * is responsible for taking a set of <a href="#!/api/Ext.data.operation.Operation" rel="Ext.data.operation.Operation" class="docClass">Ext.data.operation.Operation</a> objects and a
 * <a href="#!/api/Ext.data.Request" rel="Ext.data.Request" class="docClass">Ext.data.Request</a> object and modifying that request based on the Operations.
 *
 * <p>For example a <a href="#!/api/Ext.data.writer.Json" rel="Ext.data.writer.Json" class="docClass">Ext.data.writer.Json</a> would format the Operations and their
 * <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> instances based on the config options passed to the JsonWriter's
 * constructor.</p>
 *
 * <p>Writers are not needed for any kind of local storage - whether via a
 * <a href="#!/api/Ext.data.proxy.WebStorage" rel="Ext.data.proxy.WebStorage" class="docClass">Web Storage proxy</a> (see
 * <a href="#!/api/Ext.data.proxy.LocalStorage" rel="Ext.data.proxy.LocalStorage" class="docClass">localStorage</a> and
 * <a href="#!/api/Ext.data.proxy.SessionStorage" rel="Ext.data.proxy.SessionStorage" class="docClass">sessionStorage</a>)
 * or just in memory via a <a href="#!/api/Ext.data.proxy.Memory" rel="Ext.data.proxy.Memory" class="docClass">MemoryProxy</a>.</p>
 *
 * <h1>Dates</h1>
 *
 * <p>Before sending dates to the server, they can be formatted using an <a href="#!/api/Ext.Date" rel="Ext.Date" class="docClass">Ext.Date</a>
 * format. These formats can be specified both on the field and the writer itself. In terms
 * of precedence, from highest to lowest:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.data.writer.Writer-cfg-dateFormat" rel="Ext.data.writer.Writer-cfg-dateFormat" class="docClass">Writer.dateFormat</a> The writer <code>dateFormat</code> will always have the
 * highest precedence.</li>
 * <li><a href="#!/api/Ext.data.field.Date-cfg-dateWriteFormat" rel="Ext.data.field.Date-cfg-dateWriteFormat" class="docClass">Ext.data.field.Date.dateWriteFormat</a> The <code>dateWriteFormat</code> given to the field
 * instance. This is handled by <a href="#!/api/Ext.data.field.Date-method-serialize" rel="Ext.data.field.Date-method-serialize" class="docClass">Ext.data.field.Date.serialize</a>.</li>
 * <li><a href="#!/api/Ext.data.field.Date-cfg-dateFormat" rel="Ext.data.field.Date-cfg-dateFormat" class="docClass">Field.dateFormat</a> This is handled by the field's
 * <code>serialize</code> method.</li>
 * <li><a href="#!/api/Ext.data.field.Date-cfg-dateReadFormat" rel="Ext.data.field.Date-cfg-dateReadFormat" class="docClass">Field.dateReadFormat</a> Also handled by the
 * field's <code>serialize</code> method.</li>
 * </ul>
 */
public class DataWriter extends ext.Base implements ext.mixin.IFactoryable {
  /**
   * Creates new Writer.
   * @param config Config object.
   */
  public function DataWriter(config:ext.data.writer.DataWriter = null) {
    super();
  }

  /**
   * <code>true</code> in this class to identify an object as an instantiated Writer, or subclass thereof.
   * @default true
   */
  public native function get isWriter():Boolean;

  /**
   * @private
   */
  public native function set isWriter(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This object contains the options passed to <code><a href="#!/api/Ext.data.Model-method-getData" rel="Ext.data.Model-method-getData" class="docClass">Ext.data.Model.getData</a></code> when
   * writing <code><a href="#!/api/Ext.data.Model-property-phantom" rel="Ext.data.Model-property-phantom" class="docClass">Ext.data.Model.phantom</a></code> records or when <code>writeAllFields</code> is set
   * to <code>true</code>.
   *
   * <p><em>NOTE:</em> The <code>serialize</code> option cannot be used here.</p>
   * @default {persist: true}
   * @see #getAllDataOptions()
   * @see #setAllDataOptions()
   */
  public native function get allDataOptions():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set allDataOptions(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * When specified this property causes the <code><a href="#!/api/Ext.data.Model-cfg-idProperty" rel="Ext.data.Model-cfg-idProperty" class="docClass">Ext.data.Model.idProperty</a></code> of
   * newly created records to be sent to the server as this name instead of the
   * value of the <code>idProperty</code>.
   *
   * <p>For example, by default, the following code:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Person', {
   *      idProperty: 'id',  // this is the default value (for clarity)
   *
   *      fields: [ 'name' ]
   *  });
   *
   *  var person = new Person({
   *      // no id provided, so one is generated
   *      name: 'Clark Kent'
   *  });
   * </code></pre>
   *
   * <p>Will send this to the server:</p>
   *
   * <pre><code> {
   *      id: 'Person-1',
   *      name: 'Clark Kent'
   *  }
   * </code></pre>
   *
   * <p>This can be an issue if the server expects an integer for the "id" property.
   * You can use <code><a href="#!/api/Ext.data.Model-cfg-identifier" rel="Ext.data.Model-cfg-identifier" class="docClass">Ext.data.Model.identifier</a></code> to produce identifiers that
   * the server would recognize or use this config to send the client's id in a
   * different property.</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Person', {
   *      idProperty: 'id',  // this is the default value (for clarity)
   *
   *      proxy: {
   *          writer: {
   *              clientIdProperty: 'clientId'
   *          }
   *      },
   *
   *      fields: [ 'name' ]
   *  });
   * </code></pre>
   *
   * <p>Given the above, the server is sent this data now:</p>
   *
   * <pre><code> {
   *      clientId: 'Person-1',
   *      name: 'Clark Kent'
   *  }
   * </code></pre>
   *
   * <p>While this config provides the behavior of <code><a href="#!/api/Ext.data.Model-cfg-clientIdProperty" rel="Ext.data.Model-cfg-clientIdProperty" class="docClass">Ext.data.Model.clientIdProperty</a></code>
   * from previous releases, this property is not as useful as a suitable
   * <code><a href="#!/api/Ext.data.Model-cfg-identifier" rel="Ext.data.Model-cfg-identifier" class="docClass">Ext.data.Model.identifier</a></code> due to id's appearing in foreign-key fields
   * and in <code><a href="#!/api/Ext.data.Model-cfg-manyToMany" rel="Ext.data.Model-cfg-manyToMany" class="docClass">Ext.data.Model.manyToMany</a></code> associations.</p>
   *
   * <p>See <code><a href="#!/api/Ext.data.Model-cfg-identifier" rel="Ext.data.Model-cfg-identifier" class="docClass">Ext.data.Model.identifier</a></code> for more on id generation.</p>
   * @see #getClientIdProperty()
   * @see #setClientIdProperty()
   */
  public native function get clientIdProperty():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set clientIdProperty(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This is used for each field of type date in the model to format the value before
   * it is sent to the server.
   * @see #getDateFormat()
   * @see #setDateFormat()
   */
  public native function get dateFormat():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set dateFormat(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This property is used to read the key for each value that will be sent to the
   * server.
   *
   * <p>For example:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Person', {
   *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
   *     fields: [{
   *         name: 'first',
   *         mapping: 'firstName'
   *     }, {
   *         name: 'last',
   *         mapping: 'lastName'
   *     }, {
   *         name: 'age'
   *     }]
   * });
   *
   * new <a href="#!/api/Ext.data.writer.Writer" rel="Ext.data.writer.Writer" class="docClass">Ext.data.writer.Writer</a>({
   *     nameProperty: 'mapping'
   * });
   *
   * // This will be sent to the server
   * {
   *     firstName: 'first name value',
   *     lastName: 'last name value',
   *     age: 1
   * }
   * </code></pre>
   *
   * <p>If the value is not present, the field name will always be used.</p>
   * @default 'name'
   * @see #getNameProperty()
   * @see #setNameProperty()
   */
  public native function get nameProperty():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set nameProperty(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This object contains the options passed to <code><a href="#!/api/Ext.data.Model-method-getData" rel="Ext.data.Model-method-getData" class="docClass">Ext.data.Model.getData</a></code> when
   * writing non <code><a href="#!/api/Ext.data.Model-property-phantom" rel="Ext.data.Model-property-phantom" class="docClass">Ext.data.Model.phantom</a></code> records or when <code>writeAllFields</code> is
   * set to <code>false</code>.
   *
   * <p><em>NOTE:</em> The <code>serialize</code> option cannot be used here.</p>
   * @default {changes: true, critical: true}
   * @see #getPartialDataOptions()
   * @see #setPartialDataOptions()
   */
  public native function get partialDataOptions():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set partialDataOptions(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If a transform function is set, it will be invoked just before <a href="#!/api/Ext.data.writer.Writer-method-writeRecords" rel="Ext.data.writer.Writer-method-writeRecords" class="docClass">writeRecords</a>
   * executes. It is passed the unserialized data object and the <a href="#!/api/Ext.data.Request" rel="Ext.data.Request" class="docClass">request</a>
   * object. The transform function returns a data object, which can be a modified version of the original
   * data object, or a completely new data object. The transform can be a function, or an object
   * with a 'fn' key and an optional 'scope' key. Example usage:
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
   *     model: 'User',
   *     proxy: {
   *         type: 'ajax',
   *         url : 'users.json',
   *         writer: {
   *             type: 'json',
   *             transform: {
   *                 fn: function(data, request) {
   *                     // do some manipulation of the unserialized data object
   *                     return data;
   *                 },
   *                 scope: this
   *             }
   *         }
   *     },
   * });
   * </code></pre>
   * @see #getTransform()
   * @see #setTransform()
   */
  public native function get transform():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set transform(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to write all fields from the record to the
   * server. If set to <code>false</code> it will only send the fields that were modified. Note
   * that any fields that have <code><a href="#!/api/Ext.data.field.Field-cfg-persist" rel="Ext.data.field.Field-cfg-persist" class="docClass">Ext.data.field.Field.persist</a></code> set to <code>false</code>
   * will still be ignored while those with <code><a href="#!/api/Ext.data.field.Field-cfg-critical" rel="Ext.data.field.Field-cfg-critical" class="docClass">Ext.data.field.Field.critical</a></code>
   * set to <code>true</code> will be included.
   *
   * <p>The exact set of fields written is determined by <code>allDataOptions</code> (when <code>true</code>)
   * or <code>partialDataOptions</code> (when <code>false</code>). This option is ignored and treated as
   * <code>true</code> when writing <code><a href="#!/api/Ext.data.Model-property-phantom" rel="Ext.data.Model-property-phantom" class="docClass">Ext.data.Model.phantom</a></code> records.</p>
   *
   * <p>It is seldom a good idea to use this config. Rather use <code>allDataOptions</code> or
   * <code>partialDataOptions</code> to control what fields are sent for records based on their
   * <code><a href="#!/api/Ext.data.Model-property-phantom" rel="Ext.data.Model-property-phantom" class="docClass">Ext.data.Model.phantom</a></code> state.</p>
   *
   * <p>In the previous release, this was default <code>true</code>.</p>
   * @default false
   * @see #getWriteAllFields()
   * @see #setWriteAllFields()
   */
  public native function get writeAllFields():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set writeAllFields(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * By default, each record's id is always included in the output for non-phantom
   * records since in most cases the id will be required on the server to process
   * the record action. This is helpful since the id will normally not be modified,
   * and so would not be sent to the server unless <a href="#!/api/Ext.data.writer.Writer-cfg-writeAllFields" rel="Ext.data.writer.Writer-cfg-writeAllFields" class="docClass">writeAllFields</a> was
   * explicitly enabled.
   *
   * <p>However, there are cases where it is not desirable for the record id to be passed
   * in the data directly. For example, when using a RESTful API the record id would
   * typically be appended to the url instead.</p>
   * @default true
   * @see #getWriteRecordId()
   * @see #setWriteRecordId()
   */
  public native function get writeRecordId():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set writeRecordId(value:Boolean):void;

  /**
   * @inheritDoc
   */
  public native function get factoryConfig():Object;

  /**
   * @inheritDoc
   */
  public native function set factoryConfig(value:Object):void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.writer.Writer-cfg-allDataOptions" rel="Ext.data.writer.Writer-cfg-allDataOptions" class="docClass">allDataOptions</a>.
   */
  public native function getAllDataOptions():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.writer.Writer-cfg-clientIdProperty" rel="Ext.data.writer.Writer-cfg-clientIdProperty" class="docClass">clientIdProperty</a>.
   */
  public native function getClientIdProperty():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.writer.Writer-cfg-dateFormat" rel="Ext.data.writer.Writer-cfg-dateFormat" class="docClass">dateFormat</a>.
   */
  public native function getDateFormat():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.writer.Writer-cfg-nameProperty" rel="Ext.data.writer.Writer-cfg-nameProperty" class="docClass">nameProperty</a>.
   */
  public native function getNameProperty():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.writer.Writer-cfg-partialDataOptions" rel="Ext.data.writer.Writer-cfg-partialDataOptions" class="docClass">partialDataOptions</a>.
   */
  public native function getPartialDataOptions():Object;

  /**
   * Formats the data for each record before sending it to the server. This method should
   * be overridden to format the data in a way that differs from the default.
   * @param record The record that we are writing to the server.
   * @param operation An operation object.
   * @return An object literal of name/value keys to be written to the server.
   * By default this method returns the data property on the record.
   */
  public native function getRecordData(record:ext.data.Model, operation:ext.data.operation.Operation = null):Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.writer.Writer-cfg-transform" rel="Ext.data.writer.Writer-cfg-transform" class="docClass">transform</a>.
   */
  public native function getTransform():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.writer.Writer-cfg-writeAllFields" rel="Ext.data.writer.Writer-cfg-writeAllFields" class="docClass">writeAllFields</a>.
   */
  public native function getWriteAllFields():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.writer.Writer-cfg-writeRecordId" rel="Ext.data.writer.Writer-cfg-writeRecordId" class="docClass">writeRecordId</a>.
   */
  public native function getWriteRecordId():Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.data.writer.Writer-cfg-allDataOptions" rel="Ext.data.writer.Writer-cfg-allDataOptions" class="docClass">allDataOptions</a>.
   * @param allDataOptions The new value.
   */
  public native function setAllDataOptions(allDataOptions:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.writer.Writer-cfg-clientIdProperty" rel="Ext.data.writer.Writer-cfg-clientIdProperty" class="docClass">clientIdProperty</a>.
   * @param clientIdProperty The new value.
   */
  public native function setClientIdProperty(clientIdProperty:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.writer.Writer-cfg-dateFormat" rel="Ext.data.writer.Writer-cfg-dateFormat" class="docClass">dateFormat</a>.
   * @param dateFormat The new value.
   */
  public native function setDateFormat(dateFormat:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.writer.Writer-cfg-nameProperty" rel="Ext.data.writer.Writer-cfg-nameProperty" class="docClass">nameProperty</a>.
   * @param nameProperty The new value.
   */
  public native function setNameProperty(nameProperty:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.writer.Writer-cfg-partialDataOptions" rel="Ext.data.writer.Writer-cfg-partialDataOptions" class="docClass">partialDataOptions</a>.
   * @param partialDataOptions The new value.
   */
  public native function setPartialDataOptions(partialDataOptions:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.writer.Writer-cfg-transform" rel="Ext.data.writer.Writer-cfg-transform" class="docClass">transform</a>.
   * @param transform The new value.
   */
  public native function setTransform(transform:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.writer.Writer-cfg-writeAllFields" rel="Ext.data.writer.Writer-cfg-writeAllFields" class="docClass">writeAllFields</a>.
   * @param writeAllFields The new value.
   */
  public native function setWriteAllFields(writeAllFields:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.writer.Writer-cfg-writeRecordId" rel="Ext.data.writer.Writer-cfg-writeRecordId" class="docClass">writeRecordId</a>.
   * @param writeRecordId The new value.
   */
  public native function setWriteRecordId(writeRecordId:Boolean):void;

  /**
   * Prepares a Proxy's <a href="#!/api/Ext.data.Request" rel="Ext.data.Request" class="docClass">Ext.data.Request</a> object.
   * @param request The request object.
   * @return The modified request object.
   */
  public native function write(request:ext.data.Request):ext.data.Request;

  /**
   * Write the record data to the request in the appropriate format.
   * @param request The request.
   * @param data An array of objects containing data.
   * @return The request.
   */
  protected native function writeRecords(request:ext.data.Request, data:Array):ext.data.Request;
}
}