package ext.dom {

[Uses("ext.dom.SDomQuery")]
[Uses("ext.Base")]
[Native("Ext.dom.Query", require)]
/**
 * Provides high performance selector/xpath processing by compiling queries into reusable functions. New pseudo classes
 * and matchers can be plugged. It works on HTML and XML documents (if a content node is passed in).
 *
 * <p>DomQuery supports most of the <a href="http://www.w3.org/TR/2005/WD-css3-selectors-20051215/#selectors">CSS3 selectors spec</a>, along with some custom selectors and basic XPath.</p>
 *
 * <p>All selectors, attribute filters and pseudos below can be combined infinitely in any order. For example
 * <code>div.foo:nth-child(odd)[&#64;foo=bar].bar:first</code> would be a perfectly valid selector. Node filters are processed
 * in the order in which they appear, which allows you to optimize your queries for your document structure.</p>
 *
 * <h2>Simple Selectors</h2>
 *
 * <p>For performance reasons, some query methods accept selectors that are termed as <strong>simple selectors</strong>. A simple
 * selector is a selector that does not include contextual information about any parent/sibling elements.</p>
 *
 * <p>Some examples of valid simple selectors:</p>
 *
 * <pre><code>var simple = '.foo'; // Only asking for the class name on the element
 * var simple = 'div.bar'; // Only asking for the tag/class name on the element
 * var simple = '[href];' // Asking for an attribute on the element.
 * var simple = ':not(.foo)'; // Only asking for the non-matches against the class name
 * var simple = 'span:first-child'; // Doesn't require any contextual information about the parent node
 * </code></pre>
 *
 * <p>Simple examples of invalid simple selectors:</p>
 *
 * <pre><code>var notSimple = 'div.foo div.bar'; // Requires matching a parent node by class name
 * var notSimple = 'span + div'; //  Requires matching a sibling by tag name
 * </code></pre>
 *
 * <h2>Element Selectors:</h2>
 *
 * <ul>
 * <li><strong><code>*</code></strong> any element</li>
 * <li><strong><code>E</code></strong> an element with the tag E</li>
 * <li><strong><code>E F</code></strong> All descendent elements of E that have the tag F</li>
 * <li><strong><code>E &gt; F</code></strong> or <strong>E/F</strong> all direct children elements of E that have the tag F</li>
 * <li><strong><code>E + F</code></strong> all elements with the tag F that are immediately preceded by an element with the tag E</li>
 * <li><strong><code>E ~ F</code></strong> all elements with the tag F that are preceded by a sibling element with the tag E</li>
 * </ul>
 *
 *
 * <h2>Attribute Selectors:</h2>
 *
 * <p>The use of <code>&#64;</code> and quotes are optional. For example, <code>div[&#64;foo='bar']</code> is also a valid attribute selector.</p>
 *
 * <ul>
 * <li><strong><code>E[foo]</code></strong> has an attribute "foo"</li>
 * <li><strong><code>E[foo=bar]</code></strong> has an attribute "foo" that equals "bar"</li>
 * <li><strong><code>E[foo^=bar]</code></strong> has an attribute "foo" that starts with "bar"</li>
 * <li><strong><code>E[foo$=bar]</code></strong> has an attribute "foo" that ends with "bar"</li>
 * <li><strong><code>E[foo*=bar]</code></strong> has an attribute "foo" that contains the substring "bar"</li>
 * <li><strong><code>E[foo%=2]</code></strong> has an attribute "foo" that is evenly divisible by 2</li>
 * <li><strong><code>E[foo!=bar]</code></strong> attribute "foo" does not equal "bar"</li>
 * </ul>
 *
 *
 * <h2>Pseudo Classes:</h2>
 *
 * <ul>
 * <li><strong><code>E:first-child</code></strong> E is the first child of its parent</li>
 * <li><strong><code>E:last-child</code></strong> E is the last child of its parent</li>
 * <li><strong><code>E:nth-child(_n_)</code></strong> E is the _n_th child of its parent (1 based as per the spec)</li>
 * <li><strong><code>E:nth-child(odd)</code></strong> E is an odd child of its parent</li>
 * <li><strong><code>E:nth-child(even)</code></strong> E is an even child of its parent</li>
 * <li><strong><code>E:only-child</code></strong> E is the only child of its parent</li>
 * <li><strong><code>E:checked</code></strong> E is an element that is has a checked attribute that is true (e.g. a radio or checkbox)</li>
 * <li><strong><code>E:first</code></strong> the first E in the resultset</li>
 * <li><strong><code>E:last</code></strong> the last E in the resultset</li>
 * <li><strong><code>E:nth(_n_)</code></strong> the _n_th E in the resultset (1 based)</li>
 * <li><strong><code>E:odd</code></strong> shortcut for :nth-child(odd)</li>
 * <li><strong><code>E:even</code></strong> shortcut for :nth-child(even)</li>
 * <li><strong><code>E:contains(foo)</code></strong> E's innerHTML contains the substring "foo"</li>
 * <li><strong><code>E:nodeValue(foo)</code></strong> E contains a textNode with a nodeValue that equals "foo"</li>
 * <li><strong><code>E:not(S)</code></strong> an E element that does not match simple selector S</li>
 * <li><strong><code>E:has(S)</code></strong> an E element that has a descendent that matches simple selector S</li>
 * <li><strong><code>E:next(S)</code></strong> an E element whose next sibling matches simple selector S</li>
 * <li><strong><code>E:prev(S)</code></strong> an E element whose previous sibling matches simple selector S</li>
 * <li><strong><code>E:any(S1|S2|S2)</code></strong> an E element which matches any of the simple selectors S1, S2 or S3</li>
 * <li><strong><code>E:visible(true)</code></strong> an E element which is deeply visible according to <a href="#!/api/Ext.dom.Element-method-isVisible" rel="Ext.dom.Element-method-isVisible" class="docClass">Ext.dom.Element.isVisible</a></li>
 * </ul>
 *
 *
 * <h2>CSS Value Selectors:</h2>
 *
 * <ul>
 * <li><strong><code>E{display=none}</code></strong> css value "display" that equals "none"</li>
 * <li><strong><code>E{display^=none}</code></strong> css value "display" that starts with "none"</li>
 * <li><strong><code>E{display$=none}</code></strong> css value "display" that ends with "none"</li>
 * <li><strong><code>E{display*=none}</code></strong> css value "display" that contains the substring "none"</li>
 * <li><strong><code>E{display%=2}</code></strong> css value "display" that is evenly divisible by 2</li>
 * <li><strong><code>E{display!=none}</code></strong> css value "display" that does not equal "none"</li>
 * </ul>
 *
 *
 * <h2>XML Namespaces:</h2>
 *
 * <ul>
 * <li><strong><code>ns|E</code></strong> an element with tag E and namespace prefix ns</li>
 * </ul>
 */
public const DomQuery:ext.dom.SDomQuery;
}