package ext.form {
import ext.Component;

[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.form.Label", require)]
/**
 * Produces a standalone <code>&lt;label /&gt;</code> element which can be inserted into a form and be associated with a field
 * in that form using the <a href="#!/api/Ext.form.Label-cfg-forId" rel="Ext.form.Label-cfg-forId" class="docClass">forId</a> property.
 *
 * <p><strong>NOTE:</strong> in most cases it will be more appropriate to use the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a>
 * and associated config properties (<a href="#!/api/Ext.form.Labelable-cfg-labelAlign" rel="Ext.form.Labelable-cfg-labelAlign" class="docClass">Ext.form.Labelable.labelAlign</a>, <a href="#!/api/Ext.form.Labelable-cfg-labelWidth" rel="Ext.form.Labelable-cfg-labelWidth" class="docClass">Ext.form.Labelable.labelWidth</a>,
 * etc.) in field components themselves, as that allows labels to be uniformly sized throughout the form.
 * <a href="#!/api/Ext.form.Label" rel="Ext.form.Label" class="docClass">Ext.form.Label</a> should only be used when your layout can not be achieved with the standard
 * <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">field layout</a>.</p>
 *
 * <p>You will likely be associating the label with a field component that extends <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a>, so
 * you should make sure the <a href="#!/api/Ext.form.Label-cfg-forId" rel="Ext.form.Label-cfg-forId" class="docClass">forId</a> is set to the same value as the <a href="#!/api/Ext.form.field.Base-cfg-inputId" rel="Ext.form.field.Base-cfg-inputId" class="docClass">inputId</a>
 * of that field.</p>
 *
 * <p>The label's text can be set using either the <a href="#!/api/Ext.form.Label-cfg-text" rel="Ext.form.Label-cfg-text" class="docClass">text</a> or <a href="#!/api/Ext.form.Label-cfg-html" rel="Ext.form.Label-cfg-html" class="docClass">html</a> configuration properties; the
 * difference between the two is that the former will automatically escape HTML characters when rendering, while
 * the latter will not.</p>
 *
 * <h1>Example</h1>
 *
 * <p>This example creates a Label after its associated Text field, an arrangement that cannot currently
 * be achieved using the standard Field layout's labelAlign.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'Field with Label',
 *     width: 400,
 *     bodyPadding: 10,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     layout: {
 *         type: 'hbox',
 *         align: 'middle'
 *     },
 *     items: [{
 *         xtype: 'textfield',
 *         hideLabel: true,
 *         flex: 1
 *     }, {
 *         xtype: 'label',
 *         forId: 'myFieldId',
 *         text: 'My Awesome Field',
 *         margin: '0 0 0 10'
 *     }]
 * });
 * </code></pre>
 */
public class Label extends ext.Component {
  /**
   * @param config @inheritDoc
   */
  public function Label(config:ext.form.Label = null) {
    super();
  }

  [ExtConfig]
  /**
   * An HTML fragment that will be used as the label's innerHTML.
   * Note that if <a href="#!/api/Ext.form.Label-cfg-text" rel="Ext.form.Label-cfg-text" class="docClass">text</a> is specified it will take precedence and this value will be ignored.
   * @default ''
   */
  override public native function get html():*;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set html(value:*):void;

  [ExtConfig]
  /**
   * The id of the input element to which this label will be bound via the standard HTML 'for'
   * attribute. If not specified, the attribute will not be added to the label. In most cases you will be
   * associating the label with a <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> component, so you should make sure this matches
   * the <a href="#!/api/Ext.form.field.Base-cfg-inputId" rel="Ext.form.field.Base-cfg-inputId" class="docClass">inputId</a> of that field.
   */
  public native function get forId():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set forId(value:String):void;

  [ExtConfig]
  /**
   * The plain text to display within the label. If you need to include HTML
   * tags within the label's innerHTML, use the <a href="#!/api/Ext.form.Label-cfg-html" rel="Ext.form.Label-cfg-html" class="docClass">html</a> config instead.
   * @default ''
   * @see #setText()
   */
  public native function get text():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set text(value:String):void;

  /**
   * Updates the label's innerHTML with the specified string.
   * @param text The new label text
   * @param encode False to skip HTML-encoding the text when rendering it
   * to the label. This might be useful if you want to include tags in the label's innerHTML rather
   * than rendering them as string literals per the default logic.
   * @default true
   * @return this
   */
  public native function setText(text:String, encode:Boolean = true):ext.form.Label;
}
}