package ext.form.field {
import ext.data.AbstractStore;
import ext.data.Model;
import ext.data.Store;
import ext.util.Collection;
import ext.util.IStoreHolder;

[Uses("ext.form.field.Picker")]
[Uses("ext.util.StoreHolder")]
[Uses("ext.Base")]
[Uses("ext.form.field.TextField")]
[Uses("ext.form.field.BaseField")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Native("Ext.form.field.ComboBox", require)]
/**
 * Fires before the deselected item is removed from the collection
 * @eventType ext.form.field.events.ComboBox_record_indexEvent.BEFORE_DESELECT
 */
[Event(name = 'onBeforeDeselect', type = 'ext.form.field.events.ComboBox_record_indexEvent')]
/**
 * Fires before all queries are processed. Return false to cancel the query or set the queryPlan's cancel
 * property to true.
 * @eventType ext.form.field.events.ComboBox_queryPlanEvent.BEFORE_QUERY
 */
[Event(name = 'onBeforeQuery', type = 'ext.form.field.events.ComboBox_queryPlanEvent')]
/**
 * Fires before the selected item is added to the collection
 * @eventType ext.form.field.events.ComboBox_record_indexEvent.BEFORE_SELECT
 */
[Event(name = 'onBeforeSelect', type = 'ext.form.field.events.ComboBox_record_indexEvent')]
/**
 * Fires when at least one list item is selected.
 * @eventType ext.form.field.events.ComboBox_recordEvent.SELECT
 */
[Event(name = 'onSelect', type = 'ext.form.field.events.ComboBox_recordEvent')]
/**
 * A combobox control with support for autocomplete, remote loading, and many other features.
 *
 * <p>A ComboBox is like a combination of a traditional HTML text <code>&lt;input&gt;</code> field and a <code>&lt;select&gt;</code>
 * field; the user is able to type freely into the field, and/or pick values from a dropdown selection
 * list. The user can input any value by default, even if it does not appear in the selection list;
 * to prevent free-form values and restrict them to items in the list, set <a href="#!/api/Ext.form.field.ComboBox-cfg-forceSelection" rel="Ext.form.field.ComboBox-cfg-forceSelection" class="docClass">forceSelection</a> to <code>true</code>.</p>
 *
 * <p>The selection list's options are populated from any <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>, including remote
 * stores. The data items in the store are mapped to each option's displayed text and backing value via
 * the <a href="#!/api/Ext.form.field.ComboBox-cfg-valueField" rel="Ext.form.field.ComboBox-cfg-valueField" class="docClass">valueField</a> and <a href="#!/api/Ext.form.field.ComboBox-cfg-displayField" rel="Ext.form.field.ComboBox-cfg-displayField" class="docClass">displayField</a> configurations, respectively.</p>
 *
 * <p>If your store is not remote, i.e. it depends only on local data and is loaded up front, you should be
 * sure to set the <a href="#!/api/Ext.form.field.ComboBox-cfg-queryMode" rel="Ext.form.field.ComboBox-cfg-queryMode" class="docClass">queryMode</a> to <code>'local'</code>, as this will improve responsiveness for the user.</p>
 *
 * <h1>Example usage:</h1>
 *
 * <pre class='inline-example '><code>// The data store containing the list of states
 * var states = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     fields: ['abbr', 'name'],
 *     data : [
 *         {"abbr":"AL", "name":"Alabama"},
 *         {"abbr":"AK", "name":"Alaska"},
 *         {"abbr":"AZ", "name":"Arizona"}
 *     ]
 * });
 *
 * // Create the combo box, attached to the states data store
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.ComboBox" rel="Ext.form.field.ComboBox" class="docClass">Ext.form.ComboBox</a>', {
 *     fieldLabel: 'Choose State',
 *     store: states,
 *     queryMode: 'local',
 *     displayField: 'name',
 *     valueField: 'abbr',
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <h1>Events</h1>
 *
 * <p>To do something when something in ComboBox is selected, configure the select event:</p>
 *
 * <pre><code>var cb = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.ComboBox" rel="Ext.form.field.ComboBox" class="docClass">Ext.form.ComboBox</a>', {
 *     // all of your config options
 *     listeners:{
 *          scope: yourScope,
 *          'select': yourFunction
 *     }
 * });
 *
 * // Alternatively, you can assign events after the object is created:
 * var cb = new <a href="#!/api/Ext.form.field.ComboBox" rel="Ext.form.field.ComboBox" class="docClass">Ext.form.field.ComboBox</a>(yourOptions);
 * cb.on('select', yourFunction, yourScope);
 * </code></pre>
 *
 * <h1>Multiple Selection</h1>
 *
 * <p>The <a href="#!/api/Ext.form.field.ComboBox-cfg-multiSelect" rel="Ext.form.field.ComboBox-cfg-multiSelect" class="docClass">multiSelect</a> config is deprecated.  For multiple selection use
 * <a href="#!/api/Ext.form.field.Tag" rel="Ext.form.field.Tag" class="docClass">Ext.form.field.Tag</a> or <a href="#!/api/Ext.view.MultiSelector" rel="Ext.view.MultiSelector" class="docClass">Ext.view.MultiSelector</a>.</p>
 *
 * <h1>Filtered Stores</h1>
 *
 * <p>If you have a local store that is already filtered, you can use the <a href="#!/api/Ext.form.field.ComboBox-property-lastQuery" rel="Ext.form.field.ComboBox-property-lastQuery" class="docClass">lastQuery</a> config option
 * to prevent the store from having the filter being cleared on first expand.</p>
 *
 * <h2>Customized combobox</h2>
 *
 * <p>Both the text shown in dropdown menu and text field can be easily customized:</p>
 *
 * <pre class='inline-example '><code>var states = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     fields: ['abbr', 'name'],
 *     data : [
 *         {"abbr":"AL", "name":"Alabama"},
 *         {"abbr":"AK", "name":"Alaska"},
 *         {"abbr":"AZ", "name":"Arizona"}
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.ComboBox" rel="Ext.form.field.ComboBox" class="docClass">Ext.form.ComboBox</a>', {
 *     fieldLabel: 'Choose State',
 *     store: states,
 *     queryMode: 'local',
 *     valueField: 'abbr',
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     // Template for the dropdown menu.
 *     // Note the use of the "x-list-plain" and "x-boundlist-item" class,
 *     // this is required to make the items selectable.
 *     tpl: <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a>',
 *         '&lt;ul class="x-list-plain"&gt;&lt;tpl for="."&gt;',
 *             '&lt;li role="option" class="x-boundlist-item"&gt;{abbr} - {name}&lt;/li&gt;',
 *         '&lt;/tpl&gt;&lt;/ul&gt;'
 *     ),
 *     // template for the content inside text field
 *     displayTpl: <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a>',
 *         '&lt;tpl for="."&gt;',
 *             '{abbr} - {name}',
 *         '&lt;/tpl&gt;'
 *     )
 * });
 * </code></pre>
 *
 * <p>See also the <a href="#!/api/Ext.form.field.ComboBox-cfg-listConfig" rel="Ext.form.field.ComboBox-cfg-listConfig" class="docClass">listConfig</a> option for additional configuration of the dropdown.</p>
 */
public class ComboBox extends ext.form.field.Picker implements ext.util.IStoreHolder {
  /**
   * @param config @inheritDoc
   */
  public function ComboBox(config:ext.form.field.ComboBox = null) {
    super();
  }

  [ExtConfig]
  /**
   * An additional CSS class used to style the trigger button. The trigger will always get the <a href="#!/api/Ext.form.trigger.Trigger-property-baseCls" rel="Ext.form.trigger.Trigger-property-baseCls" class="docClass">Ext.form.trigger.Trigger.baseCls</a>
   * by default and <code>triggerCls</code> will be <strong>appended</strong> if specified.
   * @default 'x-form-arrow-trigger'
   */
  override public native function get triggerCls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set triggerCls(value:String):void;

  /**
   * The value of the match string used to filter the store. Delete this property to force a requery. Example use:
   *
   * <pre><code>var combo = new <a href="#!/api/Ext.form.field.ComboBox" rel="Ext.form.field.ComboBox" class="docClass">Ext.form.field.ComboBox</a>({
   *     ...
   *     queryMode: 'remote',
   *     listeners: {
   *         // delete the previous query in the beforequery event or set
   *         // combo.lastQuery = null (this will reload the store the next time it expands)
   *         beforequery: function(qe){
   *             delete qe.combo.lastQuery;
   *         }
   *     }
   * });
   * </code></pre>
   *
   * <p>To make sure the filter in the store is not cleared the first time the ComboBox trigger is used configure the
   * combo with <code>lastQuery=''</code>. Example use:</p>
   *
   * <pre><code>var combo = new <a href="#!/api/Ext.form.field.ComboBox" rel="Ext.form.field.ComboBox" class="docClass">Ext.form.field.ComboBox</a>({
   *     ...
   *     queryMode: 'local',
   *     triggerAction: 'all',
   *     lastQuery: ''
   * });
   * </code></pre>
   */
  public native function get lastQuery():String;

  /**
   * @private
   */
  public native function set lastQuery(value:String):void;

  [ExtConfig]
  /**
   * The text query to send to the server to return all records for the list with no filtering
   * @default ''
   */
  public native function get allQuery():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allQuery(value:String):void;

  [ExtConfig]
  /**
   * Configure as <code>true</code> to allow matching of the typed characters at any position in the <a href="#!/api/Ext.form.field.ComboBox-cfg-valueField" rel="Ext.form.field.ComboBox-cfg-valueField" class="docClass">valueField</a>'s value.
   * @default false
   */
  public native function get anyMatch():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set anyMatch(value:Boolean):void;

  [ExtConfig]
  /**
   * This option controls whether to <em>initially</em> load the store when a value is set so that
   * the display value can be determined from the appropriate record.
   * The store will only be loaded in a limited set of circumstances:
   * - The store is not currently loading.
   * - The store does not have a pending <a href="#!/api/Ext.data.Store-cfg-autoLoad" rel="Ext.data.Store-cfg-autoLoad" class="docClass">Ext.data.Store.autoLoad</a>.
   * - The store has not been loaded before.
   * @default false
   */
  public native function get autoLoadOnValue():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoLoadOnValue(value:Boolean):void;

  [ExtConfig]
  /**
   * <code>true</code> to automatically highlight the first result gathered by the data store in the dropdown list when it is
   * opened. A false value would cause nothing in the list to be highlighted automatically, so
   * the user would have to manually highlight an item before pressing the enter or <a href="#!/api/Ext.form.field.ComboBox-cfg-selectOnTab" rel="Ext.form.field.ComboBox-cfg-selectOnTab" class="docClass">tab</a> key to
   * select it (unless the value of (<a href="#!/api/Ext.form.field.ComboBox-cfg-typeAhead" rel="Ext.form.field.ComboBox-cfg-typeAhead" class="docClass">typeAhead</a>) were true), or use the mouse to select a value.
   * @default true
   */
  public native function get autoSelect():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoSelect(value:Boolean):void;

  [ExtConfig]
  /**
   * When <code>true</code>, the last selected record in the dropdown
   * list will be re-selected upon <a href="#!/api/Ext.form.field.ComboBox-cfg-autoSelect" rel="Ext.form.field.ComboBox-cfg-autoSelect" class="docClass">autoSelect</a>. Set to <code>false</code> to always select the first
   * record in the drop-down list.
   * For accessible applications it is recommended to set this option to <code>false</code>.
   * @default true
   */
  public native function get autoSelectLast():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoSelectLast(value:Boolean):void;

  [ExtConfig]
  /**
   * Configure as <code>true</code> to make the filtering match with exact case matching
   * @default false
   */
  public native function get caseSensitive():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set caseSensitive(value:Boolean):void;

  [ExtConfig]
  /**
   * <em>When <a href="#!/api/Ext.form.field.ComboBox-cfg-queryMode" rel="Ext.form.field.ComboBox-cfg-queryMode" class="docClass">queryMode</a> is <code>'local'</code> only</em>
   *
   * <p>As text is entered, the underlying store is filtered to match the value. When this option is <code>true</code>,
   * any filtering applied by this field will be cleared when focus is removed &amp; reinstated on focus.
   * If <code>false</code>, the filters will be left in place.</p>
   * @default true
   */
  public native function get clearFilterOnBlur():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set clearFilterOnBlur(value:Boolean):void;

  [Deprecated(replacement = "ext.form.field.Tag", since = "5.1.0")]
  [ExtConfig]
  /**
   * Has no effect if <a href="#!/api/Ext.form.field.ComboBox-cfg-multiSelect" rel="Ext.form.field.ComboBox-cfg-multiSelect" class="docClass">multiSelect</a> is <code>false</code>
   *
   * <p>Configure as true to automatically collapse the pick list after a selection is made.</p>
   * @default false
   */
  public native function get collapseOnSelect():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set collapseOnSelect(value:Boolean):void;

  [ExtConfig]
  /**
   * Set of options that will be used as defaults for the user-configured <a href="#!/api/Ext.form.field.ComboBox-cfg-listConfig" rel="Ext.form.field.ComboBox-cfg-listConfig" class="docClass">listConfig</a> object.
   * @default {loadingHeight: 70, minWidth: 70, maxHeight: 300, shadow: 'sides'}
   */
  public native function get defaultListConfig():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultListConfig(value:Object):void;

  [Deprecated(replacement = "ext.form.field.Tag", since = "5.1.0")]
  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The character(s) used to separate the <a href="#!/api/Ext.form.field.ComboBox-cfg-displayField" rel="Ext.form.field.ComboBox-cfg-displayField" class="docClass">display values</a> of multiple selected items when
   * <code><a href="#!/api/Ext.form.field.ComboBox-cfg-multiSelect" rel="Ext.form.field.ComboBox-cfg-multiSelect" class="docClass">multiSelect</a> = true</code>.
   * @default ', '
   * @see #getDelimiter()
   * @see #setDelimiter()
   */
  public native function get delimiter():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set delimiter(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The underlying <a href="#!/api/Ext.data.field.Field-cfg-name" rel="Ext.data.field.Field-cfg-name" class="docClass">data field name</a> to bind to this ComboBox.
   *
   * <p>See also <code><a href="#!/api/Ext.form.field.ComboBox-cfg-valueField" rel="Ext.form.field.ComboBox-cfg-valueField" class="docClass">valueField</a></code>.</p>
   * @default 'text'
   * @see #getDisplayField()
   * @see #setDisplayField()
   */
  public native function get displayField():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set displayField(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The template to be used to display selected records inside the text field. An array of the selected records' data
   * will be passed to the template. Defaults to:
   *
   * <pre><code>'&lt;tpl for="."&gt;' +
   *     '{[typeof values === "string" ? values : values["' + me.displayField + '"]]}' +
   *     '&lt;tpl if="xindex &lt; xcount"&gt;' + me.delimiter + '&lt;/tpl&gt;' +
   * '&lt;/tpl&gt;'
   * </code></pre>
   *
   * <p>By default only the immediate data of the record is passed (no associated data). The <a href="#!/api/Ext.form.field.ComboBox-method-getRecordDisplayData" rel="Ext.form.field.ComboBox-method-getRecordDisplayData" class="docClass">getRecordDisplayData</a> can
   * be overridden to extend this.</p>
   * @see #getDisplayTpl()
   * @see #setDisplayTpl()
   */
  public native function get displayTpl():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set displayTpl(value:*):void;

  [ExtConfig]
  /**
   * <em>When <a href="#!/api/Ext.form.field.ComboBox-cfg-queryMode" rel="Ext.form.field.ComboBox-cfg-queryMode" class="docClass">queryMode</a> is <code>'local'</code> only</em>
   *
   * <p>Set to <code>true</code> to have the ComboBox use the typed value as a RegExp source to filter the store to get possible matches.
   * Invalid regex values will be ignored.</p>
   */
  public native function get enableRegEx():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableRegEx(value:Boolean):void;

  [ExtConfig]
  /**
   * <code>true</code> to restrict the selected value to one of the values in the list, <code>false</code> to allow the user to set
   * arbitrary text into the field.
   * @default false
   */
  public native function get forceSelection():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set forceSelection(value:Boolean):void;

  [ExtConfig]
  /**
   * <code>false</code> to not allow the component to resize itself when its data changes
   * (and its <a href="#!/api/Ext.form.field.ComboBox-cfg-grow" rel="Ext.form.field.ComboBox-cfg-grow" class="docClass">grow</a> property is <code>true</code>)
   * @default true
   */
  public native function get growToLongestValue():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set growToLongestValue(value:Boolean):void;

  [ExtConfig]
  /**
   * The name of an underlying hidden field which will be synchronized with the underlying value of the combo.
   * This option is useful if the combo is part of a form element doing a regular form post. The hidden field
   * will not be created unless a hiddenName is specified.
   * @default ""
   */
  public native function get hiddenName():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hiddenName(value:String):void;

  [ExtConfig]
  /**
   * An optional set of configuration properties that will be passed to the <a href="#!/api/Ext.view.BoundList" rel="Ext.view.BoundList" class="docClass">Ext.view.BoundList</a>'s constructor.
   * Any configuration that is valid for BoundList can be included. Some of the more useful ones are:
   *
   * <ul>
   * <li><a href="#!/api/Ext.view.BoundList-cfg-cls" rel="Ext.view.BoundList-cfg-cls" class="docClass">cls</a> - defaults to empty</li>
   * <li><a href="#!/api/Ext.view.BoundList-cfg-emptyText" rel="Ext.view.BoundList-cfg-emptyText" class="docClass">emptyText</a> - defaults to empty string</li>
   * <li><a href="#!/api/Ext.view.BoundList-cfg-itemSelector" rel="Ext.view.BoundList-cfg-itemSelector" class="docClass">itemSelector</a> - defaults to the value defined in BoundList</li>
   * <li><a href="#!/api/Ext.view.BoundList-cfg-loadingText" rel="Ext.view.BoundList-cfg-loadingText" class="docClass">loadingText</a> - defaults to <code>'Loading...'</code></li>
   * <li><a href="#!/api/Ext.view.BoundList-cfg-minWidth" rel="Ext.view.BoundList-cfg-minWidth" class="docClass">minWidth</a> - defaults to <code>70</code></li>
   * <li><a href="#!/api/Ext.view.BoundList-cfg-maxWidth" rel="Ext.view.BoundList-cfg-maxWidth" class="docClass">maxWidth</a> - defaults to <code>undefined</code></li>
   * <li><a href="#!/api/Ext.view.BoundList-cfg-maxHeight" rel="Ext.view.BoundList-cfg-maxHeight" class="docClass">maxHeight</a> - defaults to <code>300</code></li>
   * <li><a href="#!/api/Ext.view.BoundList-cfg-resizable" rel="Ext.view.BoundList-cfg-resizable" class="docClass">resizable</a> - defaults to <code>false</code></li>
   * <li><a href="#!/api/Ext.view.BoundList-cfg-shadow" rel="Ext.view.BoundList-cfg-shadow" class="docClass">shadow</a> - defaults to <code>'sides'</code></li>
   * <li><a href="#!/api/Ext.view.BoundList-cfg-width" rel="Ext.view.BoundList-cfg-width" class="docClass">width</a> - defaults to <code>undefined</code> (automatically set to the width of the ComboBox
   * field if <a href="#!/api/Ext.form.field.ComboBox-cfg-matchFieldWidth" rel="Ext.form.field.ComboBox-cfg-matchFieldWidth" class="docClass">matchFieldWidth</a> is true)</li>
   * <li><p><a href="#!/api/Ext.view.BoundList-method-getInnerTpl" rel="Ext.view.BoundList-method-getInnerTpl" class="docClass">getInnerTpl</a> A function which returns a template string which renders
   * the ComboBox's <a href="#!/api/Ext.form.field.ComboBox-cfg-displayField" rel="Ext.form.field.ComboBox-cfg-displayField" class="docClass">displayField</a> value in the dropdown. This defaults to just outputting the raw value,
   * but may use any <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">XTemplate</a> methods to produce output.</p>
   *
   * <p>The running template is configured with some extra properties that provide some context:</p>
   *
   * <pre><code>- field <a href="#!/api/Ext.form.field.ComboBox" rel="Ext.form.field.ComboBox" class="docClass">ComboBox</a> This combobox
   * - store <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a> This combobox's data store
   * </code></pre></li>
   * </ul>
   */
  public native function get listConfig():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set listConfig(value:Object):void;

  [ExtConfig]
  /**
   * The minimum number of characters the user must type before autocomplete and <a href="#!/api/Ext.form.field.ComboBox-cfg-typeAhead" rel="Ext.form.field.ComboBox-cfg-typeAhead" class="docClass">typeAhead</a> activate.
   *
   * <p>Defaults to <code>4</code> if <code><a href="#!/api/Ext.form.field.ComboBox-cfg-queryMode" rel="Ext.form.field.ComboBox-cfg-queryMode" class="docClass">queryMode</a> = 'remote'</code> or <code>0</code> if <code><a href="#!/api/Ext.form.field.ComboBox-cfg-queryMode" rel="Ext.form.field.ComboBox-cfg-queryMode" class="docClass">queryMode</a> = 'local'</code>,
   * does not apply if <code><a href="#!/api/Ext.form.field.Trigger-cfg-editable" rel="Ext.form.field.Trigger-cfg-editable" class="docClass">editable</a> = false</code>.</p>
   */
  public native function get minChars():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set minChars(value:Number):void;

  [Deprecated(replacement = "ext.view.MultiSelectorView", since = "5.1.0")]
  [ExtConfig]
  /**
   * If set to <code>true</code>, allows the combo field to hold more than one value at a time, and allows selecting multiple
   * items from the dropdown list. The combo's text field will show all selected values separated by the
   * <a href="#!/api/Ext.form.field.ComboBox-cfg-delimiter" rel="Ext.form.field.ComboBox-cfg-delimiter" class="docClass">delimiter</a>.
   * @default false
   */
  public native function get multiSelect():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set multiSelect(value:Boolean):void;

  [ExtConfig]
  /**
   * If greater than <code>0</code>, a <a href="#!/api/Ext.toolbar.Paging" rel="Ext.toolbar.Paging" class="docClass">Ext.toolbar.Paging</a> is displayed in the footer of the dropdown list and the
   * <a href="#!/api/Ext.form.field.ComboBox-method-doQuery" rel="Ext.form.field.ComboBox-method-doQuery" class="docClass">filter queries</a> will execute with page start and <a href="#!/api/Ext.view.BoundList-cfg-pageSize" rel="Ext.view.BoundList-cfg-pageSize" class="docClass">limit</a>
   * parameters. Only applies when <code><a href="#!/api/Ext.form.field.ComboBox-cfg-queryMode" rel="Ext.form.field.ComboBox-cfg-queryMode" class="docClass">queryMode</a> = 'remote'</code>.
   * @default 0
   */
  public native function get pageSize():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set pageSize(value:Number):void;

  [Deprecated(replacement = "ext.form.field.Tag", since = "5.1.0")]
  [ExtConfig]
  /**
   * Has no effect if <a href="#!/api/Ext.form.field.ComboBox-cfg-multiSelect" rel="Ext.form.field.ComboBox-cfg-multiSelect" class="docClass">multiSelect</a> is <code>false</code>
   *
   * <p>Configure as <code>false</code> to automatically collapse the pick list after a selection is made.</p>
   * @default true
   */
  public native function get pinList():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set pinList(value:Boolean):void;

  [ExtConfig]
  /**
   * When true, this prevents the combo from re-querying (either locally or remotely) when the current query
   * is the same as the previous query.
   * @default true
   */
  public native function get queryCaching():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set queryCaching(value:Boolean):void;

  [ExtConfig]
  /**
   * The length of time in milliseconds to delay between the start of typing and sending the query to filter the
   * dropdown list.
   *
   * <p>Defaults to <code>500</code> if <code><a href="#!/api/Ext.form.field.ComboBox-cfg-queryMode" rel="Ext.form.field.ComboBox-cfg-queryMode" class="docClass">queryMode</a> = 'remote'</code> or <code>10</code> if <code><a href="#!/api/Ext.form.field.ComboBox-cfg-queryMode" rel="Ext.form.field.ComboBox-cfg-queryMode" class="docClass">queryMode</a> = 'local'</code></p>
   */
  public native function get queryDelay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set queryDelay(value:Number):void;

  [ExtConfig]
  /**
   * The mode in which the ComboBox uses the configured Store. Acceptable values are:
   *
   * <ul>
   * <li><p><strong><code>'remote'</code></strong> :</p>
   *
   * <p>In <code>queryMode: 'remote'</code>, the ComboBox loads its Store dynamically based upon user interaction.</p>
   *
   * <p>This is typically used for "autocomplete" type inputs, and after the user finishes typing, the Store is <a href="#!/api/Ext.data.Store-method-load" rel="Ext.data.Store-method-load" class="docClass">load</a>ed.</p>
   *
   * <p>A parameter containing the typed string is sent in the load request. The default parameter name for the input
   * string is <code>query</code>, but this can be configured using the <a href="#!/api/Ext.form.field.ComboBox-cfg-queryParam" rel="Ext.form.field.ComboBox-cfg-queryParam" class="docClass">queryParam</a> config.</p>
   *
   * <p>In <code>queryMode: 'remote'</code>, the Store may be configured with <code><a href="#!/api/Ext.data.Store-cfg-remoteFilter" rel="Ext.data.Store-cfg-remoteFilter" class="docClass">remoteFilter</a>:
   * true</code>, and further filters may be <em>programatically</em> added to the Store which are then passed with every load
   * request which allows the server to further refine the returned dataset.</p>
   *
   * <p>Typically, in an autocomplete situation, <a href="#!/api/Ext.form.field.ComboBox-cfg-hideTrigger" rel="Ext.form.field.ComboBox-cfg-hideTrigger" class="docClass">hideTrigger</a> is configured <code>true</code> because it has no meaning for
   * autocomplete.</p></li>
   * <li><p><strong><code>'local'</code></strong> :</p>
   *
   * <p>ComboBox loads local data</p>
   *
   * <pre><code>var combo = new <a href="#!/api/Ext.form.field.ComboBox" rel="Ext.form.field.ComboBox" class="docClass">Ext.form.field.ComboBox</a>({
   *     renderTo: document.body,
   *     queryMode: 'local',
   *     store: new <a href="#!/api/Ext.data.ArrayStore" rel="Ext.data.ArrayStore" class="docClass">Ext.data.ArrayStore</a>({
   *         id: 0,
   *         fields: [
   *             'myId',  // numeric value is the key
   *             'displayText'
   *         ],
   *         data: [[1, 'item1'], [2, 'item2']]  // data is local
   *     }),
   *     valueField: 'myId',
   *     displayField: 'displayText',
   *     triggerAction: 'all'
   * });
   * </code></pre></li>
   * </ul>
   * @default 'remote'
   */
  public native function get queryMode():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set queryMode(value:String):void;

  [ExtConfig]
  /**
   * Name of the parameter used by the Store to pass the typed string when the ComboBox is configured with
   * <code><a href="#!/api/Ext.form.field.ComboBox-cfg-queryMode" rel="Ext.form.field.ComboBox-cfg-queryMode" class="docClass">queryMode</a>: 'remote'</code>. If explicitly set to a falsy value it will not be sent.
   * @default 'query'
   */
  public native function get queryParam():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set queryParam(value:String):void;

  [ExtConfig]
  /**
   * Whether the Tab key should select the currently highlighted item.
   * @default true
   */
  public native function get selectOnTab():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set selectOnTab(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The selected model. Typically used with <a href="#!/api/Ext.form.field.ComboBox-cfg-bind" rel="Ext.form.field.ComboBox-cfg-bind" class="docClass">binding</a>.
   * @see #getSelection()
   * @see #setSelection()
   */
  public native function get selection():ext.data.Model;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set selection(value:ext.data.Model):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The data source to which the combo / tagfield is bound. Acceptable values for this
   * property are:
   *
   * <ul>
   * <li><strong>any <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a> class / subclass</strong></li>
   * <li><strong>an <a href="#!/api/Ext.data.Store-cfg-storeId" rel="Ext.data.Store-cfg-storeId" class="docClass">ID of a store</a></strong></li>
   * <li><p><strong>an Array</strong> : Arrays will be converted to a <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> internally,
   * automatically generating <a href="#!/api/Ext.data.field.Field-cfg-name" rel="Ext.data.field.Field-cfg-name" class="docClass">field names</a> to work with all
   * data components.</p>
   *
   * <ul>
   * <li><p><strong>1-dimensional array</strong> : (e.g., <code>['Foo','Bar']</code>)</p>
   *
   * <p>A 1-dimensional array will automatically be expanded (each array item will be
   * used for both the combo <a href="#!/api/Ext.form.field.ComboBox-cfg-valueField" rel="Ext.form.field.ComboBox-cfg-valueField" class="docClass">valueField</a> and <a href="#!/api/Ext.form.field.ComboBox-cfg-displayField" rel="Ext.form.field.ComboBox-cfg-displayField" class="docClass">displayField</a>)</p></li>
   * <li><p><strong>2-dimensional array</strong> : (e.g., <code>[['f','Foo'],['b','Bar']]</code>)</p>
   *
   * <p>For a multi-dimensional array, the value in index 0 of each item will be assumed
   * to be the combo <a href="#!/api/Ext.form.field.ComboBox-cfg-valueField" rel="Ext.form.field.ComboBox-cfg-valueField" class="docClass">valueField</a>, while the value at index 1 is assumed to be
   * the combo <a href="#!/api/Ext.form.field.ComboBox-cfg-displayField" rel="Ext.form.field.ComboBox-cfg-displayField" class="docClass">displayField</a>.</p></li>
   * </ul>
   * </li>
   * <li><p><strong>a <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a> config object</strong>.  When passing a config you can
   * specify the store type by alias.  Passing a config object with a store type will
   * dynamically create a new store of that type when the combo / tagfield is
   * instantiated.</p>
   *
   * <p><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.store.States', {
   *     extend: '<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>',
   *     alias: 'store.states',
   *     fields: ['name']
   * });</p>
   *
   * <p><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *     xtype: 'combobox',
   *     renderTo: document.body,
   *     store: {
   *         type: 'states',
   *         data: [{
   *             name: 'California'
   *         }]
   *     },
   *     queryMode: 'local',
   *     displayField: 'name',
   *     valueField: 'name'
   * });</p></li>
   * </ul>
   *
   *
   * <p>See also <a href="#!/api/Ext.form.field.ComboBox-cfg-queryMode" rel="Ext.form.field.ComboBox-cfg-queryMode" class="docClass">queryMode</a>.</p>
   * @see #getStore()
   * @see #setStore()
   */
  public native function get store():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set store(value:*):void;

  [ExtConfig]
  /**
   * The id, DOM node or <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> of an existing HTML <code>&lt;select&gt;</code> element to convert into a ComboBox. The
   * target select's options will be used to build the options in the ComboBox dropdown; a configured <a href="#!/api/Ext.form.field.ComboBox-cfg-store" rel="Ext.form.field.ComboBox-cfg-store" class="docClass">store</a>
   * will take precedence over this.
   */
  public native function get transform():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set transform(value:*):void;

  [ExtConfig]
  /**
   * <code>true</code> to automatically render this combo box in place of the select element that is being
   * <a href="#!/api/Ext.form.field.ComboBox-cfg-transform" rel="Ext.form.field.ComboBox-cfg-transform" class="docClass">transformed</a>. If <code>false</code>, this combo will be rendered using the normal rendering,
   * either as part of a layout, or using <a href="#!/api/Ext.form.field.ComboBox-cfg-renderTo" rel="Ext.form.field.ComboBox-cfg-renderTo" class="docClass">renderTo</a> or <a href="#!/api/Ext.form.field.ComboBox-method-render" rel="Ext.form.field.ComboBox-method-render" class="docClass">render</a>.
   * @default true
   */
  public native function get transformInPlace():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set transformInPlace(value:Boolean):void;

  [ExtConfig]
  /**
   * The action to execute when the trigger is clicked.
   *
   * <ul>
   * <li><p><strong><code>'all'</code></strong> :</p>
   *
   * <p><a href="#!/api/Ext.form.field.ComboBox-method-doQuery" rel="Ext.form.field.ComboBox-method-doQuery" class="docClass">run the query</a> specified by the <code><a href="#!/api/Ext.form.field.ComboBox-cfg-allQuery" rel="Ext.form.field.ComboBox-cfg-allQuery" class="docClass">allQuery</a></code> config option</p></li>
   * <li><p><strong><code>'last'</code></strong> :</p>
   *
   * <p><a href="#!/api/Ext.form.field.ComboBox-method-doQuery" rel="Ext.form.field.ComboBox-method-doQuery" class="docClass">run the query</a> using the <code><a href="#!/api/Ext.form.field.ComboBox-property-lastQuery" rel="Ext.form.field.ComboBox-property-lastQuery" class="docClass">last query value</a></code>.</p></li>
   * <li><p><strong><code>'query'</code></strong> :</p>
   *
   * <p><a href="#!/api/Ext.form.field.ComboBox-method-doQuery" rel="Ext.form.field.ComboBox-method-doQuery" class="docClass">run the query</a> using the <a href="#!/api/Ext.form.field.Base-method-getRawValue" rel="Ext.form.field.Base-method-getRawValue" class="docClass">raw value</a>.</p></li>
   * </ul>
   *
   *
   * <p>See also <code><a href="#!/api/Ext.form.field.ComboBox-cfg-queryParam" rel="Ext.form.field.ComboBox-cfg-queryParam" class="docClass">queryParam</a></code>.</p>
   * @default 'all'
   */
  public native function get triggerAction():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set triggerAction(value:String):void;

  [ExtConfig]
  /**
   * <code>true</code> to populate and autoselect the remainder of the text being typed after a configurable delay
   * (<a href="#!/api/Ext.form.field.ComboBox-cfg-typeAheadDelay" rel="Ext.form.field.ComboBox-cfg-typeAheadDelay" class="docClass">typeAheadDelay</a>) if it matches a known value.
   * @default false
   */
  public native function get typeAhead():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set typeAhead(value:Boolean):void;

  [ExtConfig]
  /**
   * The length of time in milliseconds to wait until the typeahead text is displayed if <code><a href="#!/api/Ext.form.field.ComboBox-cfg-typeAhead" rel="Ext.form.field.ComboBox-cfg-typeAhead" class="docClass">typeAhead</a> = true</code>
   * @default 250
   */
  public native function get typeAheadDelay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set typeAheadDelay(value:Number):void;

  [ExtConfig]
  /**
   * The underlying <a href="#!/api/Ext.data.field.Field-cfg-name" rel="Ext.data.field.Field-cfg-name" class="docClass">data value name</a> to bind to this ComboBox.
   *
   * <p><strong>Note</strong>: use of a <code>valueField</code> requires the user to make a selection in order for a value to be mapped. See also
   * <code><a href="#!/api/Ext.form.field.ComboBox-cfg-displayField" rel="Ext.form.field.ComboBox-cfg-displayField" class="docClass">displayField</a></code>.</p>
   *
   * <p>Defaults to match the value of the <a href="#!/api/Ext.form.field.ComboBox-cfg-displayField" rel="Ext.form.field.ComboBox-cfg-displayField" class="docClass">displayField</a> config.</p>
   */
  public native function get valueField():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set valueField(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * When using a name/value combo, if the value passed to setValue is not found in the store, valueNotFoundText will
   * be displayed as the field text if defined. If this default text is used, it means there
   * is no value set and no validation will occur on this field.
   * @see #getValueNotFoundText()
   * @see #setValueNotFoundText()
   */
  public native function get valueNotFoundText():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set valueNotFoundText(value:String):void;

  /**
   * @inheritDoc
   */
  public native function get autoDestroyBoundStore():Boolean;

  /**
   * @inheritDoc
   */
  public native function set autoDestroyBoundStore(value:Boolean):void;

  /**
   * Sets the specified value(s) into the field. For each value, if a record is found in the <a href="#!/api/Ext.form.field.ComboBox-cfg-store" rel="Ext.form.field.ComboBox-cfg-store" class="docClass">store</a> that
   * matches based on the <a href="#!/api/Ext.form.field.ComboBox-cfg-valueField" rel="Ext.form.field.ComboBox-cfg-valueField" class="docClass">valueField</a>, then that record's <a href="#!/api/Ext.form.field.ComboBox-cfg-displayField" rel="Ext.form.field.ComboBox-cfg-displayField" class="docClass">displayField</a> will be displayed in the
   * field. If no match is found, and the <a href="#!/api/Ext.form.field.ComboBox-cfg-valueNotFoundText" rel="Ext.form.field.ComboBox-cfg-valueNotFoundText" class="docClass">valueNotFoundText</a> config option is defined, then that will be
   * displayed as the default field text. Otherwise a blank value will be shown, although the value will still be set.
   * @param value The value to set
   * @return this
   */
  override public native function setValue(value:*):ext.form.field.IField;

  /**
   * Adds a value or values to the current value of the field
   * @param value The value or values to add to the current value, see <a href="#!/api/Ext.form.field.ComboBox-method-setValue" rel="Ext.form.field.ComboBox-method-setValue" class="docClass">setValue</a>
   */
  public native function addValue(value:*):void;

  /**
   * A method called when the filtering caused by the <a href="#!/api/Ext.form.field.ComboBox-method-doQuery" rel="Ext.form.field.ComboBox-method-doQuery" class="docClass">doQuery</a> call is complete and the store has been
   * either filtered locally (if <a href="#!/api/Ext.form.field.ComboBox-cfg-queryMode" rel="Ext.form.field.ComboBox-cfg-queryMode" class="docClass">queryMode</a> is <code>"local"</code>), or has been loaded using the specified filtering.
   * @param queryPlan An object containing details about the query was executed.
   * @param {String} queryPlan.query The query value to be used to match against the ComboBox's <a href="#!/api/Ext.form.field.ComboBox-cfg-valueField" rel="Ext.form.field.ComboBox-cfg-valueField" class="docClass">valueField</a>.
   * @param {Boolean} queryPlan.forceAll If <code>true</code>, causes the query to be executed even if the minChars threshold is not met.
   * @param {Boolean} queryPlan.cancel A boolean value which, if set to <code>true</code> upon return, causes the query not to be executed.
   * @param {Boolean} queryPlan.rawQuery If <code>true</code> indicates that the raw input field value is being used, and upon store load,
   * the input field value should <strong>not</strong> be overwritten.
   */
  public native function afterQuery(queryPlan:Object):void;

  /**
   * A method which may modify aspects of how the store is to be filtered (if <a href="#!/api/Ext.form.field.ComboBox-cfg-queryMode" rel="Ext.form.field.ComboBox-cfg-queryMode" class="docClass">queryMode</a> is <code>"local"</code>)
   * of loaded (if <a href="#!/api/Ext.form.field.ComboBox-cfg-queryMode" rel="Ext.form.field.ComboBox-cfg-queryMode" class="docClass">queryMode</a> is <code>"remote"</code>).
   *
   * <p>This is called by the {&#64;link <a href="#!/api/Ext.form.field.ComboBox-method-doQuery" rel="Ext.form.field.ComboBox-method-doQuery" class="docClass">doQuery</a> method, and may be overridden in subclasses to modify
   * the default behaviour.</p>
   *
   * <p>This method is passed an object containing information about the upcoming query operation which it may modify
   * before returning.</p>
   * @param queryPlan An object containing details about the query to be executed.
   * @param {String} queryPlan.query The query value to be used to match against the ComboBox's <a href="#!/api/Ext.form.field.ComboBox-cfg-valueField" rel="Ext.form.field.ComboBox-cfg-valueField" class="docClass">valueField</a>.
   * @param {String} queryPlan.lastQuery The query value used the last time a store query was made.
   * @param {Boolean} queryPlan.forceAll If <code>true</code>, causes the query to be executed even if the minChars threshold is not met.
   * @param {Boolean} queryPlan.cancel A boolean value which, if set to <code>true</code> upon return, causes the query not to be executed.
   * @param {Boolean} queryPlan.rawQuery If <code>true</code> indicates that the raw input field value is being used, and upon store load,
   * the input field value should <strong>not</strong> be overwritten.
   */
  public native function beforeQuery(queryPlan:Object):void;

  /**
   * Clears any value currently set in the ComboBox.
   */
  public native function clearValue():void;

  /**
   * Executes a query to filter the dropdown list. Fires the <a href="#!/api/Ext.form.field.ComboBox-event-beforequery" rel="Ext.form.field.ComboBox-event-beforequery" class="docClass">beforequery</a> event prior to performing the query
   * allowing the query action to be canceled if needed.
   * @param queryString The string to use to filter available items by matching against the configured <a href="#!/api/Ext.form.field.ComboBox-cfg-valueField" rel="Ext.form.field.ComboBox-cfg-valueField" class="docClass">valueField</a>.
   * @param forceAll <code>true</code> to force the query to execute even if there are currently fewer characters in
   * the field than the minimum specified by the <code><a href="#!/api/Ext.form.field.ComboBox-cfg-minChars" rel="Ext.form.field.ComboBox-cfg-minChars" class="docClass">minChars</a></code> config option. It also clears any filter
   * previously saved in the current store.
   * @default false
   * @param rawQuery Pass as true if the raw typed value is being used as the query string. This causes the
   * resulting store load to leave the raw value undisturbed.
   * @default false
   * @return true if the query was permitted to run, false if it was cancelled by a <a href="#!/api/Ext.form.field.ComboBox-event-beforequery" rel="Ext.form.field.ComboBox-event-beforequery" class="docClass">beforequery</a>
   * handler.
   */
  public native function doQuery(queryString:String, forceAll:Boolean = false, rawQuery:Boolean = false):void;

  /**
   * Finds the record by searching for a specific field/value combination.
   * @param field The name of the field to test.
   * @param value The value to match the field against.
   * @return The matched record or false.
   */
  public native function findRecord(field:String, value:Object):ext.data.Model;

  /**
   * Finds the record by searching values in the <a href="#!/api/Ext.form.field.ComboBox-cfg-displayField" rel="Ext.form.field.ComboBox-cfg-displayField" class="docClass">displayField</a>.
   * @param value The value to match the field against.
   * @return The matched record or <code>false</code>.
   */
  public native function findRecordByDisplay(value:Object):ext.data.Model;

  /**
   * Finds the record by searching values in the <a href="#!/api/Ext.form.field.ComboBox-cfg-valueField" rel="Ext.form.field.ComboBox-cfg-valueField" class="docClass">valueField</a>.
   * @param value The value to match the field against.
   * @return The matched record or <code>false</code>.
   */
  public native function findRecordByValue(value:Object):ext.data.Model;

  [Deprecated(replacement = "ext.form.field.Tag", since = "5.1.0")]
  /**
   * Returns the value of <a href="#!/api/Ext.form.field.ComboBox-cfg-delimiter" rel="Ext.form.field.ComboBox-cfg-delimiter" class="docClass">delimiter</a>.
   */
  public native function getDelimiter():String;

  /**
   * Returns the value of <a href="#!/api/Ext.form.field.ComboBox-cfg-displayField" rel="Ext.form.field.ComboBox-cfg-displayField" class="docClass">displayField</a>.
   */
  public native function getDisplayField():String;

  /**
   * Returns the value of <a href="#!/api/Ext.form.field.ComboBox-cfg-displayTpl" rel="Ext.form.field.ComboBox-cfg-displayTpl" class="docClass">displayTpl</a>.
   */
  public native function getDisplayTpl():*;

  /**
   * Returns the <code><a href="#!/api/Ext.util.FilterCollection" rel="Ext.util.FilterCollection" class="docClass">Ext.util.FilterCollection</a></code>. Unless <code>autoCreate</code> is explicitly passed
   * as <code>false</code> this collection will be automatically created if it does not yet exist.
   * @param autoCreate Pass <code>false</code> to disable auto-creation of the collection.
   * @default true
   * @return The collection of filters.
   */
  public native function getFilters(autoCreate:Boolean = true):ext.util.Collection;

  /**
   * Returns the combobox's selection.
   * @return The selected record
   */
  public native function getSelection():ext.data.Model;

  /**
   * Gets the current store instance.
   * @return The store, null if one does not exist.
   */
  public native function getStore():ext.data.AbstractStore;

  /**
   * Returns the value of <a href="#!/api/Ext.form.field.ComboBox-cfg-valueNotFoundText" rel="Ext.form.field.ComboBox-cfg-valueNotFoundText" class="docClass">valueNotFoundText</a>.
   */
  public native function getValueNotFoundText():String;

  /**
   * Selects an item by a <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a>, or by a key value.
   */
  public native function select(r:Object):void;

  [Deprecated(replacement = "ext.form.field.Tag", since = "5.1.0")]
  /**
   * Sets the value of <a href="#!/api/Ext.form.field.ComboBox-cfg-delimiter" rel="Ext.form.field.ComboBox-cfg-delimiter" class="docClass">delimiter</a>.
   * @param delimiter The new value.
   */
  public native function setDelimiter(delimiter:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.form.field.ComboBox-cfg-displayField" rel="Ext.form.field.ComboBox-cfg-displayField" class="docClass">displayField</a>.
   * @param displayField The new value.
   */
  public native function setDisplayField(displayField:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.form.field.ComboBox-cfg-displayTpl" rel="Ext.form.field.ComboBox-cfg-displayTpl" class="docClass">displayTpl</a>.
   * @param displayTpl The new value.
   */
  public native function setDisplayTpl(displayTpl:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.form.field.ComboBox-cfg-selection" rel="Ext.form.field.ComboBox-cfg-selection" class="docClass">selection</a>.
   * @param selection The new value.
   */
  public native function setSelection(selection:ext.data.Model):void;

  /**
   * Sets the store to the specified store.
   * @since 5.0.0
   */
  public native function setStore(store:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.form.field.ComboBox-cfg-valueNotFoundText" rel="Ext.form.field.ComboBox-cfg-valueNotFoundText" class="docClass">valueNotFoundText</a>.
   * @param valueNotFoundText The new value.
   */
  public native function setValueNotFoundText(valueNotFoundText:String):void;

  /**
   * @inheritDoc
   */
  public native function bindStore(store:ext.data.Store = undefined):void;

  /**
   * Binds listeners for this component to the store. By default it will add
   * anything bound by the getStoreListeners method, however it can be overridden
   * in a subclass to provide any more complicated handling.
   * @param store The store to bind to
   */
  protected native function bindStoreListeners(store:ext.data.AbstractStore):void;

  /**
   * Gets data for each record to be used for constructing the display value with
   * the <a href="#!/api/Ext.form.field.ComboBox-cfg-displayTpl" rel="Ext.form.field.ComboBox-cfg-displayTpl" class="docClass">displayTpl</a>. This may be overridden to provide access to associated records.
   * @param record The record.
   * @return The data to be passed for each record to the <a href="#!/api/Ext.form.field.ComboBox-cfg-displayTpl" rel="Ext.form.field.ComboBox-cfg-displayTpl" class="docClass">displayTpl</a>.
   */
  protected native function getRecordDisplayData(record:ext.data.Model):Object;

  /**
   * Unbinds listeners from this component to the store. By default it will remove
   * anything bound by the bindStoreListeners method, however it can be overridden
   * in a subclass to provide any more complicated handling.
   * @param store The store to unbind from
   */
  protected native function unbindStoreListeners(store:ext.data.AbstractStore):void;
}
}