package ext.form.field {
import ext.Base;
import js.HTMLElement;

[Uses("ext.Base")]
[Native("Ext.form.field.Field", require)]
/**
 * Fires when the value of a field is changed. The value of a field is
 * checked for changes when the field's <a href="#!/api/Ext.form.field.Field-method-setValue" rel="Ext.form.field.Field-method-setValue" class="docClass">setValue</a> method
 * is called and when any of the events listed in
 * <a href="#!/api/Ext.form.field.Base-cfg-checkChangeEvents" rel="Ext.form.field.Base-cfg-checkChangeEvents" class="docClass">checkChangeEvents</a> are fired.
 * @eventType ext.form.field.events.Field_newValue_oldValueEvent.CHANGE
 */
[Event(name = 'onChange', type = 'ext.form.field.events.Field_newValue_oldValueEvent')]
/**
 * Fires when a change in the field's <a href="#!/api/Ext.form.field.Field-method-isDirty" rel="Ext.form.field.Field-method-isDirty" class="docClass">isDirty</a> state is detected.
 * @eventType ext.form.field.events.Field_isDirtyEvent.DIRTY_CHANGE
 */
[Event(name = 'onDirtyChange', type = 'ext.form.field.events.Field_isDirtyEvent')]
/**
 * Fires when a change in the field's validity is detected.
 * @eventType ext.form.field.events.Field_isValidEvent.VALIDITY_CHANGE
 */
[Event(name = 'onValidityChange', type = 'ext.form.field.events.Field_isValidEvent')]
/**
 * This mixin provides a common interface for the logical behavior and state of form fields, including:
 *
 * <ul>
 * <li>Getter and setter methods for field values</li>
 * <li>Events and methods for tracking value and validity changes</li>
 * <li>Methods for triggering validation</li>
 * </ul>
 *
 *
 * <p><strong>NOTE</strong>: When implementing custom fields, it is most likely that you will want to extend the <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a>
 * component class rather than using this mixin directly, as BaseField contains additional logic for generating an
 * actual DOM complete with <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">label and error message</a> display and a form input field,
 * plus methods that bind the Field value getters and setters to the input field's value.</p>
 *
 * <p>If you do want to implement this mixin directly and don't want to extend <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a>, then
 * you will most likely want to override the following methods with custom implementations: <a href="#!/api/Ext.form.field.Field-method-getValue" rel="Ext.form.field.Field-method-getValue" class="docClass">getValue</a>,
 * <a href="#!/api/Ext.form.field.Field-method-setValue" rel="Ext.form.field.Field-method-setValue" class="docClass">setValue</a>, and <a href="#!/api/Ext.form.field.Field-method-getErrors" rel="Ext.form.field.Field-method-getErrors" class="docClass">getErrors</a>. Other methods may be overridden as needed but their base
 * implementations should be sufficient for common cases. You will also need to make sure that <a href="#!/api/Ext.form.field.Field-method-initField" rel="Ext.form.field.Field-method-initField" class="docClass">initField</a>
 * is called during the component's initialization.</p>
 */
public class Field extends ext.Base implements ext.form.field.IField {
  /**
   * @param config @inheritDoc
   */
  public function Field(config:ext.form.field.Field = null) {
    super();
  }

  /**
   * Flag denoting that this component is a Field. Always true.
   * @default true
   */
  public native function get isFormField():Boolean;

  /**
   * @private
   */
  public native function set isFormField(value:Boolean):void;

  /**
   * The original value of the field as configured in the <a href="#!/api/Ext.form.field.Field-cfg-value" rel="Ext.form.field.Field-cfg-value" class="docClass">value</a> configuration, or as loaded by the last
   * form load operation if the form's <a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a> setting is <code>true</code>.
   */
  public native function get originalValue():*;

  /**
   * @private
   */
  public native function set originalValue(value:*):void;

  [ExtConfig]
  /**
   * True to disable the field. Disabled Fields will not be <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submitted</a>.
   * @default false
   */
  public native function get disabled():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set disabled(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the field. By default this is used as the parameter name when including the
   * <a href="#!/api/Ext.form.field.Field-method-getSubmitData" rel="Ext.form.field.Field-method-getSubmitData" class="docClass">field value</a> in a <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">form submit()</a>. To prevent the field from
   * being included in the form submit, set <a href="#!/api/Ext.form.field.Field-cfg-submitValue" rel="Ext.form.field.Field-cfg-submitValue" class="docClass">submitValue</a> to false.
   * @see #getName()
   */
  public native function get name():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set name(value:String):void;

  [ExtConfig]
  /**
   * Setting this to false will prevent the field from being <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submitted</a> even when it is
   * not disabled.
   * @default true
   */
  public native function get submitValue():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set submitValue(value:Boolean):void;

  [ExtConfig]
  /**
   * Specifies whether this field should be validated immediately whenever a change in its value is detected.
   * If the validation results in a change in the field's validity, a <a href="#!/api/Ext.form.field.Field-event-validitychange" rel="Ext.form.field.Field-event-validitychange" class="docClass">validitychange</a> event will be
   * fired. This allows the field to show feedback about the validity of its contents immediately as the user is
   * typing.
   *
   * <p>When set to false, feedback will not be immediate. However the form will still be validated before submitting if
   * the clientValidation option to <a href="#!/api/Ext.form.Basic-method-doAction" rel="Ext.form.Basic-method-doAction" class="docClass">Ext.form.Basic.doAction</a> is enabled, or if the field or form are validated
   * manually.</p>
   *
   * <p>See also <a href="#!/api/Ext.form.field.Base-cfg-checkChangeEvents" rel="Ext.form.field.Base-cfg-checkChangeEvents" class="docClass">Ext.form.field.Base.checkChangeEvents</a> for controlling how changes to the field's value are
   * detected.</p>
   * @default true
   */
  public native function get validateOnChange():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set validateOnChange(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This property, when a <code>String</code>, contributes its value to the error state of this
   * instance as reported by <code>getErrors</code>.
   * @see #getValidation()
   * @see #setValidation()
   */
  public native function get validation():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set validation(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A value to initialize this field with.
   * @see #getValue()
   * @see #setValue()
   */
  public native function get value():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set value(value:Object):void;

  [ExtConfig]
  /**
   * The event name(s) to use to publish the <a href="#!/api/Ext.form.field.Field-cfg-value" rel="Ext.form.field.Field-cfg-value" class="docClass">value</a>
   * <a href="#!/api/Ext.form.field.Base-cfg-bind" rel="Ext.form.field.Base-cfg-bind" class="docClass">Ext.form.field.Base.bind</a> for this field.
   * @default 'change'
   * @since 5.0.1
   */
  public native function get valuePublishEvent():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set valuePublishEvent(value:*):void;

  /**
   * A utility for grouping a set of modifications which may trigger value changes into a single transaction, to
   * prevent excessive firing of <a href="#!/api/Ext.form.field.Field-event-change" rel="Ext.form.field.Field-event-change" class="docClass">change</a> events. This is useful for instance if the field has sub-fields which
   * are being updated as a group; you don't want the container field to check its own changed state for each subfield
   * change.
   * @param fn The function to call with change checks suspended.
   */
  public native function batchChanges(fn:Function):void;

  /**
   * Checks whether the value of the field has changed since the last time it was checked.
   * If the value has changed, it:
   *
   * <ol>
   * <li>Fires the <a href="#!/api/Ext.form.field.Field-event-change" rel="Ext.form.field.Field-event-change" class="docClass">change event</a>,</li>
   * <li>Performs validation if the <a href="#!/api/Ext.form.field.Field-cfg-validateOnChange" rel="Ext.form.field.Field-cfg-validateOnChange" class="docClass">validateOnChange</a> config is enabled, firing the
   * <a href="#!/api/Ext.form.field.Field-event-validitychange" rel="Ext.form.field.Field-event-validitychange" class="docClass">validitychange event</a> if the validity has changed, and</li>
   * <li>Checks the <a href="#!/api/Ext.form.field.Field-method-isDirty" rel="Ext.form.field.Field-method-isDirty" class="docClass">dirty state</a> of the field and fires the <a href="#!/api/Ext.form.field.Field-event-dirtychange" rel="Ext.form.field.Field-event-dirtychange" class="docClass">dirtychange event</a>
   * if it has changed.</li>
   * </ol>
   */
  public native function checkChange():void;

  /**
   * Checks the <a href="#!/api/Ext.form.field.Field-method-isDirty" rel="Ext.form.field.Field-method-isDirty" class="docClass">isDirty</a> state of the field and if it has changed since the last time it was checked,
   * fires the <a href="#!/api/Ext.form.field.Field-event-dirtychange" rel="Ext.form.field.Field-event-dirtychange" class="docClass">dirtychange</a> event.
   */
  public native function checkDirty():void;

  /**
   * Cleans up values initialized by this Field mixin on the current instance.
   * Components using this mixin should call this method before being destroyed.
   */
  public native function cleanupField():void;

  /**
   * Clear any invalid styles/messages for this field. Components using this mixin should implement this method to
   * update the components rendering to clear any existing messages.
   *
   * <p><strong>Note</strong>: this method does not cause the Field's <a href="#!/api/Ext.form.field.Field-method-validate" rel="Ext.form.field.Field-method-validate" class="docClass">validate</a> or <a href="#!/api/Ext.form.field.Field-method-isValid" rel="Ext.form.field.Field-method-isValid" class="docClass">isValid</a> methods to return <code>true</code>
   * if the value does not <em>pass</em> validation. So simply clearing a field's errors will not necessarily allow
   * submission of forms submitted with the <a href="#!/api/Ext.form.action.Submit-cfg-clientValidation" rel="Ext.form.action.Submit-cfg-clientValidation" class="docClass">Ext.form.action.Submit.clientValidation</a> option set.</p>
   */
  public native function clearInvalid():void;

  /**
   * Only relevant if the instance's <a href="#!/api/Ext.form.field.Field-method-isFileUpload" rel="Ext.form.field.Field-method-isFileUpload" class="docClass">isFileUpload</a> method returns true. Returns a reference to the file input
   * DOM element holding the user's selected file. The input will be appended into the submission form and will not be
   * returned, so this method should also create a replacement.
   */
  public native function extractFileInput():js.HTMLElement;

  /**
   * Runs this field's validators and returns an array of error messages for any validation failures. This is called
   * internally during validation and would not usually need to be used manually.
   *
   * <p>Each subclass should override or augment the return value to provide their own errors.</p>
   * @param value The value to get errors for (defaults to the current field value)
   * @return All error messages for this field; an empty Array if none.
   */
  public native function getErrors(value:*):Array;

  /**
   * Returns the value(s) that should be saved to the <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> instance for this field, when <a href="#!/api/Ext.form.Basic-method-updateRecord" rel="Ext.form.Basic-method-updateRecord" class="docClass">Ext.form.Basic.updateRecord</a> is called. Typically this will be an object with a single name-value pair, the name
   * being this field's <a href="#!/api/Ext.form.field.Field-method-getName" rel="Ext.form.field.Field-method-getName" class="docClass">name</a> and the value being its current data value. More advanced field
   * implementations may return more than one name-value pair. The returned values will be saved to the corresponding
   * field names in the Model.
   *
   * <p>Note that the values returned from this method are not guaranteed to have been successfully <a href="#!/api/Ext.form.field.Field-method-validate" rel="Ext.form.field.Field-method-validate" class="docClass">validated</a>.</p>
   * @return A mapping of submit parameter names to values; each value should be a string, or an array of
   * strings if that particular name has multiple values. It can also return null if there are no parameters to be
   * submitted.
   */
  public native function getModelData(includeEmptyText:Object, isSubmitting:Object):Object;

  /**
   * Returns the <a href="#!/api/Ext.form.field.Field-cfg-name" rel="Ext.form.field.Field-cfg-name" class="docClass">name</a> attribute of the field. This is used as the parameter name
   * when including the field value in a <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">form submit()</a>.
   * @return name The field <a href="#!/api/Ext.form.field.Field-cfg-name" rel="Ext.form.field.Field-cfg-name" class="docClass">name</a>
   */
  public native function getName():String;

  /**
   * Returns the parameter(s) that would be included in a standard form submit for this field. Typically this will be
   * an object with a single name-value pair, the name being this field's <a href="#!/api/Ext.form.field.Field-method-getName" rel="Ext.form.field.Field-method-getName" class="docClass">name</a> and the value being
   * its current stringified value. More advanced field implementations may return more than one name-value pair.
   *
   * <p>Note that the values returned from this method are not guaranteed to have been successfully <a href="#!/api/Ext.form.field.Field-method-validate" rel="Ext.form.field.Field-method-validate" class="docClass">validated</a>.</p>
   * @return A mapping of submit parameter names to values; each value should be a string, or an array of
   * strings if that particular name has multiple values. It can also return null if there are no parameters to be
   * submitted.
   */
  public native function getSubmitData():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.form.field.Field-cfg-validation" rel="Ext.form.field.Field-cfg-validation" class="docClass">validation</a>.
   */
  public native function getValidation():*;

  /**
   * Returns the current data value of the field. The type of value returned is particular to the type of the
   * particular field (e.g. a Date object for <a href="#!/api/Ext.form.field.Date" rel="Ext.form.field.Date" class="docClass">Ext.form.field.Date</a>).
   * @return value The field value
   */
  public native function getValue():*;

  /**
   * Initializes this Field mixin on the current instance. Components using this mixin should call this method during
   * their own initialization process.
   */
  public native function initField():void;

  /**
   * Initializes the field's value based on the initial config.
   */
  public native function initValue():void;

  /**
   * Returns true if the value of this Field has been changed from its <a href="#!/api/Ext.form.field.Field-property-originalValue" rel="Ext.form.field.Field-property-originalValue" class="docClass">originalValue</a>.
   * Will always return false if the field is disabled.
   *
   * <p>Note that if the owning <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">form</a> was configured with
   * <a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a> then the <a href="#!/api/Ext.form.field.Field-property-originalValue" rel="Ext.form.field.Field-property-originalValue" class="docClass">originalValue</a> is updated when
   * the values are loaded by <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a>.<a href="#!/api/Ext.form.Basic-method-setValues" rel="Ext.form.Basic-method-setValues" class="docClass">setValues</a>.</p>
   * @return True if this field has been changed from its original value (and is not disabled),
   * false otherwise.
   */
  public native function isDirty():Boolean;

  /**
   * Returns whether two field <a href="#!/api/Ext.form.field.Field-method-getValue" rel="Ext.form.field.Field-method-getValue" class="docClass">values</a> are logically equal. Field implementations may override this
   * to provide custom comparison logic appropriate for the particular field's data type.
   * @param value1 The first value to compare
   * @param value2 The second value to compare
   * @return True if the values are equal, false if inequal.
   */
  public native function isEqual(value1:Object, value2:Object):Boolean;

  /**
   * Returns whether this Field is a file upload field; if it returns true, forms will use special techniques for
   * <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submitting the form</a> via AJAX. See <a href="#!/api/Ext.form.Basic-method-hasUpload" rel="Ext.form.Basic-method-hasUpload" class="docClass">Ext.form.Basic.hasUpload</a> for details. If
   * this returns true, the <a href="#!/api/Ext.form.field.Field-method-extractFileInput" rel="Ext.form.field.Field-method-extractFileInput" class="docClass">extractFileInput</a> method must also be implemented to return the corresponding file
   * input element.
   */
  public native function isFileUpload():Boolean;

  /**
   * Returns whether or not the field value is currently valid by <a href="#!/api/Ext.form.field.Field-method-getErrors" rel="Ext.form.field.Field-method-getErrors" class="docClass">validating</a> the field's current
   * value. The <a href="#!/api/Ext.form.field.Field-event-validitychange" rel="Ext.form.field.Field-event-validitychange" class="docClass">validitychange</a> event will not be fired; use <a href="#!/api/Ext.form.field.Field-method-validate" rel="Ext.form.field.Field-method-validate" class="docClass">validate</a> instead if you want the event
   * to fire. <strong>Note</strong>: <a href="#!/api/Ext.form.field.Field-cfg-disabled" rel="Ext.form.field.Field-cfg-disabled" class="docClass">disabled</a> fields are always treated as valid.
   *
   * <p>Implementations are encouraged to ensure that this method does not have side-effects such as triggering error
   * message display.</p>
   * @return True if the value is valid, else false
   */
  public native function isValid():Boolean;

  /**
   * Display one or more error messages associated with this field, using
   * <a href="#!/api/Ext.form.Labelable-cfg-msgTarget" rel="Ext.form.Labelable-cfg-msgTarget" class="docClass">Ext.form.Labelable.msgTarget</a> to determine how to display the messages and
   * applying <a href="#!/api/Ext.form.Labelable-cfg-invalidCls" rel="Ext.form.Labelable-cfg-invalidCls" class="docClass">Ext.form.Labelable.invalidCls</a> to the field's UI element.
   *
   * <pre><code>var formPanel = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
   *     title: 'Contact Info',
   *     width: 300,
   *     bodyPadding: 10,
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     items: [{
   *         xtype: 'textfield',
   *         name: 'name',
   *         id: 'nameId',
   *         fieldLabel: 'Name'
   *     }],
   *     bbar: [{
   *         text: 'Mark both fields invalid',
   *         handler: function() {
   *             var nameField = formPanel.getForm().findField('name');
   *             nameField.markInvalid('Name invalid message');
   *
   *             // multiple error string syntax
   *             // nameField.markInvalid(['First message', 'Second message']);
   *         }
   *     }]
   * });
   * </code></pre>
   *
   * <p><strong>Note</strong>: this method does not cause the Field's <a href="#!/api/Ext.form.field.Field-method-validate" rel="Ext.form.field.Field-method-validate" class="docClass">validate</a> or
   * <a href="#!/api/Ext.form.field.Field-method-isValid" rel="Ext.form.field.Field-method-isValid" class="docClass">isValid</a> methods to return <code>false</code> if the value does <em>pass</em> validation.
   * So simply marking a Field as invalid will not prevent submission of forms
   * submitted with the <a href="#!/api/Ext.form.action.Submit-cfg-clientValidation" rel="Ext.form.action.Submit-cfg-clientValidation" class="docClass">Ext.form.action.Submit.clientValidation</a> option set.</p>
   * @param errors The validation message(s) to display.
   */
  public native function markInvalid(errors:*):void;

  /**
   * Resets the current field value to the originally loaded value and clears any validation messages. See <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a>.<a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a>
   */
  public native function reset():void;

  /**
   * Resets the field's <a href="#!/api/Ext.form.field.Field-property-originalValue" rel="Ext.form.field.Field-property-originalValue" class="docClass">originalValue</a> property so it matches the current <a href="#!/api/Ext.form.field.Field-method-getValue" rel="Ext.form.field.Field-method-getValue" class="docClass">value</a>. This is
   * called by <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a>.<a href="#!/api/Ext.form.Basic-method-setValues" rel="Ext.form.Basic-method-setValues" class="docClass">setValues</a> if the form's
   * <a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a> property is set to true.
   */
  public native function resetOriginalValue():void;

  /**
   * Sets the value of <a href="#!/api/Ext.form.field.Field-cfg-validation" rel="Ext.form.field.Field-cfg-validation" class="docClass">validation</a>.
   * @param validation The new value.
   */
  public native function setValidation(validation:*):void;

  /**
   * Sets a data value into the field and runs the change detection and validation.
   * @param value The value to set
   * @return this
   */
  public native function setValue(value:*):ext.form.field.IField;

  /**
   * Returns whether or not the field value is currently valid by <a href="#!/api/Ext.form.field.Field-method-getErrors" rel="Ext.form.field.Field-method-getErrors" class="docClass">validating</a> the field's current
   * value, and fires the <a href="#!/api/Ext.form.field.Field-event-validitychange" rel="Ext.form.field.Field-event-validitychange" class="docClass">validitychange</a> event if the field's validity has changed since the last validation.
   * <strong>Note</strong>: <a href="#!/api/Ext.form.field.Field-cfg-disabled" rel="Ext.form.field.Field-cfg-disabled" class="docClass">disabled</a> fields are always treated as valid.
   *
   * <p>Custom implementations of this method are allowed to have side-effects such as triggering error message display.
   * To validate without side-effects, use <a href="#!/api/Ext.form.field.Field-method-isValid" rel="Ext.form.field.Field-method-isValid" class="docClass">isValid</a>.</p>
   * @return True if the value is valid, else false
   */
  public native function validate():Boolean;

  /**
   * Template method before a field is reset.
   */
  protected native function beforeReset():void;
}
}