package ext.form.field {
import ext.button.Button;
import ext.dom.Element;

[Uses("ext.form.field.TextField")]
[Uses("ext.form.field.BaseField")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Native("Ext.form.field.File", require)]
/**
 * Fires when the underlying file input field's value has changed from the user selecting a new file from the system
 * file selection dialog.
 * @eventType ext.form.field.events.FileUploadField_valueEvent.CHANGE
 */
[Event(name = 'onChange', type = 'ext.form.field.events.FileUploadField_valueEvent')]
/**
 * A file upload field which has custom styling and allows control over the button text and other
 * features of <a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">text fields</a> like <a href="#!/api/Ext.form.field.Text-cfg-emptyText" rel="Ext.form.field.Text-cfg-emptyText" class="docClass">empty text</a>.
 * It uses a hidden file input element behind the scenes to allow user selection of a file and to
 * perform the actual upload during <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">form submit</a>.
 *
 * <p>Because there is no secure cross-browser way to programmatically set the value of a file input,
 * the standard Field <code>setValue</code> method is not implemented. The <code><a href="#!/api/Ext.form.field.File-method-getValue" rel="Ext.form.field.File-method-getValue" class="docClass">getValue</a></code> method will return
 * a value that is browser-dependent; some have just the file name, some have a full path, some use
 * a fake path.</p>
 *
 * <p><strong>IMPORTANT:</strong> File uploads are not performed using normal 'Ajax' techniques; see the description for
 * <a href="#!/api/Ext.form.Basic-method-hasUpload" rel="Ext.form.Basic-method-hasUpload" class="docClass">Ext.form.Basic.hasUpload</a> for details.</p>
 *
 * <h1>Example Usage</h1>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'Upload a Photo',
 *     width: 400,
 *     bodyPadding: 10,
 *     frame: true,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     items: [{
 *         xtype: 'filefield',
 *         name: 'photo',
 *         fieldLabel: 'Photo',
 *         labelWidth: 50,
 *         msgTarget: 'side',
 *         allowBlank: false,
 *         anchor: '100%',
 *         buttonText: 'Select Photo...'
 *     }],
 *
 *     buttons: [{
 *         text: 'Upload',
 *         handler: function() {
 *             var form = this.up('form').getForm();
 *             if(form.isValid()) {
 *                 form.submit({
 *                     url: 'photo-upload.php',
 *                     waitMsg: 'Uploading your photo...',
 *                     success: function(fp, o) {
 *                         <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Success', 'Your photo "' + o.result.file + '" has been uploaded.');
 *                     }
 *                 });
 *             }
 *         }
 *     }]
 * });
 * </code></pre>
 */
public class FileUploadField extends ext.form.field.TextField {
  /**
   * @param config @inheritDoc
   */
  public function FileUploadField(config:ext.form.field.FileUploadField = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Overridden to undefined as <a href="#!/api/Ext.form.field.File-cfg-emptyText" rel="Ext.form.field.File-cfg-emptyText" class="docClass">emptyText</a> is not supported with <a href="#!/api/Ext.form.field.File-cfg-inputType" rel="Ext.form.field.File-cfg-inputType" class="docClass">inputType</a>:'file' and should be avoided.
   * The default text to place into an empty field.
   * @see #getEmptyText()
   * @see #setEmptyText()
   */
  override public native function get emptyText():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set emptyText(value:String):void;

  [ExtConfig]
  /**
   * Unlike with other form fields, the readOnly config defaults to true in File field.
   * @default true
   * @see #setReadOnly()
   */
  override public native function get readOnly():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set readOnly(value:Boolean):void;

  /**
   * A reference to the trigger Button component created for this upload field. Only populated after this component is
   * rendered.
   */
  public native function get button():ext.button.Button;

  /**
   * @private
   */
  public native function set button(value:ext.button.Button):void;

  /**
   * A reference to the invisible file input element created for this upload field. Only populated after this
   * component is rendered.
   */
  public native function get fileInputEl():ext.dom.Element;

  /**
   * @private
   */
  public native function set fileInputEl(value:ext.dom.Element):void;

  [ExtConfig]
  /**
   * An optional list of file MIME types accepted by this field.
   * This string will be rendered in to the <code>accept</code> attribute of the file input and should
   * conform to HTML requirements: http://www.w3.org/TR/html-markup/input.file.html
   * @since 6.2.0
   */
  public native function get accept():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set accept(value:String):void;

  [ExtConfig]
  /**
   * Specify optional custom button <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a> config (eg. iconCls, text) for the upload button
   */
  public native function get buttonConfig():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set buttonConfig(value:Object):void;

  [ExtConfig]
  /**
   * The number of pixels of space reserved between the button and the text field. Note that this only
   * applies if <a href="#!/api/Ext.form.field.File-cfg-buttonOnly" rel="Ext.form.field.File-cfg-buttonOnly" class="docClass">buttonOnly</a> = false.
   * @default 3
   */
  public native function get buttonMargin():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set buttonMargin(value:Number):void;

  [ExtConfig]
  /**
   * True to display the file upload field as a button with no visible text field. If true, all
   * inherited Text members will still be available.
   * @default false
   */
  public native function get buttonOnly():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set buttonOnly(value:Boolean):void;

  [ExtConfig]
  /**
   * The button text to display on the upload button. Note that if you supply a value for
   * <a href="#!/api/Ext.form.field.File-cfg-buttonConfig" rel="Ext.form.field.File-cfg-buttonConfig" class="docClass">buttonConfig</a>, the buttonConfig.text value will be used instead if available.
   * @default 'Browse...'
   */
  public native function get buttonText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set buttonText(value:String):void;

  [ExtConfig]
  /**
   * True to clear the selected file value when the form this field belongs to
   * is submitted to the server.
   * @default true
   */
  public native function get clearOnSubmit():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set clearOnSubmit(value:Boolean):void;

  /**
   * Overridden to do nothing
   * @param value The value to set
   */
  override public native function setValue(value:*):ext.form.field.IField;

  /**
   * Gets the markup to be inserted into the subTplMarkup.
   */
  public native function getTriggerMarkup():String;
}
}