package ext.form.field {

[Uses("ext.form.field.SpinnerField")]
[Uses("ext.form.field.TextField")]
[Uses("ext.form.field.BaseField")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Native("Ext.form.field.Number", require)]
/**
 * A numeric text field that provides automatic keystroke filtering to disallow non-numeric characters,
 * and numeric validation to limit the value to a range of valid numbers. The range of acceptable number
 * values can be controlled by setting the <a href="#!/api/Ext.form.field.Number-cfg-minValue" rel="Ext.form.field.Number-cfg-minValue" class="docClass">minValue</a> and <a href="#!/api/Ext.form.field.Number-cfg-maxValue" rel="Ext.form.field.Number-cfg-maxValue" class="docClass">maxValue</a> configs, and fractional
 * decimals can be disallowed by setting <a href="#!/api/Ext.form.field.Number-cfg-allowDecimals" rel="Ext.form.field.Number-cfg-allowDecimals" class="docClass">allowDecimals</a> to <code>false</code>.
 *
 * <p>By default, the number field is also rendered with a set of up/down spinner buttons and has
 * up/down arrow key and mouse wheel event listeners attached for incrementing/decrementing the value by the
 * <a href="#!/api/Ext.form.field.Number-cfg-step" rel="Ext.form.field.Number-cfg-step" class="docClass">step</a> value. To hide the spinner buttons set <code><a href="#!/api/Ext.form.field.Number-cfg-hideTrigger" rel="Ext.form.field.Number-cfg-hideTrigger" class="docClass">hideTrigger</a>:true</code>; to disable
 * the arrow key and mouse wheel handlers set <code><a href="#!/api/Ext.form.field.Number-cfg-keyNavEnabled" rel="Ext.form.field.Number-cfg-keyNavEnabled" class="docClass">keyNavEnabled</a>:false</code> and
 * <code><a href="#!/api/Ext.form.field.Number-cfg-mouseWheelEnabled" rel="Ext.form.field.Number-cfg-mouseWheelEnabled" class="docClass">mouseWheelEnabled</a>:false</code>. See the example below.</p>
 *
 * <h1>Example usage</h1>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'On The Wall',
 *     width: 300,
 *     bodyPadding: 10,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     items: [{
 *         xtype: 'numberfield',
 *         anchor: '100%',
 *         name: 'bottles',
 *         fieldLabel: 'Bottles of Beer',
 *         value: 99,
 *         maxValue: 99,
 *         minValue: 0
 *     }],
 *     buttons: [{
 *         text: 'Take one down, pass it around',
 *         handler: function() {
 *             this.up('form').down('[name=bottles]').spinDown();
 *         }
 *     }]
 * });
 * </code></pre>
 *
 * <h1>Removing UI Enhancements</h1>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'Personal Info',
 *     width: 300,
 *     bodyPadding: 10,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     items: [{
 *         xtype: 'numberfield',
 *         anchor: '100%',
 *         name: 'age',
 *         fieldLabel: 'Age',
 *         minValue: 0, //prevents negative numbers
 *
 *         // Remove spinner buttons, and arrow key and mouse wheel listeners
 *         hideTrigger: true,
 *         keyNavEnabled: false,
 *         mouseWheelEnabled: false
 *     }]
 * });
 * </code></pre>
 *
 * <h1>Using Step</h1>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     title: 'Step',
 *     width: 300,
 *     bodyPadding: 10,
 *     items: [{
 *         xtype: 'numberfield',
 *         anchor: '100%',
 *         name: 'evens',
 *         fieldLabel: 'Even Numbers',
 *
 *         // Set step so it skips every other number
 *         step: 2,
 *         value: 0,
 *
 *         // Add change handler to force user-entered numbers to evens
 *         listeners: {
 *             change: function(field, value) {
 *                 value = parseInt(value, 10);
 *                 field.setValue(value + value % 2);
 *             }
 *         }
 *     }]
 * });
 * </code></pre>
 */
public class NumberField extends ext.form.field.SpinnerField {
  /**
   * @param config @inheritDoc
   */
  public function NumberField(config:ext.form.field.NumberField = null) {
    super();
  }

  [ExtConfig]
  /**
   * False to disallow decimal values
   * @default true
   */
  public native function get allowDecimals():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowDecimals(value:Boolean):void;

  [ExtConfig]
  /**
   * Set to <code>false</code> to disallow Exponential number notation
   * @default true
   */
  public native function get allowExponential():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowExponential(value:Boolean):void;

  [ExtConfig]
  /**
   * True to automatically strip not allowed characters from the field.
   * @default false
   */
  public native function get autoStripChars():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoStripChars(value:Boolean):void;

  [ExtConfig]
  /**
   * The base set of characters to evaluate as valid numbers.
   * @default '0123456789'
   */
  public native function get baseChars():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set baseChars(value:String):void;

  [ExtConfig]
  /**
   * The maximum precision to display after the decimal separator
   * @default 2
   */
  public native function get decimalPrecision():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set decimalPrecision(value:Number):void;

  [ExtConfig]
  /**
   * Character(s) to allow as the decimal separator. Defaults to <a href="#!/api/Ext.util.Format-property-decimalSeparator" rel="Ext.util.Format-property-decimalSeparator" class="docClass">decimalSeparator</a>.
   */
  public native function get decimalSeparator():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set decimalSeparator(value:String):void;

  [ExtConfig]
  /**
   * Error text to display if the maximum value validation fails.
   * @default 'The maximum value for this field is {0}'
   */
  public native function get maxText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maxText(value:String):void;

  [ExtConfig]
  /**
   * The maximum allowed value. Will be used by the field's validation logic, and for
   * <a href="#!/api/Ext.form.field.Spinner-method-setSpinUpEnabled" rel="Ext.form.field.Spinner-method-setSpinUpEnabled" class="docClass">enabling/disabling the up spinner button</a>.
   *
   * <p>Defaults to Number.MAX_VALUE.</p>
   * @see #setMaxValue()
   */
  public native function get maxValue():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set maxValue(value:Number):void;

  [ExtConfig]
  /**
   * Error text to display if the minimum value validation fails.
   * @default 'The minimum value for this field is {0}'
   */
  public native function get minText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set minText(value:String):void;

  [ExtConfig]
  /**
   * The minimum allowed value. Will be used by the field's validation logic,
   * and for <a href="#!/api/Ext.form.field.Spinner-method-setSpinUpEnabled" rel="Ext.form.field.Spinner-method-setSpinUpEnabled" class="docClass">enabling/disabling the down spinner button</a>.
   *
   * <p>Defaults to Number.NEGATIVE_INFINITY.</p>
   * @see #setMinValue()
   */
  public native function get minValue():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set minValue(value:Number):void;

  [ExtConfig]
  /**
   * Error text to display if the value is not a valid number. For example, this can happen if a valid character like
   * '.' or '-' is left in the field with no number.
   * @default '{0} is not a valid number'
   */
  public native function get nanText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set nanText(value:String):void;

  [ExtConfig]
  /**
   * Error text to display if the value is negative and <a href="#!/api/Ext.form.field.Number-cfg-minValue" rel="Ext.form.field.Number-cfg-minValue" class="docClass">minValue</a> is set to 0. This is used instead of the
   * <a href="#!/api/Ext.form.field.Number-cfg-minText" rel="Ext.form.field.Number-cfg-minText" class="docClass">minText</a> in that circumstance only.
   * @default 'The value cannot be negative'
   */
  public native function get negativeText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set negativeText(value:String):void;

  [ExtConfig]
  /**
   * Specifies a numeric interval by which the field's value will be incremented or decremented when the user invokes
   * the spinner.
   * @default 1
   */
  public native function get step():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set step(value:Number):void;

  [ExtConfig]
  /**
   * False to ensure that the <a href="#!/api/Ext.form.field.Number-method-getSubmitValue" rel="Ext.form.field.Number-method-getSubmitValue" class="docClass">getSubmitValue</a> method strips
   * always uses <code>.</code> as the separator, regardless of the <a href="#!/api/Ext.form.field.Number-cfg-decimalSeparator" rel="Ext.form.field.Number-cfg-decimalSeparator" class="docClass">decimalSeparator</a>
   * configuration.
   * @default true
   */
  public native function get submitLocaleSeparator():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set submitLocaleSeparator(value:Boolean):void;

  /**
   * Runs all of Number's validations and returns an array of any errors. Note that this first runs Text's
   * validations, so the returned array is an amalgamation of all field errors. The additional validations run test
   * that the value is a number, and that it is within the configured min and max values.
   * @param value The value to get errors for (defaults to the current field value)
   * @return All validation errors for this field
   */
  override public native function getErrors(value:*):Array;

  /**
   * Replaces any existing <a href="#!/api/Ext.form.field.Number-cfg-maxValue" rel="Ext.form.field.Number-cfg-maxValue" class="docClass">maxValue</a> with the new value.
   * @param value The maximum value
   */
  public native function setMaxValue(value:Number):void;

  /**
   * Replaces any existing <a href="#!/api/Ext.form.field.Number-cfg-minValue" rel="Ext.form.field.Number-cfg-minValue" class="docClass">minValue</a> with the new value.
   * @param value The minimum value
   */
  public native function setMinValue(value:Number):void;
}
}