package ext.fx {
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.fx.Anim", require)]
/**
 * Fires when the animation is complete.
 * @eventType ext.fx.events.Anim_startTimeEvent.AFTER_ANIMATE
 */
[Event(name = 'onAfterAnimate', type = 'ext.fx.events.Anim_startTimeEvent')]
/**
 * Fires before the animation starts. A handler can return false to cancel the animation.
 * @eventType ext.fx.events.AnimEvent.BEFORE_ANIMATE
 */
[Event(name = 'onBeforeAnimate', type = 'ext.fx.events.AnimEvent')]
/**
 * Fires when the animation's last frame has been set.
 * @eventType ext.fx.events.Anim_startTimeEvent.LAST_FRAME
 */
[Event(name = 'onLastFrame', type = 'ext.fx.events.Anim_startTimeEvent')]
/**
 * This class manages animation for a specific <a href="#!/api/Ext.fx.Anim-cfg-target" rel="Ext.fx.Anim-cfg-target" class="docClass">target</a>. The animation allows
 * animation of various properties on the target, such as size, position, color and others.
 *
 * <h2>Starting Conditions</h2>
 *
 * <p>The starting conditions for the animation are provided by the <a href="#!/api/Ext.fx.Anim-cfg-from" rel="Ext.fx.Anim-cfg-from" class="docClass">from</a> configuration.
 * Any/all of the properties in the <a href="#!/api/Ext.fx.Anim-cfg-from" rel="Ext.fx.Anim-cfg-from" class="docClass">from</a> configuration can be specified. If a particular
 * property is not defined, the starting value for that property will be read directly from the target.</p>
 *
 * <h2>End Conditions</h2>
 *
 * <p>The ending conditions for the animation are provided by the <a href="#!/api/Ext.fx.Anim-cfg-to" rel="Ext.fx.Anim-cfg-to" class="docClass">to</a> configuration. These mark
 * the final values once the animations has finished. The values in the <a href="#!/api/Ext.fx.Anim-cfg-from" rel="Ext.fx.Anim-cfg-from" class="docClass">from</a> can mirror
 * those in the <a href="#!/api/Ext.fx.Anim-cfg-to" rel="Ext.fx.Anim-cfg-to" class="docClass">to</a> configuration to provide a starting point.</p>
 *
 * <h2>Other Options</h2>
 *
 * <ul>
 * <li><a href="#!/api/Ext.fx.Anim-cfg-duration" rel="Ext.fx.Anim-cfg-duration" class="docClass">duration</a>: Specifies the time period of the animation.</li>
 * <li><a href="#!/api/Ext.fx.Anim-cfg-easing" rel="Ext.fx.Anim-cfg-easing" class="docClass">easing</a>: Specifies the easing of the animation.</li>
 * <li><a href="#!/api/Ext.fx.Anim-cfg-iterations" rel="Ext.fx.Anim-cfg-iterations" class="docClass">iterations</a>: Allows the animation to repeat a number of times.</li>
 * <li><a href="#!/api/Ext.fx.Anim-cfg-alternate" rel="Ext.fx.Anim-cfg-alternate" class="docClass">alternate</a>: Used in conjunction with <a href="#!/api/Ext.fx.Anim-cfg-iterations" rel="Ext.fx.Anim-cfg-iterations" class="docClass">iterations</a>, reverses the direction every second iteration.</li>
 * </ul>
 *
 *
 * <h2>Example Code</h2>
 *
 * <pre class='inline-example '><code>var myComponent = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>', {
 *     renderTo: document.body,
 *     width: 200,
 *     height: 200,
 *     style: 'border: 1px solid red;'
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a>', {
 *     target: myComponent,
 *     duration: 1000,
 *     from: {
 *         width: 400 //starting width 400
 *     },
 *     to: {
 *         width: 300, //end width 300
 *         height: 300 // end height 300
 *     }
 * });
 * </code></pre>
 */
public class Anim extends ext.mixin.Observable {
  /**
   * @param config @inheritDoc
   */
  public function Anim(config:ext.fx.Anim = null) {
    super();
  }

  /**
   * Current iteration the animation is running.
   * @default 0
   */
  public native function get currentIteration():Number;

  /**
   * @private
   */
  public native function set currentIteration(value:Number):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Anim, or subclass thereof.
   * @default true
   */
  public native function get isAnimation():Boolean;

  /**
   * @private
   */
  public native function set isAnimation(value:Boolean):void;

  /**
   * Flag to determine if the animation is paused. Only set this to true if you need to
   * keep the Anim instance around to be un-paused later; otherwise call <a href="#!/api/Ext.fx.Anim-method-end" rel="Ext.fx.Anim-method-end" class="docClass">end</a>.
   * @default false
   */
  public native function get paused():Boolean;

  /**
   * @private
   */
  public native function set paused(value:Boolean):void;

  /**
   * Flag to determine if the animation has started
   * @default false
   */
  public native function get running():Boolean;

  /**
   * @private
   */
  public native function set running(value:Boolean):void;

  /**
   * Starting time of the animation.
   * @default 0
   */
  public native function get startTime():Date;

  /**
   * @private
   */
  public native function set startTime(value:Date):void;

  [ExtConfig]
  /**
   * Used in conjunction with iterations to reverse the animation each time an iteration completes.
   * @default false
   */
  public native function get alternate():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set alternate(value:Boolean):void;

  [ExtConfig]
  /**
   * <code>true</code> to immediately force this animation to its final state. This can be useful
   * in cases where you want the final effect of an animation, but need to the actual
   * animation dynamically. Also see the <a href="#!/api/Ext.fx.Anim-method-jumpToEnd" rel="Ext.fx.Anim-method-jumpToEnd" class="docClass">jumpToEnd</a> method.
   * @default false
   */
  public native function get autoEnd():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoEnd(value:Boolean):void;

  [ExtConfig]
  /**
   * A function to be run after the animation has completed.
   * &#64;controllable
   */
  public native function get callback():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set callback(value:*):void;

  [ExtConfig]
  /**
   * Time to delay before starting the animation.
   * @default 0
   */
  public native function get delay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set delay(value:Number):void;

  [ExtConfig]
  /**
   * Time in milliseconds for a single animation to last. If the <a href="#!/api/Ext.fx.Anim-cfg-iterations" rel="Ext.fx.Anim-cfg-iterations" class="docClass">iterations</a> property is
   * specified, then each animate will take the same duration for each iteration.
   * @default 250
   */
  public native function get duration():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set duration(value:Number):void;

  [ExtConfig]
  /**
   * Currently only for Component Animation: Only set a component's outer element size bypassing layouts.
   * Set to true to do full layouts for every frame of the animation.
   * @default false
   */
  public native function get dynamic():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dynamic(value:Boolean):void;

  [ExtConfig]
  /**
   * This describes how the intermediate values used during a transition will be calculated.
   * It allows for a transition to change speed over its duration.
   *
   * <ul>
   * <li>backIn</li>
   * <li>backOut</li>
   * <li>bounceIn</li>
   * <li>bounceOut</li>
   * <li>ease</li>
   * <li>easeIn</li>
   * <li>easeOut</li>
   * <li>easeInOut</li>
   * <li>elasticIn</li>
   * <li>elasticOut</li>
   * <li>cubic-bezier(x1, y1, x2, y2)</li>
   * </ul>
   *
   *
   * <p>Note that cubic-bezier will create a custom easing curve following the CSS3 <a href="http://www.w3.org/TR/css3-transitions/#transition-timing-function_tag">transition-timing-function</a>
   * specification.  The four values specify points P1 and P2 of the curve as (x1, y1, x2, y2). All values must
   * be in the range [0, 1] or the definition is invalid.</p>
   * @default 'ease'
   */
  public native function get easing():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set easing(value:String):void;

  [ExtConfig]
  /**
   * An object containing property/value pairs for the beginning of the animation.  If not specified, the current state of the
   * Ext.fx.target will be used. For example:
   *
   * <pre><code>from: {
   *     opacity: 0,       // Transparent
   *     color: '#ffffff', // White
   *     left: 0
   * }
   * </code></pre>
   */
  public native function get from():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set from(value:Object):void;

  [ExtConfig]
  /**
   * Number of times to execute the animation.
   * @default 1
   */
  public native function get iterations():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set iterations(value:Number):void;

  [ExtConfig]
  /**
   * Animation keyframes follow the CSS3 Animation configuration pattern. 'from' is always considered '0%' and 'to'
   * is considered '100%'. <strong>Every keyframe declaration must have a keyframe rule for 0% and 100%, possibly defined using
   * "from" or "to".</strong>  A keyframe declaration without these keyframe selectors is invalid and will not be available for
   * animation.  The keyframe declaration for a keyframe rule consists of properties and values. Properties that are unable to
   * be animated are ignored in these rules, with the exception of 'easing' which can be changed at each keyframe. For example:
   *
   * <pre><code>keyframes : {
   *     '0%': {
   *         left: 100
   *     },
   *     '40%': {
   *         left: 150
   *     },
   *     '60%': {
   *         left: 75
   *     },
   *     '100%': {
   *         left: 100
   *     }
   * }
   * </code></pre>
   */
  public native function get keyframes():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set keyframes(value:Object):void;

  [ExtConfig]
  /**
   * <code>true</code> to remove the target when the animation is complete, using the appropriate removal
   * method for the target. For example, a component will be destroyed, elements will be removed.
   */
  public native function get remove():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set remove(value:Boolean):void;

  [ExtConfig]
  /**
   * Run the animation from the end to the beginning
   * Defaults to false.
   * @default false
   */
  public native function get reverse():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set reverse(value:Boolean):void;

  [ExtConfig]
  /**
   * The scope that the <a href="#!/api/Ext.fx.Anim-cfg-callback" rel="Ext.fx.Anim-cfg-callback" class="docClass">callback</a> function will be called with
   */
  public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scope(value:Object):void;

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.fx.target.Target" rel="Ext.fx.target.Target" class="docClass">Ext.fx.target.Target</a> to apply the animation to.  This should only be specified when creating an <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> directly.
   * The target does not need to be a <a href="#!/api/Ext.fx.target.Target" rel="Ext.fx.target.Target" class="docClass">Ext.fx.target.Target</a> instance, it can be the underlying object. For example, you can
   * pass a Component, Element or Sprite as the target and the Anim will create the appropriate <a href="#!/api/Ext.fx.target.Target" rel="Ext.fx.target.Target" class="docClass">Ext.fx.target.Target</a> object
   * automatically.
   */
  public native function get target():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set target(value:*):void;

  [ExtConfig]
  /**
   * An object containing property/value pairs for the end of the animation. For example:
   *
   * <pre><code>to: {
   *     opacity: 1,       // Opaque
   *     color: '#00ff00', // Green
   *     left: 500
   * }
   * </code></pre>
   */
  public native function get to():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set to(value:Object):void;

  /**
   * Fire afteranimate event and end the animation. Usually called automatically when the
   * animation reaches its final frame, but can also be called manually to preemptively
   * stop and destroy the running animation.
   */
  public native function end(suppressEvent:* = undefined):void;

  /**
   * Immediately force this animation to its final state.
   */
  public native function jumpToEnd():void;
}
}