package ext.grid {
import ext.panel.TablePanel;

[Uses("ext.panel.TablePanel")]
[Uses("ext.grid.locking.Lockable")]
[Uses("ext.Base")]
[Uses("ext.panel.Panel")]
[Uses("ext.container.DockingContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.grid.Panel", require)]
/**
 * Fires before a reconfigure to enable modification of incoming Store and columns.
 * @eventType ext.grid.events.GridPanel_store_columns_oldStore_oldColumnsEvent.BEFORE_RECONFIGURE
 */
[Event(name = 'onBeforeReconfigure', type = 'ext.grid.events.GridPanel_store_columns_oldStore_oldColumnsEvent')]
/**
 * Fires after a reconfigure.
 * @eventType ext.grid.events.GridPanel_store_columns_oldStore_oldColumnsEvent.RECONFIGURE
 */
[Event(name = 'onReconfigure', type = 'ext.grid.events.GridPanel_store_columns_oldStore_oldColumnsEvent')]
/**
 * Grids are an excellent way of showing large amounts of tabular data on the client side.
 * Essentially a supercharged <code>&lt;table&gt;</code>, GridPanel makes it easy to fetch, sort and filter
 * large amounts of data.
 *
 * <p>Grids are composed of two main pieces - a <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a> full of data and
 * a set of columns to render.</p>
 *
 * <h2>Basic GridPanel</h2>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     storeId: 'simpsonsStore',
 *     fields:[ 'name', 'email', 'phone'],
 *     data: [
 *         { name: 'Lisa', email: 'lisa&#64;simpsons.com', phone: '555-111-1224' },
 *         { name: 'Bart', email: 'bart&#64;simpsons.com', phone: '555-222-1234' },
 *         { name: 'Homer', email: 'homer&#64;simpsons.com', phone: '555-222-1244' },
 *         { name: 'Marge', email: 'marge&#64;simpsons.com', phone: '555-222-1254' }
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Simpsons',
 *     store: <a href="#!/api/Ext.data.StoreManager-method-lookup" rel="Ext.data.StoreManager-method-lookup" class="docClass">Ext.data.StoreManager.lookup</a>('simpsonsStore'),
 *     columns: [
 *         { text: 'Name', dataIndex: 'name' },
 *         { text: 'Email', dataIndex: 'email', flex: 1 },
 *         { text: 'Phone', dataIndex: 'phone' }
 *     ],
 *     height: 200,
 *     width: 400,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>The code above produces a simple grid with three columns. We specified a Store which
 * will load JSON data inline.
 * In most apps we would be placing the grid inside another container and wouldn't need to
 * use the <a href="#!/api/Ext.grid.Panel-cfg-height" rel="Ext.grid.Panel-cfg-height" class="docClass">height</a>, <a href="#!/api/Ext.grid.Panel-cfg-width" rel="Ext.grid.Panel-cfg-width" class="docClass">width</a> and <a href="#!/api/Ext.grid.Panel-cfg-renderTo" rel="Ext.grid.Panel-cfg-renderTo" class="docClass">renderTo</a> configurations but they
 * are included here to make it easy to get up and running.</p>
 *
 * <p>The grid we created above will contain a header bar with a title ('Simpsons'), a row of
 * column headers directly underneath and finally the grid rows under the headers.</p>
 *
 * <p><strong>Height config with bufferedRenderer: true</strong></p>
 *
 * <p>The <a href="#!/api/Ext.grid.Panel-cfg-height" rel="Ext.grid.Panel-cfg-height" class="docClass">height</a> config must be set when creating a grid using
 * <a href="#!/api/Ext.grid.Panel-cfg-bufferedRenderer" rel="Ext.grid.Panel-cfg-bufferedRenderer" class="docClass">bufferedRenderer</a>: true <em>and</em> the grid's height is not managed
 * by an owning container layout.  In Ext JS 5.x bufferedRendering is true by default.</p>
 *
 * <h2>Configuring columns</h2>
 *
 * <p>By default, each column is sortable and will toggle between ASC and DESC sorting when
 * you click on its header. Each column header is also reorderable by default, and each
 * gains a drop-down menu with options to hide and show columns.  It's easy to configure
 * each column - here we use the same example as above and just modify the columns config:</p>
 *
 * <pre><code>columns: [
 *     {
 *         text: 'Name',
 *         dataIndex: 'name',
 *         sortable: false,
 *         hideable: false,
 *         flex: 1
 *     },
 *     {
 *         text: 'Email',
 *         dataIndex: 'email',
 *         hidden: true
 *     },
 *     {
 *         text: 'Phone',
 *         dataIndex: 'phone',
 *         width: 100
 *     }
 * ]
 * </code></pre>
 *
 * <p>We turned off sorting and hiding on the 'Name' column so clicking its header now has no
 * effect. We also made the Email column hidden by default (it can be shown again by using
 * the menu on any other column). We also set the Phone column to a fixed with of 100px
 * and flexed the Name column, which means it takes up all remaining width after the other
 * columns have been accounted for. See the <a href="#!/api/Ext.grid.column.Column" rel="Ext.grid.column.Column" class="docClass">column docs</a> for
 * more details.</p>
 *
 * <h2>Renderers</h2>
 *
 * <p>As well as customizing columns, it's easy to alter the rendering of individual cells
 * using renderers. A renderer is tied to a particular column and is passed the value that
 * would be rendered into each cell in that column. For example, we could define a
 * renderer function for the email column to turn each email address into a mailto link:</p>
 *
 * <pre><code>columns: [
 *     {
 *         text: 'Email',
 *         dataIndex: 'email',
 *         renderer: function(value) {
 *             return <a href="#!/api/Ext.String-method-format" rel="Ext.String-method-format" class="docClass">Ext.String.format</a>('&lt;a href="mailto:{0}"&gt;{1}&lt;/a&gt;', value, value);
 *         }
 *     }
 * ]
 * </code></pre>
 *
 * <p>See the <a href="#!/api/Ext.grid.column.Column" rel="Ext.grid.column.Column" class="docClass">column docs</a> for more information on renderers.</p>
 *
 * <h2>Selection Models</h2>
 *
 * <p>Sometimes you simply want to render data for viewing, but usually it's
 * necessary to interact with or update that data. Grids use a concept called a Selection
 * Model, which is simply a mechanism for selecting some part of the data in the grid. The
 * two main types of Selection Model are RowSelectionModel, where entire rows are
 * selected, and CellSelectionModel, where individual cells are selected.</p>
 *
 * <p>Grids use a Row Selection Model by default, but this is easy to customize like so:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     selModel: 'cellmodel',
 *     store: ...
 * });
 * </code></pre>
 *
 * <p>Specifying the <code>cellmodel</code> changes a couple of things. Firstly, clicking on a cell now
 * selects just that cell (using a <a href="#!/api/Ext.selection.RowModel" rel="Ext.selection.RowModel" class="docClass">rowmodel</a> will select the
 * entire row), and secondly the keyboard navigation will walk from cell to cell instead
 * of row to row. Cell-based selection models are usually used in conjunction with
 * editing.</p>
 *
 * <p>You may also utilize selModel as a config object for an instance of <a href="#!/api/Ext.selection.Model" rel="Ext.selection.Model" class="docClass">Ext.selection.Model</a>.</p>
 *
 * <p>For example:</p>
 *
 * <pre><code>selModel: {
 *     selType: 'cellmodel',
 *     mode   : 'MULTI'
 * }
 * </code></pre>
 *
 * <p>This allows you to modify additional selection model configurations such as:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.selection.Model-cfg-mode" rel="Ext.selection.Model-cfg-mode" class="docClass">mode</a> - Specifies whether user may select multiple
 * rows or single rows</li>
 * <li><a href="#!/api/Ext.selection.Model-cfg-allowDeselect" rel="Ext.selection.Model-cfg-allowDeselect" class="docClass">allowDeselect</a> - Specifies whether user may
 * deselect records (when in SINGLE mode)</li>
 * <li><a href="#!/api/Ext.selection.Model-cfg-ignoreRightMouseSelection" rel="Ext.selection.Model-cfg-ignoreRightMouseSelection" class="docClass">ignoreRightMouseSelection</a> - Specifies
 * whether user may ignore right clicks
 * for selection purposes</li>
 * </ul>
 *
 *
 * <h2>Sorting &amp; Filtering</h2>
 *
 * <p>Every grid is attached to a <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a>, which provides multi-sort and
 * filtering capabilities. It's
 * easy to set up a grid to be sorted from the start:</p>
 *
 * <pre><code>var myGrid = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     store: {
 *         fields: ['name', 'email', 'phone'],
 *         sorters: ['name', 'phone']
 *     },
 *     columns: [
 *         { text: 'Name',  dataIndex: 'name' },
 *         { text: 'Email', dataIndex: 'email' }
 *     ]
 * });
 * </code></pre>
 *
 * <p>Sorting at run time is easily accomplished by simply clicking each column header. If
 * you need to perform sorting on more than one field at run time it's easy to do so by
 * adding new sorters to the store:</p>
 *
 * <pre><code>myGrid.store.sort([
 *     { property: 'name',  direction: 'ASC' },
 *     { property: 'email', direction: 'DESC' }
 * ]);
 * </code></pre>
 *
 * <p>See <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> for examples of filtering.</p>
 *
 * <h2>State saving</h2>
 *
 * <p>When configured <a href="#!/api/Ext.grid.Panel-cfg-stateful" rel="Ext.grid.Panel-cfg-stateful" class="docClass">stateful</a>, grids save their column state (order and width)
 * encapsulated within the default Panel state of changed width and height and
 * collapsed/expanded state.</p>
 *
 * <p>On a <code>stateful</code> grid, not only should the Grid have a <a href="#!/api/Ext.grid.Panel-cfg-stateId" rel="Ext.grid.Panel-cfg-stateId" class="docClass">stateId</a>, each
 * <a href="#!/api/Ext.grid.Panel-cfg-columns" rel="Ext.grid.Panel-cfg-columns" class="docClass">column</a> of the grid should also be configured with a
 * <a href="#!/api/Ext.grid.column.Column-cfg-stateId" rel="Ext.grid.column.Column-cfg-stateId" class="docClass">stateId</a> which identifies that column locally
 * within the grid.</p>
 *
 * <p>Omitting the <code>stateId</code> config from the columns results in columns with generated
 * internal ID's.  The generated ID's are subject to change on each page load
 * making it impossible for the state manager to restore the previous state of the
 * columns.</p>
 *
 * <h2>Plugins and Features</h2>
 *
 * <p>Grid supports addition of extra functionality through features and plugins:</p>
 *
 * <ul>
 * <li><p><a href="#!/api/Ext.grid.plugin.CellEditing" rel="Ext.grid.plugin.CellEditing" class="docClass">CellEditing</a> - editing grid contents one cell at a time.</p></li>
 * <li><p><a href="#!/api/Ext.grid.plugin.RowEditing" rel="Ext.grid.plugin.RowEditing" class="docClass">RowEditing</a> - editing grid contents an entire row at a time.</p></li>
 * <li><p><a href="#!/api/Ext.grid.plugin.DragDrop" rel="Ext.grid.plugin.DragDrop" class="docClass">DragDrop</a> - drag-drop reordering of grid rows.</p></li>
 * <li><p><a href="#!/api/Ext.toolbar.Paging" rel="Ext.toolbar.Paging" class="docClass">Paging toolbar</a> - paging through large sets of data.</p></li>
 * <li><p><a href="#!/api/Ext.grid.plugin.BufferedRenderer" rel="Ext.grid.plugin.BufferedRenderer" class="docClass">Infinite scrolling</a> - another way to handle large sets of data.</p></li>
 * <li><p><a href="#!/api/Ext.grid.column.RowNumberer" rel="Ext.grid.column.RowNumberer" class="docClass">RowNumberer</a> - automatically numbered rows.</p></li>
 * <li><p><a href="#!/api/Ext.grid.feature.Grouping" rel="Ext.grid.feature.Grouping" class="docClass">Grouping</a> - grouping together rows having the same value in a particular field.</p></li>
 * <li><p><a href="#!/api/Ext.grid.feature.Summary" rel="Ext.grid.feature.Summary" class="docClass">Summary</a> - a summary row at the bottom of a grid.</p></li>
 * <li><p><a href="#!/api/Ext.grid.feature.GroupingSummary" rel="Ext.grid.feature.GroupingSummary" class="docClass">GroupingSummary</a> - a summary row at the bottom of each group.</p></li>
 * </ul>
 */
public class GridPanel extends ext.panel.TablePanel {
  /**
   * @param config @inheritDoc
   */
  public function GridPanel(config:ext.grid.GridPanel = null) {
    super();
  }

  [ExtConfig]
  /**
   * False to remove row line styling
   * @default true
   */
  override public native function get rowLines():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set rowLines(value:Boolean):void;
}
}