package ext.grid.column {
import ext.mixin.IStyleCacher;

[Uses("ext.grid.column.Column")]
[Uses("ext.mixin.StyleCacher")]
[Uses("ext.grid.header.HeaderContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.util.FocusableContainer")]
[Native("Ext.grid.column.Widget", require)]
/**
 * A widget column is configured with a <a href="#!/api/Ext.grid.column.Widget-cfg-widget" rel="Ext.grid.column.Widget-cfg-widget" class="docClass">widget</a> config object which specifies an
 * <a href="#!/api/Ext.Component-cfg-xtype" rel="Ext.Component-cfg-xtype" class="docClass">xtype</a> to indicate which type of Widget or Component belongs
 * in the cells of this column.
 *
 * <p>When a widget cell is rendered, a <a href="#!/api/Ext.Widget" rel="Ext.Widget" class="docClass">Widget</a> or <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a> of the specified type
 * is rendered into that cell.</p>
 *
 * <p>There are two ways of setting values in a cell widget.</p>
 *
 * <p>Ths simplest way is to use data binding. Each cell widget has a <a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">ViewModel</a> injected which inherits from any ViewModel
 * that the grid is using, and contains two extra properties:</p>
 *
 * <ul>
 * <li><code>record</code> : <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a><br>The record which backs the grid row.</li>
 * <li><code>recordIndex</code> : Number<br>The index in the dataset of the record which backs the grid row.</li>
 * </ul>
 *
 *
 * <p>The widget configuration may contain a <a href="#!/api/Ext.grid.column.Widget-cfg-bind" rel="Ext.grid.column.Widget-cfg-bind" class="docClass">bind</a> config which uses the ViewModel's data.</p>
 *
 * <p>The deprecated way is to configure the column with a <a href="#!/api/Ext.grid.column.Widget-cfg-dataIndex" rel="Ext.grid.column.Widget-cfg-dataIndex" class="docClass">dataIndex</a>. The widget's
 * <a href="#!/api/Ext.Component-property-defaultBindProperty" rel="Ext.Component-property-defaultBindProperty" class="docClass">defaultBindProperty</a> will be set using the
 * specified field from the associated record.</p>
 *
 * <p>In the example below we are monitoring the throughput of electricity substations. The capacity being
 * used as a proportion of the maximum rated capacity is displayed as a progress bar. As new data arrives and the
 * instantaneous usage value is updated, the <code>capacityUsed</code> field updates itself, and the record's
 * change is broadcast to all bindings. The <a href="#!/api/Ext.Progress" rel="Ext.Progress" class="docClass">Progress Bar Widget</a>'s
 * <a href="#!/api/Ext.Progress-property-defaultBindProperty" rel="Ext.Progress-property-defaultBindProperty" class="docClass">defaultBindProperty</a> (which is
 * "value") is set to the calculated <code>capacityUsed</code>.</p>
 *
 * <pre class='inline-example '><code>var grid = new <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>({
 *     title: 'Substation power monitor',
 *     width: 600,
 *     viewConfig: {
 *         enableTextSelection: false,
 *         markDirty: false
 *     },
 *     columns: [{
 *         text: 'Id',
 *         dataIndex: 'id',
 *         width: 120
 *     }, {
 *         text: 'Rating',
 *         dataIndex: 'maxCapacity',
 *         width: 80
 *     }, {
 *         text: 'Avg.',
 *         dataIndex: 'avg',
 *         width: 85,
 *         formatter: 'number("0.00")'
 *     }, {
 *         text: 'Max',
 *         dataIndex: 'max',
 *         width: 80
 *     }, {
 *         text: 'Instant',
 *         dataIndex: 'instant',
 *         width: 80
 *     }, {
 *         text: '%Capacity',
 *         width: 150,
 *
 *         // This is our Widget column
 *         xtype: 'widgetcolumn',
 *
 *         // This is the widget definition for each cell.
 *         // The Progress widget class's defaultBindProperty is 'value'
 *         // so its "value" setting is taken from the ViewModel's record "capacityUsed" field
 *         widget: {
 *             xtype: 'progressbarwidget',
 *             bind: '{record.capacityUsed}',
 *             textTpl: [
 *                 '{percent:number("0")}% capacity'
 *             ]
 *         }
 *     }],
 *     renderTo: document.body,
 *     disableSelection: true,
 *     store: {
 *        fields: [{
 *            name: 'id',
 *            type: 'string'
 *        }, {
 *            name: 'maxCapacity',
 *            type: 'int'
 *        }, {
 *            name: 'avg',
 *            type: 'int',
 *            calculate: function(data) {
 *                // Make this depend upon the instant field being set which sets the sampleCount and total.
 *                // Use subscript format to access the other pseudo fields which are set by the instant field's converter
 *                return data.instant &amp;&amp; data['total'] / data['sampleCount'];
 *            }
 *        }, {
 *            name: 'max',
 *            type: 'int',
 *            calculate: function(data) {
 *                // This will be seen to depend on the "instant" field.
 *                // Use subscript format to access this field's current value to avoid circular dependency error.
 *                return (data['max'] || 0) &lt; data.instant ? data.instant : data['max'];
 *            }
 *        }, {
 *            name: 'instant',
 *            type: 'int',
 *
 *            // Upon every update of instantaneous power throughput,
 *            // update the sample count and total so that the max field can calculate itself
 *            convert: function(value, rec) {
 *                rec.data.sampleCount = (rec.data.sampleCount || 0) + 1;
 *                rec.data.total = (rec.data.total || 0) + value;
 *                return value;
 *            },
 *           depends: []
 *        }, {
 *            name: 'capacityUsed',
 *            calculate: function(data) {
 *                return data.instant / data.maxCapacity;
 *            }
 *        }],
 *        data: [{
 *            id: 'Substation A',
 *            maxCapacity: 1000,
 *            avg: 770,
 *            max: 950,
 *            instant: 685
 *        }, {
 *            id: 'Substation B',
 *            maxCapacity: 1000,
 *            avg: 819,
 *            max: 992,
 *            instant: 749
 *        }, {
 *            id: 'Substation C',
 *            maxCapacity: 1000,
 *            avg: 588,
 *              max: 936,
 *            instant: 833
 *        }, {
 *            id: 'Substation D',
 *            maxCapacity: 1000,
 *            avg: 639,
 *            max: 917,
 *            instant: 825
 *        }]
 *    }
 * });
 *
 * // Fake data updating...
 * // Change one record per second to a random power value
 * <a href="#!/api/Ext-method-interval" rel="Ext-method-interval" class="docClass">Ext.interval</a>(function() {
 *     var recIdx = <a href="#!/api/Ext.Number-method-randomInt" rel="Ext.Number-method-randomInt" class="docClass">Ext.Number.randomInt</a>(0, 3),
 *         newPowerReading = <a href="#!/api/Ext.Number-method-randomInt" rel="Ext.Number-method-randomInt" class="docClass">Ext.Number.randomInt</a>(500, 1000);
 *
 *     grid.store.getAt(recIdx).set('instant', newPowerReading);
 * }, 1000);
 * </code></pre>
 */
public class WidgetColumn extends ext.grid.column.Column implements ext.mixin.IStyleCacher {
  /**
   * @param config @inheritDoc
   */
  public function WidgetColumn(config:ext.grid.column.WidgetColumn = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A map of xtype to <a href="#!/api/Ext.Component-cfg-ui" rel="Ext.Component-cfg-ui" class="docClass">Ext.Component.ui</a> names to use when using Components in this column.
   *
   * <p>Currently <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Button</a> and all subclasses of <a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">TextField</a> default
   * to using <code>ui: "default"</code> when in a WidgetColumn except for in the "classic" theme, when they use ui "grid-cell".</p>
   * @default {}
   * @see #getDefaultWidgetUI()
   * @see #setDefaultWidgetUI()
   */
  public native function get defaultWidgetUI():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set defaultWidgetUI(value:Object):void;

  [ExtConfig]
  /**
   * A function that will be called when a widget is attached to a record. This may be useful for
   * doing any post-processing.
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *     xtype: 'grid',
   *     title: 'Student progress report',
   *     width: 250,
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     disableSelection: true,
   *     store: {
   *         fields: ['name', 'isHonorStudent'],
   *         data: [{
   *             name: 'Finn',
   *             isHonorStudent: true
   *         }, {
   *             name: 'Jake',
   *             isHonorStudent: false
   *         }]
   *     },
   *     columns: [{
   *         text: 'Name',
   *         dataIndex: 'name',
   *         flex: 1
   *     }, {
   *         xtype: 'widgetcolumn',
   *         text: 'Honor Roll',
   *         dataIndex: 'isHonorStudent',
   *         width: 150,
   *         widget: {
   *             xtype: 'button',
   *             handler: function() {
   *                 // print certificate handler
   *             }
   *         },
   *         // called when the widget is initially instantiated
   *         // on the widget column
   *         onWidgetAttach: function(col, widget, rec) {
   *             widget.setText('Print Certificate');
   *             widget.setDisabled(!rec.get('isHonorStudent'));
   *         }
   *     }]
   * });
   * </code></pre>
   */
  public native function get onWidgetAttach():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set onWidgetAttach(value:*):void;

  [ExtConfig]
  /**
   * Prevent grid selection upon click on the widget.
   * @default true
   */
  public native function get stopSelection():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set stopSelection(value:Boolean):void;

  [ExtConfig]
  /**
   * A config object containing an <a href="#!/api/Ext.Component-cfg-xtype" rel="Ext.Component-cfg-xtype" class="docClass">xtype</a>.
   *
   * <p>This is used to create the widgets or components which are rendered into the cells of this column.</p>
   *
   * <p>The rendered component has a <a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">ViewModel</a> injected which inherits from any ViewModel
   * that the grid is using, and contains two extra properties:</p>
   *
   * <ul>
   * <li><code>record</code> : <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a><br>The record which backs the grid row.</li>
   * <li><code>recordIndex</code> : Number<br>The index in the dataset of the record which backs the grid row.</li>
   * </ul>
   *
   *
   * <p>The widget configuration may contain a <a href="#!/api/Ext.grid.column.Widget-cfg-bind" rel="Ext.grid.column.Widget-cfg-bind" class="docClass">bind</a> config which uses the ViewModel's data.</p>
   *
   * <p>The derecated way of obtaining data from the record is still supported if the widget does <em>not</em> use a <a href="#!/api/Ext.grid.column.Widget-cfg-bind" rel="Ext.grid.column.Widget-cfg-bind" class="docClass">bind</a> config.</p>
   *
   * <p>This column's <a href="#!/api/Ext.grid.column.Widget-cfg-dataIndex" rel="Ext.grid.column.Widget-cfg-dataIndex" class="docClass">dataIndex</a> is used to update the widget/component's <a href="#!/api/Ext.Component-property-defaultBindProperty" rel="Ext.Component-property-defaultBindProperty" class="docClass">defaultBindProperty</a>.</p>
   *
   * <p>The widget will be decorated with 2 methods:
   * <code>getWidgetRecord</code> - Returns the <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">record</a> the widget is associated with.
   * <code>getWidgetColumn</code> - Returns the <a href="#!/api/Ext.grid.column.Widget" rel="Ext.grid.column.Widget" class="docClass">column</a> the widget
   * was associated with.</p>
   */
  public native function get widget():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set widget(value:Object):void;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.column.Widget-cfg-defaultWidgetUI" rel="Ext.grid.column.Widget-cfg-defaultWidgetUI" class="docClass">defaultWidgetUI</a>.
   */
  public native function getDefaultWidgetUI():Object;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.column.Widget-cfg-defaultWidgetUI" rel="Ext.grid.column.Widget-cfg-defaultWidgetUI" class="docClass">defaultWidgetUI</a>.
   * @param defaultWidgetUI The new value.
   */
  public native function setDefaultWidgetUI(defaultWidgetUI:Object):void;
}
}