package ext.grid.feature {
import ext.data.Model;

[Uses("ext.grid.feature.Feature")]
[Uses("ext.util.Observable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.grid.feature.RowBody", require)]
/**
 * The rowbody feature enhances the grid's markup to have an additional
 * tr -> td -> div which spans the entire width of the original row.
 *
 * <p>This is useful to to associate additional information with a particular
 * record in an Ext.grid.Grid.</p>
 *
 * <p>Rowbodies are initially hidden unless you override <a href="#!/api/Ext.grid.feature.RowBody-method-getAdditionalData" rel="Ext.grid.feature.RowBody-method-getAdditionalData" class="docClass">getAdditionalData</a>.</p>
 *
 * <p>The events fired by RowBody are relayed to the owning
 * <a href="#!/api/Ext.view.Table" rel="Ext.view.Table" class="docClass">grid view</a> (and subsequently the owning grid).</p>
 *
 * <h1>Example</h1>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Animal', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['name', 'latin', 'desc', 'lifespan']
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     width: 400,
 *     height: 300,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     store: {
 *         model: 'Animal',
 *         data: [{
 *             name: 'Tiger',
 *             latin: 'Panthera tigris',
 *             desc: 'The largest cat species, weighing up to 306 kg (670 lb).',
 *             lifespan: '20 - 26 years (in captivity)'
 *         }, {
 *             name: 'Roman snail',
 *             latin: 'Helix pomatia',
 *             desc: 'A species of large, edible, air-breathing land snail.',
 *             lifespan: '20 - 35 years'
 *         }, {
 *             name: 'Yellow-winged darter',
 *             latin: 'Sympetrum flaveolum',
 *             desc: 'A dragonfly found in Europe and mid and Northern China.',
 *             lifespan: '4 - 6 weeks'
 *         }, {
 *             name: 'Superb Fairy-wren',
 *             latin: 'Malurus cyaneus',
 *             desc: 'Common and familiar across south-eastern Australia.',
 *             lifespan: '5 - 6 years'
 *         }]
 *     },
 *     columns: [{
 *         dataIndex: 'name',
 *         text: 'Common name',
 *         width: 125
 *     }, {
 *         dataIndex: 'latin',
 *         text: 'Scientific name',
 *         flex: 1
 *     }],
 *     features: [{
 *         ftype: 'rowbody',
 *         getAdditionalData: function (data, idx, record, orig) {
 *             // Usually you would style the my-body-class in a CSS file
 *             return {
 *                 rowBody: '&lt;div style="padding: 1em"&gt;' + record.get("desc") + '&lt;/div&gt;',
 *                 rowBodyCls: "my-body-class"
 *             };
 *         }
 *     }],
 *     listeners: {
 *         rowbodyclick: function(view, rowEl, e, eOpts) {
 *             var itemEl = <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>(rowEl).up(view.itemSelector),
 *                 rec = view.getRecord(itemEl);
 *
 *             <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>(rec.get('name') + ' life span', rec.get('lifespan'));
 *         }
 *     }
 * });
 * </code></pre>
 *
 * <p> # Cell Editing and Cell Selection Model</p>
 *
 * <p>Note that if <a href="#!/api/Ext.grid.plugin.CellEditing" rel="Ext.grid.plugin.CellEditing" class="docClass">cell editing</a> or the <a href="#!/api/Ext.selection.CellModel" rel="Ext.selection.CellModel" class="docClass">cell selection model</a> are going
 * to be used, then the <a href="#!/api/Ext.grid.feature.RowBody" rel="Ext.grid.feature.RowBody" class="docClass">RowBody</a> feature, or <a href="#!/api/Ext.grid.plugin.RowExpander" rel="Ext.grid.plugin.RowExpander" class="docClass">RowExpander</a> plugin MUST
 * be used for intra-cell navigation to be correct.</p>
 *
 * <p><strong>Note:</strong> The <a href="#!/api/Ext.grid.plugin.RowExpander" rel="Ext.grid.plugin.RowExpander" class="docClass">rowexpander</a> plugin and the rowbody
 * feature are exclusive and cannot both be set on the same grid / tree.</p>
 */
public class RowBody extends ext.grid.feature.Feature {
  /**
   * @param config @inheritDoc
   */
  public function RowBody(config:ext.grid.feature.RowBody = null) {
    super();
  }

  [ExtConfig]
  /**
   * Configure as <code>true</code> to put the row expander body <em>before</em> the data row.
   * @default false
   */
  public native function get bodyBefore():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set bodyBefore(value:Boolean):void;

  /**
   * Provides additional data to the prepareData call within the grid view.
   * The rowbody feature adds 3 additional variables into the grid view's template.
   * These are <code>rowBody</code>, <code>rowBodyCls</code>, and <code>rowBodyColspan</code>.
   *
   * <ul>
   * <li><strong>rowBody:</strong> <em>{String}</em> The HTML to display in the row body element.  Defaults
   * to <em>undefined</em>.</li>
   * <li><strong>rowBodyCls:</strong> <em>{String}</em> An optional CSS class (or multiple classes
   * separated by spaces) to apply to the row body element.  Defaults to
   * <a href="#!/api/Ext.grid.feature.RowBody-property-rowBodyCls" rel="Ext.grid.feature.RowBody-property-rowBodyCls" class="docClass">rowBodyCls</a>.</li>
   * <li><strong>rowBodyColspan:</strong> <em>{Number}</em> The number of columns that the row body element
   * should span.  Defaults to the number of visible columns.</li>
   * </ul>
   * @param data The data for this particular record.
   * @param idx The row index for this record.
   * @param record The record instance
   * @param orig The original result from the prepareData call to massage.
   * @return An object containing additional variables for use in the grid
   * view's template
   */
  protected native function getAdditionalData(data:Object, idx:Number, record:ext.data.Model, orig:Object):Object;
}
}