package ext.grid.locking {

[Uses("ext.grid.locking.Lockable")]
[Mixin("ext.grid.locking.Lockable")]
/**
 * Lockable is a private mixin which injects lockable behavior into any
 * TablePanel subclass such as GridPanel or TreePanel. TablePanel will
 * automatically inject the <a href="#!/api/Ext.grid.locking.Lockable" rel="Ext.grid.locking.Lockable" class="docClass">Ext.grid.locking.Lockable</a> mixin in when one of the
 * these conditions are met:
 *
 * <ul>
 * <li>The TablePanel has the lockable configuration set to true</li>
 * <li>One of the columns in the TablePanel has locked set to true/false</li>
 * </ul>
 *
 *
 * <p>Each TablePanel subclass must register an alias. It should have an array
 * of configurations to copy to the 2 separate tablepanels that will be generated
 * to note what configurations should be copied. These are named normalCfgCopy and
 * lockedCfgCopy respectively.</p>
 *
 * <p>Configurations which are specified in this class will be available on any grid or
 * tree which is using the lockable functionality.</p>
 *
 * <p>By default the two grids, "locked" and "normal" will be arranged using an <a href="#!/api/Ext.layout.container.HBox" rel="Ext.layout.container.HBox" class="docClass">hbox</a>
 * layout. If the lockable grid is configured with <code><a href="#!/api/Ext.grid.locking.Lockable-cfg-split" rel="Ext.grid.locking.Lockable-cfg-split" class="docClass">split:true</a></code>, a vertical splitter
 * will be placed between the two grids to resize them.</p>
 *
 * <p>It is possible to override the layout of the lockable grid, or example, you may wish to
 * use a border layout and have one of the grids collapsible.</p>
 * @see ext.grid.locking.Lockable
 */
public interface ILockable {
  [ExtConfig]
  /**
   * By default, a lockable grid uses an <a href="#!/api/Ext.layout.container.HBox" rel="Ext.layout.container.HBox" class="docClass">HBox</a> layout to arrange
   * the two grids (possibly separated by a splitter).
   *
   * <p>Using this config it is possible to specify a different layout to arrange the two grids.</p>
   */
  function get layout():*;

  [ExtConfig]
  function set layout(value:*):void;

  [ExtConfig]
  /**
   * Any special configuration options for the locked part of the grid
   */
  function get lockedGridConfig():Object;

  [ExtConfig]
  function set lockedGridConfig(value:Object):void;

  [ExtConfig]
  /**
   * A view configuration to be applied to the
   * locked side of the grid. Any conflicting configurations between lockedViewConfig
   * and viewConfig will be overwritten by the lockedViewConfig.
   */
  function get lockedViewConfig():Object;

  [ExtConfig]
  function set lockedViewConfig(value:Object):void;

  [ExtConfig]
  /**
   * Any special configuration options for the normal part of the grid
   */
  function get normalGridConfig():Object;

  [ExtConfig]
  function set normalGridConfig(value:Object):void;

  [ExtConfig]
  /**
   * A view configuration to be applied to the
   * normal/unlocked side of the grid. Any conflicting configurations between normalViewConfig
   * and viewConfig will be overwritten by the normalViewConfig.
   */
  function get normalViewConfig():Object;

  [ExtConfig]
  function set normalViewConfig(value:Object):void;

  [ExtConfig]
  /**
   * Configure as <code>true</code> to place a resizing <a href="#!/api/Ext.resizer.Splitter" rel="Ext.resizer.Splitter" class="docClass">splitter</a> between the locked
   * and unlocked columns. May also be a configuration object for the Splitter.
   * @default false
   */
  function get split():*;

  [ExtConfig]
  function set split(value:*):void;

  [ExtConfig]
  /**
   * An array of events that, when fired, should trigger this object to
   * save its state. Defaults to none. <code>stateEvents</code> may be any type
   * of event supported by this object, including browser or custom events
   * (e.g., <code>['click', 'customerchange']</code>).
   *
   * <p>See <code>stateful</code> for an explanation of saving and
   * restoring object state.</p>
   */
  function get stateEvents():Array;

  [ExtConfig]
  function set stateEvents(value:Array):void;

  [ExtConfig]
  /**
   * The xtype of the subgrid to specify. If this is
   * not specified lockable will determine the subgrid xtype to create by the
   * following rule. Use the superclasses xtype if the superclass is NOT
   * tablepanel, otherwise use the xtype itself.
   */
  function get subGridXType():String;

  [ExtConfig]
  function set subGridXType(value:String):void;

  [ExtConfig]
  /**
   * Synchronize rowHeight between the normal and
   * locked grid view. This is turned on by default. If your grid is guaranteed
   * to have rows of all the same height, you should set this to false to
   * optimize performance.
   * @default true
   */
  function get syncRowHeight():Boolean;

  [ExtConfig]
  function set syncRowHeight(value:Boolean):void;

  /**
   * Synchronizes the row heights between the locked and non locked portion of the grid for each
   * row. If one row is smaller than the other, the height will be increased to match the larger one.
   */
  function syncRowHeights():void;
}
}