package ext.grid.plugin {
import ext.data.Model;
import ext.grid.CellContext;

[Uses("ext.grid.plugin.GridEditingPlugin")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.plugin.AbstractPlugin")]
[Uses("ext.Plugin")]
[Native("Ext.grid.plugin.RowEditing", require)]
/**
 * The <a href="#!/api/Ext.grid.plugin.RowEditing" rel="Ext.grid.plugin.RowEditing" class="docClass">Ext.grid.plugin.RowEditing</a> plugin injects editing at a row level for a Grid. When editing begins,
 * a small floating dialog will be shown for the appropriate row. Each editable column will show a field
 * for editing. There is a button to save or cancel all changes for the edit.
 *
 * <p>The field that will be used for the editor is defined at the
 * <a href="#!/api/Ext.grid.column.Column-cfg-editor" rel="Ext.grid.column.Column-cfg-editor" class="docClass">editor</a>. The editor can be a field instance or a field configuration.
 * If an editor is not specified for a particular column then that column won't be editable and the value of
 * the column will be displayed. To provide a custom renderer for non-editable values, use the
 * <a href="#!/api/Ext.grid.column.Column-cfg-editRenderer" rel="Ext.grid.column.Column-cfg-editRenderer" class="docClass">editRenderer</a> configuration on the column.</p>
 *
 * <p>The editor may be shared for each column in the grid, or a different one may be specified for each column.
 * An appropriate field type should be chosen to match the data structure that it will be editing. For example,
 * to edit a date, it would be useful to specify <a href="#!/api/Ext.form.field.Date" rel="Ext.form.field.Date" class="docClass">Ext.form.field.Date</a> as the editor.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     storeId: 'simpsonsStore',
 *     fields:[ 'name', 'email', 'phone'],
 *     data: [
 *         { name: 'Lisa', email: 'lisa&#64;simpsons.com', phone: '555-111-1224' },
 *         { name: 'Bart', email: 'bart&#64;simpsons.com', phone: '555-222-1234' },
 *         { name: 'Homer', email: 'homer&#64;simpsons.com', phone: '555-222-1244' },
 *         { name: 'Marge', email: 'marge&#64;simpsons.com', phone: '555-222-1254' }
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Simpsons',
 *     store: <a href="#!/api/Ext.data.StoreManager-method-lookup" rel="Ext.data.StoreManager-method-lookup" class="docClass">Ext.data.StoreManager.lookup</a>('simpsonsStore'),
 *     columns: [
 *         {header: 'Name', dataIndex: 'name', editor: 'textfield'},
 *         {header: 'Email', dataIndex: 'email', flex:1,
 *             editor: {
 *                 xtype: 'textfield',
 *                 allowBlank: false
 *             }
 *         },
 *         {header: 'Phone', dataIndex: 'phone'}
 *     ],
 *     selModel: 'rowmodel',
 *     plugins: {
 *         ptype: 'rowediting',
 *         clicksToEdit: 1
 *     },
 *     height: 200,
 *     width: 400,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class RowEditingPlugin extends ext.grid.plugin.GridEditingPlugin {
  /**
   * @param config @inheritDoc
   */
  public function RowEditingPlugin(config:ext.grid.plugin.RowEditingPlugin = null) {
    super();
  }

  [ExtConfig]
  /**
   * <code>true</code> to automatically cancel any pending changes when the row editor begins editing
   * a new row. <code>false</code> to force the user to explicitly cancel the pending changes.
   * Note that this option is mutually exclusive with <a href="#!/api/Ext.grid.plugin.RowEditing-cfg-autoUpdate" rel="Ext.grid.plugin.RowEditing-cfg-autoUpdate" class="docClass">autoUpdate</a>.
   * @default true
   */
  public native function get autoCancel():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoCancel(value:Boolean):void;

  [ExtConfig]
  /**
   * Set this to <code>true</code> to automatically confirm any pending changes when the row editor
   * begins editing a new row. When <code>false</code>, the user will need to explicitly confirm
   * the pending changes.
   * Note that if this is set to <code>true</code>, <a href="#!/api/Ext.grid.plugin.RowEditing-cfg-autoCancel" rel="Ext.grid.plugin.RowEditing-cfg-autoCancel" class="docClass">autoCancel</a> will be set to <code>false</code>.
   * @default false
   */
  public native function get autoUpdate():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoUpdate(value:Boolean):void;

  [ExtConfig]
  /**
   * The number of clicks to move the row editor to a new row while it is visible and actively editing another row.
   * This will default to the same value as <a href="#!/api/Ext.grid.plugin.Editing-cfg-clicksToEdit" rel="Ext.grid.plugin.Editing-cfg-clicksToEdit" class="docClass">clicksToEdit</a>.
   */
  public native function get clicksToMoveEditor():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set clicksToMoveEditor(value:Number):void;

  [ExtConfig]
  /**
   * True to show a <a href="#!/api/Ext.tip.ToolTip" rel="Ext.tip.ToolTip" class="docClass">tooltip</a> that summarizes all validation errors present
   * in the row editor. Set to false to prevent the tooltip from showing.
   * @default true
   */
  public native function get errorSummary():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set errorSummary(value:Boolean):void;

  [ExtConfig]
  /**
   * The ARIA label template for screen readers to announce when row editing starts.
   * This label can be a <a href="#!/api/Ext.String-method-format" rel="Ext.String-method-format" class="docClass">Ext.String.format</a> template, with the only parameter
   * being the row number. Note that row numbers start at base <a href="#!/api/Ext.grid.plugin.RowEditing-cfg-formAriaLabelRowBase" rel="Ext.grid.plugin.RowEditing-cfg-formAriaLabelRowBase" class="docClass">formAriaLabelRowBase</a>.
   * @default "'Editing row {0}'"
   */
  public native function get formAriaLabel():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set formAriaLabel(value:String):void;

  [ExtConfig]
  /**
   * Screen readers will announce grid column header as first row of the ARIA table,
   * so the first actual data row is #2 for screen reader users. If your grid has
   * more than one column header row, you might want to increase this number.
   * If the column header is not visible, the base will be decreased automatically.
   * @default 2
   */
  public native function get formAriaLabelRowBase():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set formAriaLabelRowBase(value:Number):void;

  [ExtConfig]
  /**
   * If configured as <code>true</code>, then canceling an edit on a newly inserted
   * record which has not been modified will delete that record from the store.
   * @default false
   */
  public native function get removeUnmodified():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set removeUnmodified(value:Boolean):void;

  /**
   * Starts editing the specified record, using the specified Column definition to define which field is being edited.
   * @param record The Store data record which backs the row to be edited.
   * @param columnHeader The Column object defining the column field to be focused, or index of the column.
   * If not specified, it will default to the first visible column.
   * @return <code>true</code> if editing was started, <code>false</code> otherwise.
   */
  public native function startEdit(record:ext.data.Model, columnHeader:* = undefined):Boolean;

  /**
   * This method is called when actionable mode is requested for a cell.
   * @param position The position at which actionable mode was requested.
   * @return <code>false</code> Actionable mode is <em>not</em> entered for RowEditing.
   */
  protected native function activateCell(position:ext.grid.CellContext):Boolean;

  /**
   * Called by TableView#resumeActionableMode to resume actionable processing after
   * the view has been changed.
   * @param position The position at which to resume actionable processing.
   * @return <code>true</code> if this Actionable has successfully resumed.
   */
  protected native function resume(position:ext.grid.CellContext):Boolean;

  /**
   * Called by TableView#suspendActionableMode to suspend actionable processing while
   * the view is being changed.
   */
  protected native function suspend():void;
}
}