package ext.layout.container {

[Uses("ext.layout.container.VBoxLayout")]
[Uses("ext.layout.container.BoxLayout")]
[Uses("ext.layout.container.ContainerLayout")]
[Uses("ext.layout.Layout")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Native("Ext.layout.container.Accordion", require)]
/**
 * This is a layout that manages multiple Panels in an expandable accordion style such that by default only
 * one Panel can be expanded at any given time (set <a href="#!/api/Ext.layout.container.Accordion-cfg-multi" rel="Ext.layout.container.Accordion-cfg-multi" class="docClass">multi</a> config to have more open). Each Panel has
 * built-in support for expanding and collapsing.
 *
 * <p>Note: Only Ext Panels and all subclasses of <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a> may be used in an accordion layout Container.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {
 *     title: 'Accordion Layout',
 *     width: 300,
 *     height: 300,
 *     defaults: {
 *         // applied to each contained panel
 *         bodyStyle: 'padding:15px'
 *     },
 *     layout: {
 *         // layout-specific configs go here
 *         type: 'accordion',
 *         titleCollapse: false,
 *         animate: true,
 *         activeOnTop: true
 *     },
 *     items: [{
 *         title: 'Panel 1',
 *         html: 'Panel content!'
 *     },{
 *         title: 'Panel 2',
 *         html: 'Panel content!'
 *     },{
 *         title: 'Panel 3',
 *         html: 'Panel content!'
 *     }],
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class AccordionLayout extends ext.layout.container.VBoxLayout {
  /**
   * @param config @inheritDoc
   */
  public function AccordionLayout(config:ext.layout.container.AccordionLayout = null) {
    super();
  }

  [ExtConfig]
  /**
   * Only valid when <a href="#!/api/Ext.layout.container.Accordion-cfg-multi" rel="Ext.layout.container.Accordion-cfg-multi" class="docClass">multi</a> is <code>false</code> and <a href="#!/api/Ext.layout.container.Accordion-cfg-animate" rel="Ext.layout.container.Accordion-cfg-animate" class="docClass">animate</a> is <code>false</code>.
   *
   * <p>True to swap the position of each panel as it is expanded so that it becomes the first item in the container,
   * false to keep the panels in the rendered order.</p>
   * @default false
   */
  public native function get activeOnTop():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set activeOnTop(value:Boolean):void;

  [ExtConfig]
  /**
   * True to slide the contained panels open and closed during expand/collapse using animation, false to open and
   * close directly with no animation. Note: The layout performs animated collapsing
   * and expanding, <em>not</em> the child Panels.
   * @default true
   */
  public native function get animated():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set animate(value:Boolean):void;

  [ExtConfig]
  /**
   * Child Panels have their width actively managed to fit within the accordion's width.
   */
  public native function get autoWidth():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoWidth(value:Boolean):void;

  [ExtConfig]
  /**
   * True to make sure the collapse/expand toggle button always renders first (to the left of) any other tools
   * in the contained Panels' title bars, false to render it last. By default, this will use the
   * <a href="#!/api/Ext.panel.Panel-cfg-collapseFirst" rel="Ext.panel.Panel-cfg-collapseFirst" class="docClass">Ext.panel.Panel.collapseFirst</a> setting on the panel. If the config option is specified on the layout,
   * it will override the panel value.
   */
  public native function get collapseFirst():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set collapseFirst(value:Boolean):void;

  [ExtConfig]
  /**
   * True to adjust the active item's height to fill the available space in the container, false to use the
   * item's current height, or auto height if not explicitly set.
   * @default true
   */
  public native function get fill():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set fill(value:Boolean):void;

  [ExtConfig]
  /**
   * True to hide the contained Panels' collapse/expand toggle buttons, false to display them.
   * When set to true, <a href="#!/api/Ext.layout.container.Accordion-cfg-titleCollapse" rel="Ext.layout.container.Accordion-cfg-titleCollapse" class="docClass">titleCollapse</a> is automatically set to true.
   * @default false
   */
  public native function get hideCollapseTool():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideCollapseTool(value:Boolean):void;

  [ExtConfig]
  /**
   * Set to true to enable multiple accordion items to be open at once.
   * @default false
   */
  public native function get multi():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set multi(value:Boolean):void;

  [ExtConfig]
  /**
   * True to allow expand/collapse of each contained panel by clicking anywhere on the title bar, false to allow
   * expand/collapse only when the toggle tool button is clicked.  When set to false,
   * <a href="#!/api/Ext.layout.container.Accordion-cfg-hideCollapseTool" rel="Ext.layout.container.Accordion-cfg-hideCollapseTool" class="docClass">hideCollapseTool</a> should be false also. An explicit <a href="#!/api/Ext.panel.Panel-cfg-titleCollapse" rel="Ext.panel.Panel-cfg-titleCollapse" class="docClass">Ext.panel.Panel.titleCollapse</a> declared
   * on the panel will override this setting.
   * @default true
   */
  public native function get titleCollapse():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set titleCollapse(value:Boolean):void;

  [ExtConfig]
  /**
   * When <code>true</code>, pressing Down or Right arrow key on the
   * focused last accordion panel header will navigate to the first panel; pressing Up
   * or Left arrow key on the focused first accordion panel header will navigate to the
   * last panel.
   * Set this to <code>false</code> to prevent keyboard navigation from wrapping over the edges.
   * @default true
   */
  public native function get wrapOver():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set wrapOver(value:Boolean):void;
}
}