package ext.menu {
import ext.Component;
import ext.mixin.IQueryable;

[Uses("ext.Component")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.menu.Item", require)]
/**
 * Fires when this item is activated
 * @eventType ext.menu.events.ItemEvent.ACTIVATE
 */
[Event(name = 'onActivate', type = 'ext.menu.events.ItemEvent')]
/**
 * Fires when this item is clicked
 * @eventType ext.menu.events.Item_eEvent.CLICK
 */
[Event(name = 'onClick', type = 'ext.menu.events.Item_eEvent')]
/**
 * Fires when this item is deactivated
 * @eventType ext.menu.events.ItemEvent.DEACTIVATE
 */
[Event(name = 'onDeactivate', type = 'ext.menu.events.ItemEvent')]
/**
 * Fired when the item's icon is changed by the <a href="#!/api/Ext.menu.Item-method-setIcon" rel="Ext.menu.Item-method-setIcon" class="docClass">setIcon</a> or <a href="#!/api/Ext.menu.Item-method-setIconCls" rel="Ext.menu.Item-method-setIconCls" class="docClass">setIconCls</a> methods.
 * @eventType ext.menu.events.Item_oldIcon_newIconEvent.ICON_CHANGE
 */
[Event(name = 'onIconChange', type = 'ext.menu.events.Item_oldIcon_newIconEvent')]
/**
 * Fired when the item's text is changed by the <a href="#!/api/Ext.menu.Item-method-setText" rel="Ext.menu.Item-method-setText" class="docClass">setText</a> method.
 * @eventType ext.menu.events.Item_oldText_newTextEvent.TEXT_CHANGE
 */
[Event(name = 'onTextChange', type = 'ext.menu.events.Item_oldText_newTextEvent')]
/**
 * A base class for all menu items that require menu-related functionality such as click handling,
 * sub-menus, icons, etc.
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">Ext.menu.Menu</a>', {
 *     width: 100,
 *     height: 100,
 *     floating: false,  // usually you want this set to True (default)
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),  // usually rendered by it's containing component
 *     items: [{
 *         text: 'icon item',
 *         iconCls: 'add16'
 *     },{
 *         text: 'text item'
 *     },{
 *         text: 'plain item',
 *         plain: true
 *     }]
 * });
 * </code></pre>
 */
public class Item extends ext.Component implements ext.mixin.IQueryable {
  /**
   * @param config @inheritDoc
   */
  public function Item(config:ext.menu.Item = null) {
    super();
  }

  [ExtConfig]
  /**
   * The CSS class added to the menu item when the item is disabled.
   * @default Ext.baseCSSPrefix + 'menu-item-disabled'
   */
  override public native function get disabledCls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set disabledCls(value:String):void;

  /**
   * Whether or not this item is currently activated
   * @default false
   */
  public native function get activated():Boolean;

  /**
   * @private
   */
  public native function set activated(value:Boolean):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Menu Item, or subclass thereof.
   * @default true
   */
  public native function get isMenuItem():Boolean;

  /**
   * @private
   */
  public native function set isMenuItem(value:Boolean):void;

  /**
   * The parent Menu of this item.
   */
  public native function get parentMenu():ext.menu.Menu;

  /**
   * @private
   */
  public native function set parentMenu(value:ext.menu.Menu):void;

  [ExtConfig]
  /**
   * The CSS class added to the menu item when the item is focused.
   * @default Ext.baseCSSPrefix + 'menu-item-active'
   */
  public native function get activeCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set activeCls(value:String):void;

  [Deprecated(replacement = "focusable", since = "5.1.0")]
  [ExtConfig]
  /**
   * Whether or not this menu item can be focused.
   */
  public native function get canActivate():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set canActivate(value:Boolean):void;

  [ExtConfig]
  /**
   * The delay in milliseconds to wait before hiding the menu after clicking the menu item.
   * This only has an effect when <code>hideOnClick: true</code>.
   * @default 0
   */
  public native function get clickHideDelay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set clickHideDelay(value:Number):void;

  [ExtConfig]
  /**
   * Whether or not to destroy any associated sub-menu when this item is destroyed.
   * @default true
   */
  public native function get destroyMenu():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set destroyMenu(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A numeric unicode character code to use as the icon.  The default font-family
   * for glyphs can be set globally using
   * <a href="#!/api/Ext.app.Application-cfg-glyphFontFamily" rel="Ext.app.Application-cfg-glyphFontFamily" class="docClass">glyphFontFamily</a> application
   * config or the <a href="#!/api/Ext-method-setGlyphFontFamily" rel="Ext-method-setGlyphFontFamily" class="docClass">Ext.setGlyphFontFamily()</a> method.
   * It is initially set to <code>'Pictos'</code>.
   *
   * <p>The following shows how to set the glyph using the font icons provided in the
   * SDK (assuming the font-family has been configured globally):</p>
   *
   * <pre><code>// assumes the glyphFontFamily is "Pictos"
   * glyph: 'x48'       // the "home" icon (H character)
   *
   * // assumes the glyphFontFamily is "Pictos"
   * glyph: 72          // The "home" icon (H character)
   *
   * // assumes the glyphFontFamily is "Pictos"
   * glyph: 'H'         // the "home" icon
   * </code></pre>
   *
   * <p>Alternatively, this config option accepts a string with the charCode and
   * font-family separated by the <code>&#64;</code> symbol.</p>
   *
   * <pre><code>// using Font Awesome
   * glyph: 'xf015&#64;FontAwesome'     // the "home" icon
   *
   * // using Pictos
   * glyph: 'H&#64;Pictos'              // the "home" icon
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   * @see #getGlyph()
   * @see #setGlyph()
   */
  public native function get glyph():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set glyph(value:*):void;

  [ExtConfig]
  /**
   * A function called when the menu item is clicked (can be used instead of <a href="#!/api/Ext.menu.Item-event-click" rel="Ext.menu.Item-event-click" class="docClass">click</a> event).
   * <ul>
   *   <li><code>item</code> : ext.menu.Item
   * The item that was clicked
   *   </li>
   *   <li><code>e</code> : ext.event.Event
   * The underlying <a href="#!/api/Ext.event.Event" rel="Ext.event.Event" class="docClass">Ext.event.Event</a>.
   * &#64;controllable
   *   </li>
   * </ul>
   * @see #setHandler()
   */
  public native function get handler():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set handler(value:*):void;

  [ExtConfig]
  /**
   * Whether to not to hide the owning menu when this item is clicked.
   * @default true
   */
  public native function get hideOnClick():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideOnClick(value:Boolean):void;

  [ExtConfig]
  /**
   * The href attribute to use for the underlying anchor link.
   * @default '#'
   */
  public native function get href():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set href(value:String):void;

  [ExtConfig]
  /**
   * The target attribute to use for the underlying anchor link.
   */
  public native function get hrefTarget():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hrefTarget(value:String):void;

  [ExtConfig]
  /**
   * Path to an image to use as an icon.
   *
   * <p>For instructions on how you can use icon fonts including those distributed in
   * the SDK see <a href="#!/api/Ext.menu.Item-cfg-iconCls" rel="Ext.menu.Item-cfg-iconCls" class="docClass">iconCls</a>.</p>
   * @default Ext#BLANK_IMAGE_URL
   * @see #setIcon()
   */
  public native function get icon():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set icon(value:String):void;

  [ExtConfig]
  /**
   * One or more space separated CSS classes to be applied to the icon element.<br/>
   * The CSS rule(s) applied should specify a background image to be used as the
   * icon.
   *
   * <p>An example of specifying a custom icon class would be something like:</p>
   *
   * <pre><code>// specify the property in the config for the class:
   * iconCls: 'my-home-icon'
   *
   * // css rule specifying the background image to be used as the icon image:
   * .my-home-icon {
   *     background-image: url(../images/my-home-icon.gif) !important;
   * }
   * </code></pre>
   *
   * <p>In addition to specifying your own classes, you can use the font icons
   * provided in the SDK using the following syntax:</p>
   *
   * <pre><code>// using Font Awesome
   * iconCls: 'x-fa fa-home'
   *
   * // using Pictos
   * iconCls: 'pictos pictos-home'
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   * @see #setIconCls()
   */
  public native function get iconCls():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set iconCls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The sub-menu associated with this item, if one was configured.
   * @see #getMenu()
   * @see #setMenu()
   */
  public native function get menu():ext.menu.Menu;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set menu(value:ext.menu.Menu):void;

  [ExtConfig]
  /**
   * The default <a href="#!/api/Ext.util.Positionable-method-getAlignToXY" rel="Ext.util.Positionable-method-getAlignToXY" class="docClass">Ext.util.Positionable.getAlignToXY</a> anchor position value for this
   * item's sub-menu relative to this item's position.
   * @default 'tl-tr?'
   */
  public native function get menuAlign():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set menuAlign(value:String):void;

  [ExtConfig]
  /**
   * The delay in milliseconds before this item's sub-menu expands after this item is moused over.
   * @default 200
   */
  public native function get menuExpandDelay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set menuExpandDelay(value:Number):void;

  [ExtConfig]
  /**
   * The delay in milliseconds before this item's sub-menu hides after this item is moused out.
   * @default 200
   */
  public native function get menuHideDelay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set menuHideDelay(value:Number):void;

  [ExtConfig]
  /**
   * Whether or not this item is plain text/html with no icon or visual submenu indication.
   */
  public native function get plain():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set plain(value:Boolean):void;

  [ExtConfig]
  /**
   * The text/html to display in this item.
   * @see #setText()
   */
  public native function get text():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set text(value:String):void;

  [ExtConfig]
  /**
   * The tooltip for the button - can be a string to be used as innerHTML (html tags are accepted) or
   * QuickTips config object.
   * @see #setTooltip()
   */
  public native function get tooltip():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set tooltip(value:*):void;

  [ExtConfig]
  /**
   * The type of tooltip to use. Either 'qtip' for QuickTips or 'title' for title attribute.
   * @default 'qtip'
   */
  public native function get tooltipType():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tooltipType(value:String):void;

  /**
   * Retrieves the first direct child of this container which matches the passed selector or component.
   * The passed in selector must comply with an <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector, or it can be an actual <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>.
   * @param selector An <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector. If no selector is
   * specified, the first child will be returned.
   * @return The matching child <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> (or <code>null</code> if no match was found).
   */
  public native function child(selector:* = undefined):ext.Component;

  /**
   * Retrieves the first descendant of this container which matches the passed selector.
   * The passed in selector must comply with an <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector, or it can be an actual <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>.
   * @param selector An <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector or <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>. If no selector is
   * specified, the first child will be returned.
   * @return The matching descendant <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> (or <code>null</code> if no match was found).
   */
  public native function down(selector:* = undefined):ext.Component;

  /**
   * Returns the value of <a href="#!/api/Ext.menu.Item-cfg-glyph" rel="Ext.menu.Item-cfg-glyph" class="docClass">glyph</a>.
   */
  public native function getGlyph():*;

  /**
   * Get the attached sub-menu for this item.
   * @return The sub-menu. <code>null</code> if it doesn't exist.
   */
  public native function getMenu():ext.menu.Menu;

  /**
   * Retrieves all descendant components which match the passed selector.
   * Executes an <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a> using this container as its root.
   * @param selector Selector complying to an <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector.
   * If no selector is specified all items will be returned.
   * @return Components which matched the selector
   */
  public native function query(selector:String = null):Array;

  /**
   * Retrieves all descendant components which match the passed function.
   * The function should return false for components that are to be
   * excluded from the selection.
   * @param fn The matcher function. It will be called with a single argument,
   * the component being tested.
   * @param scope The scope in which to run the function. If not specified,
   * it will default to the active component.
   * @return Components matched by the passed function
   */
  public native function queryBy(fn:Function, scope:Object = null):Array;

  /**
   * Finds a component at any level under this container matching the id/itemId.
   * This is a shorthand for calling ct.down('#' + id);
   * @param id The id to find
   * @return The matching id, null if not found
   */
  public native function queryById(id:String):ext.Component;

  /**
   * Sets the value of <a href="#!/api/Ext.menu.Item-cfg-glyph" rel="Ext.menu.Item-cfg-glyph" class="docClass">glyph</a>.
   * @param glyph The new value.
   */
  public native function setGlyph(glyph:*):void;

  /**
   * Sets the <a href="#!/api/Ext.menu.Item-event-click" rel="Ext.menu.Item-event-click" class="docClass">click</a> handler of this item
   * @param fn The handler function
   * @param scope The scope of the handler function
   */
  public native function setHandler(fn:Function, scope:Object = null):void;

  /**
   * Sets the <a href="#!/api/Ext.menu.Item-cfg-icon" rel="Ext.menu.Item-cfg-icon" class="docClass">icon</a> on this item.
   * @param icon The new icon URL. If this <code>MenuItem</code> was configured with a <a href="#!/api/Ext.menu.Item-cfg-glyph" rel="Ext.menu.Item-cfg-glyph" class="docClass">glyph</a>,
   * this may be a glyph configuration. See <a href="#!/api/Ext.menu.Item-cfg-glyph" rel="Ext.menu.Item-cfg-glyph" class="docClass">glyph</a>.
   */
  public native function setIcon(icon:String):void;

  /**
   * Sets the <a href="#!/api/Ext.menu.Item-cfg-iconCls" rel="Ext.menu.Item-cfg-iconCls" class="docClass">iconCls</a> of this item
   * @param iconCls The CSS class to set to <a href="#!/api/Ext.menu.Item-cfg-iconCls" rel="Ext.menu.Item-cfg-iconCls" class="docClass">iconCls</a>
   */
  public native function setIconCls(iconCls:String):void;

  /**
   * Set a child menu for this item. See the <a href="#!/api/Ext.menu.Item-cfg-menu" rel="Ext.menu.Item-cfg-menu" class="docClass">menu</a> configuration.
   * @param menu A menu, or menu configuration. null may be
   * passed to remove the menu.
   * @param destroyMenu True to destroy any existing menu. False to
   * prevent destruction. If not specified, the <a href="#!/api/Ext.menu.Item-cfg-destroyMenu" rel="Ext.menu.Item-cfg-destroyMenu" class="docClass">destroyMenu</a> configuration
   * will be used.
   */
  public native function setMenu(menu:*, destroyMenu:Boolean = false):void;

  /**
   * Sets the <a href="#!/api/Ext.menu.Item-cfg-text" rel="Ext.menu.Item-cfg-text" class="docClass">text</a> of this item
   * @param text The <a href="#!/api/Ext.menu.Item-cfg-text" rel="Ext.menu.Item-cfg-text" class="docClass">text</a>
   */
  public native function setText(text:String):void;

  /**
   * Sets the tooltip for this menu item.
   * @param tooltip This may be:
   * <ul>
   * <li><strong>String</strong> : A string to be used as innerHTML (html tags are accepted) to show in a tooltip</li>
   * <li><strong>Object</strong> : A configuration object for <a href="#!/api/Ext.tip.QuickTipManager-method-register" rel="Ext.tip.QuickTipManager-method-register" class="docClass">Ext.tip.QuickTipManager.register</a>.</li>
   * </ul>
   * @return this
   */
  public native function setTooltip(tooltip:*):ext.menu.Item;

  /**
   * Traverses the tree rooted at this node in post-order mode, calling the passed function on the nodes at each level.
   * That is the function is called upon each node <strong>after</strong> being called on its children).
   *
   * <p>This method is used at each level down the cascade. Currently <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>s
   * and <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">TreeModel</a>s are queryable.</p>
   *
   * <p>If you have tree-structured data, you can make your nodes queryable, and use ComponentQuery on them.</p>
   * @param selector A ComponentQuery selector used to filter candidate nodes before calling the function.
   * An empty string matches any node.
   * @param fn The function to call. Return <code>false</code> to aborl the traverse.
   * @param {Object} fn.node The node being visited.
   * @param scope The context (<code>this</code> reference) in which the function is executed.
   * @param extraArgs A set of arguments to be appended to the function's argument list to pass down extra data known to the caller
   * <strong>after</strong> the node being visited.
   */
  public native function visitPostOrder(selector:Object, fn:Function, scope:Object = null, extraArgs:Array = null):void;

  /**
   * Traverses the tree rooted at this node in pre-order mode, calling the passed function on the nodes at each level.
   * That is the function is called upon each node <strong>before</strong> being called on its children).
   *
   * <p>This method is used at each level down the cascade. Currently <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>s
   * and <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">TreeModel</a>s are queryable.</p>
   *
   * <p>If you have tree-structured data, you can make your nodes queryable, and use ComponentQuery on them.</p>
   * @param selector A ComponentQuery selector used to filter candidate nodes before calling the function.
   * An empty string matches any node.
   * @param fn The function to call. Return <code>false</code> to aborl the traverse.
   * @param {Object} fn.node The node being visited.
   * @param scope The context (<code>this</code> reference) in which the function is executed.
   * @param extraArgs A set of arguments to be appended to the function's argument list to pass down extra data known to the caller
   * <strong>after</strong> the node being visited.
   */
  public native function visitPreOrder(selector:Object, fn:Function, scope:Object = null, extraArgs:Array = null):void;
}
}