package ext.mixin {
import ext.Base;
import ext.app.ViewController;
import ext.app.ViewModel;
import ext.data.Session;

[Uses("ext.Base")]
[Native("Ext.mixin.Bindable", require)]
[ExcludeClass]
/**
 * This class is intended as a mixin for classes that want to provide a "bind" config that
 * connects to a <code>ViewModel</code>.
 */
public class Bindable extends ext.Base implements ext.mixin.IBindable {
  /**
   * @param config @inheritDoc
   */
  public function Bindable(config:ext.mixin.Bindable = null) {
    super();
  }

  /**
   * This property is used to determine the property of a <code>bind</code> config that is just
   * the value. For example, if <code>defaultBindProperty="value"</code>, then this shorthand
   * <code>bind</code> config:
   *
   * <pre><code> bind: '{name}'
   * </code></pre>
   *
   * <p>Is equivalent to this object form:</p>
   *
   * <pre><code> bind: {
   *      value: '{name}'
   *  }
   * </code></pre>
   *
   * <p>The <code>defaultBindProperty</code> is set to "value" for form fields and to "store" for
   * grids and trees.</p>
   */
  public native function get defaultBindProperty():String;

  /**
   * @private
   */
  public native function set defaultBindProperty(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Setting this config option adds or removes data bindings for other configs.
   * For example, to bind the <code>title</code> config:
   *
   * <pre><code> var panel = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'panel',
   *      bind: {
   *          title: 'Hello {user.name}'
   *      }
   *  });
   * </code></pre>
   *
   * <p>To dynamically add bindings:</p>
   *
   * <pre><code> panel.setBind({
   *      title: 'Greetings {user.name}!'
   *  });
   * </code></pre>
   *
   * <p>To remove bindings:</p>
   *
   * <pre><code> panel.setBind({
   *      title: null
   *  });
   * </code></pre>
   *
   * <p>The bind expressions are presented to <code><a href="#!/api/Ext.app.ViewModel-method-bind" rel="Ext.app.ViewModel-method-bind" class="docClass">Ext.app.ViewModel.bind</a></code>. The
   * <code>ViewModel</code> instance is determined by <code>lookupViewModel</code>.</p>
   * @default {$value: null, lazy: true}
   * @see #getBind()
   * @see #setBind()
   */
  public native function get bind():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set bind(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A string alias, a configuration object or an instance of a <code>ViewController</code> for
   * this container. Sample usage:
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.UserController', {
   *     alias: 'controller.user'
   * });
   *
   * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('UserContainer', {
   *     extend: 'Ext.container.container',
   *     controller: 'user'
   * });
   * // Or
   * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('UserContainer', {
   *     extend: 'Ext.container.container',
   *     controller: {
   *         type: 'user',
   *         someConfig: true
   *     }
   * });
   *
   * // Can also instance at runtime
   * var ctrl = new MyApp.UserController();
   * var view = new UserContainer({
   *     controller: ctrl
   * });
   * </code></pre>
   * @see #getController()
   * @see #setController()
   */
  public native function get controller():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set controller(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If <code>true</code>, this component will be the default scope (this pointer) for events
   * specified with string names so that the scope can be dynamically resolved. The
   * component will automatically become the defaultListenerScope if a
   * <a href="#!/api/Ext.mixin.Bindable-cfg-controller" rel="Ext.mixin.Bindable-cfg-controller" class="docClass">controller</a> is specified.
   *
   * <p>See the introductory docs for <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a> for some sample
   * usages.</p>
   *
   * <p><strong>NOTE</strong>: This value can only be reliably set at construction time. Setting it
   * after that time may not correctly rewire all of the potentially effected
   * listeners.</p>
   * @default false
   * @see #getDefaultListenerScope()
   * @see #setDefaultListenerScope()
   */
  public native function get defaultListenerScope():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set defaultListenerScope(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * One or more names of config properties that this component should publish
   * to its ViewModel. Generally speaking, only properties defined in a class config
   * block (including ancestor config blocks and mixins) are eligible for publishing
   * to the viewModel. Some components override this and publish their most useful
   * configs by default.
   *
   * <p><strong>Note:</strong> We'll discuss publishing properties <strong>not</strong> found in the config block below.</p>
   *
   * <p>Values determined to be invalid by component (often form fields and model validations)
   * will not be published to the ViewModel.</p>
   *
   * <p>This config uses the <code><a href="#!/api/Ext.mixin.Bindable-cfg-reference" rel="Ext.mixin.Bindable-cfg-reference" class="docClass">reference</a></code> to determine the name of the data
   * object to place in the <code>ViewModel</code>. If <code>reference</code> is not set then this config
   * is ignored.</p>
   *
   * <p>By using this config and <code><a href="#!/api/Ext.mixin.Bindable-cfg-reference" rel="Ext.mixin.Bindable-cfg-reference" class="docClass">reference</a></code> you can bind configs between
   * components. For example:</p>
   *
   * <pre><code> ...
   *      items: [{
   *          xtype: 'textfield',
   *          reference: 'somefield',  // component's name in the ViewModel
   *          publishes: 'value' // value is not published by default
   *      },{
   *          ...
   *      },{
   *          xtype: 'displayfield',
   *          bind: 'You have entered "{somefield.value}"'
   *      }]
   *  ...
   * </code></pre>
   *
   * <p>Classes must provide this config as an Object:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.foo.Bar', {
   *      publishes: {
   *          foo: true,
   *          bar: true
   *      }
   *  });
   * </code></pre>
   *
   * <p>This is required for the config system to properly merge values from derived
   * classes.</p>
   *
   * <p>For instances this value can be specified as a value as show above or an array
   * or object as follows:</p>
   *
   * <pre><code> {
   *      xtype: 'textfield',
   *      reference: 'somefield',
   *      publishes: [
   *          'value',
   *          'rawValue',
   *          'dirty'
   *      ]
   *  }
   *
   *  // This achieves the same result as the above array form.
   *  {
   *      xtype: 'textfield',
   *      reference: 'somefield',
   *      publishes: {
   *          value: true,
   *          rawValue: true,
   *          dirty: true
   *      }
   *  }
   * </code></pre>
   *
   * <p>In some cases, users may want to publish a property to the viewModel that is not found in a class
   * config block. In these situations, you may utilize <a href="#!/api/Ext.mixin.Bindable-method-publishState" rel="Ext.mixin.Bindable-method-publishState" class="docClass">publishState</a> if the property has a
   * setter method.  Let's use <a href="#!/api/Ext.form.Labelable-method-setFieldLabel" rel="Ext.form.Labelable-method-setFieldLabel" class="docClass">setFieldLabel</a> as an example:</p>
   *
   * <pre><code>  setFieldLabel: function(fieldLabel) {
   *       this.callParent(arguments);
   *       this.publishState('fieldLabel', fieldLabel);
   *   }
   * </code></pre>
   *
   * <p>With the above chunk of code, fieldLabel may now be published to the viewModel.</p>
   * @since 5.0.0
   * @see #getPublishes()
   * @see #setPublishes()
   */
  public native function get publishes():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set publishes(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Specifies a name for this component inside its component hierarchy. This name
   * must be unique within its <a href="#!/api/Ext.container.Container-cfg-referenceHolder" rel="Ext.container.Container-cfg-referenceHolder" class="docClass">view</a>
   * or its <a href="#!/api/Ext.app.ViewController" rel="Ext.app.ViewController" class="docClass">ViewController</a>. See the documentation in
   * <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a> for more information about references.
   *
   * <p><strong>Note</strong>: Valid identifiers start with a letter or underscore and are followed
   * by zero or more additional letters, underscores or digits. References are case
   * sensitive.</p>
   * @see #getReference()
   * @see #setReference()
   */
  public native function get reference():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set reference(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If provided this creates a new <code>Session</code> instance for this component. If this
   * is a <code>Container</code>, this will then be inherited by all child components.
   *
   * <p>To create a new session you can specify <code>true</code>:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'viewport',
   *      session: true,
   *
   *      items: [{
   *          ...
   *      }]
   *  });
   * </code></pre>
   *
   * <p>Alternatively, a config object can be provided:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'viewport',
   *      session: {
   *          ...
   *      },
   *
   *      items: [{
   *          ...
   *      }]
   *  });
   * </code></pre>
   * @default null
   * @see #getSession()
   * @see #setSession()
   */
  public native function get session():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set session(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This object holds a map of <code>config</code> properties that will update their binding
   * as they are modified. For example, <code>value</code> is a key added by form fields. The
   * form of this config is the same as <code><a href="#!/api/Ext.mixin.Bindable-cfg-publishes" rel="Ext.mixin.Bindable-cfg-publishes" class="docClass">publishes</a></code>.
   *
   * <p>This config is defined so that updaters are not created and added for all
   * bound properties since most cannot be modified by the end-user and hence are
   * not appropriate for two-way binding.</p>
   * @see #getTwoWayBindable()
   * @see #setTwoWayBindable()
   */
  public native function get twoWayBindable():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set twoWayBindable(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The <code>ViewModel</code> is a data provider for this component and its children. The
   * data contained in the <code>ViewModel</code> is typically used by adding <code>bind</code> configs
   * to the components that want present or edit this data.
   *
   * <p>When set, the <code>ViewModel</code> is created and links to any inherited <code>viewModel</code>
   * instance from an ancestor container as the "parent". The <code>ViewModel</code> hierarchy,
   * once established, only supports creation or destruction of children. The
   * parent of a <code>ViewModel</code> cannot be changed on the fly.</p>
   *
   * <p>If this is a root-level <code>ViewModel</code>, the data model connection is made to this
   * component's associated <code><a href="#!/api/Ext.data.Session" rel="Ext.data.Session" class="docClass">Data Session</a></code>. This is
   * determined by calling <code>getInheritedSession</code>.</p>
   * @default {$value: null, lazy: true}
   * @see #getViewModel()
   * @see #setViewModel()
   */
  public native function get viewModel():ext.app.ViewModel;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set viewModel(value:ext.app.ViewModel):void;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-bind" rel="Ext.mixin.Bindable-cfg-bind" class="docClass">bind</a>.
   */
  public native function getBind():Object;

  /**
   * Returns the <a href="#!/api/Ext.app.ViewController" rel="Ext.app.ViewController" class="docClass">Ext.app.ViewController</a> instance associated with this
   * component via the <a href="#!/api/Ext.mixin.Bindable-cfg-controller" rel="Ext.mixin.Bindable-cfg-controller" class="docClass">controller</a> config or <a href="#!/api/Ext.mixin.Bindable-method-setController" rel="Ext.mixin.Bindable-method-setController" class="docClass">setController</a> method.
   * @return Returns this component's ViewController or
   * null if one was not configured
   */
  public native function getController():ext.app.ViewController;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-defaultListenerScope" rel="Ext.mixin.Bindable-cfg-defaultListenerScope" class="docClass">defaultListenerScope</a>.
   */
  public native function getDefaultListenerScope():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-publishes" rel="Ext.mixin.Bindable-cfg-publishes" class="docClass">publishes</a>.
   * @since 5.0.0
   */
  public native function getPublishes():*;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-reference" rel="Ext.mixin.Bindable-cfg-reference" class="docClass">reference</a>.
   */
  public native function getReference():String;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-session" rel="Ext.mixin.Bindable-cfg-session" class="docClass">session</a>.
   */
  public native function getSession():*;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-twoWayBindable" rel="Ext.mixin.Bindable-cfg-twoWayBindable" class="docClass">twoWayBindable</a>.
   */
  public native function getTwoWayBindable():*;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-viewModel" rel="Ext.mixin.Bindable-cfg-viewModel" class="docClass">viewModel</a>.
   */
  public native function getViewModel():*;

  /**
   * Gets the controller that controls this view. May be a controller that belongs
   * to a view higher in the hierarchy.
   * @since 5.0.1
   * @param skipThis <code>true</code> to not consider the controller directly attached
   * to this view (if it exists).
   * @default false
   * @return The controller. <code>null</code> if no controller is found.
   */
  public native function lookupController(skipThis:Boolean = false):ext.app.ViewController;

  /**
   * Returns the <code><a href="#!/api/Ext.data.Session" rel="Ext.data.Session" class="docClass">Ext.data.Session</a></code> for this instance. This property may come
   * from this instance's <code><a href="#!/api/Ext.mixin.Bindable-cfg-session" rel="Ext.mixin.Bindable-cfg-session" class="docClass">session</a></code> or be inherited from this object's parent.
   * @since 5.0.0
   * @param skipThis Pass <code>true</code> to ignore a <code>session</code> configured on
   * this instance and only consider an inherited session.
   * @default false
   */
  public native function lookupSession(skipThis:Boolean = false):ext.data.Session;

  /**
   * Returns the <code><a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">Ext.app.ViewModel</a></code> for this instance. This property may come from this
   * this instance's <code><a href="#!/api/Ext.mixin.Bindable-cfg-viewModel" rel="Ext.mixin.Bindable-cfg-viewModel" class="docClass">viewModel</a></code> or be inherited from this object's parent.
   * @since 5.0.0
   * @param skipThis Pass <code>true</code> to ignore a <code>viewModel</code> configured on
   * this instance and only consider an inherited view model.
   * @default false
   */
  public native function lookupViewModel(skipThis:Boolean = false):ext.app.ViewModel;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-bind" rel="Ext.mixin.Bindable-cfg-bind" class="docClass">bind</a>.
   * @param bind The new value.
   */
  public native function setBind(bind:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-controller" rel="Ext.mixin.Bindable-cfg-controller" class="docClass">controller</a>.
   * @param controller The new value.
   */
  public native function setController(controller:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-defaultListenerScope" rel="Ext.mixin.Bindable-cfg-defaultListenerScope" class="docClass">defaultListenerScope</a>.
   * @param defaultListenerScope The new value.
   */
  public native function setDefaultListenerScope(defaultListenerScope:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-publishes" rel="Ext.mixin.Bindable-cfg-publishes" class="docClass">publishes</a>.
   * @since 5.0.0
   * @param publishes The new value.
   */
  public native function setPublishes(publishes:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-reference" rel="Ext.mixin.Bindable-cfg-reference" class="docClass">reference</a>.
   * @param reference The new value.
   */
  public native function setReference(reference:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-session" rel="Ext.mixin.Bindable-cfg-session" class="docClass">session</a>.
   * @param session The new value.
   */
  public native function setSession(session:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-twoWayBindable" rel="Ext.mixin.Bindable-cfg-twoWayBindable" class="docClass">twoWayBindable</a>.
   * @param twoWayBindable The new value.
   */
  public native function setTwoWayBindable(twoWayBindable:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-viewModel" rel="Ext.mixin.Bindable-cfg-viewModel" class="docClass">viewModel</a>.
   * @param viewModel The new value.
   */
  public native function setViewModel(viewModel:*):void;

  /**
   * Called by <code>getInherited</code> to initialize the inheritedState the first time it is
   * requested.
   */
  protected native function initInheritedState(inheritedState:Object):void;

  /**
   * Checks if a particular binding is synchronizing the value.
   * @param name The name of the property being bound to.
   * @return <code>true</code> if the binding is syncing.
   */
  protected native function isSyncing(name:String):Boolean;

  /**
   * Publish this components state to the <code>ViewModel</code>. If no arguments are given (or if
   * this is the first call), the entire state is published. This state is determined by
   * the <code>publishes</code> property.
   *
   * <p>This method is called only by component authors.</p>
   * @since 5.0.0
   * @param property The name of the property to update.
   * @param value The value of <code>property</code>. Only needed if <code>property</code> is given.
   */
  protected native function publishState(property:String = null, value:Object = null):void;
}
}