package ext.mixin {

[Uses("ext.mixin.Factoryable")]
[Mixin("ext.mixin.Factoryable")]
/**
 * This mixin automates use of <code><a href="#!/api/Ext.Factory" rel="Ext.Factory" class="docClass">Ext.Factory</a></code>. When mixed in to a class, the <code>alias</code> of the
 * class is retrieved and combined with an optional <code>factoryConfig</code> property on that class
 * to produce the configuration to pass to <code><a href="#!/api/Ext.Factory" rel="Ext.Factory" class="docClass">Ext.Factory</a></code>.
 *
 * <p>The factory method created by <code><a href="#!/api/Ext.Factory" rel="Ext.Factory" class="docClass">Ext.Factory</a></code> is also added as a static method to the
 * target class.</p>
 *
 * <p>Given a class declared like so:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.bar.Thing', {
 *      mixins: [
 *          '<a href="#!/api/Ext.mixin.Factoryable" rel="Ext.mixin.Factoryable" class="docClass">Ext.mixin.Factoryable</a>'
 *      ],
 *
 *      alias: 'bar.thing',  // this is detected by Factoryable
 *
 *      factoryConfig: {
 *          defaultType: 'thing',  // this is the default deduced from the alias
 *          // other configs
 *      },
 *
 *      ...
 *  });
 * </code></pre>
 *
 * <p>The produced factory function can be used to create instances using the following
 * forms:</p>
 *
 * <pre><code> var obj;
 *
 *  obj = App.bar.Thing.create('thing'); // same as "new App.bar.Thing()"
 *
 *  obj = App.bar.Thing.create({
 *      type: 'thing'       // same as above
 *  });
 *
 *  obj = App.bar.Thing.create({
 *      xclass: 'App.bar.Thing'  // same as above
 *  });
 *
 *  var obj2 = App.bar.Thing.create(obj);
 *  // obj === obj2  (passing an instance returns the instance)
 * </code></pre>
 *
 * <p>Alternatively the produced factory is available as a static method of <code><a href="#!/api/Ext.Factory" rel="Ext.Factory" class="docClass">Ext.Factory</a></code>.</p>
 * @see ext.mixin.Factoryable
 */
public interface IFactoryable {
  /**
   * If this property is specified by the target class of this mixin its properties are
   * used to configure the created <code><a href="#!/api/Ext.Factory" rel="Ext.Factory" class="docClass">Ext.Factory</a></code>.
   */
  function get factoryConfig():Object;

  function set factoryConfig(value:Object):void;
}
}