package ext.mixin {
import ext.IMixin;

[Uses("ext.Mixin")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Base")]
[Mixin("ext.mixin.Inheritable")]
/**
 * A mixin that provides the functionality for inheritable configs. This allows linking
 * components and containers via a prototype-chained object for accessing inherited
 * values.
 *
 * <h2>Getting Inherited Properties</h2>
 *
 * <p>A component's inherited state is used to keep track of aspects of a component's state
 * that might be influenced by its ancestors like "collapsed" and "hidden". For example:</p>
 *
 * <pre><code> var hidden = this.getInheritedConfig('hidden');
 * </code></pre>
 *
 * <p>The above will produce <code>true</code> if this or any ancestor component has its <code>hidden</code> config
 * set to <code>true</code>.</p>
 *
 * <h2>Chained Objects</h2>
 *
 * <p>Inheritable properties are implemented by chaining each component's inherited state
 * object to its parent container's inherited state object via the prototype. The result
 * is such that if a component's <code>inheritedState</code> does not have it's own property, it
 * inherits the property from the nearest ancestor that does.</p>
 *
 * <p>In the case of a Container, two state objects are created. The primary ("outer") object
 * is used for reading inherited properties. It is also what a child will prototype chain
 * to if that child is not part of the container's <code>items</code> collection. Anything in the
 * <code>items</code> collection will chain to the inheritedStateInner object instead. This object is
 * prototype chained to inheritedState but allows for Container's layout to set inherited
 * properties that specifically apply only to children of the container. This inner object
 * is unlikely to be needed by user code.</p>
 *
 * <h2>Publishing Inherited Properties</h2>
 *
 * <p>The first step to publishing inherited properties is to override <code>initInheritedState</code>
 * and add properties that have inheritable values.</p>
 *
 * <pre><code> initInheritedState: function (inheritedState) {
 *      this.callParent(arguments);
 *
 *      if (this.getHidden()) {
 *          inheritedState.hidden = true;
 *      }
 *  }
 * </code></pre>
 *
 * <p>The above is important because <code>initInheritedState</code> is called whenever the object needs
 * to be repopulated. As you can see, only <code>true</code> values are added to <code>inheritedState</code> in
 * this case because <code>false</code> would mask a <code>hidden</code> value of <code>true</code> from an ancestor.</p>
 *
 * <p>If these values change dynamically, these properties must be maintained. For example:</p>
 *
 * <pre><code> updateHidden: function (hidden) {
 *      var inherited = this.getInherited();
 *
 *      if (hidden) {
 *          inherited.hidden = true;
 *      } else {
 *          // Unmask whatever may be inherited:
 *          delete inherited.hidden;
 *      }
 *  }
 * </code></pre>
 *
 * <h2>Proper Usage</h2>
 *
 * <p>ALWAYS access inherited state using <code>getInherited</code> or <code>getInheritedConfig</code>, not by
 * accessing <code>inheritedState</code> directly.</p>
 *
 * <p>The <code>inheritedState</code> property does not exist until the first call to <code>getInherited</code>. At
 * that point <code>getInherited</code> walks up the component tree to establish the <code>inheritedState</code>
 * prototype chain. Additionally the <code>inheritedState</code> property should NOT be relied upon
 * even after the initial call to <code>getInherited</code> because it is possible for it to become
 * invalid.</p>
 *
 * <p>Invalidation typically happens when a component is moved to a new container. In such a
 * case the <code>inheritedState</code> remains invalid until the next time <code>getInherited</code> is called
 * on the component or one of its descendants.</p>
 * @see ext.mixin.Inheritable
 */
public interface IInheritable extends ext.IMixin {
  /**
   * This method returns an object containing the inherited properties for this instance.
   * @since 5.0.0
   * @param inner Pass <code>true</code> to return <code>inheritedStateInner</code> instead
   * of the normal <code>inheritedState</code> object. This is only needed internally and should
   * not be passed by user code.
   * @default false
   * @return The <code>inheritedState</code> object containing inherited properties.
   */
  function getInherited(inner:Boolean = false):Object;

  /**
   * This method returns the value of a config property that may be inherited from some
   * ancestor.
   *
   * <p>In some cases, a config may be explicitly set on a component with the intent of
   * <em>only</em> being presented to its children while that component should act upon the
   * inherited value (see <code>referenceHolder</code> for example). In these cases the <code>skipThis</code>
   * parameter should be specified as <code>true</code>.</p>
   * @since 5.0.0
   * @param property The name of the config property to return.
   * @param skipThis Pass <code>true</code> if the property should be ignored if
   * found on this instance. In other words, <code>true</code> means the property must be inherited
   * and not explicitly set on this instance.
   * @default false
   * @return The value of the requested <code>property</code>.
   */
  function getInheritedConfig(property:String, skipThis:Boolean = false):*;

  /**
   * Called when this Inheritable is added to a parent
   */
  function onInheritedAdd(instanced:Boolean):void;

  /**
   * Called when this inheritable is removed from a parent
   * @param destroying <code>true</code> if this item will be destroyed by it's container
   */
  function onInheritedRemove(destroying:Boolean):void;
}
}