package ext.mixin {

[Uses("ext.mixin.Responsive")]
[Mixin("ext.mixin.Responsive")]
/**
 * This mixin provides its user with a <code>responsiveConfig</code> config that allows the class
 * to conditionally control config properties.
 *
 * <p>For example:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('ResponsiveClass', {
 *      mixin: [
 *          '<a href="#!/api/Ext.mixin.Responsive" rel="Ext.mixin.Responsive" class="docClass">Ext.mixin.Responsive</a>'
 *      ],
 *
 *      responsiveConfig: {
 *          portrait: {
 *          },
 *
 *          landscape: {
 *          }
 *      }
 *  });
 * </code></pre>
 *
 * <p>For a config to participate as a responsiveConfig it must have a "setter" method. In
 * the below example, a "setRegion" method must exist.</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
 *      xtype: 'viewport',
 *      layout: 'border',
 *
 *      items: [{
 *          title: 'Some Title',
 *          plugins: 'responsive',
 *
 *          responsiveConfig: {
 *              'width &lt; 800': {
 *                  region: 'north'
 *              },
 *              'width &gt;= 800': {
 *                  region: 'west'
 *              }
 *          }
 *      }]
 *  });
 * </code></pre>
 *
 * <p>To use responsiveConfig the class must be defined using the <a href="#!/api/Ext.mixin.Responsive" rel="Ext.mixin.Responsive" class="docClass">Ext.mixin.Responsive</a> mixin.</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.view.Foo', {
 *      extend: '<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>',
 *      xtype: 'foo',
 *      mixins: [
 *           '<a href="#!/api/Ext.mixin.Responsive" rel="Ext.mixin.Responsive" class="docClass">Ext.mixin.Responsive</a>'
 *      ],
 *      ...
 *  });
 * </code></pre>
 *
 * <p>Otherwise, you will need to use the responsive plugin if the class is not one you authored.</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {
 *      renderTo: document.body,
 *      plugins: 'responsive',
 *      ...
 *  });
 * </code></pre>
 *
 * <p> <em>Note:</em> There is the exception of <code><a href="#!/api/Ext.container.Viewport" rel="Ext.container.Viewport" class="docClass">Ext.container.Viewport</a></code> or other classes using <code><a href="#!/api/Ext.plugin.Viewport" rel="Ext.plugin.Viewport" class="docClass">Ext.plugin.Viewport</a></code>.
 *  In those cases, the viewport plugin inherits from <code><a href="#!/api/Ext.plugin.Responsive" rel="Ext.plugin.Responsive" class="docClass">Ext.plugin.Responsive</a></code>.</p>
 *
 * <p>For details see <code><a href="#!/api/Ext.mixin.Responsive-cfg-responsiveConfig" rel="Ext.mixin.Responsive-cfg-responsiveConfig" class="docClass">responsiveConfig</a></code>.</p>
 * @see ext.mixin.Responsive
 */
public interface IResponsive {
  [ExtConfig]
  /**
   * This object consists of keys that represent the conditions on which configs
   * will be applied. For example:
   *
   * <pre><code> responsiveConfig: {
   *      landscape: {
   *          region: 'west'
   *      },
   *      portrait: {
   *          region: 'north'
   *      }
   *  }
   * </code></pre>
   *
   * <p>In this case the keys ("landscape" and "portrait") are the criteria (or "rules")
   * and the object to their right contains the configs that will apply when that
   * rule is true.</p>
   *
   * <p>These rules can be any valid JavaScript expression but the following values
   * are considered in scope:</p>
   *
   * <ul>
   * <li><code>landscape</code> - True if the device orientation is landscape (always <code>true</code> on
   * desktop devices).</li>
   * <li><code>portrait</code> - True if the device orientation is portrait (always <code>false</code> on
   * desktop devices).</li>
   * <li><code>tall</code> - True if <code>width</code> &lt; <code>height</code> regardless of device type.</li>
   * <li><code>wide</code> - True if <code>width</code> > <code>height</code> regardless of device type.</li>
   * <li><code>width</code> - The width of the viewport in pixels.</li>
   * <li><code>height</code> - The height of the viewport in pixels.</li>
   * <li><code>platform</code> - An object containing various booleans describing the platform
   * (see <code><a href="#!/api/Ext-property-platformTags" rel="Ext-property-platformTags" class="docClass">Ext.platformTags</a></code>). The properties of this
   * object are also available implicitly (without "platform." prefix) but this
   * sub-object may be useful to resolve ambiguity (for example, if one of the
   * <code><a href="#!/api/Ext.mixin.Responsive-cfg-responsiveFormulas" rel="Ext.mixin.Responsive-cfg-responsiveFormulas" class="docClass">responsiveFormulas</a></code> overlaps and hides any of these properties).
   * Previous to Ext JS 5.1, the <code>platformTags</code> were only available using this
   * prefix.</li>
   * </ul>
   *
   *
   * <p>A more complex example:</p>
   *
   * <pre><code> responsiveConfig: {
   *      'desktop || width &gt; 800': {
   *          region: 'west'
   *      },
   *
   *      '!(desktop || width &gt; 800)': {
   *          region: 'north'
   *      }
   *  }
   * </code></pre>
   *
   * <p><strong>NOTE</strong>: If multiple rules set a single config (like above), it is important
   * that the rules be mutually exclusive. That is, only one rule should set each
   * config. If multiple rules are actively setting a single config, the order of
   * these (and therefore the config's value) is unspecified.</p>
   *
   * <p>For a config to participate as a <code>responsiveConfig</code> it must have a "setter"
   * method. In the above example, a "setRegion" method must exist.</p>
   * @since 5.0.0
   */
  function get responsiveConfig():Object;

  [ExtConfig]
  function set responsiveConfig(value:Object):void;

  [ExtConfig]
  /**
   * It is common when using <code>responsiveConfig</code> to have recurring expressions that
   * make for complex configurations. Using <code>responsiveFormulas</code> allows you to cut
   * down on this repetition by adding new properties to the "scope" for the rules
   * in a <code>responsiveConfig</code>.
   *
   * <p>For example:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.view.main.Main', {
   *      extend: '<a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a>',
   *
   *      mixins: [
   *          '<a href="#!/api/Ext.mixin.Responsive" rel="Ext.mixin.Responsive" class="docClass">Ext.mixin.Responsive</a>'
   *      ],
   *
   *      responsiveFormulas: {
   *          small: 'width &lt; 600',
   *
   *          medium: 'width &gt;= 600 &amp;&amp; width &lt; 800',
   *
   *          large: 'width &gt;= 800',
   *
   *          tuesday: function (context) {
   *              return (new Date()).getDay() === 2;
   *          }
   *      }
   *  });
   * </code></pre>
   *
   * <p>With the above declaration, any <code>responsiveConfig</code> can now use these values
   * like so:</p>
   *
   * <pre><code> responsiveConfig: {
   *      small: {
   *          hidden: true
   *      },
   *      'medium &amp;&amp; !tuesday': {
   *          hidden: false,
   *          region: 'north'
   *      },
   *      large: {
   *          hidden: false,
   *          region: 'west'
   *      }
   *  }
   * </code></pre>
   * @since 5.0.1
   */
  function get responsiveFormulas():Object;

  [ExtConfig]
  function set responsiveFormulas(value:Object):void;

  /**
   * This method removes this instance from the Responsive collection.
   */
  function destroy(...params):void;
}
}