package ext.mixin {
import ext.Base;
import ext.plugin.AbstractPlugin;

[Uses("ext.Base")]
[Native("Ext.mixin.Pluggable", require)]
/**
 * This mixin provides support for a <code>plugins</code> config and related API's.
 *
 * <p>If this mixin is used for non-Components, the statements regarding the host being a
 * Component can be translated accordingly. The only requirement on the user of this class
 * is that the plugins actually used be appropriate for their host.</p>
 *
 * <p>While <code><a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a></code> in the Classic Toolkit supports <code>plugins</code>, it does not use this
 * class to provide that support. This is due to backwards compatibility in regard to
 * timing changes this implementation would present.</p>
 *
 * <p><strong>Important:</strong> To ensure plugins are destroyed, call <code>setPlugins(null)</code>.</p>
 */
public class Pluggable extends ext.Base implements ext.mixin.IPluggable {
  [ExtConfig]
  /**
   * An object or array of objects that will provide custom functionality for this
   * component. If a string is provided or a string is one of the elements of the
   * array, that string is treated as the <code>type</code> alias. For example, "listpaging"
   * is the type alias for <code>Ext.plugin.ListPaging</code>. The full alias includes the
   * "plugin." prefix (i.e., 'plugin.listpaging').
   *
   * <p>Plugins should derive from <code><a href="#!/api/Ext.plugin.Abstract" rel="Ext.plugin.Abstract" class="docClass">Ext.plugin.Abstract</a></code> but this is not required. The
   * only requirement for a valid plugin is that it contain an <code>init()</code> method that
   * accepts a reference to the owning component.</p>
   *
   * <p>When a component is created, if any plugins are available, the component will
   * call the <code><a href="#!/api/Ext.plugin.Abstract-method-init" rel="Ext.plugin.Abstract-method-init" class="docClass">init</a></code> method on each plugin,
   * passing a reference to itself. Each plugin can then call methods or respond to
   * events on the component as needed to provide its functionality.</p>
   *
   * <h2>Example code</h2>
   *
   * <p>A plugin by alias:</p>
   *
   * <pre><code> var list = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'list',
   *      itemTpl: '&lt;div class="item"&gt;{title}&lt;/div&gt;',
   *      store: 'Items',
   *
   *      plugins: 'listpaging'
   *  });
   * </code></pre>
   *
   * <p>Multiple plugins by alias:</p>
   *
   * <pre><code> var list = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'list',
   *      itemTpl: '&lt;div class="item"&gt;{title}&lt;/div&gt;',
   *      store: 'Items',
   *
   *      plugins: ['listpaging', 'pullrefresh']
   *  });
   * </code></pre>
   *
   * <p>Single plugin by class name with config options:</p>
   *
   * <pre><code> var list = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'list',
   *      itemTpl: '&lt;div class="item"&gt;{title}&lt;/div&gt;',
   *      store: 'Items',
   *
   *      plugins: {
   *          type: 'listpaging',
   *          autoPaging: true
   *      }
   *  });
   * </code></pre>
   *
   * <p>Multiple plugins by type and class name with config options:</p>
   *
   * <pre><code> var list = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'list',
   *      itemTpl: '&lt;div class="item"&gt;{title}&lt;/div&gt;',
   *      store: 'Items',
   *
   *      plugins: [{
   *          xclass: 'Ext.plugin.PullRefresh',
   *          pullRefreshText: 'Pull to refresh...'
   *      }, {
   *          type: 'listpaging',
   *          autoPaging: true
   *      }]
   *  });
   * </code></pre>
   */
  public native function get plugins():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set plugins(value:Array):void;

  /**
   * Adds a plugin. For example:
   *
   * <pre><code> list.addPlugin('pullrefresh');
   * </code></pre>
   *
   * <p>Or:</p>
   *
   * <pre><code> list.addPlugin({
   *      type: 'pullrefresh',
   *      pullRefreshText: 'Pull to refresh...'
   *  });
   * </code></pre>
   * @since 6.2.0
   * @param plugin The plugin or config object or
   * alias to add.
   */
  public native function addPlugin(plugin:*):void;

  /**
   * Removes and destroys a plugin.
   *
   * <p><strong>Note:</strong> Not all plugins are designed to be removable. Consult the documentation
   * for the specific plugin in question to be sure.</p>
   * @since 6.2.0
   * @param plugin The plugin or its <code>id</code> to remove.
   * @return plugin instance or <code>null</code> if not found.
   */
  public native function destroyPlugin(plugin:*):ext.plugin.AbstractPlugin;

  /**
   * Retrieves plugin by its <code>type</code> alias. For example:
   *
   * <pre><code> var list = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'list',
   *      itemTpl: '&lt;div class="item"&gt;{title}&lt;/div&gt;',
   *      store: 'Items',
   *
   *      plugins: ['listpaging', 'pullrefresh']
   *  });
   *
   *  list.findPlugin('pullrefresh').setPullRefreshText('Pull to refresh...');
   * </code></pre>
   *
   * <p><strong>Note:</strong> See also <a href="#!/api/Ext.mixin.Pluggable-method-getPlugin" rel="Ext.mixin.Pluggable-method-getPlugin" class="docClass">getPlugin</a>.</p>
   * @since 6.2.0
   * @param type The Plugin's <code>type</code> as specified by the class's
   * <a href="#!/api/Ext.Class-cfg-alias" rel="Ext.Class-cfg-alias" class="docClass">alias</a> configuration.
   * @return plugin instance or <code>null</code> if not found.
   */
  public native function findPlugin(type:String):ext.plugin.AbstractPlugin;

  /**
   * Retrieves a plugin by its <code>id</code>.
   *
   * <pre><code> var list = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'list',
   *      itemTpl: '&lt;div class="item"&gt;{title}&lt;/div&gt;',
   *      store: 'Items',
   *
   *      plugins: {
   *          xclass: 'Ext.plugin.PullRefresh',
   *          id: 'foo'
   *      }
   *  });
   *
   *  list.getPlugin('foo').setPullRefreshText('Pull to refresh...');
   * </code></pre>
   *
   * <p><strong>Note:</strong> See also <a href="#!/api/Ext.mixin.Pluggable-method-findPlugin" rel="Ext.mixin.Pluggable-method-findPlugin" class="docClass">findPlugin</a>.</p>
   * @since 6.2.0
   * @param id The <code>id</code> of the plugin.
   * @return plugin instance or <code>null</code> if not found.
   */
  public native function getPlugin(id:String):ext.plugin.AbstractPlugin;

  /**
   * Removes and (optionally) destroys a plugin.
   *
   * <p><strong>Note:</strong> Not all plugins are designed to be removable. Consult the documentation
   * for the specific plugin in question to be sure.</p>
   * @since 6.2.0
   * @param plugin The plugin or its <code>id</code> to remove.
   * @param destroy Pass <code>true</code> to not call <code>destroy()</code> on the plugin.
   * @return plugin instance or <code>null</code> if not found.
   */
  public native function removePlugin(plugin:*, destroy:Boolean = false):ext.plugin.AbstractPlugin;
}
}