package ext.plugin {
import ext.Component;
import ext.dom.Element;

[Uses("ext.plugin.AbstractPlugin")]
[Uses("ext.Plugin")]
[Uses("ext.Base")]
[Native("Ext.plugin.AbstractClipboard", require)]
/**
 * This base class manages clipboard data transfer for a component. As an abstract class,
 * applications use derived classes such as <code><a href="#!/api/Ext.grid.plugin.Clipboard" rel="Ext.grid.plugin.Clipboard" class="docClass">Ext.grid.plugin.Clipboard</a></code> instead
 * and seldom use this class directly.
 *
 * <h2>Operation</h2>
 *
 * <p>Components that interact with the clipboard do so in two directions: copy and paste.
 * When copying to the clipboard, a component will often provide multiple data formats.
 * On paste, the consumer of the data can then decide what format it prefers and ignore
 * the others.</p>
 *
 * <h3>Copy (and Cut)</h3>
 *
 * <p>There are two storage locations provided for holding copied data:</p>
 *
 * <ul>
 * <li>The system clipboard, used to exchange data with other applications running
 * outside the browser.</li>
 * <li>A memory space in the browser page that can hold data for use only by other
 * components on the page. This allows for richer formats to be transferred.</li>
 * </ul>
 *
 *
 * <p>A component can copy (or cut) data in multiple formats as controlled by the
 * <code><a href="#!/api/Ext.plugin.AbstractClipboard-cfg-memory" rel="Ext.plugin.AbstractClipboard-cfg-memory" class="docClass">memory</a></code> and <code><a href="#!/api/Ext.plugin.AbstractClipboard-cfg-system" rel="Ext.plugin.AbstractClipboard-cfg-system" class="docClass">system</a></code> configs.</p>
 *
 * <h3>Paste</h3>
 *
 * <p>While there may be many formats available, when a component is ready to paste, only
 * one format can ultimately be used. This is specified by the <code><a href="#!/api/Ext.plugin.AbstractClipboard-cfg-source" rel="Ext.plugin.AbstractClipboard-cfg-source" class="docClass">source</a></code>
 * config.</p>
 *
 * <h2>Browser Limitations</h2>
 *
 * <p>At the current time, browsers have only a limited ability to interact with the system
 * clipboard. The only reliable, cross-browser, plugin-in-free technique for doing so is
 * to use invisible elements and focus tricks <strong>during</strong> the processing of clipboard key
 * presses like CTRL+C (on Windows/Linux) or CMD+C (on Mac).</p>
 */
public class AbstractClipboardPlugin extends ext.plugin.AbstractPlugin {
  /**
   * @param config @inheritDoc
   */
  public function AbstractClipboardPlugin(config:ext.plugin.AbstractClipboardPlugin = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This object is keyed by the names of the data formats supported by this plugin.
   * The property values of this object are objects with <code>get</code> and <code>put</code> properties
   * that name the methods for getting data from (copy) and putting to into (paste)
   * the associated component.
   *
   * <p>For example:</p>
   *
   * <pre><code> formats: {
   *      html: {
   *          get: 'getHtmlData',
   *          put: 'putHtmlData'
   *      }
   *  }
   * </code></pre>
   *
   * <p>This declares support for the "html" data format and indicates that the
   * <code>getHtmlData</code> method should be called to copy HTML data from the component,
   * while the <code>putHtmlData</code> should be called to paste HTML data into the component.</p>
   *
   * <p>By default, all derived classes must support a "text" format:</p>
   *
   * <pre><code> formats: {
   *      text: {
   *          get: 'getTextData',
   *          put: 'putTextData'
   *      }
   *  }
   * </code></pre>
   *
   * <p>To understand the method signatures required to implement a data format, see the
   * documentation for <code><a href="#!/api/Ext.plugin.AbstractClipboard-method-getTextData" rel="Ext.plugin.AbstractClipboard-method-getTextData" class="docClass">getTextData</a></code> and  <code><a href="#!/api/Ext.plugin.AbstractClipboard-method-putTextData" rel="Ext.plugin.AbstractClipboard-method-putTextData" class="docClass">putTextData</a></code>.</p>
   *
   * <p>The format name "system" is not allowed.</p>
   * @default {text: {get: 'getTextData', put: 'putTextData'}}
   * @see #getFormats()
   * @see #setFormats()
   */
  public native function get formats():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set formats(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The data format(s) to copy to the private, memory clipboard. By default, data
   * is not saved to the memory clipboard. Specify <code>true</code> to include all formats
   * of data, or a string to copy a single format, or an array of strings to copy
   * multiple formats.
   * @see #getMemory()
   * @see #setMemory()
   */
  public native function get memory():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set memory(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The format or formats in order of preference when pasting data. This list can
   * be any of the valid formats, plus the name "system". When a paste occurs, this
   * config is consulted. The first format specified by this config that has data
   * available in the private memory space is used. If "system" is encountered in
   * the list, whatever data is available on the system clipboard is chosen. At
   * that point, no further source formats will be considered.
   * @default "system"
   * @see #getSource()
   * @see #setSource()
   */
  public native function get source():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set source(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The data format to set in the system clipboard. By default, the "text"
   * format is used. Based on the type of derived class, other formats may be
   * possible.
   * @default "text"
   * @see #getSystem()
   * @see #setSystem()
   */
  public native function get system():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set system(value:String):void;

  /**
   * Returns the value of <a href="#!/api/Ext.plugin.AbstractClipboard-cfg-memory" rel="Ext.plugin.AbstractClipboard-cfg-memory" class="docClass">memory</a>.
   */
  public native function getMemory():*;

  /**
   * Returns the value of <a href="#!/api/Ext.plugin.AbstractClipboard-cfg-source" rel="Ext.plugin.AbstractClipboard-cfg-source" class="docClass">source</a>.
   */
  public native function getSource():*;

  /**
   * Returns the value of <a href="#!/api/Ext.plugin.AbstractClipboard-cfg-system" rel="Ext.plugin.AbstractClipboard-cfg-system" class="docClass">system</a>.
   */
  public native function getSystem():String;

  /**
   * Returns the element target to listen to copy/paste.
   * @param comp The component this plugin is initialized on.
   * @return The element target.
   */
  public native function getTarget(comp:ext.Component):ext.dom.Element;

  /**
   * This method returns the selected data in text format.
   * @param format The name of the format (i.e., "text").
   * @param erase Pass <code>true</code> to erase (cut) the data, <code>false</code> to just copy.
   * @return The data in text format.
   */
  public native function getTextData(format:String, erase:Boolean):String;

  /**
   * This method pastes the given text data.
   * @param data The data in the indicated <code>format</code>.
   * @param format The name of the format (i.e., "text").
   */
  public native function putTextData(data:Object, format:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.plugin.AbstractClipboard-cfg-memory" rel="Ext.plugin.AbstractClipboard-cfg-memory" class="docClass">memory</a>.
   * @param memory The new value.
   */
  public native function setMemory(memory:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.plugin.AbstractClipboard-cfg-source" rel="Ext.plugin.AbstractClipboard-cfg-source" class="docClass">source</a>.
   * @param source The new value.
   */
  public native function setSource(source:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.plugin.AbstractClipboard-cfg-system" rel="Ext.plugin.AbstractClipboard-cfg-system" class="docClass">system</a>.
   * @param system The new value.
   */
  public native function setSystem(system:String):void;

  /**
   * Returns the value of <a href="#!/api/Ext.plugin.AbstractClipboard-cfg-formats" rel="Ext.plugin.AbstractClipboard-cfg-formats" class="docClass">formats</a>.
   */
  protected native function getFormats():Object;

  /**
   * Sets the value of <a href="#!/api/Ext.plugin.AbstractClipboard-cfg-formats" rel="Ext.plugin.AbstractClipboard-cfg-formats" class="docClass">formats</a>.
   * @param formats The new value.
   */
  protected native function setFormats(formats:Object):void;
}
}