package ext.plugin {
import ext.Base;
import ext.Component;
import ext.Plugin;

[Uses("ext.Base")]
[Uses("ext.Plugin")]
[Native("Ext.plugin.Abstract", require)]
[ExtConfig]
/**
 * This is the base class from which all plugins should extend.
 *
 * <p>This class defines the essential API of plugins as used by Components by defining the
 * following methods:</p>
 *
 * <ul>
 * <li><p><code>init</code> : The plugin initialization method which the host Component calls during
 *  Component initialization. The Component passes itself as the sole parameter.
 *  Subclasses should set up bidirectional links between the plugin and its host
 *  Component here.</p></li>
 * <li><p><code>destroy</code> : The plugin cleanup method which the host Component calls at Component
 *  destruction time. Use this method to break links between the plugin and the
 *  Component and to free any allocated resources.</p></li>
 * </ul>
 */
public class AbstractPlugin extends ext.Base implements ext.Plugin {
  /**
   * Initializes the plugin.
   * @param config Configuration object.
   */
  public function AbstractPlugin(config:ext.plugin.AbstractPlugin = null) {
    super();
  }

  /**
   * The value <code>true</code> to identify objects of this class or a subclass thereof.
   * @default true
   */
  public native function get isPlugin():Boolean;

  [ExtConfig]
  /**
   * An identifier for the plugin that can be set at creation time to later retrieve the
   * plugin using the getPlugin method. For example:
   *
   * <pre><code> var panel = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'panel',
   *
   *      plugins: [{
   *          id: 'foo',
   *          ...
   *      }]
   *  });
   *
   *  // later on:
   *  var plugin = panel.getPlugin('foo');
   * </code></pre>
   * @since 6.2.0
   */
  public native function get id():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set id(value:String):void;

  [ExtConfig("id")]
  /**
   * Alias for <code>id</code> property to make it accessible from MXML.
   * @copy #id
   * @see #id
   */
  public native function set id_(value:String):void;

  [Deprecated(message = "Use &lt;code>id&lt;/code> instead\n", since = "6.2.0")]
  [ExtConfig]
  public native function get pluginId():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set pluginId(value:String):void;

  [ExtConfig]
  /**
   * The configured list of stateEvents used to (optionally) participate in Owner Component's state management.
   */
  public native function get stateEvents():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set stateEvents(value:*):void;

  /**
   * The component to which this plugin is attached.
   */
  [Bindable(event = "DUMMY")]
  public native function get cmp():ext.Component;

  /**
   * The destroy method is invoked by the owning Component at the time the Component is
   * being destroyed.
   */
  override public native function destroy(...params):void;

  /**
   * The applyState method is invoked by the client Component's State mixin after initComponent method has been run for the client.
   *
   * <p>The supplied implementation is empty. If plugin Subclasses are to (optionally) participate in the client Component's
   * state management, implementers should provide a suitable method to utilize it.</p>
   * @param state The current plugin state object to be applied.
   * @param allState The current aggregate state of the Component and all plugins.
   */
  public native function applyState(state:Object, allState:Object):void;

  /**
   * Creates clone of the plugin.
   * @param overrideCfg Additional config for the derived plugin.
   */
  public native function clonePlugin(overrideCfg:Object = null):void;

  /**
   * Plugins that can be disconnected from their host component should implement
   * this method.
   * @since 6.2.0
   */
  public native function detachCmp():void;

  /**
   * The base implementation just sets the plugin's <code>disabled</code> flag to <code>true</code>
   *
   * <p>Plugin subclasses which need more complex processing may implement an overriding implementation.</p>
   */
  public native function disable():void;

  /**
   * The base implementation just sets the plugin's <code>disabled</code> flag to <code>false</code>
   *
   * <p>Plugin subclasses which need more complex processing may implement an overriding implementation.</p>
   */
  public native function enable():void;

  /**
   * Returns the component to which this plugin is attached.
   * @return The owning host component.
   */
  public native function getCmp():ext.Component;

  /**
   * The getState method is invoked by the client Component's State mixin when one or more of the the specified <a href="#!/api/Ext.plugin.Abstract-cfg-stateEvents" rel="Ext.plugin.Abstract-cfg-stateEvents" class="docClass">stateEvents</a> are raised.
   *
   * <p>The supplied implementation is empty. If plugin Subclasses are to (optionally) participate in the client Component's
   * state management, implementers should provide a suitable method which returns a state object.</p>
   * @return state
   */
  public native function getState():Object;

  /**
   * The init method is invoked to formally associate the host component and the plugin.
   *
   * <p>Subclasses should perform initialization and set up any requires links between the
   * plugin and its host Component in their own implementation of this method.</p>
   * @param host The host Component which owns this plugin.
   */
  public native function init(host:ext.Component):void;

  /**
   * Sets the host component to which this plugin is attached. For a plugin to be
   * removable without being destroyed, this method should be provided and be prepared
   * to receive <code>null</code> for the component.
   * @param host The owning host component.
   */
  public native function setCmp(host:ext.Component):void;
}
}