package ext.selection {
import ext.grid.CellContext;

[Uses("ext.selection.DataViewSelectionModel")]
[Uses("ext.selection.SelectionModel")]
[Uses("ext.util.StoreHolder")]
[Uses("ext.Base")]
[Uses("ext.mixin.Observable")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.selection.CellModel", require)]
/**
 * Fired after a cell is deselected
 * @eventType ext.selection.events.CellSelectionModel_record_row_columnEvent.DESELECT
 */
[Event(name = 'onDeselect', type = 'ext.selection.events.CellSelectionModel_record_row_columnEvent')]
/**
 * Fired after a cell is selected
 * @eventType ext.selection.events.CellSelectionModel_record_row_columnEvent.SELECT
 */
[Event(name = 'onSelect', type = 'ext.selection.events.CellSelectionModel_record_row_columnEvent')]
/**
 * A selection model for <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">grid panels</a> which allows selection of a single cell at a time.
 *
 * <p>Implements cell based navigation via keyboard.</p>
 *
 * <pre class='inline-example '><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     fields: ['name', 'email', 'phone'],
 *     data: [
 *         { name: 'Lisa', email: 'lisa&#64;simpsons.com',  phone: '555-111-1224' },
 *         { name: 'Bart', email: 'bart&#64;simpsons.com',  phone: '555-222-1234' },
 *         { name: 'Homer', email: 'homer&#64;simpsons.com', phone: '555-222-1244' },
 *         { name: 'Marge', email: 'marge&#64;simpsons.com', phone: '555-222-1254' }
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Simpsons',
 *     store: store,
 *     width: 400,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     columns: [
 *         { text: 'Name',  dataIndex: 'name' },
 *         { text: 'Email', dataIndex: 'email', flex: 1 },
 *         { text: 'Phone', dataIndex: 'phone' }
 *     ],
 *     selModel: 'cellmodel'
 * });
 * </code></pre>
 */
public class CellSelectionModel extends ext.selection.DataViewSelectionModel {
  /**
   * @param config @inheritDoc
   */
  public function CellSelectionModel(config:ext.selection.CellSelectionModel = null) {
    super();
  }

  [ExtConfig]
  /**
   * Turns on/off keyboard navigation within the grid.
   * @default true
   */
  override public native function get enableKeyNav():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set enableKeyNav(value:Boolean):void;

  [ExtConfig]
  /**
   * Mode of selection.  Valid values are:
   *
   * <ul>
   * <li><strong>"SINGLE"</strong> - Only allows selecting one item at a time. This is the default.</li>
   * </ul>
   */
  override public native function get mode():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set mode(value:String):void;

  [ExtConfig]
  /**
   * Set this configuration to true to prevent wrapping around of selection as
   * a user navigates to the first or last column.
   * @default false
   */
  public native function get preventWrap():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set preventWrap(value:Boolean):void;

  /**
   * Selects a cell by row / column.
   *
   * <pre><code>var grid = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
   *     title: 'Simpsons',
   *     store: {
   *         fields: ['name', 'email', 'phone'],
   *         data: [{
   *             name: "Lisa",
   *             email: "lisa&#64;simpsons.com",
   *             phone: "555-111-1224"
   *         }]
   *     },
   *     columns: [{
   *         text: 'Name',
   *         dataIndex: 'name'
   *     }, {
   *         text: 'Email',
   *         dataIndex: 'email',
   *         hidden: true
   *     }, {
   *         text: 'Phone',
   *         dataIndex: 'phone',
   *         flex: 1
   *     }],
   *     height: 200,
   *     width: 400,
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     selType: 'cellmodel',
   *     tbar: [{
   *         text: 'Select position Object',
   *         handler: function() {
   *             grid.getSelectionModel().select({
   *                 row: grid.getStore().getAt(0),
   *                 column: grid.down('gridcolumn[dataIndex=name]')
   *             });
   *         }
   *     }, {
   *         text: 'Select position by Number',
   *         handler: function() {
   *             grid.getSelectionModel().select({
   *                 row: 0,
   *                 column: 1
   *             });
   *         }
   *     }]
   * });
   * </code></pre>
   * @param records An array of records or an index
   * @param keepExisting True to retain existing selections
   * @default false
   * @param suppressEvent True to not fire a select event
   * @default false
   */
  override public native function select(records:*, keepExisting:Boolean = false, suppressEvent:Boolean = false):void;

  [Deprecated(replacement = "getPosition", since = "5.0.1")]
  /**
   * Returns the current position in the format {row: row, column: column}
   */
  public native function getCurrentPosition():Object;

  /**
   * Returns the current position in the format {row: row, column: column}
   * @return A CellContext object describing the current cell.
   */
  public native function getPosition():ext.grid.CellContext;

  [Deprecated(replacement = "setPosition", since = "5.0.1")]
  /**
   * Sets the current position.
   * @param position The position to set. May be an object of the form <code>{row:1, column:2}</code>
   * @param suppressEvent True to suppress selection events
   */
  public native function setCurrentPosition(position:*, suppressEvent:Boolean):void;

  /**
   * Sets the current position.
   *
   * <p>Note that if passing a column index, it is the index within the <em>visible</em> column set.</p>
   * @param position The position to set. May be an object of the form <code>{row:1, column:2}</code>
   * @param suppressEvent True to suppress selection events
   */
  public native function setPosition(position:*, suppressEvent:Boolean):void;
}
}