package ext.state {
import ext.Base;

[Uses("ext.Base")]
[Native("Ext.state.Stateful", require)]
/**
 * Fires before the state of the object is restored. Return false from an event handler to stop the restore.
 * @eventType ext.state.events.Stateful_stateEvent.BEFORE_STATE_RESTORE
 */
[Event(name = 'onBeforeStateRestore', type = 'ext.state.events.Stateful_stateEvent')]
/**
 * Fires before the state of the object is saved to the configured state provider. Return false to stop the save.
 * @eventType ext.state.events.Stateful_stateEvent.BEFORE_STATE_SAVE
 */
[Event(name = 'onBeforeStateSave', type = 'ext.state.events.Stateful_stateEvent')]
/**
 * Fires after the state of the object is restored.
 * @eventType ext.state.events.Stateful_stateEvent.STATE_RESTORE
 */
[Event(name = 'onStateRestore', type = 'ext.state.events.Stateful_stateEvent')]
/**
 * Fires after the state of the object is saved to the configured state provider.
 * @eventType ext.state.events.Stateful_stateEvent.STATE_SAVE
 */
[Event(name = 'onStateSave', type = 'ext.state.events.Stateful_stateEvent')]
/**
 * A mixin for being able to save the state of an object to an underlying
 * <a href="#!/api/Ext.state.Provider" rel="Ext.state.Provider" class="docClass">Ext.state.Provider</a>.
 */
public class Stateful extends ext.Base implements ext.state.IStateful {
  /**
   * @param config @inheritDoc
   */
  public function Stateful(config:ext.state.Stateful = null) {
    super();
  }

  [ExtConfig]
  /**
   * A buffer to be applied if many state events are fired within a short period.
   * @default 100
   */
  public native function get saveDelay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set saveDelay(value:Number):void;

  [ExtConfig]
  /**
   * An array of events that, when fired, should trigger this object to
   * save its state. Defaults to none. <code>stateEvents</code> may be any type
   * of event supported by this object, including browser or custom events
   * (e.g., <code>['click', 'customerchange']</code>).
   *
   * <p>See <code><a href="#!/api/Ext.state.Stateful-cfg-stateful" rel="Ext.state.Stateful-cfg-stateful" class="docClass">stateful</a></code> for an explanation of saving and
   * restoring object state.</p>
   */
  public native function get stateEvents():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set stateEvents(value:Array):void;

  [ExtConfig]
  /**
   * The unique id for this object to use for state management purposes.
   *
   * <p>See <a href="#!/api/Ext.state.Stateful-cfg-stateful" rel="Ext.state.Stateful-cfg-stateful" class="docClass">stateful</a> for an explanation of saving and restoring state.</p>
   */
  public native function get stateId():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set stateId(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A flag which causes the object to attempt to restore the state of
   * internal properties from a saved state on startup. The object must have
   * a <a href="#!/api/Ext.state.Stateful-cfg-stateId" rel="Ext.state.Stateful-cfg-stateId" class="docClass">stateId</a> for state to be managed.
   *
   * <p>Auto-generated ids are not guaranteed to be stable across page loads and
   * cannot be relied upon to save and restore the same state for a object.</p>
   *
   * <p>For state saving to work, the state manager's provider must have been
   * set to an implementation of <a href="#!/api/Ext.state.Provider" rel="Ext.state.Provider" class="docClass">Ext.state.Provider</a> which overrides the
   * <a href="#!/api/Ext.state.Provider-method-set" rel="Ext.state.Provider-method-set" class="docClass">set</a> and <a href="#!/api/Ext.state.Provider-method-get" rel="Ext.state.Provider-method-get" class="docClass">get</a>
   * methods to save and recall name/value pairs. A built-in implementation,
   * <a href="#!/api/Ext.state.CookieProvider" rel="Ext.state.CookieProvider" class="docClass">Ext.state.CookieProvider</a> is available.</p>
   *
   * <p>To set the state provider for the current page:</p>
   *
   * <p>   <a href="#!/api/Ext.state.Manager-method-setProvider" rel="Ext.state.Manager-method-setProvider" class="docClass">Ext.state.Manager.setProvider</a>(new <a href="#!/api/Ext.state.CookieProvider" rel="Ext.state.CookieProvider" class="docClass">Ext.state.CookieProvider</a>({
   *        expires: new Date(new Date().getTime()+(1000<em>60</em>60<em>24</em>7)), //7 days from now
   *    }));</p>
   *
   * <p>A stateful object attempts to save state when one of the events
   * listed in the <a href="#!/api/Ext.state.Stateful-cfg-stateEvents" rel="Ext.state.Stateful-cfg-stateEvents" class="docClass">stateEvents</a> configuration fires.</p>
   *
   * <p>To save state, a stateful object first serializes its state by
   * calling <em><a href="#!/api/Ext.state.Stateful-method-getState" rel="Ext.state.Stateful-method-getState" class="docClass">getState</a></em>.</p>
   *
   * <p>The Component base class implements <a href="#!/api/Ext.state.Stateful-method-getState" rel="Ext.state.Stateful-method-getState" class="docClass">getState</a> to save its width and
   * height within the state only if they were initially configured, and have
   * changed from the configured value.</p>
   *
   * <p>The Panel class saves its collapsed state in addition to that.</p>
   *
   * <p>The Grid class saves its column state and store state (sorters and filters and grouper) in addition to its superclass state.</p>
   *
   * <p>If there is more application state to be save, the developer must provide an implementation which
   * first calls the superclass method to inherit the above behaviour, and then injects new properties
   * into the returned object.</p>
   *
   * <p>The value yielded by <code>getState</code> is passed to <a href="#!/api/Ext.state.Manager-method-set" rel="Ext.state.Manager-method-set" class="docClass">Ext.state.Manager.set</a>
   * which uses the configured <a href="#!/api/Ext.state.Provider" rel="Ext.state.Provider" class="docClass">Ext.state.Provider</a> to save the object
   * keyed by the <a href="#!/api/Ext.state.Stateful-cfg-stateId" rel="Ext.state.Stateful-cfg-stateId" class="docClass">stateId</a>.</p>
   *
   * <p>During construction, a stateful object attempts to <em>restore</em> its state by calling
   * <a href="#!/api/Ext.state.Manager-method-get" rel="Ext.state.Manager-method-get" class="docClass">Ext.state.Manager.get</a> passing the <a href="#!/api/Ext.state.Stateful-cfg-stateId" rel="Ext.state.Stateful-cfg-stateId" class="docClass">stateId</a></p>
   *
   * <p>The resulting object is passed to <a href="#!/api/Ext.state.Stateful-method-applyState" rel="Ext.state.Stateful-method-applyState" class="docClass">applyState</a>*. The default implementation of
   * <a href="#!/api/Ext.state.Stateful-method-applyState" rel="Ext.state.Stateful-method-applyState" class="docClass">applyState</a> simply copies properties into the object, but a developer may
   * override this to support restoration of more complex application state.</p>
   *
   * <p>You can perform extra processing on state save and restore by attaching
   * handlers to the <a href="#!/api/Ext.state.Stateful-event-beforestaterestore" rel="Ext.state.Stateful-event-beforestaterestore" class="docClass">beforestaterestore</a>, <a href="#!/api/Ext.state.Stateful-event-staterestore" rel="Ext.state.Stateful-event-staterestore" class="docClass">staterestore</a>,
   * <a href="#!/api/Ext.state.Stateful-event-beforestatesave" rel="Ext.state.Stateful-event-beforestatesave" class="docClass">beforestatesave</a> and <a href="#!/api/Ext.state.Stateful-event-statesave" rel="Ext.state.Stateful-event-statesave" class="docClass">statesave</a> events. In some simple cases,
   * passing an object for the <code>stateful</code> config may suffice. If an object is
   * provided, the properties of that object are used to include or exclude stateful
   * properties returned by <code>getState</code>. For example:</p>
   *
   * <pre><code> stateful: {
   *      height: false, // never persist the height
   *      width: true    // always persist the width
   *  }
   * </code></pre>
   *
   * <p>The above is roughly equivalent to the following:</p>
   *
   * <pre><code> getState: function () {
   *      var state = this.callParent();
   *
   *      delete state.height;
   *      state.width = this.width;
   *
   *      return state;
   *  }
   * </code></pre>
   * @default false
   * @see #getStateful()
   * @see #setStateful()
   */
  public native function get stateful():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set stateful(value:*):void;

  /**
   * Destroys this stateful object.
   */
  override public native function destroy(...params):void;

  /**
   * Add events that will trigger the state to be saved. If the first argument is an
   * array, each element of that array is the name of a state event. Otherwise, each
   * argument passed to this method is the name of a state event.
   * @param events The event name or an array of event names.
   */
  public native function addStateEvents(events:*):void;

  /**
   * Applies the state to the object. This should be overridden in subclasses to do
   * more complex state operations. By default it applies the state properties onto
   * the current object.
   * @param state The state
   */
  public native function applyState(state:Object):void;

  /**
   * Gets the current state of the object. By default this function returns null,
   * it should be overridden in subclasses to implement methods for getting the state.
   * @return The current state
   */
  public native function getState():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.state.Stateful-cfg-stateful" rel="Ext.state.Stateful-cfg-stateful" class="docClass">stateful</a>.
   */
  public native function getStateful():*;

  /**
   * Conditionally saves a single property from this object to the given state object.
   * The idea is to only save state which has changed from the initial state so that
   * current software settings do not override future software settings. Only those
   * values that are user-changed state should be saved.
   * @param propName The name of the property to save.
   * @param state The state object in to which to save the property.
   * @param stateName The name to use for the property in state.
   * @return True if the property was saved, false if not.
   */
  public native function savePropToState(propName:String, state:Object, stateName:String = null):Boolean;

  /**
   * Gathers additional named properties of the instance and adds their current values
   * to the passed state object.
   * @param propNames The name (or array of names) of the property to save.
   * @param state The state object in to which to save the property values.
   * @return state
   */
  public native function savePropsToState(propNames:*, state:Object):Object;

  /**
   * Saves the state of the object to the persistence store.
   */
  public native function saveState():void;

  /**
   * Sets the value of <a href="#!/api/Ext.state.Stateful-cfg-stateful" rel="Ext.state.Stateful-cfg-stateful" class="docClass">stateful</a>.
   * @param stateful The new value.
   */
  public native function setStateful(stateful:*):void;
}
}