package ext.tree.plugin {
import ext.grid.ViewDropZone;
import ext.plugin.AbstractPlugin;
import ext.view.ViewDragZone;

[Uses("ext.plugin.AbstractPlugin")]
[Uses("ext.Plugin")]
[Uses("ext.Base")]
[Native("Ext.tree.plugin.TreeViewDragDrop", require)]
/**
 * <strong>This event is fired through the <a href="#!/api/Ext.tree.View" rel="Ext.tree.View" class="docClass">TreeView</a> and its owning
 * <a href="#!/api/Ext.tree.Panel" rel="Ext.tree.Panel" class="docClass">Tree</a>. You can add listeners to the tree or tree <a href="#!/api/Ext.tree.Panel-cfg-viewConfig" rel="Ext.tree.Panel-cfg-viewConfig" class="docClass">view config</a> object</strong>
 *
 * <p>Fired when a drop gesture has been triggered by a mouseup event in a valid drop
 * position in the tree view.</p>
 *
 * <p>Returning <code>false</code> to this event signals that the drop gesture was invalid and
 * animates the drag proxy back to the point from which the drag began.</p>
 *
 * <p>The dropHandlers parameter can be used to defer the processing of this event. For
 * example, you can force the handler to wait for the result of a message box
 * confirmation or an asynchronous server call (<em>see the details of the dropHandlers
 * property for more information</em>).</p>
 *
 * <pre><code>tree.on('beforedrop', function(node, data, overModel, dropPosition, dropHandlers) {
 *     // Defer the handling
 *     dropHandlers.wait = true;
 *     <a href="#!/api/Ext.MessageBox-method-confirm" rel="Ext.MessageBox-method-confirm" class="docClass">Ext.MessageBox.confirm</a>('Drop', 'Are you sure', function(btn){
 *         if (btn === 'yes') {
 *             dropHandlers.processDrop();
 *         } else {
 *             dropHandlers.cancelDrop();
 *         }
 *     });
 * });
 * </code></pre>
 *
 * <p>Any other return value continues with the data transfer operation unless the wait
 * property is set.</p>
 * @eventType ext.tree.plugin.events.TreeViewDragDropPlugin_node_data_overModel_dropPosition_dropHandlersEvent.BEFORE_DROP
 */
[Event(name = 'onBeforeDrop', type = 'ext.tree.plugin.events.TreeViewDragDropPlugin_node_data_overModel_dropPosition_dropHandlersEvent')]
/**
 * <strong>This event is fired through the <a href="#!/api/Ext.tree.View" rel="Ext.tree.View" class="docClass">TreeView</a> and its owning
 * <a href="#!/api/Ext.tree.Panel" rel="Ext.tree.Panel" class="docClass">Tree</a>. You can add listeners to the tree or tree <a href="#!/api/Ext.tree.Panel-cfg-viewConfig" rel="Ext.tree.Panel-cfg-viewConfig" class="docClass">view config</a> object</strong>
 *
 * <p>Fired when a drop operation has been completed and the data has been moved or
 * copied.</p>
 * @eventType ext.tree.plugin.events.TreeViewDragDropPlugin_node_data_overModel_dropPositionEvent.DROP
 */
[Event(name = 'onDrop', type = 'ext.tree.plugin.events.TreeViewDragDropPlugin_node_data_overModel_dropPositionEvent')]
/**
 * This plugin provides drag and drop functionality for a <a href="#!/api/Ext.tree.View" rel="Ext.tree.View" class="docClass">TreeView</a>.
 *
 * <p>A specialized instance of <a href="#!/api/Ext.dd.DragZone" rel="Ext.dd.DragZone" class="docClass">DragZone</a> and <a href="#!/api/Ext.dd.DropZone" rel="Ext.dd.DropZone" class="docClass">DropZone</a> are attached to the tree view.  The DropZone will participate in drops
 * from DragZones having the same <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-ddGroup" rel="Ext.tree.plugin.TreeViewDragDrop-cfg-ddGroup" class="docClass">ddGroup</a> including drops from within the same
 * tree.</p>
 *
 * <p>During the drop operation a data object is passed to a participating DropZone's drop
 * handlers.  The drag data object has the following properties:</p>
 *
 * <ul>
 * <li><p><strong>copy:</strong> Boolean <br> The value of <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-copy" rel="Ext.tree.plugin.TreeViewDragDrop-cfg-copy" class="docClass">copy</a>.  Or <code>true</code> if <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-allowCopy" rel="Ext.tree.plugin.TreeViewDragDrop-cfg-allowCopy" class="docClass">allowCopy</a> is true
 * <strong>and</strong> the control key was pressed as the drag operation began.</p></li>
 * <li><p><strong>view:</strong> <a href="#!/api/Ext.tree.View" rel="Ext.tree.View" class="docClass">TreeView</a> <br> The source tree view from which the
 * drag originated</p></li>
 * <li><p><strong>ddel:</strong> HTMLElement <br> The drag proxy element which moves with the cursor</p></li>
 * <li><p><strong>item:</strong> HTMLElement <br> The tree view node upon which the mousedown event was
 * registered</p></li>
 * <li><p><strong>records:</strong> Array <br> An Array of <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a>s
 * representing the selected data being dragged from the source tree view.</p></li>
 * </ul>
 *
 *
 * <p>By adding this plugin to a view, two new events will be fired from the client
 * tree view as well as its owning Tree: <code><a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-event-beforedrop" rel="Ext.tree.plugin.TreeViewDragDrop-event-beforedrop" class="docClass">beforedrop</a></code> and <code><a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-event-drop" rel="Ext.tree.plugin.TreeViewDragDrop-event-drop" class="docClass">drop</a></code>.</p>
 *
 * <pre><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">Ext.data.TreeStore</a>', {
 *     root: {
 *         expanded: true,
 *         children: [{
 *             text: "detention",
 *             leaf: true
 *         }, {
 *             text: "homework",
 *             expanded: true,
 *             children: [{
 *                 text: "book report",
 *                 leaf: true
 *             }, {
 *                 text: "algebra",
 *                 leaf: true
 *             }]
 *         }, {
 *             text: "buy lottery tickets",
 *             leaf: true
 *         }]
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tree.Panel" rel="Ext.tree.Panel" class="docClass">Ext.tree.Panel</a>', {
 *     title: 'Simple Tree',
 *     width: 200,
 *     height: 200,
 *     store: store,
 *     rootVisible: false,
 *     renderTo: document.body,
 *     viewConfig: {
 *         plugins: {
 *             ptype: 'treeviewdragdrop',
 *             dragText: 'Drag and drop to reorganize'
 *         }
 *     }
 * });
 * </code></pre>
 */
public class TreeViewDragDropPlugin extends ext.plugin.AbstractPlugin {
  /**
   * @param config @inheritDoc
   */
  public function TreeViewDragDropPlugin(config:ext.tree.plugin.TreeViewDragDropPlugin = null) {
    super();
  }

  [ExtConfig]
  /**
   * True if drops on the tree container (outside of a specific tree node) are allowed.
   * @default false
   */
  public native function get allowContainerDrops():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowContainerDrops(value:Boolean):void;

  [ExtConfig]
  /**
   * Set as <code>true</code> to allow the user to hold down the control key at the start of the
   * drag operation and copy the dragged records between grids.  Otherwise, dragged
   * records will be moved.
   *
   * <p><strong>Note:</strong> This only applies to records dragged between two different grids with
   * unique stores.</p>
   *
   * <p>See <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-copy" rel="Ext.tree.plugin.TreeViewDragDrop-cfg-copy" class="docClass">copy</a> to enable the copying of all dragged records.</p>
   * @default false
   */
  public native function get allowCopy():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowCopy(value:Boolean):void;

  [ExtConfig]
  /**
   * Allow inserting a dragged node between an expanded parent node and its first child that will become a sibling of
   * the parent when dropped.
   * @default false
   */
  public native function get allowParentInserts():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowParentInserts(value:Boolean):void;

  [ExtConfig]
  /**
   * True if the tree should only allow append drops (use for trees which are sorted).
   * @default false
   */
  public native function get appendOnly():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set appendOnly(value:Boolean):void;

  [ExtConfig]
  /**
   * True to register this container with the Scrollmanager for auto scrolling during drag operations.
   * A <a href="#!/api/Ext.dd.ScrollManager" rel="Ext.dd.ScrollManager" class="docClass">Ext.dd.ScrollManager</a> configuration may also be passed.
   * @default false
   */
  public native function get containerScroll():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set containerScroll(value:*):void;

  [ExtConfig]
  /**
   * Set as <code>true</code> to copy the records from the source grid to the destination drop
   * grid.  Otherwise, dragged records will be moved.
   *
   * <p><strong>Note:</strong> This only applies to records dragged between two different grids with
   * unique stores.</p>
   *
   * <p>See <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-allowCopy" rel="Ext.tree.plugin.TreeViewDragDrop-cfg-allowCopy" class="docClass">allowCopy</a> to allow only control-drag operations to copy records.</p>
   * @default false
   */
  public native function get copy():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set copy(value:Boolean):void;

  [ExtConfig]
  /**
   * A named drag drop group to which this object belongs. If a group is specified, then both the DragZones and
   * DropZone used by this plugin will only interact with other drag drop objects in the same group.
   * @default TreeDD
   */
  public native function get ddGroup():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ddGroup(value:String):void;

  [ExtConfig]
  /**
   * The name of the model field that is used to display the text for the nodes
   * @default text
   */
  public native function get displayField():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set displayField(value:String):void;

  [ExtConfig]
  /**
   * The ddGroup to which the <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-property-dragZone" rel="Ext.tree.plugin.TreeViewDragDrop-property-dragZone" class="docClass">DragZone</a> will belong.
   *
   * <p>This defines which other DropZones the DragZone will interact with. Drag/DropZones only interact with other
   * Drag/DropZones which are members of the same ddGroup.</p>
   */
  public native function get dragGroup():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dragGroup(value:String):void;

  [ExtConfig]
  /**
   * The text to show while dragging.
   *
   * <p>Two placeholders can be used in the text:</p>
   *
   * <ul>
   * <li><code>{0}</code> The number of selected items.</li>
   * <li><code>{1}</code> 's' when more than 1 items (only useful for English).</li>
   * </ul>
   *
   *
   * <p><strong>NOTE:</strong> The node's <a href="#!/api/Ext.tree.Panel-cfg-displayField" rel="Ext.tree.Panel-cfg-displayField" class="docClass">text</a> will be shown
   * when a single node is dragged unless <code>dragText</code> is a simple text string.</p>
   * @default '{0} selected node{1}'
   */
  public native function get dragText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dragText(value:String):void;

  [ExtConfig]
  /**
   * An <a href="#!/api/Ext.view.DragZone" rel="Ext.view.DragZone" class="docClass">DragZone</a> which handles mousedown and dragging of records from the grid.
   */
  public native function get dragZone():ext.view.ViewDragZone;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dragZone(value:ext.view.ViewDragZone):void;

  [ExtConfig]
  /**
   * The ddGroup to which the <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-property-dropZone" rel="Ext.tree.plugin.TreeViewDragDrop-property-dropZone" class="docClass">DropZone</a> will belong.
   *
   * <p>This defines which other DragZones the DropZone will interact with. Drag/DropZones only interact with other
   * Drag/DropZones which are members of the same <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-ddGroup" rel="Ext.tree.plugin.TreeViewDragDrop-cfg-ddGroup" class="docClass">ddGroup</a>.</p>
   */
  public native function get dropGroup():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dropGroup(value:String):void;

  [ExtConfig]
  /**
   * An <a href="#!/api/Ext.grid.ViewDropZone" rel="Ext.grid.ViewDropZone" class="docClass">DropZone</a> which handles mouseover and dropping records in any grid which shares the same <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-dropGroup" rel="Ext.tree.plugin.TreeViewDragDrop-cfg-dropGroup" class="docClass">dropGroup</a>.
   */
  public native function get dropZone():ext.grid.ViewDropZone;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dropZone(value:ext.grid.ViewDropZone):void;

  [ExtConfig]
  /**
   * Set to <code>false</code> to disallow dragging items from the View.
   * @default true
   */
  public native function get enableDrag():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableDrag(value:Boolean):void;

  [ExtConfig]
  /**
   * Set to <code>false</code> to disallow the View from accepting drop gestures.
   * @default true
   */
  public native function get enableDrop():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableDrop(value:Boolean):void;

  [ExtConfig]
  /**
   * The delay in milliseconds to wait before expanding a target tree node while dragging a droppable node over the
   * target.
   * @default 1000
   */
  public native function get expandDelay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set expandDelay(value:Number):void;

  [ExtConfig]
  /**
   * The color to use when visually highlighting the dragged or dropped node (default value is light blue).
   * The color must be a 6 digit hex value, without a preceding '#'. See also <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightOnDrop" rel="Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightOnDrop" class="docClass">nodeHighlightOnDrop</a> and
   * <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightOnRepair" rel="Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightOnRepair" class="docClass">nodeHighlightOnRepair</a>.
   * @default c3daf9
   */
  public native function get nodeHighlightColor():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set nodeHighlightColor(value:String):void;

  [ExtConfig]
  /**
   * Whether or not to highlight any nodes after they are
   * successfully dropped on their target. Defaults to the value of <code><a href="#!/api/Ext-property-enableFx" rel="Ext-property-enableFx" class="docClass">Ext.enableFx</a></code>.
   * See also <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightColor" rel="Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightColor" class="docClass">nodeHighlightColor</a> and <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightOnRepair" rel="Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightOnRepair" class="docClass">nodeHighlightOnRepair</a>.
   */
  public native function get nodeHighlightOnDrop():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set nodeHighlightOnDrop(value:Boolean):void;

  [ExtConfig]
  /**
   * Whether or not to highlight any nodes after they are
   * repaired from an unsuccessful drag/drop. Defaults to the value of <code><a href="#!/api/Ext-property-enableFx" rel="Ext-property-enableFx" class="docClass">Ext.enableFx</a></code>.
   * See also <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightColor" rel="Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightColor" class="docClass">nodeHighlightColor</a> and <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightOnDrop" rel="Ext.tree.plugin.TreeViewDragDrop-cfg-nodeHighlightOnDrop" class="docClass">nodeHighlightOnDrop</a>.
   */
  public native function get nodeHighlightOnRepair():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set nodeHighlightOnRepair(value:Boolean):void;

  [ExtConfig]
  /**
   * Configure as <code>true</code> to sort the target node into the current tree sort order after the dropped node is added.
   * @default false
   */
  public native function get sortOnDrop():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set sortOnDrop(value:Boolean):void;
}
}