package ext.util {
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.util.HashMap", require)]
/**
 * Fires when a new item is added to the hash.
 * @eventType ext.util.events.HashMap_key_valueEvent.ADD
 */
[Event(name = 'onAdd', type = 'ext.util.events.HashMap_key_valueEvent')]
/**
 * Fires when the hash is cleared.
 * @eventType ext.util.events.HashMapEvent.CLEAR
 */
[Event(name = 'onClear', type = 'ext.util.events.HashMapEvent')]
/**
 * Fires when an item is removed from the hash.
 * @eventType ext.util.events.HashMap_key_valueEvent.REMOVE
 */
[Event(name = 'onRemove', type = 'ext.util.events.HashMap_key_valueEvent')]
/**
 * Fires when an item is replaced in the hash.
 * @eventType ext.util.events.HashMap_key_value_oldEvent.REPLACE
 */
[Event(name = 'onReplace', type = 'ext.util.events.HashMap_key_value_oldEvent')]
/**
 * Represents a collection of a set of key and value pairs. Each key in the HashMap
 * must be unique, the same key cannot exist twice. Access to items is provided via
 * the key only. Sample usage:
 *
 * <pre><code>var map = new <a href="#!/api/Ext.util.HashMap" rel="Ext.util.HashMap" class="docClass">Ext.util.HashMap</a>();
 * map.add('key1', 1);
 * map.add('key2', 2);
 * map.add('key3', 3);
 *
 * map.each(function(key, value, length){
 *     console.log(key, value, length);
 * });
 * </code></pre>
 *
 * <p>The HashMap is an unordered class,
 * there is no guarantee when iterating over the items that they will be in any particular
 * order. If this is required, then use a <a href="#!/api/Ext.util.MixedCollection" rel="Ext.util.MixedCollection" class="docClass">Ext.util.MixedCollection</a>.</p>
 */
public class HashMap extends ext.mixin.Observable {
  /**
   * Creates new HashMap.
   * @param config Config object.
   */
  public function HashMap(config:ext.util.HashMap = null) {
    super();
  }

  /**
   * Mutation counter which is incremented upon add and remove.
   * @default 0
   */
  public native function get generation():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A function that is used to retrieve a default key for a passed object.
   * A default is provided that returns the <code>id</code> property on the object. This function is only used
   * if the <code>add</code> method is called with a single argument.
   * @see #getKeyFn()
   * @see #setKeyFn()
   */
  public native function get keyFn():Function;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set keyFn(value:Function):void;

  /**
   * Adds an item to the collection. Fires the <a href="#!/api/Ext.util.HashMap-event-add" rel="Ext.util.HashMap-event-add" class="docClass">add</a> event when complete.
   * @param key The key to associate with the item, or the new item.
   * <p>If a <a href="#!/api/Ext.util.HashMap-method-getKey" rel="Ext.util.HashMap-method-getKey" class="docClass">getKey</a> implementation was specified for this HashMap,
   * or if the key of the stored items is in a property called <code>id</code>,
   * the HashMap will be able to <em>derive</em> the key for the new item.
   * In this case just pass the new item in this parameter.</p>
   * @param o The item to add.
   * @return The item added.
   */
  public native function add(key:*, o:Object = null):Object;

  /**
   * Removes all items from the hash.
   * @return this
   */
  public native function clear():ext.util.HashMap;

  /**
   * Performs a shallow copy on this hash.
   * @return The new hash object.
   */
  public native function clone():ext.util.HashMap;

  /**
   * Checks whether a value exists in the hash.
   * @param value The value to check for.
   * @return True if the value exists in the dictionary.
   */
  public native function contains(value:Object):Boolean;

  /**
   * Checks whether a key exists in the hash.
   * @param key The key to check for.
   * @return True if they key exists in the hash.
   */
  public native function containsKey(key:String):Boolean;

  /**
   * Executes the specified function once for each item in the hash.
   * Returning false from the function will cease iteration.
   * @param fn The function to execute.
   * @param {String} fn.key The key of the item.
   * @param {Number} fn.value The value of the item.
   * @param {Number} fn.length The total number of items in the hash.
   * @param scope The scope to execute in. Defaults to <tt>this</tt>.
   * @return this
   */
  public native function each(fn:Function, scope:Object = null):ext.util.HashMap;

  /**
   * Retrieves an item with a particular key.
   * @param key The key to lookup.
   * @return The value at that key. If it doesn't exist, <code>undefined</code> is returned.
   */
  public native function get(key:String):Object;

  /**
   * Gets the number of items in the hash.
   * @return The number of items in the hash.
   */
  public native function getCount():Number;

  /**
   * Extracts the key from an object. This is a default implementation, it may be overridden
   * @param o The object to get the key from
   * @return The key to use.
   */
  public native function getKey(o:Object):String;

  /**
   * Returns the value of <a href="#!/api/Ext.util.HashMap-cfg-keyFn" rel="Ext.util.HashMap-cfg-keyFn" class="docClass">keyFn</a>.
   */
  public native function getKeyFn():Function;

  /**
   * Return all of the keys in the hash.
   * @return An array of keys.
   */
  public native function getKeys():Array;

  /**
   * Return all of the values in the hash.
   * @return An array of values.
   */
  public native function getValues():Array;

  /**
   * Remove an item from the hash.
   * @param o The value of the item to remove.
   * @return True if the item was successfully removed.
   */
  public native function remove(o:Object):Boolean;

  /**
   * Remove an item from the hash.
   * @param key The key to remove.
   * @return True if the item was successfully removed.
   */
  public native function removeAtKey(key:String):Boolean;

  /**
   * An alias for <a href="#!/api/Ext.util.HashMap-method-removeAtKey" rel="Ext.util.HashMap-method-removeAtKey" class="docClass">removeAtKey</a>
   * @param key The key to remove.
   * @return True if the item was successfully removed.
   */
  public native function removeByKey(key:String):Boolean;

  /**
   * Replaces an item in the hash. If the key doesn't exist, the
   * <a href="#!/api/Ext.util.HashMap-method-add" rel="Ext.util.HashMap-method-add" class="docClass">add</a> method will be used.
   * @param key The key of the item.
   * @param value The new value for the item.
   * @return The new value of the item.
   */
  public native function replace(key:String, value:Object):Object;

  /**
   * Sets the value of <a href="#!/api/Ext.util.HashMap-cfg-keyFn" rel="Ext.util.HashMap-cfg-keyFn" class="docClass">keyFn</a>.
   * @param keyFn The new value.
   */
  public native function setKeyFn(keyFn:Function):void;
}
}