package ext.util {
import ext.Component;

[Uses("ext.util.Focusable")]
[Mixin("ext.util.Focusable")]
/**
 * A mixin for individually focusable things (Components, Widgets, etc)
 * @see ext.util.Focusable
 */
public interface IFocusable {
  /**
   * <code>true</code> for interactive Components, <code>false</code> for static Components.
   * For Containers, this property reflects interactiveness of the
   * Container itself, not its children. See <a href="#!/api/Ext.util.Focusable-method-isFocusable" rel="Ext.util.Focusable-method-isFocusable" class="docClass">isFocusable</a>.
   *
   * <p><strong>Note:</strong> Plain components are static, so not focusable.</p>
   * @default false
   */
  function get focusable():Boolean;

  [ExtConfig]
  /**
   * CSS class that will be added to focused
   * Component, and removed when Component blurs.
   * @default 'x-focus'
   */
  function get focusCls():String;

  [ExtConfig]
  function set focusCls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * DOM tabIndex attribute for this Focusable's
   * focusEl.
   * @see #getTabIndex()
   * @see #setTabIndex()
   */
  function get tabIndex():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  function set tabIndex(value:Number):void;

  /**
   * Try to focus this component.
   *
   * <p>If this component is disabled, a close relation will be targeted for focus instead
   * to keep focus localized for keyboard users.</p>
   * @param selectText If applicable, <code>true</code> to also select all the text in this component, or an array consisting of start and end (defaults to start) position of selection.
   * @param delay Delay the focus this number of milliseconds (true for 10 milliseconds).
   * @param callback Only needed if the <code>delay</code> parameter is used. A function to call upon focus.
   * @param scope Only needed if the <code>delay</code> parameter is used. The scope (<code>this</code> reference) in which to execute the callback.
   * @return The focused Component. Usually <code>this</code> Component. Some Containers may
   * delegate focus to a descendant Component (<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Window</a>s can do this through their
   * <a href="#!/api/Ext.window.Window-cfg-defaultFocus" rel="Ext.window.Window-cfg-defaultFocus" class="docClass">defaultFocus</a> config option. If this component is disabled, a closely
   * related component will be focused and that will be returned.
   */
  function focus(selectText:* = undefined, delay:* = undefined, callback:Function = null, scope:Function = null):ext.Component;

  /**
   * Return the actual tabIndex for this Focusable.
   * @return tabIndex attribute value
   */
  function getTabIndex():Number;

  /**
   * Determine if this Focusable can receive focus at this time.
   *
   * <p>Note that Containers can be non-focusable themselves while delegating
   * focus treatment to a child Component; see
   * <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">#defaultFocus</a> for more information.</p>
   * @param deep Optionally determine if the container itself
   * is focusable, or if container's focus is delegated to a child component
   * and that child is focusable.
   * @default false
   * @return True if component is focusable, false if not.
   */
  function isFocusable(deep:Boolean = false):Boolean;

  /**
   * Set the tabIndex property for this Focusable. If the focusEl
   * is available, set tabIndex attribute on it, too.
   * @param newTabIndex new tabIndex to set
   */
  function setTabIndex(newTabIndex:Number):void;
}
}