package ext.util {
import js.HTMLElement;

[Uses("ext.util.Renderable")]
[Mixin("ext.util.Renderable")]
/**
 * Given a component hierarchy of this:
 *
 * <pre><code> {
 *      xtype: 'panel',
 *      id: 'ContainerA',
 *      layout: 'hbox',
 *      renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *      items: [
 *          {
 *              id: 'ContainerB',
 *              xtype: 'container',
 *              items: [
 *                  { id: 'ComponentA' }
 *              ]
 *          }
 *      ]
 *  }
 * </code></pre>
 *
 * <p>The rendering of the above proceeds roughly like this:</p>
 *
 * <ul>
 * <li>ContainerA's initComponent calls <a href="#!/api/Ext.util.Renderable-method-render" rel="Ext.util.Renderable-method-render" class="docClass">render</a> passing the <code>renderTo</code> property as the
 * container argument.</li>
 * <li><code>render</code> calls the <code>getRenderTree</code> method to get a complete <a href="#!/api/Ext.dom.Helper" rel="Ext.dom.Helper" class="docClass">Ext.dom.Helper</a> spec.</li>
 * <li><code>getRenderTree</code> fires the "beforerender" event and calls the <a href="#!/api/Ext.util.Renderable-method-beforeRender" rel="Ext.util.Renderable-method-beforeRender" class="docClass">beforeRender</a>
 * method. Its result is obtained by calling <a href="#!/api/Ext.util.Renderable-method-getElConfig" rel="Ext.util.Renderable-method-getElConfig" class="docClass">getElConfig</a>.</li>
 * <li>The <a href="#!/api/Ext.util.Renderable-method-getElConfig" rel="Ext.util.Renderable-method-getElConfig" class="docClass">getElConfig</a> method uses the <code>renderTpl</code> and its render data as the content
 * of the <code>autoEl</code> described element.</li>
 * <li>The result of <code>getRenderTree</code> is passed to <a href="#!/api/Ext.dom.Helper-method-append" rel="Ext.dom.Helper-method-append" class="docClass">Ext.dom.Helper.append</a>.</li>
 * <li>The <code>renderTpl</code> contains calls to render things like docked items, container items
 * and raw markup (such as the <code>html</code> or <code>tpl</code> config properties). These calls are to
 * methods added to the <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a> instance by #setupRenderTpl.</li>
 * <li>The #setupRenderTpl method adds methods such as <code>renderItems</code>, <code>renderContent</code>, etc.
 * to the template. These are directed to "doRenderItems", "doRenderContent" etc..</li>
 * <li>The #setupRenderTpl calls traverse from components to their <a href="#!/api/Ext.layout.Layout" rel="Ext.layout.Layout" class="docClass">Ext.layout.Layout</a>
 * object.</li>
 * <li>When a container is rendered, it also has a <code>renderTpl</code>. This is processed when the
 * <code>renderContainer</code> method is called in the component's <code>renderTpl</code>. This call goes to
 * <a href="#!/api/Ext.layout.container.Container-method-doRenderContainer" rel="Ext.layout.container.Container-method-doRenderContainer" class="docClass">Ext.layout.container.Container.doRenderContainer</a>. This method repeats this
 * process for all components in the container.</li>
 * <li>After the top-most component's markup is generated and placed in to the DOM, the next
 * step is to link elements to their components and finish calling the component methods
 * <code>onRender</code> and <code>afterRender</code> as well as fire the corresponding events.</li>
 * <li>The first step in this is to call <a href="#!/api/Ext.util.Renderable-method-finishRender" rel="Ext.util.Renderable-method-finishRender" class="docClass">finishRender</a>. This method descends the
 * component hierarchy and calls <code>onRender</code> and fires the <code>render</code> event. These calls
 * are delivered top-down to approximate the timing of these calls/events from previous
 * versions.</li>
 * <li>During the pass, the component's <code>el</code> is set. Likewise, the <code>renderSelectors</code> and
 * <code>childEls</code> are applied to capture references to the component's elements.</li>
 * <li>These calls are also made on the <a href="#!/api/Ext.layout.container.Container" rel="Ext.layout.container.Container" class="docClass">Ext.layout.container.Container</a> layout to
 * capture its elements. Both of these classes use <a href="#!/api/Ext.util.ElementContainer" rel="Ext.util.ElementContainer" class="docClass">Ext.util.ElementContainer</a> to
 * handle <code>childEls</code> processing.</li>
 * </ul>
 * @see ext.util.Renderable
 */
public interface IRenderable {
  /**
   * Ensures that this component is attached to <code>document.body</code>. If the component was
   * rendered to <a href="#!/api/Ext-method-getDetachedBody" rel="Ext-method-getDetachedBody" class="docClass">Ext.getDetachedBody</a>, then it will be appended to <code>document.body</code>.
   * Any configured position is also restored.
   * @param runLayout True to run the component's layout.
   * @default false
   */
  function ensureAttachedToBody(runLayout:Boolean = false):void;

  /**
   * This function takes the position argument passed to onRender and returns a
   * DOM element that you can use in the insertBefore.
   * @param position Index, element id or element you want
   * to put this component before.
   * @return DOM element that you can use in the insertBefore
   */
  function getInsertPosition(position:*):js.HTMLElement;

  /**
   * Renders the Component into the passed HTML element.
   *
   * <p><strong>If you are using a <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Container</a> object to house this
   * Component, then do not use the render method.</strong></p>
   *
   * <p>A Container's child Components are rendered by that Container's
   * <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a> manager when the Container is first rendered.</p>
   *
   * <p>When creating complex UIs, it is important to remember that sizing and positioning
   * of child items is the responsibility of the Container's <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a>
   * manager.  If you expect child items to be sized in response to user interactions, you must
   * configure the Container with a layout manager which creates and manages the type of layout you
   * have in mind.</p>
   *
   * <p><strong>Omitting the Container's <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a> config means that a basic
   * layout manager is used which does nothing but render child components sequentially into the
   * Container. No sizing or positioning will be performed in this situation.</strong></p>
   * @param container The element this Component should be
   * rendered into. If it is being created from existing markup, this should be omitted.
   * @param position The element ID or DOM node index within the container <strong>before</strong>
   * which this component will be inserted (defaults to appending to the end of the container)
   */
  function render(container:* = undefined, position:* = undefined):void;
}
}