package ext.util {
import ext.Base;
import ext.event.Event;

[Uses("ext.Base")]
[Native("Ext.util.KeyNav", require)]
/**
 * Provides a convenient wrapper for normalized keyboard navigation. KeyNav allows you to bind navigation keys to
 * function calls that will get called when the keys are pressed, providing an easy way to implement custom navigation
 * schemes for any UI component.
 *
 * <p>The following are all of the possible keys that can be implemented: enter, space, left, right, up, down, tab, esc,
 * pageUp, pageDown, del, backspace, home, end.</p>
 *
 * <p>Usage:</p>
 *
 * <pre><code>var nav = new <a href="#!/api/Ext.util.KeyNav" rel="Ext.util.KeyNav" class="docClass">Ext.util.KeyNav</a>({
 *     target : "my-element",
 *     left   : function(e){
 *         this.moveLeft(e.ctrlKey);
 *     },
 *     right  : function(e){
 *         this.moveRight(e.ctrlKey);
 *     },
 *     enter  : function(e){
 *         this.save();
 *     },
 *
 *     // Binding may be a function specifiying fn, scope and defaultEventAction
 *     esc: {
 *         fn: this.onEsc,
 *         defaultEventAction: false
 *     },
 *
 *     // Binding may be keyed by a single character
 *     A: {
 *         ctrl: true,
 *         fn: selectAll
 *     },
 *
 *     // Binding may be keyed by a key code (45 = INSERT)
 *     45: {
 *         fn: doInsert
 *     }
 *     scope : this
 * });
 * </code></pre>
 */
public class KeyNav extends ext.Base {
  /**
   * @param config @inheritDoc
   */
  public function KeyNav(config:ext.util.KeyNav = null) {
    super();
  }

  /**
   * The last key event that this KeyMap handled.
   */
  public native function get lastKeyEvent():ext.event.Event;

  /**
   * @private
   */
  public native function set lastKeyEvent(value:ext.event.Event):void;

  [ExtConfig]
  /**
   * The method to call on the <a href="#!/api/Ext.event.Event" rel="Ext.event.Event" class="docClass">Ext.event.Event</a> after this KeyNav intercepts a key.
   * Valid values are <a href="#!/api/Ext.event.Event-method-stopEvent" rel="Ext.event.Event-method-stopEvent" class="docClass">Ext.event.Event.stopEvent</a>, <a href="#!/api/Ext.event.Event-method-preventDefault" rel="Ext.event.Event-method-preventDefault" class="docClass">Ext.event.Event.preventDefault</a>
   * and <a href="#!/api/Ext.event.Event-method-stopPropagation" rel="Ext.event.Event-method-stopPropagation" class="docClass">Ext.event.Event.stopPropagation</a>.
   *
   * <p>If a falsy value is specified, no method is called on the key event.</p>
   * @default false
   */
  public native function get defaultEventAction():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultEventAction(value:String):void;

  [ExtConfig]
  /**
   * True to disable this KeyNav instance.
   * @default false
   * @see #setDisabled()
   */
  public native function get disabled():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set disabled(value:Boolean):void;

  [ExtConfig]
  /**
   * The event to listen for to pick up key events.
   * @default 'keypress'
   */
  public native function get eventName():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set eventName(value:String):void;

  [ExtConfig]
  /**
   * Handle the keydown event instead of keypress. KeyNav automatically does this for IE since IE does not propagate
   * special keys on keypress, but setting this to true will force other browsers to also handle keydown instead of
   * keypress.
   * @default false
   */
  public native function get forceKeyDown():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set forceKeyDown(value:Boolean):void;

  [ExtConfig]
  /**
   * Configure this as <code>true</code> if there are any input fields within the <a href="#!/api/Ext.util.KeyNav-cfg-target" rel="Ext.util.KeyNav-cfg-target" class="docClass">target</a>, and this KeyNav
   * should not process events from input fields, (<code>&amp;lt;input&gt;, &amp;lt;textarea&gt; and elements with</code>contentEditable="true"`)
   * @default false
   */
  public native function get ignoreInputFields():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ignoreInputFields(value:Boolean):void;

  [ExtConfig]
  /**
   * An optional pre-existing <a href="#!/api/Ext.util.KeyMap" rel="Ext.util.KeyMap" class="docClass">KeyMap</a> to use to listen for key events. If not specified,
   * one is created.
   */
  public native function get keyMap():ext.util.KeyMap;

  [ExtConfig]
  /**
   * @private
   */
  public native function set keyMap(value:ext.util.KeyMap):void;

  [ExtConfig]
  /**
   * The priority to set on this KeyNav's listener. Listeners with a higher priority are fired before those with
   * lower priority.
   */
  public native function get priority():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set priority(value:Number):void;

  [ExtConfig]
  /**
   * An optional event processor function which accepts the argument list provided by the <a href="#!/api/Ext.util.KeyNav-cfg-eventName" rel="Ext.util.KeyNav-cfg-eventName" class="docClass">configured
   * event</a> of the <a href="#!/api/Ext.util.KeyNav-cfg-target" rel="Ext.util.KeyNav-cfg-target" class="docClass">target</a>, and returns a keyEvent for processing by the KeyMap.
   *
   * <p>This may be useful when the <a href="#!/api/Ext.util.KeyNav-cfg-target" rel="Ext.util.KeyNav-cfg-target" class="docClass">target</a> is a Component with s complex event signature. Extra information from
   * the event arguments may be injected into the event for use by the handler functions before returning it.</p>
   */
  public native function get processEvent():Function;

  [ExtConfig]
  /**
   * @private
   */
  public native function set processEvent(value:Function):void;

  [ExtConfig]
  /**
   * The scope (<code>this</code> context) in which the <a href="#!/api/Ext.util.KeyNav-cfg-processEvent" rel="Ext.util.KeyNav-cfg-processEvent" class="docClass">processEvent</a> method is executed.
   * @default this
   */
  public native function get processEventScope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set processEventScope(value:Object):void;

  [ExtConfig]
  /**
   * The object on which to listen for the event specified by the <a href="#!/api/Ext.util.KeyNav-cfg-eventName" rel="Ext.util.KeyNav-cfg-eventName" class="docClass">eventName</a> config option.
   */
  public native function get target():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set target(value:*):void;

  /**
   * Destroy this KeyNav.
   * @param params Pass <code>true</code> to remove the element associated with this KeyNav.
   */
  override public native function destroy(...params):void;

  /**
   * Disables this KeyNav.
   */
  public native function disable():void;

  /**
   * Enables this KeyNav.
   */
  public native function enable():void;

  /**
   * Convenience function for setting disabled/enabled by boolean.
   */
  public native function setDisabled(disabled:Boolean):void;
}
}