package ext.util {
import ext.Base;

[Uses("ext.Base")]
[Native("Ext.util.LocalStorage", require)]
/**
 * This class provides a common API to LocalStorage with backwards compatibility for IE.
 *
 * <p>The primary aspects of this API match the HTML5 standard API except that this class
 * provides a scoping mechanism to isolate property values by instance. This scope is
 * determined from the <code>id</code> property. Further, this class does not expose the number of
 * keys in the store as a <code>length</code> property as this cannot be maintained reliably without
 * undue cost. Instead there is a <code>getKeys</code> method that returns the cached array of keys
 * which is lazily populated on first call.</p>
 *
 * <p>For example:</p>
 *
 * <pre><code> var store = new <a href="#!/api/Ext.util.LocalStorage" rel="Ext.util.LocalStorage" class="docClass">Ext.util.LocalStorage</a>({
 *          id: 'foo'
 *      });
 *
 *  store.setItem('bar', 'stuff');
 *
 *  // Equivalent to:
 *  window.localStorage.setItem('foo-bar', 'stuff');
 * </code></pre>
 *
 * <p>In all cases, the <code>id</code> property is only used by the underlying storage and should not
 * be needed in item access calls or appear when enumerating keys.</p>
 *
 * <p>To continue with the previous example:</p>
 *
 * <pre><code> var keys = store.getKeys();
 *  console.log(keys.length);   // logs 1
 *  console.log(store.key(0));  // logs "bar"
 * </code></pre>
 *
 * <h2>Sharing Instances</h2>
 *
 * <p>The management of the underlying storage can be broken if multiple instances of this
 * class are created with the same <code>id</code> simultaneously. To avoid creating multiple instances
 * with the same <code>id</code>, use the <code>get</code> method and it will lazily create and share a single
 * instance. When you are done with the shared instance, call <code>release</code>.</p>
 *
 * <pre><code> var storage = <a href="#!/api/Ext.util.LocalStorage-static-method-get" rel="Ext.util.LocalStorage-static-method-get" class="docClass">Ext.util.LocalStorage.get</a>('id');
 *
 *  ...
 *
 *  storage.release(); // do not call `destroy` as others may be using this object
 * </code></pre>
 *
 * <p><strong>IMPORTANT:</strong> Do not mix direction instantiation and <code>get</code> with the same <code>id</code>.</p>
 *
 * <h2>Legacy IE</h2>
 *
 * <p>Older IE browsers (specifically IE7 and below) do not support <code>localStorage</code> so this
 * class provides equivalent support using the IE proprietary persistence mechanism: the
 * <a href="http://msdn.microsoft.com/en-us/library/ms531424(VS.85"><code>userData</code> behavior</a>.aspx). In
 * this mode, the <code>id</code> serves as name passed to the <code>load</code> and <code>save</code> methods and as the
 * suffix on the DOM element added to the <code>head</code>.</p>
 *
 * <p>In this mode, writes to the underlying storage are buffered and delayed for performance
 * reasons. This can be managed using the <code>flushDelay</code> config or by directly calling the
 * <code>save</code> method.</p>
 */
public class LocalStorage extends ext.Base {
  /**
   * This will be <code>true</code> if some form of local storage is supported or <code>false</code> if not.
   * @default true
   */
  public static native function get supported():Boolean;

  /**
   * Returns a shared instance of the desired local store given its <code>id</code>. When you
   * are finished with the returned object call the <code>release</code> method:
   *
   * <pre><code> var store = <a href="#!/api/Ext.util.LocalStorage-static-method-get" rel="Ext.util.LocalStorage-static-method-get" class="docClass">Ext.util.LocalStorage.get</a>('foo');
   *
   *  // .. use store
   *
   *  store.release();
   * </code></pre>
   *
   * <p><strong>NOTE:</strong> Do not mix this call with direct instantiation of the same <code>id</code>.</p>
   * @param id The <code>id</code> of the desired instance or a config object
   * with an <code>id</code> property at a minimum.
   * @return The desired instance, created if needed.
   */
  public static native function get(id:*):ext.util.LocalStorage;

  /**
   * @param config @inheritDoc
   */
  public function LocalStorage(config:ext.util.LocalStorage = null) {
    super();
  }

  /**
   * This property is set to <code>true</code> when the instance's <code>destroy</code> method is called.
   * @default false
   */
  override public native function get destroyed():Boolean;

  [ExtConfig]
  /**
   * The number of milliseconds to delay writing changes to the underlying store.
   * This applies only to legacy IE mode and helps batch multiple writes into one
   * flush to storage.
   * @default 1
   */
  public native function get flushDelay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set flushDelay(value:Number):void;

  [ExtConfig]
  /**
   * The unique identifier for this store. This config is required to scope this storage
   * distinctly from others. Ultimately, this is used to set a prefix on all keys.
   */
  public native function get id():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set id(value:String):void;

  [ExtConfig]
  /**
   * Determines if the keys collection is continuously maintained by this object. By
   * default the keys array is lazily fetched from the underlying store and when keys
   * are removed, the array is discarded. This heuristic tends to be safer than doing
   * the linear removal and array rippling to remove keys from the array on each call
   * to <code>removeItem</code>. If the cost of scanning <code>localStorage</code> for keys is high enough
   * and if the keys are frequently needed, then this flag can be set to <code>false</code> to
   * instruct this class to maintain the keys array once it has been determined.
   * @default true
   */
  public native function get lazyKeys():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set lazyKeys(value:Boolean):void;

  [ExtConfig]
  /**
   * The prefix to apply to all <code>localStorage</code> keys manages by this instance. This does
   * not apply to the legacy IE mechanism but only to the HTML5 <code>localStorage</code> keys. If
   * not provided, the <code>id</code> property initializes this value with <code>"id-"</code>.
   * @default ''
   */
  public native function get prefix():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set prefix(value:String):void;

  [ExtConfig]
  /**
   * Specify this as <code>true</code> to use <code>sessionStorage</code> instead of the default <code>localStoreage</code>.
   * This option is not supported in legacy IE browsers (IE 6 and 7) and is ignored.
   * @default false
   */
  public native function get session():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set session(value:Boolean):void;

  /**
   * Destroys this instance and for legacy IE, ensures data is flushed to persistent
   * storage. This method should not be called directly on instances returned by the
   * <code>get</code> method. Call <code>release</code> instead for such instances.
   *
   * <p><em>NOTE:</em> For non-legacy IE browsers, there is no harm in failing to call this
   * method. In legacy IE, however, failing to call this method can result in memory
   * leaks.</p>
   */
  override public native function destroy(...params):void;

  /**
   * Removes all of the keys of this storage.
   * <strong>NOTE:</strong> This method conforms to the standard HTML5 Storage interface.
   */
  public native function clear():void;

  /**
   * Returns the value associated with the given <code>key</code>.
   * <strong>NOTE:</strong> This method conforms to the standard HTML5 Storage interface.
   * @param key The key.
   * @return The value associated with the given <code>key</code>.
   */
  public native function getItem(key:String):String;

  /**
   * Returns the keys for this storage.
   * @return The keys for this storage. This array should be considered as
   * readonly.
   */
  public native function getKeys():Array;

  /**
   * Returns the specified key given its <code>index</code>. These keys have the scoping prefix
   * removed so they match what was passed to <code>setItem</code>.
   * <strong>NOTE:</strong> This method conforms to the standard HTML5 Storage interface.
   * @param index The index of the desired key.
   * @return The key.
   */
  public native function key(index:Number):String;

  /**
   * Call this method when finished with an instance returned by <code>get</code> instead of calling
   * <code>destroy</code>. When the last shared use of this instance calls <code>release</code>, the <code>destroy</code>
   * method is called automatically.
   *
   * <p><em>NOTE:</em> Failing to call this method will result in memory leaks.</p>
   */
  public native function release():void;

  /**
   * Removes the value associated with the given <code>key</code>.
   * <strong>NOTE:</strong> This method conforms to the standard HTML5 Storage interface.
   * @param key The key.
   */
  public native function removeItem(key:String):void;

  /**
   * This method ensures the content of the store is saved to the underlying storage.
   * This applies only to legacy IE. This is not normally called by user code but can
   * be called to ensure storage is saved.
   * @param delay
   * @default 0
   */
  public native function save(delay:Number = 0):void;

  /**
   * Sets the value associated with the given <code>key</code>.
   * <strong>NOTE:</strong> This method conforms to the standard HTML5 Storage interface.
   * @param key The key.
   * @param value The new associated value for <code>key</code>.
   */
  public native function setItem(key:String, value:String):void;
}
}