package ext.util {

[Uses("ext.util.AbstractMixedCollection")]
[Uses("ext.util.Sortable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.util.MixedCollection", require)]
/**
 * Fires before a sort occurs.
 * @eventType ext.util.events.MixedCollection_me_sortersEvent.BEFORE_SORT
 */
[Event(name = 'onBeforeSort', type = 'ext.util.events.MixedCollection_me_sortersEvent')]
/**
 * Represents a collection of a set of key and value pairs. Each key in the MixedCollection
 * must be unique, the same key cannot exist twice. This collection is ordered, items in the
 * collection can be accessed by index  or via the key. Newly added items are added to
 * the end of the collection. This class is similar to <a href="#!/api/Ext.util.HashMap" rel="Ext.util.HashMap" class="docClass">Ext.util.HashMap</a> however it
 * is heavier and provides more functionality. Sample usage:
 *
 * <pre><code>var coll = new <a href="#!/api/Ext.util.MixedCollection" rel="Ext.util.MixedCollection" class="docClass">Ext.util.MixedCollection</a>();
 * coll.add('key1', 'val1');
 * coll.add('key2', 'val2');
 * coll.add('key3', 'val3');
 *
 * console.log(coll.get('key1')); // prints 'val1'
 * console.log(coll.indexOfKey('key3')); // prints 2
 * </code></pre>
 *
 * <p>The MixedCollection also has support for sorting and filtering of the values in the collection.</p>
 *
 * <pre><code>var coll = new <a href="#!/api/Ext.util.MixedCollection" rel="Ext.util.MixedCollection" class="docClass">Ext.util.MixedCollection</a>();
 * coll.add('key1', 100);
 * coll.add('key2', -100);
 * coll.add('key3', 17);
 * coll.add('key4', 0);
 * var biggerThanZero = coll.filterBy(function(value){
 *     return value &gt; 0;
 * });
 * console.log(biggerThanZero.getCount()); // prints 2
 * </code></pre>
 */
public class MixedCollection extends ext.util.AbstractMixedCollection implements ext.util.ISortable {
  /**
   * Creates new MixedCollection.
   * @param config A configuration object.
   * @param {Boolean} [config.allowFunctions] Specify <code>true</code> if the <a href="#!/api/Ext.util.MixedCollection-method-addAll" rel="Ext.util.MixedCollection-method-addAll" class="docClass">addAll</a>
   * function should add function references to the collection.
   * @default false
   * @param {Function} [config.getKey] A function that can accept an item of the type(s) stored in this MixedCollection
   * and return the key value for that item.  This is used when available to look up the key on items that
   * were passed without an explicit key parameter to a MixedCollection method.  Passing this parameter is
   * equivalent to overriding the <a href="#!/api/Ext.util.MixedCollection-method-getKey" rel="Ext.util.MixedCollection-method-getKey" class="docClass">getKey</a> method.
   */
  public function MixedCollection(config:ext.util.MixedCollection = null) {
    super();
  }

  [ExtConfig]
  /**
   * Configure as <code>true</code> if the <a href="#!/api/Ext.util.MixedCollection-method-addAll" rel="Ext.util.MixedCollection-method-addAll" class="docClass">addAll</a> function should add function references to the collection.
   */
  override public native function get allowFunctions():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set allowFunctions(value:Boolean):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Sortable, or subclass thereof.
   * @default true
   */
  public native function get isSortable():Boolean;

  /**
   * @private
   */
  public native function set isSortable(value:Boolean):void;

  [ExtConfig]
  /**
   * The default sort direction to use if one is not specified.
   * @default "ASC"
   */
  public native function get defaultSortDirection():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultSortDirection(value:String):void;

  [ExtConfig]
  /**
   * The maximum number of sorters which may be applied to this Sortable when using the "multi" insertion position
   * when adding sorters.
   *
   * <p>New sorters added using the "multi" insertion position are inserted at the top of the sorters list becoming the
   * new primary sort key.</p>
   *
   * <p>If the sorters collection has grown to longer then <strong><code>multiSortLimit</code></strong>, then the it is trimmed.</p>
   * @default 3
   */
  public native function get multiSortLimit():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set multiSortLimit(value:Number):void;

  [ExtConfig]
  /**
   * The property in each item that contains the data to sort.
   */
  public native function get sortRoot():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set sortRoot(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The initial set of <a href="#!/api/Ext.util.Sorter" rel="Ext.util.Sorter" class="docClass">Sorters</a>.
   *
   * <pre><code>sorters: [{
   *     property: 'age',
   *     direction: 'DESC'
   * }, {
   *     property: 'firstName',
   *     direction: 'ASC'
   * }]
   * </code></pre>
   * @see #getSorters()
   * @see #setSorters()
   */
  public native function get sorters():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  [ArrayElementType("ext.util.Sorter")]
  /**
   * @private
   */
  public native function set sorters(value:Array):void;

  /**
   * Calculates the insertion index of the new item based upon the comparison function passed, or the current sort order.
   * @param newItem The new object to find the insertion position of.
   * @param sorterFn The function to sort by. This is the same as the sorting function
   * passed to <a href="#!/api/Ext.util.MixedCollection-method-sortBy" rel="Ext.util.MixedCollection-method-sortBy" class="docClass">sortBy</a>. It accepts 2 items from this MixedCollection, and returns -1 0, or 1
   * depending on the relative sort positions of the 2 compared items.
   * <p>If omitted, a function <a href="#!/api/Ext.util.MixedCollection-method-generateComparator" rel="Ext.util.MixedCollection-method-generateComparator" class="docClass">generated</a> from the currently defined set of
   * <a href="#!/api/Ext.util.MixedCollection-cfg-sorters" rel="Ext.util.MixedCollection-cfg-sorters" class="docClass">sorters</a> will be used.</p>
   * @return The insertion point to add the new item into this MixedCollection at using <a href="#!/api/Ext.util.MixedCollection-method-insert" rel="Ext.util.MixedCollection-method-insert" class="docClass">insert</a>
   */
  public native function findInsertionIndex(newItem:Object, sorterFn:Function = null):Number;

  /**
   * Returns a comparator function which compares two items and returns -1, 0, or 1 depending
   * on the currently defined set of <a href="#!/api/Ext.util.Sortable-cfg-sorters" rel="Ext.util.Sortable-cfg-sorters" class="docClass">sorters</a>.
   *
   * <p>If there are no <a href="#!/api/Ext.util.Sortable-cfg-sorters" rel="Ext.util.Sortable-cfg-sorters" class="docClass">sorters</a> defined, it returns a function which returns <code>0</code> meaning
   * that no sorting will occur.</p>
   */
  public native function generateComparator():void;

  [ArrayElementType("ext.util.Sorter")]
  /**
   * Returns the value of <a href="#!/api/Ext.util.Sortable-cfg-sorters" rel="Ext.util.Sortable-cfg-sorters" class="docClass">sorters</a>.
   */
  public native function getSorters():Array;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Sortable-cfg-sorters" rel="Ext.util.Sortable-cfg-sorters" class="docClass">sorters</a>.
   * @param sorters The new value.
   */
  public native function setSorters(sorters:Array):void;

  /**
   * Updates the sorters collection and triggers sorting of this Sortable. Example usage:
   *
   * <pre><code>//sort by a single field
   * myStore.sort('myField', 'DESC');
   *
   * //sorting by multiple fields
   * myStore.sort([{
   *     property : 'age',
   *     direction: 'ASC'
   * }, {
   *     property : 'name',
   *     direction: 'DESC'
   * }]);
   * </code></pre>
   *
   * <p>Classes which use this mixin must implement a <strong><code>soSort</code></strong> method which accepts a comparator function computed from
   * the full sorter set which performs the sort in an implementation-specific way.</p>
   *
   * <p>When passing a single string argument to sort, Store maintains a ASC/DESC toggler per field, so this code:</p>
   *
   * <pre><code>store.sort('myField');
   * store.sort('myField');
   * </code></pre>
   *
   * <p>Is equivalent to this code, because Store handles the toggling automatically:</p>
   *
   * <pre><code>store.sort('myField', 'ASC');
   * store.sort('myField', 'DESC');
   * </code></pre>
   * @param sorters Either a string name of one of the fields in this Store's configured <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a>, or an array of sorter configurations.
   * @param direction The overall direction to sort the data by.
   * @default "ASC"
   * @param insertionPosition Where to put the new sorter in the collection of sorters.
   * This may take the following values:
   * <ul>
   * <li><p><code>replace</code> : This means that the new sorter(s) becomes the sole sorter set for this Sortable. This is the most useful call mode
   *         to programatically sort by multiple fields.</p></li>
   * <li><p><code>prepend</code> : This means that the new sorters are inserted as the primary sorters, unchanged, and the sorter list length must be controlled by the developer.</p></li>
   * <li><p><code>multi</code> :  This is mainly useful for implementing intuitive "Sort by this" user interfaces such as the <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">GridPanel</a>'s column sorting UI.</p>
   * <p>  This mode is only supported when passing a property name and a direction.</p>
   * <p>  This means that the new sorter is becomes the primary sorter. If the sorter was <strong>already</strong> the primary sorter, the direction
   *   of sort is toggled if no direction parameter is specified.</p>
   * <p>  The number of sorters maintained is limited by the <a href="#!/api/Ext.util.Sortable-cfg-multiSortLimit" rel="Ext.util.Sortable-cfg-multiSortLimit" class="docClass">multiSortLimit</a> configuration.</p></li>
   * <li><p><code>append</code> : This means that the new sorter becomes the last sorter.</p></li>
   * </ul>
   * @default "replace"
   * @return The new sorters.
   */
  public native function sort(sorters:* = undefined, direction:String = "ASC", insertionPosition:String = "replace"):Array;

  /**
   * Sorts the collection by a single sorter function
   * @param sorterFn The function to sort by
   */
  public native function sortBy(sorterFn:Function):void;

  /**
   * Sorts this collection by <b>key</b>s.
   * @param direction 'ASC' or 'DESC'. Defaults to 'ASC'.
   * @param fn Comparison function that defines the sort order.
   * Defaults to sorting by case insensitive string.
   */
  public native function sortByKey(direction:String = null, fn:Function = null):void;

  /**
   * Gets the first sorter from the sorters collection, excluding
   * any groupers that may be in place
   * @return The sorter, null if none exist
   */
  protected native function getFirstSorter():ext.util.Sorter;

  /**
   * Returns the number of Sorters which apply to this Sortable.
   *
   * <p>May be overridden in subclasses. <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a> in particular overrides
   * this because its groupers must contribute to the sorter count so that the sort method above executes doSort.</p>
   */
  protected native function getSorterCount():uint;
}
}