package ext.util {
import ext.Base;

[Uses("ext.Base")]
[Native("Ext.util.Positionable", require)]
[ExcludeClass]
/**
 * This mixin provides a common interface for objects that can be positioned, e.g.
 * <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Components</a> and <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Elements</a>
 */
public class Positionable extends ext.Base implements ext.util.IPositionable {
  /**
   * Aligns the element with another element relative to the specified anchor points. If
   * the other element is the document it aligns it to the viewport. The position
   * parameter is optional, and can be specified in any one of the following formats:
   *
   * <ul>
   * <li><strong>Blank</strong>: Defaults to aligning the element's top-left corner to the target's
   * bottom-left corner ("tl-bl").</li>
   * <li><strong>One anchor (deprecated)</strong>: The passed anchor position is used as the target
   * element's anchor point.  The element being aligned will position its top-left
   * corner (tl) to that point. <em>This method has been deprecated in favor of the newer
   * two anchor syntax below</em>.</li>
   * <li><strong>Two anchors</strong>: If two values from the table below are passed separated by a dash,
   * the first value is used as the element's anchor point, and the second value is
   * used as the target's anchor point.</li>
   * </ul>
   *
   *
   * <p>In addition to the anchor points, the position parameter also supports the "?"
   * character. If "?" is passed at the end of the position string, the element will
   * attempt to align as specified, but the position will be adjusted to constrain to
   * the viewport if necessary. Note that the element being aligned might be swapped to
   * align to a different position than that specified in order to enforce the viewport
   * constraints. Following are all of the supported anchor positions:</p>
   *
   * <pre>Value  Description
   * -----  -----------------------------
   * tl     The top left corner (default)
   * t      The center of the top edge
   * tr     The top right corner
   * l      The center of the left edge
   * c      In the center of the element
   * r      The center of the right edge
   * bl     The bottom left corner
   * b      The center of the bottom edge
   * br     The bottom right corner
   * </pre>
   *
   *
   * <p>Example Usage:</p>
   *
   * <pre><code>// align el to other-el using the default positioning
   * // ("tl-bl", non-constrained)
   * el.alignTo("other-el");
   *
   * // align the top left corner of el with the top right corner of other-el
   * // (constrained to viewport)
   * el.alignTo("other-el", "tr?");
   *
   * // align the bottom right corner of el with the center left edge of other-el
   * el.alignTo("other-el", "br-l?");
   *
   * // align the center of el with the bottom left corner of other-el and
   * // adjust the x position by -6 pixels (and the y position by 0)
   * el.alignTo("other-el", "c-bl", [-6, 0]);
   * </code></pre>
   *
   * <p><strong>Overridden in Ext.overrides.util.Positionable.</strong></p>
   * @param element The Positionable,
   * HTMLElement, or id of the element to align to.
   * @param position The position to align to
   * @default "tl-bl?"
   * @param offsets Offset the positioning by [x, y]
   * Element animation config object
   */
  public native function alignTo(element:*, position:String = "tl-bl?", offsets:Array = null):void;

  /**
   * Anchors an element to another element and realigns it when the window is resized.
   * @param anchorToEl The Positionable,
   * HTMLElement, or id of the element to align to.
   * @param alignment The position to align to
   * @default "tl-bl?"
   * @param offsets Offset the positioning by [x, y]
   * @param animate true for the default animation or a standard
   * Element animation config object
   * @param monitorScroll True to monitor body scroll and
   * reposition. If this parameter is a number, it is used as the buffer delay in
   * milliseconds.
   * @default 50
   * @param callback The function to call after the animation finishes
   * @return this
   */
  public native function anchorTo(anchorToEl:*, alignment:String = "tl-bl?", offsets:Array = null, animate:* = undefined, monitorScroll:* = 50, callback:Function = null):ext.util.Positionable;

  /**
   * Gets the x,y coordinates to align this element with another element. See
   * <a href="#!/api/Ext.util.Positionable-method-alignTo" rel="Ext.util.Positionable-method-alignTo" class="docClass">alignTo</a> for more info on the supported position values.
   * @param element The Positionable,
   * HTMLElement, or id of the element to align to.
   * @param position The position to align to
   * @default "tl-bl?"
   * @param offsets Offset the positioning by [x, y]
   * @return [x, y]
   */
  public native function getAlignToXY(element:*, position:String = "tl-bl?", offsets:Array = null):Array;

  /**
   * Gets the x,y coordinates specified by the anchor position on the element.
   * @param anchor The specified anchor position.
   * See <a href="#!/api/Ext.util.Positionable-method-alignTo" rel="Ext.util.Positionable-method-alignTo" class="docClass">alignTo</a> for details on supported anchor positions.
   * @default 'tl'
   * @param local True to get the local (element top/left-relative) anchor
   * position instead of page coordinates
   * @param size An object containing the size to use for calculating anchor
   * position {width: (target width), height: (target height)} (defaults to the
   * element's current size)
   * @return [x, y] An array containing the element's x and y coordinates
   */
  public native function getAnchorXY(anchor:String = 'tl', local:Boolean = false, size:Object = null):Array;

  /**
   * Return an object defining the area of this Element which can be passed to
   * <a href="#!/api/Ext.util.Positionable-method-setBox" rel="Ext.util.Positionable-method-setBox" class="docClass">setBox</a> to set another Element's size/location to match this element.
   * @param contentBox If true a box for the content of the element is
   * returned.
   * @param local If true the element's left and top relative to its
   * <code>offsetParent</code> are returned instead of page x/y.
   * @return An object in the format
   * <ul>
   *   <li><code>x</code> : Number
   * The element's X position.
   *   </li>
   *   <li><code>y</code> : Number
   * The element's Y position.
   *   </li>
   *   <li><code>width</code> : Number
   * The element's width.
   *   </li>
   *   <li><code>height</code> : Number
   * The element's height.
   *   </li>
   *   <li><code>bottom</code> : Number
   * The element's lower bound.
   *   </li>
   *   <li><code>right</code> : Number
   * The element's rightmost bound.
   * <p>The returned object may also be addressed as an Array where index 0 contains the X
   * position and index 1 contains the Y position. The result may also be used for
   * <a href="#!/api/Ext.util.Positionable-method-setXY" rel="Ext.util.Positionable-method-setXY" class="docClass">setXY</a></p>
   *   </li>
   * </ul>
   */
  public native function getBox(contentBox:Boolean = false, local:Boolean = false):Object;

  /**
   * Returns a region object that defines the client area of this element.
   *
   * <p>That is, the area <em>within</em> any scrollbars.</p>
   * @return A Region containing "top, left, bottom, right" properties.
   */
  public native function getClientRegion():ext.util.Region;

  /**
   * Returns the content region of this element for purposes of constraining or clipping floating
   * children.  That is the region within the borders and scrollbars, but not within the padding.
   */
  public native function getConstrainRegion():ext.util.Region;

  /**
   * Returns the <code>[X, Y]</code> vector by which this Positionable's element must be translated to make a best
   * attempt to constrain within the passed constraint. Returns <code>false</code> if the element
   * does not need to be moved.
   *
   * <p>Priority is given to constraining the top and left within the constraint.</p>
   *
   * <p>The constraint may either be an existing element into which the element is to be
   * constrained, or a <a href="#!/api/Ext.util.Region" rel="Ext.util.Region" class="docClass">Region</a> into which this element is to be
   * constrained.</p>
   *
   * <p>By default, any extra shadow around the element is <strong>not</strong> included in the constrain calculations - the edges
   * of the element are used as the element bounds. To constrain the shadow within the constrain region, set the
   * <code>constrainShadow</code> property on this element to <code>true</code>.</p>
   * @param constrainTo The
   * Positionable, HTMLElement, element id, or Region into which the element is to be
   * constrained.
   * @param proposedPosition A proposed <code>[X, Y]</code> position to test for validity
   * and to produce a vector for instead of using the element's current position
   * @param proposedSize A proposed <code>[width, height]</code> size to constrain
   * instead of using the element's current size
   * @return <strong>If</strong> the element <em>needs</em> to be translated, an <code>[X, Y]</code>
   * vector by which this element must be translated. Otherwise, <code>false</code>.
   */
  public native function getConstrainVector(constrainTo:* = undefined, proposedPosition:Array = null, proposedSize:Array = null):*;

  /**
   * Returns the x coordinate of this element reletive to its <code>offsetParent</code>.
   * @return The local x coordinate
   */
  public native function getLocalX():Number;

  /**
   * Returns the x and y coordinates of this element relative to its <code>offsetParent</code>.
   * @return The local XY position of the element
   */
  public native function getLocalXY():Array;

  /**
   * Returns the y coordinate of this element reletive to its <code>offsetParent</code>.
   * @return The local y coordinate
   */
  public native function getLocalY():Number;

  /**
   * Returns the offsets of this element from the passed element. The element must both
   * be part of the DOM tree and not have display:none to have page coordinates.
   * @param offsetsTo The Positionable,
   * HTMLElement, or element id to get get the offsets from.
   * @return The XY page offsets (e.g. <code>[100, -200]</code>)
   */
  public native function getOffsetsTo(offsetsTo:*):Array;

  /**
   * Returns a region object that defines the area of this element.
   * @param contentBox If true a box for the content of the element is
   * returned.
   * @return A Region containing "top, left, bottom, right" properties.
   */
  public native function getRegion(contentBox:Boolean = false):ext.util.Region;

  /**
   * Returns the <strong>content</strong> region of this element. That is the region within the borders
   * and padding.
   * @return A Region containing "top, left, bottom, right" member data.
   */
  public native function getViewRegion():ext.util.Region;

  /**
   * Gets the current X position of the DOM element based on page coordinates.
   * @return The X position of the element
   */
  public native function getX():Number;

  /**
   * Gets the current position of the DOM element based on page coordinates.
   * @return The XY position of the element
   */
  public native function getXY():Array;

  /**
   * Gets the current Y position of the DOM element based on page coordinates.
   * @return The Y position of the element
   */
  public native function getY():Number;

  /**
   * Move the element relative to its current position.
   * @param direction Possible values are:
   * <ul>
   * <li><code>"l"</code> (or <code>"left"</code>)</li>
   * <li><code>"r"</code> (or <code>"right"</code>)</li>
   * <li><code>"t"</code> (or <code>"top"</code>, or <code>"up"</code>)</li>
   * <li><code>"b"</code> (or <code>"bottom"</code>, or <code>"down"</code>)</li>
   * </ul>
   * @param distance How far to move the element in pixels
   * @param animate true for the default animation or a standard
   * Element animation config object
   */
  public native function move(direction:String, distance:Number, animate:* = undefined):void;

  /**
   * Remove any anchor to this element. See <a href="#!/api/Ext.util.Positionable-method-anchorTo" rel="Ext.util.Positionable-method-anchorTo" class="docClass">anchorTo</a>.
   * @return this
   */
  public native function removeAnchor():ext.util.Positionable;

  /**
   * Sets the element's box. If animate is true then x, y, width, and height will be
   * animated concurrently.
   * @param box The box to fill {x, y, width, height}
   * @param animate true for the default animation or a standard
   * Element animation config object
   */
  public native function setBox(box:Object, animate:* = undefined):void;

  /**
   * Sets the local x coordinate of this element using CSS style. When used on an
   * absolute positioned element this method is symmetrical with <a href="#!/api/Ext.util.Positionable-method-getLocalX" rel="Ext.util.Positionable-method-getLocalX" class="docClass">getLocalX</a>, but
   * may not be symmetrical when used on a relatively positioned element.
   * @param x The x coordinate. A value of <code>null</code> sets the left style to 'auto'.
   */
  public native function setLocalX(x:Number):void;

  /**
   * Sets the local x and y coordinates of this element using CSS style. When used on an
   * absolute positioned element this method is symmetrical with <a href="#!/api/Ext.util.Positionable-method-getLocalXY" rel="Ext.util.Positionable-method-getLocalXY" class="docClass">getLocalXY</a>, but
   * may not be symmetrical when used on a relatively positioned element.
   * @param x The x coordinate or an array containing [x, y]. A value of
   * <code>null</code> sets the left style to 'auto'
   * @param y The y coordinate, required if x is not an array. A value of
   * <code>null</code> sets the top style to 'auto'
   */
  public native function setLocalXY(x:*, y:Number = NaN):void;

  /**
   * Sets the local y coordinate of this element using CSS style. When used on an
   * absolute positioned element this method is symmetrical with <a href="#!/api/Ext.util.Positionable-method-getLocalY" rel="Ext.util.Positionable-method-getLocalY" class="docClass">getLocalY</a>, but
   * may not be symmetrical when used on a relatively positioned element.
   * @param y The y coordinate. A value of <code>null</code> sets the top style to 'auto'.
   */
  public native function setLocalY(y:Number):void;

  /**
   * Sets the X position of the DOM element based on page coordinates.
   * @param x The X position
   * @param animate True for the default animation, or a standard
   * Element animation config object
   */
  public native function setX(x:Number, animate:* = undefined):void;

  /**
   * Sets the position of the DOM element in page coordinates.
   * @param pos Contains X &amp; Y [x, y] values for new position (coordinates
   * are page-based)
   * @param animate True for the default animation, or a standard
   * Element animation config object
   */
  public native function setXY(pos:Array, animate:* = undefined):void;

  /**
   * Sets the Y position of the DOM element based on page coordinates.
   * @param y The Y position
   * @param animate True for the default animation, or a standard
   * Element animation config object
   */
  public native function setY(y:Number, animate:* = undefined):void;

  /**
   * Translates the passed page coordinates into left/top css values for the element
   * @param x The page x or an array containing [x, y]
   * @param y The page y, required if x is not an array
   * @return An object with left and top properties. e.g.
   * {left: (value), top: (value)}
   */
  public native function translatePoints(x:*, y:Number = NaN):Object;
}
}