package ext.util {
import ext.Base;

[Uses("ext.Base")]
[Native]
/**
 * General purpose inflector class that <a href="#!/api/Ext.util.Inflector-method-pluralize" rel="Ext.util.Inflector-method-pluralize" class="docClass">pluralizes</a>, <a href="#!/api/Ext.util.Inflector-method-singularize" rel="Ext.util.Inflector-method-singularize" class="docClass">singularizes</a> and
 * <a href="#!/api/Ext.util.Inflector-method-ordinalize" rel="Ext.util.Inflector-method-ordinalize" class="docClass">ordinalizes</a> words. Sample usage:
 *
 * <pre><code>//turning singular words into plurals
 * <a href="#!/api/Ext.util.Inflector-method-pluralize" rel="Ext.util.Inflector-method-pluralize" class="docClass">Ext.util.Inflector.pluralize</a>('word'); //'words'
 * <a href="#!/api/Ext.util.Inflector-method-pluralize" rel="Ext.util.Inflector-method-pluralize" class="docClass">Ext.util.Inflector.pluralize</a>('person'); //'people'
 * <a href="#!/api/Ext.util.Inflector-method-pluralize" rel="Ext.util.Inflector-method-pluralize" class="docClass">Ext.util.Inflector.pluralize</a>('sheep'); //'sheep'
 *
 * //turning plurals into singulars
 * <a href="#!/api/Ext.util.Inflector-method-singularize" rel="Ext.util.Inflector-method-singularize" class="docClass">Ext.util.Inflector.singularize</a>('words'); //'word'
 * <a href="#!/api/Ext.util.Inflector-method-singularize" rel="Ext.util.Inflector-method-singularize" class="docClass">Ext.util.Inflector.singularize</a>('people'); //'person'
 * <a href="#!/api/Ext.util.Inflector-method-singularize" rel="Ext.util.Inflector-method-singularize" class="docClass">Ext.util.Inflector.singularize</a>('sheep'); //'sheep'
 *
 * //ordinalizing numbers
 * <a href="#!/api/Ext.util.Inflector-method-ordinalize" rel="Ext.util.Inflector-method-ordinalize" class="docClass">Ext.util.Inflector.ordinalize</a>(11); //"11th"
 * <a href="#!/api/Ext.util.Inflector-method-ordinalize" rel="Ext.util.Inflector-method-ordinalize" class="docClass">Ext.util.Inflector.ordinalize</a>(21); //"21st"
 * <a href="#!/api/Ext.util.Inflector-method-ordinalize" rel="Ext.util.Inflector-method-ordinalize" class="docClass">Ext.util.Inflector.ordinalize</a>(1043); //"1043rd"
 * </code></pre>
 *
 * <h1>Customization</h1>
 *
 * <p>The Inflector comes with a default set of US English pluralization rules. These can be augmented with additional
 * rules if the default rules do not meet your application's requirements, or swapped out entirely for other languages.
 * Here is how we might add a rule that pluralizes "ox" to "oxen":</p>
 *
 * <pre><code><a href="#!/api/Ext.util.Inflector-method-plural" rel="Ext.util.Inflector-method-plural" class="docClass">Ext.util.Inflector.plural</a>(/^(ox)$/i, "$1en");
 * </code></pre>
 *
 * <p>Each rule consists of two items - a regular expression that matches one or more rules, and a replacement string. In
 * this case, the regular expression will only match the string "ox", and will replace that match with "oxen". Here's
 * how we could add the inverse rule:</p>
 *
 * <pre><code><a href="#!/api/Ext.util.Inflector-method-singular" rel="Ext.util.Inflector-method-singular" class="docClass">Ext.util.Inflector.singular</a>(/^(ox)en$/i, "$1");
 * </code></pre>
 *
 * <p>Note that the ox/oxen rules are present by default.</p>
 * <p>Type of singleton Inflector.</p>
 * @see ext.util.#Inflector ext.util.Inflector
 */
public class SInflector extends ext.Base {
  /**
   * Returns the correct <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> name for a given string. Mostly used internally by the data
   * package
   * @param word The word to classify
   * @return The classified version of the word
   */
  public native function classify(word:String):String;

  /**
   * Removes all registered pluralization rules
   */
  public native function clearPlurals():void;

  /**
   * Removes all registered singularization rules
   */
  public native function clearSingulars():void;

  /**
   * Returns true if the given word is transnumeral (the word is its own singular and plural form - e.g. sheep, fish)
   * @param word The word to test
   * @return True if the word is transnumeral
   */
  public native function isTransnumeral(word:String):Boolean;

  /**
   * Ordinalizes a given number by adding a prefix such as 'st', 'nd', 'rd' or 'th' based on the last digit of the
   * number. 21 -> 21st, 22 -> 22nd, 23 -> 23rd, 24 -> 24th etc
   * @param number The number to ordinalize
   * @return The ordinalized number
   */
  public native function ordinalize(number:Number):String;

  /**
   * Adds a new pluralization rule to the Inflector. See the intro docs for more information
   * @param matcher The matcher regex
   * @param replacer The replacement string, which can reference matches from the matcher argument
   */
  public native function plural(matcher:RegExp, replacer:String):void;

  /**
   * Returns the pluralized form of a word (e.g. <a href="#!/api/Ext.util.Inflector-method-pluralize" rel="Ext.util.Inflector-method-pluralize" class="docClass">Ext.util.Inflector.pluralize</a>('word') returns 'words')
   * @param word The word to pluralize
   * @return The pluralized form of the word
   */
  public native function pluralize(word:String):String;

  /**
   * Adds a new singularization rule to the Inflector. See the intro docs for more information
   * @param matcher The matcher regex
   * @param replacer The replacement string, which can reference matches from the matcher argument
   */
  public native function singular(matcher:RegExp, replacer:String):void;

  /**
   * Returns the singularized form of a word (e.g. <a href="#!/api/Ext.util.Inflector-method-singularize" rel="Ext.util.Inflector-method-singularize" class="docClass">Ext.util.Inflector.singularize</a>('words') returns 'word')
   * @param word The word to singularize
   * @return The singularized form of the word
   */
  public native function singularize(word:String):String;
}
}