package ext.util {
import ext.Base;

[Uses("ext.Base")]
[Native("Ext.util.TaskRunner", require)]
/**
 * Provides the ability to execute one or more arbitrary tasks in an asynchronous manner.
 *
 * <p>Generally, you can use the singleton <a href="#!/api/Ext.util.TaskManager" rel="Ext.util.TaskManager" class="docClass">Ext.TaskManager</a>.  Or you can create
 * separate TaskRunner instances to start and stop unique tasks independent of one
 * another.</p>
 *
 * <p>Example usage:</p>
 *
 * <pre class='inline-example '><code>var runner = new <a href="#!/api/Ext.util.TaskRunner" rel="Ext.util.TaskRunner" class="docClass">Ext.util.TaskRunner</a>(),
 *     clock, updateClock, task;
 *
 * clock = <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>().appendChild({
 *     id: 'clock'
 * });
 *
 * // Start a simple clock task that updates a div once per second
 * updateClock = function() {
 *     clock.setHtml(<a href="#!/api/Ext.Date-method-format" rel="Ext.Date-method-format" class="docClass">Ext.Date.format</a>(new Date(), 'g:i:s A'));
 * };
 *
 * task = runner.start({
 *     run: updateClock,
 *     interval: 1000
 * });
 * </code></pre>
 *
 * <p>The equivalent using TaskManager:</p>
 *
 * <pre class='inline-example '><code>var clock, updateClock, task;
 *
 * clock = <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>().appendChild({
 *     id: 'clock'
 * });
 *
 * // Start a simple clock task that updates a div once per second
 * updateClock = function() {
 *     clock.setHtml(<a href="#!/api/Ext.Date-method-format" rel="Ext.Date-method-format" class="docClass">Ext.Date.format</a>(new Date(), 'g:i:s A'));
 * };
 *
 * var task = <a href="#!/api/Ext.util.TaskManager-method-start" rel="Ext.util.TaskManager-method-start" class="docClass">Ext.TaskManager.start</a>({
 *     run: updateClock,
 *     interval: 1000
 * });
 * </code></pre>
 *
 * <p>To end a running task:</p>
 *
 * <pre><code> task.destroy();
 * </code></pre>
 *
 * <p>If a task needs to be started and stopped repeated over time, you can create a
 * <a href="#!/api/Ext.util.TaskRunner.Task" rel="Ext.util.TaskRunner.Task" class="docClass">Task</a> instance.</p>
 *
 * <pre><code>var runner = new <a href="#!/api/Ext.util.TaskRunner" rel="Ext.util.TaskRunner" class="docClass">Ext.util.TaskRunner</a>(),
 *     task;
 *
 * task = runner.newTask({
 *     run: function() {
 *         // useful code
 *     },
 *     interval: 1000
 * });
 *
 * task.start();
 *
 * // ...
 *
 * task.stop();
 *
 * // ...
 *
 * task.start();
 * </code></pre>
 *
 * <p>A re-usable, single-run task can be managed similar to the above:</p>
 *
 * <pre><code>var runner = new <a href="#!/api/Ext.util.TaskRunner" rel="Ext.util.TaskRunner" class="docClass">Ext.util.TaskRunner</a>(),
 *     task;
 *
 * task = runner.newTask({
 *     run: function() {
 *         // useful code
 *     },
 *     interval: 1000,
 *     repeat: 1
 * });
 *
 * task.start();
 *
 * // ...
 *
 * task.stop();
 *
 * // ...
 *
 * task.start();
 * </code></pre>
 *
 * <p>See the <a href="#!/api/Ext.util.TaskRunner-method-start" rel="Ext.util.TaskRunner-method-start" class="docClass">start</a> method for details about how to configure a Task.</p>
 *
 * <p>Also see <a href="#!/api/Ext.util.DelayedTask" rel="Ext.util.DelayedTask" class="docClass">Ext.util.DelayedTask</a>.</p>
 */
public class TaskRunner extends ext.Base {
  /**
   * @param interval The minimum precision in milliseconds supported by
   * this TaskRunner instance. Alternatively, a config object to apply to the new instance.
   * @default 10
   */
  public function TaskRunner(interval:Number = 10) {
    super();
  }

  [ExtConfig]
  /**
   * This may be configured <code>false</code> to inhibit firing of the <a href="#!/api/Ext.GlobalEvents-event-idle" rel="Ext.GlobalEvents-event-idle" class="docClass">idle event</a> after task invocation.
   * @default true
   */
  public native function get fireIdleEvent():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set fireIdleEvent(value:Boolean):void;

  [ExtConfig]
  /**
   * How often to run the task in milliseconds. Defaults to every 10ms.
   * @default 10
   */
  public native function get interval():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set interval(value:Number):void;

  /**
   * Destroys this instance, stopping all tasks that are currently running.
   */
  override public native function destroy(...params):void;

  /**
   * Creates a new <a href="#!/api/Ext.util.TaskRunner.Task" rel="Ext.util.TaskRunner.Task" class="docClass">Task</a> instance. These instances can
   * be easily started and stopped.
   * @param config The config object. For details on the supported properties,
   * see <a href="#!/api/Ext.util.TaskRunner-method-start" rel="Ext.util.TaskRunner-method-start" class="docClass">start</a>.
   * @return <a href="#!/api/Ext.util.TaskRunner.Task" rel="Ext.util.TaskRunner.Task" class="docClass">Ext.util.TaskRunner.Task</a> instance, which can be useful for method chaining.
   */
  public native function newTask(config:Object):ext.util.TaskRunnerTask;

  /**
   * Starts a new task.
   *
   * <p>Before each invocation, Ext injects the property <code>taskRunCount</code> into the task object
   * so that calculations based on the repeat count can be performed.</p>
   *
   * <p>The returned task will contain a <code>destroy</code> method that can be used to destroy the
   * task and cancel further calls. This is equivalent to the <a href="#!/api/Ext.util.TaskRunner-method-stop" rel="Ext.util.TaskRunner-method-stop" class="docClass">stop</a> method.</p>
   * @param task A config object that supports the following properties:
   * @param {Function} task.run The function to execute each time the task is invoked. The
   * function will be called at each interval and passed the <code>args</code> argument if specified,
   * and the current invocation count if not.
   * <p>If a particular scope (<code>this</code> reference) is required, be sure to specify it using
   * the <code>scope</code> argument.</p>
   * @param {Boolean} task.run.return <code>false</code> from this function to terminate the task.
   * @param {Function} task.onError The function to execute in case of unhandled
   * error on task.run.
   * @param {Number} task.interval The frequency in milliseconds with which the task
   * should be invoked.
   * @param {Array} [task.args] An array of arguments to be passed to the function
   * specified by <code>run</code>. If not specified, the current invocation count is passed.
   * @param {Boolean} [task.addCountToArgs] True to add the current invocation count as
   * one of the arguments of args.
   * Note: This only takes effect when args is specified.
   * @default false
   * @param {Object} [task.scope] The scope (<code>this</code> reference) in which to execute the
   * <code>run</code> function. Defaults to the task config object.
   * @param {Number} [task.duration] The length of time in milliseconds to invoke the task
   * before stopping automatically (defaults to indefinite).
   * @param {Number} [task.repeat] The number of times to invoke the task before stopping
   * automatically (defaults to indefinite).
   * @param {Number} [task.fireIdleEvent] If all tasks in a TaskRunner's execution
   * sweep are configured with <code>fireIdleEvent: false</code>, then the
   * <a href="#!/api/Ext.GlobalEvents-event-idle" rel="Ext.GlobalEvents-event-idle" class="docClass">idleEvent</a> is not fired when the TaskRunner's execution
   * sweep finishes.
   * @default true
   * @param {Boolean} [task.fireOnStart] True to run the task immediately instead of
   * waiting for the <em>interval's</em> initial pass to call the <em>run</em> function.
   * @default false
   */
  public native function start(task:Object):Object;

  /**
   * Stops an existing running task.
   * @param task The task to stop
   * @return The task
   */
  public native function stop(task:Object):Object;

  /**
   * Stops all tasks that are currently running.
   */
  public native function stopAll():void;
}
}